/*
 * JDialog.java
 *
 * Created on October 2, 2003, 11:02 AM
 */

/**
 *
 * @author  mdowman
 */

import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.lang.*;

public class MaterialRheologyWindow extends javax.swing.JDialog {
    
    LinkedList rheologyPhases=new LinkedList();
    ArrayList ppmColoursList=new ArrayList(); // This stores all the ppmColours - always one per PPM file - so this must be updated whenever the number of PPM files is changed.

    GenericColourChooserChangeListener genericColourChangeListener=new GenericColourChooserChangeListener();
    
    void disposeOfSubWindows() {
        genericColourChooser.dispose();
        ppmColoursDialog.dispose();
    }
    
    PPMColours getCurrentlySelectedPPMColours() {
        return (PPMColours)ppmColoursList.get(ppmColoursComboBox.getSelectedIndex());
    }
    
    void makeCorrectNumberOfPPMColours() { // This will be called when the number of PPM files defined in the output panel of mainwindow changes. This has to either add or delete PPMColours to make them match the comboBox in the ppmColoursDialog (which will already have been updated before this function is called).
        if (ppmColoursComboBox.getItemCount()>ppmColoursList.size()) { // If there are not enough PPMColours we need to add them.
            while (ppmColoursComboBox.getItemCount()>ppmColoursList.size()) { 
                ppmColoursList.add(new PPMColours());
            }
            
        } else if (ppmColoursComboBox.getItemCount()<ppmColoursList.size()) { // But if there are too many PPm Colours we need to delete some. (Because of the use of while, the ifs here are redundant, but they are left in because they seem to make sense conceptually.)
            while (ppmColoursComboBox.getItemCount()<ppmColoursList.size()) {
                ppmColoursList.remove(ppmColoursList.size()-1); // Remove the last item in the list.
            }
        }
    }
    
    GenericColourChooser genericColourChooser=new GenericColourChooser();
    int numberOfNextRheologyPhase=2; // The first new rheology phase to be added will be number two (not counting the initial one).
    
    void setYieldStressWindowObjects(boolean set) {
        yieldStressLabel1.setEnabled(set);
        yieldStressLabel2.setEnabled(set);
        B0Label.setEnabled(set);
        yieldStressLabel3.setEnabled(set);
        yieldStressMinimumTextBox.setEnabled(set);
        yieldStressMaximumTextBox.setEnabled(set);
        B0TextBox.setEnabled(set);
        BzTextBox.setEnabled(set);
        BpTextBox.setEnabled(set);
        BcTextBox.setEnabled(set);
        E0TextBox.setEnabled(set);
        EDot0TextBox.setEnabled(set);
        EaTextBox.setEnabled(set);
        EDotaTextBox.setEnabled(set);
        EnTextBox.setEnabled(set);
        EDotnTextBox.setEnabled(set);
        ETTextBox.setEnabled(set);
        E0dtTextBox.setEnabled(set);
        yieldStressLabel4.setEnabled(set);
        yieldStressLabel5.setEnabled(set);
        E0Label.setEnabled(set);
        Edot0Label.setEnabled(set);
        yieldStressLabel6.setEnabled(set);
        yieldStressLabel7.setEnabled(set);
        yieldStressLabel8.setEnabled(set);
        yieldStressLabel9.setEnabled(set);
        yieldStressLabel10.setEnabled(set);
        E0dtLabel.setEnabled(set);
    }
    
    void parseAndRemoveRheologySettings(String materialParameterPrefix, MainWindow.Files files, MainWindow.Scaling scaling) {
        files.parseAndSetBoolean(materialParameterPrefix+"reproduction", tracerReproductionCheckBox);
        
        try {
            String name=files.parseAndRemoveEllipsisGUIList(materialParameterPrefix+"name", 1);
            materialNameTextBox.setText(name);
        } catch (ParameterNotFoundException exception) { // If these parameters aren't found it doesn't matter, just do nothing - the materials already have a default colour and name.
        } catch (ParameterSyntaxErrorException exception) { // Just ignore syntax errors - they'll be added to lines with errors anyway.
        } catch (ParameterIncompatibilityException exception) {
        }
        try {
            String colour=files.parseAndRemoveEllipsisGUIList(materialParameterPrefix+"colour", 1);
            genericColourChooser.theColourChooser.setColor(Color.decode(colour));
        } catch (NumberFormatException exception) { // If the colour string can't be decoded, just do nothing - it will keep its default colour.
        } catch (ParameterNotFoundException exception) { // If these parameters aren't found it doesn't matter, just do nothing - the materials already have a default colour and name.
        } catch (ParameterSyntaxErrorException exception) {
        } catch (ParameterIncompatibilityException exception) {
        }
        
        files.parseAndSetReal(materialParameterPrefix+"T_block", blockingTemperatureTextBox, scaling.getEllipsisUnits2KelvinScaler());
        
        parseAndRemovePhases(materialParameterPrefix, files, scaling);
        
        parseAndRemoveYieldStressParameters(materialParameterPrefix, files, scaling); 
        
        parseAndRemovePPMColours(materialParameterPrefix, files);
    }
    
    void parseAndRemoveYieldStressParameters(String materialParameterPrefix, MainWindow.Files files, MainWindow.Scaling scaling) {
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_minimum", yieldStressMinimumTextBox, scaling.getEllipsisUnits2StressScaler()); // Regardless of the setting of the YIELD parameter, always read all of these in - if it's set to off they'll be there just incase it later gets set to ON - if it doesn't get switched they'll just do nothing.
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_maximum", yieldStressMaximumTextBox, scaling.getEllipsisUnits2StressScaler()); 
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_B0", B0TextBox);  // Scaling removed from this and some other parameters.
//         files.parseAndSetReal(materialParameterPrefix+"yield_stress_B0", B0TextBox, scaling.getEllipsisUnits2PascalsScaler()); 
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_Bz", BzTextBox); // Dimensionless
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_Bp", BpTextBox); // Dimensionless
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_Bc", BcTextBox);
//         files.parseAndSetReal(materialParameterPrefix+"yield_stress_Bc", BcTextBox, scaling.getEllipsisUnits2PascalsScaler());
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_E0", E0TextBox); // Dimensionless
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_Edot0", EDot0TextBox);
//         files.parseAndSetReal(materialParameterPrefix+"yield_stress_Edot0", EDot0TextBox, scaling.getEllipsisUnits2PerSecondScaler());
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_Ea", EaTextBox); // Dimensionaless
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_Edota", EDotaTextBox); // Dimensionless
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_En", EnTextBox); // Dimensionless
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_Edotn", EDotnTextBox); // Dimensionless
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_ET", ETTextBox);
         files.parseAndSetReal(materialParameterPrefix+"yield_stress_E0dt", E0dtTextBox); 
//         files.parseAndSetReal(materialParameterPrefix+"yield_stress_ET", ETTextBox, scaling.getEllipsisUnits2KelvinScaler());
//         files.parseAndSetReal(materialParameterPrefix+"yield_stress_E0dt", E0dtTextBox, scaling.getEllipsisUnits2PerSecondScaler()); 
    }
    
    void parseAndRemovePPMColours(String materialParameterPrefix, MainWindow.Files files) {
/*        String PPMColoursSettings=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"Red"); // Store all the settings incase we encounter an error.  No don't do it this way - if a colouring parameter is missing it doesn't matter.
        PPMColoursSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"Green");
        PPMColoursSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"Blue");
        PPMColoursSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"Opacity");
  */      
/* NEW VERSION IS BELOW - difference is it donesn't complain if the number of PPM colours is too many.
 // By the time we get here, the materials will already have been created, and with them the appropriate number of PPMColours, so we just need to set the extant PPMColours here.
        String Red=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Red", ppmColoursList.size()); // If there is a missing parameter it doesn't matter, we just will leave those settings at default (but it is an error if there are the wrong number of items in a list).
        String Green=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Green", ppmColoursList.size());
        String Blue=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Blue", ppmColoursList.size());
        String Opacity=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Opacity", ppmColoursList.size());
            
        String Red_hot=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Red_hot", ppmColoursList.size());
        String Green_hot=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Green_hot", ppmColoursList.size());
        String Blue_hot=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Blue_hot", ppmColoursList.size());
        String Opacity_hot=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Opacity_hot", ppmColoursList.size());

        String Red_strained=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Red_strained", ppmColoursList.size());
        String Green_strained=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Green_strained", ppmColoursList.size());
        String Blue_strained=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Blue_strained", ppmColoursList.size());
        String Opacity_strained=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Opacity_strained", ppmColoursList.size());
 */       
        // By the time we get here, the materials will already have been created, and with them the appropriate number of PPMColours, so we just need to set the extant PPMColours here.
        String Red=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Red"); // If there is a missing parameter it doesn't matter, we just will leave those settings at default (but it is an error if there are the wrong number of items in a list).
        String Green=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Green");
        String Blue=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Blue");
        String Opacity=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Opacity");
            
        String Red_hot=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Red_hot");
        String Green_hot=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Green_hot");
        String Blue_hot=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Blue_hot");
        String Opacity_hot=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Opacity_hot");

        String Red_strained=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Red_strained");
        String Green_strained=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Green_strained");
        String Blue_strained=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Blue_strained");
        String Opacity_strained=files.parseAndRemoveListCatchingExceptions(materialParameterPrefix+"Opacity_strained");

        for (int PPMIndex=1; PPMIndex<=ppmColoursList.size(); ++PPMIndex) { // Go through each PPMColours in turn
            PPMColours currentPPMColours=(PPMColours)ppmColoursList.get(PPMIndex-1); // First get the PPMColours. (-1 is needed because java containers count from 0).
             
            currentPPMColours.coldColour=Colours.RGB2JavaFormat(Statics.safeNthReal(Red, PPMIndex), Statics.safeNthReal(Green, PPMIndex), Statics.safeNthReal(Blue, PPMIndex)); // Then set the colours in it for each parameter. Any errors detected here will NOT get reported - simply because that would be a bit trickier to program.
            currentPPMColours.hotColour=Colours.RGB2JavaFormat(Statics.safeNthReal(Red_hot, PPMIndex), Statics.safeNthReal(Green_hot, PPMIndex), Statics.safeNthReal(Blue_hot, PPMIndex)); // Then set the colours in it for each parameter. Any errors detected here will NOT get reported - simply because that would be a bit trickier to program.
            currentPPMColours.strainedColour=Colours.RGB2JavaFormat(Statics.safeNthReal(Red_strained, PPMIndex), Statics.safeNthReal(Green_strained, PPMIndex), Statics.safeNthReal(Blue_strained, PPMIndex)); // Then set the colours in it for each parameter. Any errors detected here will NOT get reported - simply because that would be a bit trickier to program.
            
            currentPPMColours.coldOpacity=Statics.safeNthReal(Opacity, PPMIndex); // And then set the opacities.
            currentPPMColours.hotOpacity=Statics.safeNthReal(Opacity_hot, PPMIndex); // And then set the opacities.
            currentPPMColours.strainedOpacity=Statics.safeNthReal(Opacity_strained, PPMIndex); // And then set the opacities.
        }
        
        getCurrentlySelectedPPMColours().updateWindowObjects(); // Make sure that the PPMColours are shown properly in the window.
    }
    
    void parseAndRemovePhases(String materialParameterPrefix, MainWindow.Files files, MainWindow.Scaling scaling) {
        String phaseSettings=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"rheol_phase"); // Store all the settings incase we encounter an error.
        phaseSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"rheol_cpts");
        phaseSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"rheol_T_type");
        phaseSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"viscN0");
        phaseSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"viscT1");
        phaseSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"viscT0");
        phaseSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"viscZ");
        phaseSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"viscE");
        phaseSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"density");
        phaseSettings+=files.getAllLinesForSettingsWithThisPrefix(materialParameterPrefix+"Bulk_visc");
        
        try {
            int numberOfPhases; // If this or the next parameter are missing, no problem, just give them their default values.
            try {
                numberOfPhases=files.parseInteger(materialParameterPrefix+"phases"); // We always call this function for a newly created Material, which will have just one phase - but if there are more phases we have to create them.
            } catch (ParameterNotFoundException ex) { 
                numberOfPhases=1;
            }
            
            int numberOfComponents;
            try {
                numberOfComponents=files.parseInteger(materialParameterPrefix+"rheol_cpts");
            } catch (ParameterNotFoundException ex) {
                numberOfComponents=1;
            }
            
            if (numberOfPhases!=numberOfComponents) {
                JOptionPane.showMessageDialog(null, "Ellipsis GUI only allows one rheological component per rheology phase.\nThe phases for "+materialNameTextBox.getText()+" have been ignored.", "Error", JOptionPane.ERROR_MESSAGE);
                throw new ParameterIncompatibilityException();
            }
            parseAndRemoveThisManyPhases(materialParameterPrefix, numberOfPhases, files, scaling);
            
        } catch (ParameterSyntaxErrorException exception) { // If we can't find a valid definition for the number of phases, just assume that there is only one.
            files.linesWithErrors=files.removeLastLineFromString(files.linesWithErrors)+phaseSettings;
        } catch (ParameterNotFoundException exception) {
//                System.out.println("not found"+materialParameterPrefix); // **** Debugging code
            files.linesWithErrors+=phaseSettings;
        } catch (ParameterIncompatibilityException exception) {
//                System.out.println("incom"+materialParameterPrefix); // **** Debugging code
            files.linesWithErrors+=phaseSettings;
        }
    }
    
    void parseAndRemoveThisManyPhases(String materialParameterPrefix, int numberOfPhases, MainWindow.Files files, MainWindow.Scaling scaling) throws ParameterNotFoundException, ParameterSyntaxErrorException, ParameterIncompatibilityException {
        
        String rheol_phase;
        try {
        rheol_phase=files.parseAndRemoveList(materialParameterPrefix+"rheol_phase", numberOfPhases); // N.B. this requires that those parameters which apply only to  Arrehenius must be defined, as otherwise we would have a problem with different list lengths - presumably this is how it should work. No this has been changed = because existing input files often miss some of these off - any missing ones will just be given their default values.
        } catch (ParameterNotFoundException ex) {
            rheol_phase=files.makeListOfIntegers(0, numberOfPhases);
        }
        
        String rheol_T_type;
        try {
            rheol_T_type=files.parseAndRemoveList(materialParameterPrefix+"rheol_T_type", numberOfPhases); // parseAndRemoveList will throw an exception if the parameter isn't found.
        } catch (ParameterNotFoundException ex) {
            rheol_T_type=files.makeListOfIntegers(2, numberOfPhases);
        }
        
        String viscN0;
        try {
            viscN0=files.parseAndRemoveList(materialParameterPrefix+"viscN0", numberOfPhases);
        } catch (ParameterNotFoundException ex) {
            viscN0=files.makeListOfIntegers(1, numberOfPhases);
        }
        
        String viscT1;
        try {
            viscT1=files.parseAndRemoveList(materialParameterPrefix+"viscT1", numberOfPhases);
        } catch (ParameterNotFoundException ex) {
            viscT1=files.makeListOfIntegers(1, numberOfPhases);
        }
        
        String viscT0;
        try {
            viscT0=files.parseAndRemoveList(materialParameterPrefix+"viscT0", numberOfPhases);
        } catch (ParameterNotFoundException ex) {
            viscT0=files.makeListOfIntegers(0, numberOfPhases);
        }
        
        String viscZ;
        try {
            viscZ=files.parseAndRemoveList(materialParameterPrefix+"viscZ", numberOfPhases);
        } catch (ParameterNotFoundException ex) {
            viscZ=files.makeListOfIntegers(0, numberOfPhases);
        }
        
        String viscE;
        try {
            viscE=files.parseAndRemoveList(materialParameterPrefix+"viscE", numberOfPhases);
        } catch (ParameterNotFoundException ex) {
            viscE=files.makeListOfIntegers(0, numberOfPhases);
        }
        
        String density;
        try {
            density=files.parseAndRemoveList(materialParameterPrefix+"density", numberOfPhases);
        } catch (ParameterNotFoundException ex) {
            density=files.makeListOfIntegers(1, numberOfPhases);
        }
        
        String Bulk_visc;
        try {
            Bulk_visc=files.parseAndRemoveList(materialParameterPrefix+"Bulk_visc", numberOfPhases);
        } catch (ParameterNotFoundException ex) {
            Bulk_visc=files.makeListOfIntegers(-1, numberOfPhases);
        }

        LinkedList newRheologyPhases= new LinkedList(); // Create a new list to hold the new rheology phases
        for (int phaseIndex=1; phaseIndex<=numberOfPhases; ++phaseIndex) { //// So one by one, go through all the phases, and add them to the list in the order they occur.
            try {
                RheologyPhase newRheologyPhase=new RheologyPhase();
                newRheologyPhase.rheol_T_type=Statics.nthInteger(rheol_T_type, phaseIndex);
                newRheologyPhase.viscN0=scaling.ellipsisUnits2pascalSeconds(Statics.nthReal(viscN0, phaseIndex));  
                newRheologyPhase.viscT1=scaling.ellipsisUnits2kelvin(Statics.nthReal(viscT1, phaseIndex));
                newRheologyPhase.viscT0=Statics.nthReal(viscT0, phaseIndex); // N.B. scaling removed from this and the next three parameters.
                newRheologyPhase.viscZ=Statics.nthReal(viscZ, phaseIndex);
                newRheologyPhase.viscE=Statics.nthReal(viscE, phaseIndex);
//                newRheologyPhase.viscT0=scaling.ellipsisUnits2kelvin(Statics.nthReal(viscT0, phaseIndex));
//                newRheologyPhase.viscZ=scaling.ellipsisUnits2joulePerMolePerKelvin(Statics.nthReal(viscZ, phaseIndex));
//                newRheologyPhase.viscE=scaling.ellipsisUnits2joulesPerMole(Statics.nthReal(viscE, phaseIndex));
                newRheologyPhase.density=scaling.ellipsisUnits2density(Statics.nthReal(density, phaseIndex));
                newRheologyPhase.bulkViscosity=Statics.nthReal(Bulk_visc, phaseIndex); // Dimensionless
                
                newRheologyPhase.rheol_phase=Statics.nthInteger(rheol_phase, phaseIndex);
                
                newRheologyPhases.add(newRheologyPhase);
            } catch (ParameterSyntaxErrorException exception) {
                files.linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                throw exception; // And then re-throw the exception
            }
        }
        
        // When we get here we know that we've loaded all the phases with no errors - so now just sort them, then add their names to the combo box.
        //        rheologyPhases=new LinkedList(Arrays.sort((RheologyPhase[])newRheologyPhases.toArray())); // Sort the array so the components come in the right order. If we've not encountered an error, we can now overwrite the old rheology phases - ensures that at least one valid rheology phase is always left behind - and we don't get partially loaded rheology phases.
        Collections.sort(newRheologyPhases); // Sort the array so the components come in the right order.
        rheologyPhases=newRheologyPhases; // If we've not encountered an error, we can now overwrite the old rheology phases - ensures that at least one valid rheology phase is always left behind - and we don't get partially loaded rheology phases.
        numberOfNextRheologyPhase=rheologyPhases.size()+1;
        rheologicalComponentsComboBox.removeAllItems();
        for (int phase=1; phase<=rheologyPhases.size(); ++phase) {
            rheologicalComponentsComboBox.addItem("Phase "+phase);
//            System.out.println("adding phase");
        }
        rheologicalComponentsComboBox.setSelectedIndex(0); // Make sure the comb box points to the fist phase, and that the window components are set to that phase.
        ((RheologyPhase)rheologyPhases.get(0)).updateRheologicalComponents();
    }
    
    String getEllipsisFileSettings(String materialParameterPrefix, MainWindow.Scaling scaling) { // Output the settings stored in this window in Ellipsis file format.
        String settings="# Ellipsis GUI parameter:"+materialParameterPrefix+"name=\""+materialNameTextBox.getText()+"\"";
        settings+="\n# Ellipsis GUI parameter:"+materialParameterPrefix+"colour="+materialColourButton.getBackground().getRGB()+"\n\n";
        
        settings+=materialParameterPrefix+"reproduction="+Statics.boolean2OnOff(tracerReproductionCheckBox.isSelected());
        
        settings+=getEllipsisFileSettingsForPhases(materialParameterPrefix, scaling);
        
        settings+=materialParameterPrefix+"T_block="+scaling.kelvin2EllipsisUnits(blockingTemperatureTextBox.getText())+"\n";
        
        settings+=getEllipsisFileSettingsForYieldStress(materialParameterPrefix, scaling);
        
        settings+=getEllipsisFileSettingsForPPMColours(materialParameterPrefix);
        
        return settings;
    }
    
    String getEllipsisFileSettingsForPPMColours(String materialParameterPrefix) {
        String settings="\n# PPM Colours\n\n";
        
        String Red=materialParameterPrefix+"Red=";
        String Green=materialParameterPrefix+"Green=";
        String Blue=materialParameterPrefix+"Blue=";
        String Opacity=materialParameterPrefix+"Opacity=";
        
        String Red_hot=materialParameterPrefix+"Red_hot=";
        String Green_hot=materialParameterPrefix+"Green_hot=";
        String Blue_hot=materialParameterPrefix+"Blue_hot=";
        String Opacity_hot=materialParameterPrefix+"Opacity_hot=";
        
        String Red_strained=materialParameterPrefix+"Red_strained=";
        String Blue_strained=materialParameterPrefix+"Blue_strained=";
        String Green_strained=materialParameterPrefix+"Green_strained=";
        String Opacity_strained=materialParameterPrefix+"Opacity_strained=";
        
        for (int currentColoursIndex=0; currentColoursIndex<ppmColoursList.size(); ++currentColoursIndex) {
            PPMColours currentPPMColours=(PPMColours)ppmColoursList.get(currentColoursIndex);
            
            Red+=Colours.javaFormat2R(currentPPMColours.coldColour)+","; // Colours are stored as objects of the class Color, so they need to be converted to their R, G and B components on output.
            Green+=Colours.javaFormat2G(currentPPMColours.coldColour)+",";
            Blue+=Colours.javaFormat2B(currentPPMColours.coldColour)+",";
            Opacity+=currentPPMColours.coldOpacity+",";
            
            Red_hot+=Colours.javaFormat2R(currentPPMColours.hotColour)+",";
            Green_hot+=Colours.javaFormat2G(currentPPMColours.hotColour)+",";
            Blue_hot+=Colours.javaFormat2B(currentPPMColours.hotColour)+",";
            Opacity_hot+=currentPPMColours.hotOpacity+",";
            
            Red_strained+=Colours.javaFormat2R(currentPPMColours.strainedColour)+",";
            Green_strained+=Colours.javaFormat2G(currentPPMColours.strainedColour)+",";
            Blue_strained+=Colours.javaFormat2B(currentPPMColours.strainedColour)+",";
            Opacity_strained+=currentPPMColours.strainedOpacity+",";
        }

        settings+=Useful.chopAnyTrailingComma(Red)+"\n";
        settings+=Useful.chopAnyTrailingComma(Blue)+"\n";
        settings+=Useful.chopAnyTrailingComma(Green)+"\n";
        settings+=Useful.chopAnyTrailingComma(Opacity)+"\n";

        settings+=Useful.chopAnyTrailingComma(Red_hot)+"\n";
        settings+=Useful.chopAnyTrailingComma(Blue_hot)+"\n";
        settings+=Useful.chopAnyTrailingComma(Green_hot)+"\n";
        settings+=Useful.chopAnyTrailingComma(Opacity_hot)+"\n";

        settings+=Useful.chopAnyTrailingComma(Red_strained)+"\n";
        settings+=Useful.chopAnyTrailingComma(Blue_strained)+"\n";
        settings+=Useful.chopAnyTrailingComma(Green_strained)+"\n";
        settings+=Useful.chopAnyTrailingComma(Opacity_strained)+"\n";

        return settings;
    }
    
    String getEllipsisFileSettingsForYieldStress(String materialParameterPrefix, MainWindow.Scaling scaling) {
        if (yieldStressLabel1.isEnabled()) { // Only output yield stress settings if yield stress is enabled.
            String settings="\n# Yield Stress Parameters\n\n";
            settings+=materialParameterPrefix+"yield_stress_minimum="+scaling.stress2EllipsisUnits(yieldStressMinimumTextBox.getText())+"\n";
            settings+=materialParameterPrefix+"yield_stress_maximum="+scaling.stress2EllipsisUnits(yieldStressMaximumTextBox.getText())+"\n";
            settings+=materialParameterPrefix+"yield_stress_B0="+B0TextBox.getText()+"\n"; // Parsing removed from this and some other parameters.
//            settings+=materialParameterPrefix+"yield_stress_B0="+scaling.pascals2ellipsisUnits(B0TextBox.getText())+"\n";
            settings+=materialParameterPrefix+"yield_stress_Bz="+BzTextBox.getText()+"\n"; // Dimensionless
            settings+=materialParameterPrefix+"yield_stress_Bp="+BpTextBox.getText()+"\n"; // Dimensionless
            settings+=materialParameterPrefix+"yield_stress_Bc="+BcTextBox.getText()+"\n"; 
//            settings+=materialParameterPrefix+"yield_stress_Bc="+scaling.pascals2ellipsisUnits(BcTextBox.getText())+"\n"; 
            settings+=materialParameterPrefix+"yield_stress_E0="+E0TextBox.getText()+"\n"; // Dimensionless
            settings+=materialParameterPrefix+"yield_stress_Edot0="+EDot0TextBox.getText()+"\n"; 
//            settings+=materialParameterPrefix+"yield_stress_Edot0="+scaling.perSecond2ellipsisUnits(EDot0TextBox.getText())+"\n"; 
            settings+=materialParameterPrefix+"yield_stress_Ea="+EaTextBox.getText()+"\n"; // Dimensionless
            settings+=materialParameterPrefix+"yield_stress_Edota="+EDotaTextBox.getText()+"\n"; // Dimensionless
            settings+=materialParameterPrefix+"yield_stress_En="+EnTextBox.getText()+"\n"; // Dimensionless
            settings+=materialParameterPrefix+"yield_stress_Edotn="+EDotnTextBox.getText()+"\n"; // Dimensionless
            settings+=materialParameterPrefix+"yield_stress_ET="+ETTextBox.getText()+"\n";
            settings+=materialParameterPrefix+"yield_stress_E0dt="+E0dtTextBox.getText()+"\n";
//            settings+=materialParameterPrefix+"yield_stress_ET="+scaling.kelvin2EllipsisUnits(ETTextBox.getText())+"\n";
//            settings+=materialParameterPrefix+"yield_stress_E0dt="+scaling.perSecond2ellipsisUnits(E0dtTextBox.getText())+"\n";
            
            return settings;
        } else {
            return "";
        }
    }
    
    String getEllipsisFileSettingsForPhases(String materialParameterPrefix, MainWindow.Scaling scaling) {
        String settings="\n# Rheology Phases\n\n";
        settings+=materialParameterPrefix+"phases="+rheologyPhases.size()+"\n";
        settings+=materialParameterPrefix+"rheol_cpts="+rheologyPhases.size()+"\n";
        
        String rheol_phase=materialParameterPrefix+"rheol_phase="; // We need to build the list up, by going through each phase and adding the appropriate values to each parameter list.
        String rheol_T_type=materialParameterPrefix+"rheol_T_type=";
        String viscN0=materialParameterPrefix+"viscN0=";
        String viscT1=materialParameterPrefix+"viscT1=";
        String viscT0=materialParameterPrefix+"viscT0=";
        String viscZ=materialParameterPrefix+"viscZ=";
        String viscE=materialParameterPrefix+"viscE=";
        String density=materialParameterPrefix+"density=";
        String Bulk_visc=materialParameterPrefix+"Bulk_visc=";
        
        for (int currentPhaseIndex=0; currentPhaseIndex<rheologyPhases.size(); ++currentPhaseIndex) {
            RheologyPhase currentPhase=(RheologyPhase)rheologyPhases.get(currentPhaseIndex);
            
            rheol_phase+=currentPhaseIndex+",";
            rheol_T_type+=currentPhase.rheol_T_type+",";
            viscN0+=scaling.pascalSeconds2ellipsisUnits(currentPhase.viscN0)+",";
            viscT1+=scaling.kelvin2EllipsisUnits(currentPhase.viscT1)+",";
            viscT0+=currentPhase.viscT0+",";
            viscZ+=currentPhase.viscZ+",";
            viscE+=currentPhase.viscE+",";
//            viscT0+=scaling.kelvin2EllipsisUnits(currentPhase.viscT0)+",";
//            viscZ+=scaling.joulePerMolePerKelvin2ellipsisUnits(currentPhase.viscZ)+",";
//            viscE+=scaling.joulesPerMole2ellipsisUnits(currentPhase.viscE)+",";
            density+=scaling.density2EllipsisUnits(currentPhase.density)+",";
            Bulk_visc+=currentPhase.bulkViscosity+","; // Dimensionless
        }
        
        return settings+Useful.chopAnyTrailingComma(rheol_phase)+"\n"+Useful.chopAnyTrailingComma(rheol_T_type)+"\n"+Useful.chopAnyTrailingComma(viscN0)+"\n"+Useful.chopAnyTrailingComma(viscT1)+"\n"+Useful.chopAnyTrailingComma(viscT0)+"\n"+Useful.chopAnyTrailingComma(viscZ)+"\n"+Useful.chopAnyTrailingComma(viscE)+"\n"+Useful.chopAnyTrailingComma(density)+"\n"+Useful.chopAnyTrailingComma(Bulk_visc)+"\n";
    }
    
    private void theColourChooserChange(javax.swing.event.ChangeEvent evt) { // Every time the colour is changed we need to update the colour of the materialColourButton (that is the place where the objects colour is stored).
        Color chosenColour=genericColourChooser.theColourChooser.getColor();
        materialColourButton.setBackground(chosenColour);
        if ((chosenColour.getRed()+chosenColour.getBlue()+chosenColour.getGreen())>382) { // If the chosen colour is light, use black text on the button, otherwise use white text.
            materialColourButton.setForeground(java.awt.Color.black);
        } else {
            materialColourButton.setForeground(java.awt.Color.white);
        }
    }
    
    class RheologyPhase implements Comparable {
        // First initialise everything with it's default setting.
        int rheol_T_type=2; // 1= Grank-Kamenetskii, 2=Arrhenius
        double viscN0=1.0;
        double viscT1=1.0;
        double viscT0=0.0;
        double viscZ=0.0;
        double viscE=0.0;
        double density=1.0;
        double bulkViscosity=-1.0;
        
        int rheol_phase; // This is only used for sorting the phases after they have been read in from a file.
        
        void updateRheologicalComponents() { // Just set all the window objects based on the values in the class.
            if (rheol_T_type==2) {
                arrheniusButton.doClick();
            } else {
                frankKamenetskiiButton.doClick();
            }
            N0TextBox.setText(""+viscN0);
            T1TextBox.setText(""+viscT1);
            T0TextBox.setText(""+viscT0);
            ZTextBox.setText(""+viscZ);
            ETextBox.setText(""+viscE);
            densityTextBox.setText(""+density);
            bulkViscosityTextBox.setText(""+bulkViscosity);
        }
        
        public int compareTo(Object otherPhase) { // This is used by the Arrays.sort to determine sorting order.
            int otherRheol_phase=((RheologyPhase)otherPhase).rheol_phase;
            
            if (otherRheol_phase==rheol_phase)
                return 0;
            if (otherRheol_phase>rheol_phase)
                return -1;
            return 1;
        }
    } // End of class rheology phase
    
    // Constructor - sets all paramaters to default, gives the material a number and a colour.
    public MaterialRheologyWindow(boolean yieldStressIsEnabled, int numberOfThisMaterial) { // N.B. Integer is a reference, so it can be changed.
        initComponents();
        
        setYieldStressWindowObjects(yieldStressIsEnabled); // Either enable or disable yield stress parameter window objects, according to the setting of the text box in the general material properties window.
        
        rheologyPhases.add(new RheologyPhase() ); // Each material will start with one phase.
        ((RheologyPhase)rheologyPhases.get(0)).updateRheologicalComponents(); // So set the window items to reflect those of the first (only) phase.
        
        materialNameTextBox.setText("Material "+numberOfThisMaterial);
        //        numberOfNextMaterial=new Integer(numberOfNextMaterial.intValue()+1);
        
        // Also set each material to an initial colour here, based on the predefined colours in the Colours class.
        Color initialMaterialColour=Colours.nextColour();
        materialColourButton.setBackground(initialMaterialColour);
        if ((initialMaterialColour.getRed()+initialMaterialColour.getBlue()+initialMaterialColour.getGreen())>382) { // If the chosen colour is light, use black text on the button, otherwise use white text.
            materialColourButton.setForeground(java.awt.Color.black);
        } else {
            materialColourButton.setForeground(java.awt.Color.white);
        }
        coldColourButton.setBackground(initialMaterialColour); // And set all the PPM colours to initially be the same as the GUI colour.
        coldColourButton.setForeground(Colours.contrast(initialMaterialColour));
        hotColourButton.setBackground(initialMaterialColour); 
        hotColourButton.setForeground(Colours.contrast(initialMaterialColour));
        strainedColourButton.setBackground(initialMaterialColour); 
        strainedColourButton.setForeground(Colours.contrast(initialMaterialColour));
        
        genericColourChooser.theColourChooser.getSelectionModel().addChangeListener(genericColourChangeListener);
    }
    
    class GenericColourChooserChangeListener implements javax.swing.event.ChangeListener {
        int whatColourChooserIsBeingUsedFor;
        final int materialColour=0; // This is the material colour as used in the GUI, and appearing in the main rheology window.
        final int coldColour=1;
        final int hotColour=2;
        final int strainedColour=3;
        
        public void stateChanged(javax.swing.event.ChangeEvent evt) { // The same colour user is used for varios different things, so we need to work out which one it's being used for.
            if (whatColourChooserIsBeingUsedFor==materialColour) {
                theColourChooserChange(evt);
            } else {
                Color chosenColour=genericColourChooser.theColourChooser.getColor(); // Find the newly selected colour.
                if (whatColourChooserIsBeingUsedFor==coldColour) { // And set the appropriate button.
                    coldColourButton.setBackground(chosenColour);
                    coldColourButton.setForeground(Colours.contrast(chosenColour));
                    getCurrentlySelectedPPMColours().getSettingsFromWindow(); // This records the new colour in the appropriate PPMColours.
                } else if (whatColourChooserIsBeingUsedFor==hotColour) {
                    hotColourButton.setBackground(chosenColour);
                    hotColourButton.setForeground(Colours.contrast(chosenColour));
                    getCurrentlySelectedPPMColours().getSettingsFromWindow();
                } else if (whatColourChooserIsBeingUsedFor==strainedColour) {
                    strainedColourButton.setBackground(chosenColour);
                    strainedColourButton.setForeground(Colours.contrast(chosenColour));
                    getCurrentlySelectedPPMColours().getSettingsFromWindow();
                }
            }
        }
        
    } // End of class genericColourChooserChangeListener
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        rheologyTypeButtonGroup = new javax.swing.ButtonGroup();
        buttonGroup1 = new javax.swing.ButtonGroup();
        ppmColoursDialog = new javax.swing.JDialog();
        ppmColoursComboBox = new javax.swing.JComboBox();
        coldColourButton = new javax.swing.JButton();
        hotColourButton = new javax.swing.JButton();
        strainedColourButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        ppmColoursDialogCloseButton = new javax.swing.JButton();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        coldOpacityNumberBeanBox = new NumberBoxBean();
        hotOpacityNumberBeanBox = new NumberBoxBean();
        strainedOpacityNumberBeanBox = new NumberBoxBean();
        jLabel15 = new javax.swing.JLabel();
        materialNameTextBox = new javax.swing.JTextField();
        tracerReproductionCheckBox = new javax.swing.JCheckBox();
        jPanel1 = new javax.swing.JPanel();
        String[] initialPhase={ "Phase 1" };
        rheologicalComponentsComboBox = new JComboBox(initialPhase);
        insertPhaseButton = new javax.swing.JButton();
        deletePhaseButton = new javax.swing.JButton();
        jLabel16 = new javax.swing.JLabel();
        arrheniusButton = new javax.swing.JRadioButton();
        rheologyTypeButtonGroup.add(arrheniusButton);
        frankKamenetskiiButton = new javax.swing.JRadioButton();
        rheologyTypeButtonGroup.add(frankKamenetskiiButton);
        jPanel8 = new javax.swing.JPanel();
        jLabel17 = new javax.swing.JLabel();
        N0TextBox = new javax.swing.JTextField();
        jLabel18 = new javax.swing.JLabel();
        jLabel19 = new javax.swing.JLabel();
        jLabel20 = new javax.swing.JLabel();
        jLabel21 = new javax.swing.JLabel();
        T1TextBox = new javax.swing.JTextField();
        T0TextBox = new javax.swing.JTextField();
        ZTextBox = new javax.swing.JTextField();
        ETextBox = new javax.swing.JTextField();
        jLabel22 = new javax.swing.JLabel();
        densityTextBox = new javax.swing.JTextField();
        bulkViscosityLabel = new javax.swing.JLabel();
        bulkViscosityTextBox = new javax.swing.JTextField();
        jLabel24 = new javax.swing.JLabel();
        blockingTemperatureTextBox = new javax.swing.JTextField();
        jPanel9 = new javax.swing.JPanel();
        yieldStressLabel1 = new javax.swing.JLabel();
        yieldStressLabel2 = new javax.swing.JLabel();
        B0Label = new javax.swing.JLabel();
        yieldStressLabel3 = new javax.swing.JLabel();
        yieldStressMinimumTextBox = new javax.swing.JTextField();
        yieldStressMaximumTextBox = new javax.swing.JTextField();
        B0TextBox = new javax.swing.JTextField();
        BzTextBox = new javax.swing.JTextField();
        BpTextBox = new javax.swing.JTextField();
        BcTextBox = new javax.swing.JTextField();
        E0TextBox = new javax.swing.JTextField();
        EDot0TextBox = new javax.swing.JTextField();
        EaTextBox = new javax.swing.JTextField();
        EDotaTextBox = new javax.swing.JTextField();
        EnTextBox = new javax.swing.JTextField();
        EDotnTextBox = new javax.swing.JTextField();
        ETTextBox = new javax.swing.JTextField();
        E0dtTextBox = new javax.swing.JTextField();
        yieldStressLabel4 = new javax.swing.JLabel();
        yieldStressLabel5 = new javax.swing.JLabel();
        E0Label = new javax.swing.JLabel();
        Edot0Label = new javax.swing.JLabel();
        yieldStressLabel6 = new javax.swing.JLabel();
        yieldStressLabel7 = new javax.swing.JLabel();
        yieldStressLabel8 = new javax.swing.JLabel();
        yieldStressLabel9 = new javax.swing.JLabel();
        yieldStressLabel10 = new javax.swing.JLabel();
        E0dtLabel = new javax.swing.JLabel();
        thermodynamicAndMeltPropertiesButton = new javax.swing.JButton();
        materialColourButton = new javax.swing.JButton();
        materialRheologyWindowCloseButton = new javax.swing.JButton();
        ppmColoursButton = new javax.swing.JButton();

        ppmColoursDialog.getContentPane().setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        ppmColoursDialog.setTitle("PPM Colours");
        ppmColoursDialog.setBounds(150, 150, 330, 260);
        ppmColoursComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "PPM File 11" }));
        ppmColoursComboBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                ppmColoursComboBoxItemStateChanged(evt);
            }
        });

        ppmColoursDialog.getContentPane().add(ppmColoursComboBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(100, 20, 110, -1));

        coldColourButton.setText("Cold colour");
        coldColourButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                coldColourButtonActionPerformed(evt);
            }
        });

        ppmColoursDialog.getContentPane().add(coldColourButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 60, 130, -1));

        hotColourButton.setText("Hot colour");
        hotColourButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                hotColourButtonActionPerformed(evt);
            }
        });

        ppmColoursDialog.getContentPane().add(hotColourButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 100, 130, -1));

        strainedColourButton.setText("Strained colour");
        strainedColourButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                strainedColourButtonActionPerformed(evt);
            }
        });

        ppmColoursDialog.getContentPane().add(strainedColourButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 140, -1, -1));

        jLabel1.setText("Opacity:");
        ppmColoursDialog.getContentPane().add(jLabel1, new org.netbeans.lib.awtextra.AbsoluteConstraints(170, 60, -1, -1));

        ppmColoursDialogCloseButton.setText("Close");
        ppmColoursDialogCloseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ppmColoursDialogCloseButtonActionPerformed(evt);
            }
        });

        ppmColoursDialog.getContentPane().add(ppmColoursDialogCloseButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 190, -1, -1));

        jLabel2.setText("Opacity:");
        ppmColoursDialog.getContentPane().add(jLabel2, new org.netbeans.lib.awtextra.AbsoluteConstraints(170, 100, -1, -1));

        jLabel3.setText("Opacity:");
        ppmColoursDialog.getContentPane().add(jLabel3, new org.netbeans.lib.awtextra.AbsoluteConstraints(170, 140, -1, -1));

        coldOpacityNumberBeanBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                coldOpacityNumberBeanBoxFocusLost(evt);
            }
        });

        ppmColoursDialog.getContentPane().add(coldOpacityNumberBeanBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(230, 60, 60, -1));

        hotOpacityNumberBeanBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                hotOpacityNumberBeanBoxFocusLost(evt);
            }
        });

        ppmColoursDialog.getContentPane().add(hotOpacityNumberBeanBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(230, 100, 60, -1));

        strainedOpacityNumberBeanBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                strainedOpacityNumberBeanBoxFocusLost(evt);
            }
        });

        ppmColoursDialog.getContentPane().add(strainedOpacityNumberBeanBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(230, 140, 60, -1));

        getContentPane().setLayout(null);

        setTitle("Material Rheology");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jLabel15.setText("Name:");
        jLabel15.setToolTipText("The name which will be used to identify this material within the gui");
        getContentPane().add(jLabel15);
        jLabel15.setBounds(10, 0, 50, 37);

        getContentPane().add(materialNameTextBox);
        materialNameTextBox.setBounds(50, 10, 190, 19);

        tracerReproductionCheckBox.setText("Allow tracer reproduction");
        getContentPane().add(tracerReproductionCheckBox);
        tracerReproductionCheckBox.setBounds(10, 40, 190, 23);

        jPanel1.setLayout(null);

        jPanel1.setBorder(new javax.swing.border.TitledBorder("Rheological Components"));
        jPanel1.setToolTipText("");
        rheologicalComponentsComboBox.setToolTipText("Select the phase to display (usually only a single phase is used)");
        rheologicalComponentsComboBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                rheologicalComponentsComboBoxItemStateChanged(evt);
            }
        });

        jPanel1.add(rheologicalComponentsComboBox);
        rheologicalComponentsComboBox.setBounds(10, 20, 110, 24);

        insertPhaseButton.setText("Insert Phase");
        insertPhaseButton.setToolTipText("Insert a new phase after the phase which is currently selected");
        insertPhaseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                insertPhaseButtonActionPerformed(evt);
            }
        });

        jPanel1.add(insertPhaseButton);
        insertPhaseButton.setBounds(130, 20, 110, 25);

        deletePhaseButton.setText("Delete Phase");
        deletePhaseButton.setToolTipText("Delete the phase which is currently selected");
        deletePhaseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deletePhaseButtonActionPerformed(evt);
            }
        });

        jPanel1.add(deletePhaseButton);
        deletePhaseButton.setBounds(250, 20, 120, 25);

        jLabel16.setText("Rheology Type:");
        jPanel1.add(jLabel16);
        jLabel16.setBounds(10, 50, 100, 15);

        arrheniusButton.setText("Arrhenius");
        arrheniusButton.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                arrheniusButtonItemStateChanged(evt);
            }
        });

        jPanel1.add(arrheniusButton);
        arrheniusButton.setBounds(110, 50, 86, 23);

        frankKamenetskiiButton.setText("Frank-Kamenetskii");
        frankKamenetskiiButton.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                frankKamenetskiiButtonItemStateChanged(evt);
            }
        });

        jPanel1.add(frankKamenetskiiButton);
        frankKamenetskiiButton.setBounds(200, 50, 150, 23);

        jPanel8.setLayout(null);

        jPanel8.setBorder(new javax.swing.border.TitledBorder("Viscosity Parameters"));
        jLabel17.setText("N0:");
        jPanel8.add(jLabel17);
        jLabel17.setBounds(10, 20, 20, 15);

        N0TextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                N0TextBoxFocusLost(evt);
            }
        });

        jPanel8.add(N0TextBox);
        N0TextBox.setBounds(30, 20, 40, 19);

        jLabel18.setText("T1:");
        jPanel8.add(jLabel18);
        jLabel18.setBounds(10, 40, 19, 15);

        jLabel19.setText("T0:");
        jPanel8.add(jLabel19);
        jLabel19.setBounds(10, 60, 19, 15);

        jLabel20.setText("Z:");
        jPanel8.add(jLabel20);
        jLabel20.setBounds(10, 80, 11, 15);

        jLabel21.setText("E:");
        jPanel8.add(jLabel21);
        jLabel21.setBounds(10, 100, 10, 15);

        T1TextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                T1TextBoxFocusLost(evt);
            }
        });

        jPanel8.add(T1TextBox);
        T1TextBox.setBounds(30, 40, 40, 19);

        T0TextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                T0TextBoxFocusLost(evt);
            }
        });

        jPanel8.add(T0TextBox);
        T0TextBox.setBounds(30, 60, 40, 19);

        ZTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                ZTextBoxFocusLost(evt);
            }
        });

        jPanel8.add(ZTextBox);
        ZTextBox.setBounds(30, 80, 40, 19);

        ETextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                ETextBoxFocusLost(evt);
            }
        });

        jPanel8.add(ETextBox);
        ETextBox.setBounds(30, 100, 40, 19);

        jPanel1.add(jPanel8);
        jPanel8.setBounds(10, 70, 160, 130);

        jLabel22.setText("Density:");
        jPanel1.add(jLabel22);
        jLabel22.setBounds(180, 80, 60, 15);

        densityTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                densityTextBoxFocusLost(evt);
            }
        });

        jPanel1.add(densityTextBox);
        densityTextBox.setBounds(240, 80, 70, 19);

        bulkViscosityLabel.setToolTipText("Set this to -1 to obtain an infinite viscosity");
        bulkViscosityLabel.setText("Bulk Viscosity ("+Greek.lambda+"):");
        jPanel1.add(bulkViscosityLabel);
        bulkViscosityLabel.setBounds(180, 110, 160, 15);

        bulkViscosityTextBox.setToolTipText("Set this to -1 to obtain an infinite viscosity");
        bulkViscosityTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                bulkViscosityTextBoxFocusLost(evt);
            }
        });

        jPanel1.add(bulkViscosityTextBox);
        bulkViscosityTextBox.setBounds(300, 110, 70, 19);

        getContentPane().add(jPanel1);
        jPanel1.setBounds(10, 110, 380, 210);

        jLabel24.setText("Blocking Temperature");
        getContentPane().add(jLabel24);
        jLabel24.setBounds(10, 70, 140, 15);

        blockingTemperatureTextBox.setText("-1e38");
        blockingTemperatureTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                blockingTemperatureTextBoxFocusLost(evt);
            }
        });

        getContentPane().add(blockingTemperatureTextBox);
        blockingTemperatureTextBox.setBounds(150, 70, 42, 19);

        jPanel9.setLayout(null);

        jPanel9.setBorder(new javax.swing.border.TitledBorder("Yield Stress"));
        yieldStressLabel1.setText("Minimum:");
        yieldStressLabel1.setToolTipText("Minimum yield stress for plastic deformation");
        jPanel9.add(yieldStressLabel1);
        yieldStressLabel1.setBounds(10, 20, 170, 15);

        yieldStressLabel2.setText("Maximum:");
        yieldStressLabel2.setToolTipText("Maximum yield stress for plastic deformation");
        jPanel9.add(yieldStressLabel2);
        yieldStressLabel2.setBounds(10, 40, 170, 15);

        B0Label.setText("B0:");
        B0Label.setToolTipText("Cohesion parameter");
        B0Label.setText("B"+Greek.sub0+":");
        jPanel9.add(B0Label);
        B0Label.setBounds(10, 60, 170, 15);

        yieldStressLabel3.setText("Bz:");
        yieldStressLabel3.setToolTipText("Depth dependence parameter");
        jPanel9.add(yieldStressLabel3);
        yieldStressLabel3.setBounds(10, 80, 140, 15);

        yieldStressMinimumTextBox.setText("1e-32");
        yieldStressMinimumTextBox.setToolTipText("Minimum yield stress for plastic deformation");
        yieldStressMinimumTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                yieldStressMinimumTextBoxFocusLost(evt);
            }
        });

        jPanel9.add(yieldStressMinimumTextBox);
        yieldStressMinimumTextBox.setBounds(150, 20, 60, 19);

        yieldStressMaximumTextBox.setText("1e32");
        yieldStressMaximumTextBox.setToolTipText("Maximum yield stress for plastic deformation");
        yieldStressMaximumTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                yieldStressMaximumTextBoxFocusLost(evt);
            }
        });

        jPanel9.add(yieldStressMaximumTextBox);
        yieldStressMaximumTextBox.setBounds(150, 40, 60, 19);

        B0TextBox.setText("1e32");
        B0TextBox.setToolTipText("Cohesion parameter");
        B0TextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                B0TextBoxFocusLost(evt);
            }
        });

        jPanel9.add(B0TextBox);
        B0TextBox.setBounds(150, 60, 60, 19);

        BzTextBox.setText("0");
        BzTextBox.setToolTipText("Depth dependence parameter");
        BzTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                BzTextBoxFocusLost(evt);
            }
        });

        jPanel9.add(BzTextBox);
        BzTextBox.setBounds(150, 80, 60, 19);

        BpTextBox.setText("0");
        BpTextBox.setToolTipText("Friction coefficient");
        BpTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                BpTextBoxFocusLost(evt);
            }
        });

        jPanel9.add(BpTextBox);
        BpTextBox.setBounds(150, 100, 60, 19);

        BcTextBox.setText("1e32");
        BcTextBox.setToolTipText("Tensional cut-off for yielding");
        BcTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                BcTextBoxFocusLost(evt);
            }
        });

        jPanel9.add(BcTextBox);
        BcTextBox.setBounds(150, 120, 60, 19);

        E0TextBox.setText("1e32");
        E0TextBox.setToolTipText("Strain weakening");
        E0TextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                E0TextBoxFocusLost(evt);
            }
        });

        jPanel9.add(E0TextBox);
        E0TextBox.setBounds(150, 140, 60, 20);

        EDot0TextBox.setText("0");
        EDot0TextBox.setToolTipText("Strain rate weakening");
        EDot0TextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                EDot0TextBoxFocusLost(evt);
            }
        });

        jPanel9.add(EDot0TextBox);
        EDot0TextBox.setBounds(330, 20, 60, 19);

        EaTextBox.setText("0");
        EaTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                EaTextBoxFocusLost(evt);
            }
        });

        jPanel9.add(EaTextBox);
        EaTextBox.setBounds(330, 40, 60, 19);

        EDotaTextBox.setText("1");
        EDotaTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                EDotaTextBoxFocusLost(evt);
            }
        });

        jPanel9.add(EDotaTextBox);
        EDotaTextBox.setBounds(330, 60, 60, 19);

        EnTextBox.setText("0");
        EnTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                EnTextBoxFocusLost(evt);
            }
        });

        jPanel9.add(EnTextBox);
        EnTextBox.setBounds(330, 80, 60, 19);

        EDotnTextBox.setText("0");
        EDotnTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                EDotnTextBoxFocusLost(evt);
            }
        });

        jPanel9.add(EDotnTextBox);
        EDotnTextBox.setBounds(330, 100, 60, 19);

        ETTextBox.setText("1");
        ETTextBox.setToolTipText("T above which strain weakening is reset");
        ETTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                ETTextBoxFocusLost(evt);
            }
        });

        jPanel9.add(ETTextBox);
        ETTextBox.setBounds(330, 120, 60, 19);

        E0dtTextBox.setText("0");
        E0dtTextBox.setToolTipText("Time rate of strain reduction");
        E0dtTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                E0dtTextBoxFocusLost(evt);
            }
        });

        jPanel9.add(E0dtTextBox);
        E0dtTextBox.setBounds(330, 140, 60, 19);

        yieldStressLabel4.setText("Bp:");
        yieldStressLabel4.setToolTipText("Friction coefficient");
        jPanel9.add(yieldStressLabel4);
        yieldStressLabel4.setBounds(10, 100, 140, 15);

        yieldStressLabel5.setText("Bc:");
        yieldStressLabel5.setToolTipText("Tensional cut-off for yielding");
        jPanel9.add(yieldStressLabel5);
        yieldStressLabel5.setBounds(10, 120, 140, 15);

        E0Label.setText("E0");
        E0Label.setToolTipText("Strain weakening");
        E0Label.setText("E"+Greek.sub0+":");
        jPanel9.add(E0Label);
        E0Label.setBounds(10, 140, 140, 15);

        Edot0Label.setText("Edot0:");
        Edot0Label.setToolTipText("Strain rate weakening");
        Edot0Label.setText("Edot"+Greek.sub0+":");
        jPanel9.add(Edot0Label);
        Edot0Label.setBounds(220, 20, 110, 15);

        yieldStressLabel6.setText("Ea:");
        yieldStressLabel6.setToolTipText("");
        jPanel9.add(yieldStressLabel6);
        yieldStressLabel6.setBounds(220, 40, 110, 15);

        yieldStressLabel7.setText("Edota:");
        yieldStressLabel7.setToolTipText("");
        jPanel9.add(yieldStressLabel7);
        yieldStressLabel7.setBounds(220, 60, 110, 15);

        yieldStressLabel8.setText("En:");
        yieldStressLabel8.setToolTipText("Set this to -1 to obtain an infinite viscosity");
        jPanel9.add(yieldStressLabel8);
        yieldStressLabel8.setBounds(220, 80, 110, 15);

        yieldStressLabel9.setText("Edotn:");
        yieldStressLabel9.setToolTipText("Set this to -1 to obtain an infinite viscosity");
        jPanel9.add(yieldStressLabel9);
        yieldStressLabel9.setBounds(220, 100, 110, 15);

        yieldStressLabel10.setText("ET:");
        yieldStressLabel10.setToolTipText("T above which strain weakening is reset");
        jPanel9.add(yieldStressLabel10);
        yieldStressLabel10.setBounds(220, 120, 110, 15);

        E0dtLabel.setText("E0dt");
        E0dtLabel.setToolTipText("Time rate of strain reduction");
        E0dtLabel.setText("E"+Greek.sub0+"dt:");
        jPanel9.add(E0dtLabel);
        E0dtLabel.setBounds(220, 140, 110, 15);

        getContentPane().add(jPanel9);
        jPanel9.setBounds(10, 330, 400, 170);

        thermodynamicAndMeltPropertiesButton.setText("Thermodynamic and Melt Properties...");
        getContentPane().add(thermodynamicAndMeltPropertiesButton);
        thermodynamicAndMeltPropertiesButton.setBounds(10, 510, 290, 25);

        materialColourButton.setText("Material Colour");
        materialColourButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                materialColourButtonActionPerformed(evt);
            }
        });

        getContentPane().add(materialColourButton);
        materialColourButton.setBounds(250, 10, 140, 25);

        materialRheologyWindowCloseButton.setText("Close");
        materialRheologyWindowCloseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                materialRheologyWindowCloseButtonActionPerformed(evt);
            }
        });

        getContentPane().add(materialRheologyWindowCloseButton);
        materialRheologyWindowCloseButton.setBounds(340, 510, 69, 25);

        ppmColoursButton.setText("Edit PPM Colours");
        ppmColoursButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ppmColoursButtonActionPerformed(evt);
            }
        });

        getContentPane().add(ppmColoursButton);
        ppmColoursButton.setBounds(250, 60, 140, 25);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-440)/2, (screenSize.height-571)/2, 440, 571);
    }//GEN-END:initComponents

    private void strainedColourButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_strainedColourButtonActionPerformed
        genericColourChangeListener.whatColourChooserIsBeingUsedFor=genericColourChangeListener.strainedColour;
        genericColourChooser.theColourChooser.setColor(strainedColourButton.getBackground());
        genericColourChooser.show();
    }//GEN-LAST:event_strainedColourButtonActionPerformed

    private void hotColourButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_hotColourButtonActionPerformed
        genericColourChangeListener.whatColourChooserIsBeingUsedFor=genericColourChangeListener.hotColour;
        genericColourChooser.theColourChooser.setColor(hotColourButton.getBackground());
        genericColourChooser.show();
    }//GEN-LAST:event_hotColourButtonActionPerformed

    private void coldColourButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_coldColourButtonActionPerformed
        genericColourChangeListener.whatColourChooserIsBeingUsedFor=genericColourChangeListener.coldColour;
        genericColourChooser.theColourChooser.setColor(coldColourButton.getBackground());
        genericColourChooser.show();
    }//GEN-LAST:event_coldColourButtonActionPerformed

    private void strainedOpacityNumberBeanBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_strainedOpacityNumberBeanBoxFocusLost
        getCurrentlySelectedPPMColours().getSettingsFromWindow();
    }//GEN-LAST:event_strainedOpacityNumberBeanBoxFocusLost

    private void hotOpacityNumberBeanBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_hotOpacityNumberBeanBoxFocusLost
        getCurrentlySelectedPPMColours().getSettingsFromWindow();
    }//GEN-LAST:event_hotOpacityNumberBeanBoxFocusLost

    private void coldOpacityNumberBeanBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_coldOpacityNumberBeanBoxFocusLost
        getCurrentlySelectedPPMColours().getSettingsFromWindow();
    }//GEN-LAST:event_coldOpacityNumberBeanBoxFocusLost

    private void ppmColoursComboBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_ppmColoursComboBoxItemStateChanged
        if (ppmColoursComboBox.getSelectedIndex()>=0 && ppmColoursComboBox.getItemCount()==ppmColoursList.size()) { // If an item is selected, (and so there is at least one item in the combo box, and the PPMColoursList is up to date (i.e. has an equal number of items to the comboBox), set the window objects appropriately for the newly selected PPMColours. 
            getCurrentlySelectedPPMColours().updateWindowObjects();
        }
    }//GEN-LAST:event_ppmColoursComboBoxItemStateChanged

    private void ppmColoursButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ppmColoursButtonActionPerformed
        ppmColoursDialog.show();
    }//GEN-LAST:event_ppmColoursButtonActionPerformed

    private void ppmColoursDialogCloseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ppmColoursDialogCloseButtonActionPerformed
        ppmColoursDialog.hide();
    }//GEN-LAST:event_ppmColoursDialogCloseButtonActionPerformed

    private void inputFileEditorTextAreaFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_inputFileEditorTextAreaFocusLost
        // Add your handling code here:
    }//GEN-LAST:event_inputFileEditorTextAreaFocusLost

    private void inputFileEditorTextAreaFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_inputFileEditorTextAreaFocusGained
        // Add your handling code here:
    }//GEN-LAST:event_inputFileEditorTextAreaFocusGained

    private void inputFileEditorTextAreaComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_inputFileEditorTextAreaComponentShown
        // Add your handling code here:
    }//GEN-LAST:event_inputFileEditorTextAreaComponentShown

    private void inputFileEditorScrollPaneComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_inputFileEditorScrollPaneComponentShown
        // Add your handling code here:
    }//GEN-LAST:event_inputFileEditorScrollPaneComponentShown

    private void blockingTemperatureTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_blockingTemperatureTextBoxFocusLost
        try {
            Double.parseDouble(blockingTemperatureTextBox.getText());
        } catch (NumberFormatException ex) {
            blockingTemperatureTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_blockingTemperatureTextBoxFocusLost

    private void E0dtTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_E0dtTextBoxFocusLost
        try {
            Double.parseDouble(E0dtTextBox.getText());
        } catch (NumberFormatException ex) {
            E0dtTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_E0dtTextBoxFocusLost

    private void ETTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_ETTextBoxFocusLost
        try {
            Double.parseDouble(ETTextBox.getText());
        } catch (NumberFormatException ex) {
            ETTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_ETTextBoxFocusLost

    private void EDotnTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_EDotnTextBoxFocusLost
        try {
            Double.parseDouble(EDotnTextBox.getText());
        } catch (NumberFormatException ex) {
            EDotnTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_EDotnTextBoxFocusLost

    private void EnTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_EnTextBoxFocusLost
        try {
            Double.parseDouble(EnTextBox.getText());
        } catch (NumberFormatException ex) {
            EnTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_EnTextBoxFocusLost

    private void EDotaTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_EDotaTextBoxFocusLost
        try {
            Double.parseDouble(EDotaTextBox.getText());
        } catch (NumberFormatException ex) {
            EDotaTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_EDotaTextBoxFocusLost

    private void EaTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_EaTextBoxFocusLost
        try {
            Double.parseDouble(EaTextBox.getText());
        } catch (NumberFormatException ex) {
            EaTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_EaTextBoxFocusLost

    private void EDot0TextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_EDot0TextBoxFocusLost
        try {
            Double.parseDouble(EDot0TextBox.getText());
        } catch (NumberFormatException ex) {
            EDot0TextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_EDot0TextBoxFocusLost

    private void E0TextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_E0TextBoxFocusLost
        try {
            Double.parseDouble(E0TextBox.getText());
        } catch (NumberFormatException ex) {
            E0TextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_E0TextBoxFocusLost

    private void BcTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_BcTextBoxFocusLost
        try {
            Double.parseDouble(BcTextBox.getText());
        } catch (NumberFormatException ex) {
            BcTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_BcTextBoxFocusLost

    private void BpTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_BpTextBoxFocusLost
        try {
            Double.parseDouble(BpTextBox.getText());
        } catch (NumberFormatException ex) {
            BpTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_BpTextBoxFocusLost

    private void BzTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_BzTextBoxFocusLost
        try {
            Double.parseDouble(BzTextBox.getText());
        } catch (NumberFormatException ex) {
            BzTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_BzTextBoxFocusLost

    private void B0TextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_B0TextBoxFocusLost
        try {
            Double.parseDouble(B0TextBox.getText());
        } catch (NumberFormatException ex) {
            B0TextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_B0TextBoxFocusLost

    private void yieldStressMaximumTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_yieldStressMaximumTextBoxFocusLost
        try {
            Double.parseDouble(yieldStressMaximumTextBox.getText());
        } catch (NumberFormatException ex) {
            yieldStressMaximumTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_yieldStressMaximumTextBoxFocusLost

    private void yieldStressMinimumTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_yieldStressMinimumTextBoxFocusLost
        try {
            Double.parseDouble(yieldStressMinimumTextBox.getText());
        } catch (NumberFormatException ex) {
            yieldStressMinimumTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_yieldStressMinimumTextBoxFocusLost
    
    // After any change is made to a phase, we have to store the new value in the phase in the list of phases.
    private void bulkViscosityTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_bulkViscosityTextBoxFocusLost
        try {
            currentPhase().bulkViscosity=Double.parseDouble(bulkViscosityTextBox.getText());
        } catch (NumberFormatException ex) {
            MessageBoxes.warnAboutBadDouble(this);
            currentPhase().updateRheologicalComponents();
        }
    }//GEN-LAST:event_bulkViscosityTextBoxFocusLost
    
    // After any change is made to a phase, we have to store the new value in the phase in the list of phases.
    private void densityTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_densityTextBoxFocusLost
        try {
            currentPhase().density=Double.parseDouble(densityTextBox.getText());
        } catch (NumberFormatException ex) {
            MessageBoxes.warnAboutBadDouble(this);
            currentPhase().updateRheologicalComponents();
        }
    }//GEN-LAST:event_densityTextBoxFocusLost
    
    // After any change is made to a phase, we have to store the new value in the phase in the list of phases.
    private void ETextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_ETextBoxFocusLost
        try {
            currentPhase().viscE=Double.parseDouble(ETextBox.getText());
        } catch (NumberFormatException ex) {
            MessageBoxes.warnAboutBadDouble(this);
            currentPhase().updateRheologicalComponents();
        }
    }//GEN-LAST:event_ETextBoxFocusLost
    
    // After any change is made to a phase, we have to store the new value in the phase in the list of phases.
    private void ZTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_ZTextBoxFocusLost
        try {
            currentPhase().viscZ=Double.parseDouble(ZTextBox.getText());
        } catch (NumberFormatException ex) {
            MessageBoxes.warnAboutBadDouble(this);
            currentPhase().updateRheologicalComponents();
        }
    }//GEN-LAST:event_ZTextBoxFocusLost
    
    // After any change is made to a phase, we have to store the new value in the phase in the list of phases.
    private void T0TextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_T0TextBoxFocusLost
        try {
            currentPhase().viscT0=Double.parseDouble(T0TextBox.getText());
        } catch (NumberFormatException ex) {
            MessageBoxes.warnAboutBadDouble(this);
            currentPhase().updateRheologicalComponents();
        }
    }//GEN-LAST:event_T0TextBoxFocusLost
    
    // After any change is made to a phase, we have to store the new value in the phase in the list of phases.
    private void T1TextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_T1TextBoxFocusLost
        try {
            currentPhase().viscT1=Double.parseDouble(T1TextBox.getText());
        } catch (NumberFormatException ex) {
            MessageBoxes.warnAboutBadDouble(this);
            currentPhase().updateRheologicalComponents();
        }
    }//GEN-LAST:event_T1TextBoxFocusLost
    
    // After any change is made to a phase, we have to store the new value in the phase in the list of phases.
    private void N0TextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_N0TextBoxFocusLost
        try {
            currentPhase().viscN0=Double.parseDouble(N0TextBox.getText());
        } catch (NumberFormatException ex) {
            MessageBoxes.warnAboutBadDouble(this);
            currentPhase().updateRheologicalComponents();
        }
    }//GEN-LAST:event_N0TextBoxFocusLost
    
    // After any change is made to a phase, we have to store the new value in the phase in the list of phases, and we also have to enable/disable appropriate other window items.
    private void frankKamenetskiiButtonItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_frankKamenetskiiButtonItemStateChanged
        updateRheologyType();
    }//GEN-LAST:event_frankKamenetskiiButtonItemStateChanged
    
    // After any change is made to a phase, we have to store the new value in the phase in the list of phases, and we also have to enable/disable appropriate other window items.
    private void arrheniusButtonItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_arrheniusButtonItemStateChanged
        updateRheologyType();
    }//GEN-LAST:event_arrheniusButtonItemStateChanged
    
    private void updateRheologyType() {
        if (arrheniusButton.isSelected()) {
            currentPhase().rheol_T_type=2;
            T0TextBox.setEnabled(true);
            ZTextBox.setEnabled(true);
            ETextBox.setEnabled(true);
        } else {
            currentPhase().rheol_T_type=1;
            T0TextBox.setEnabled(false);
            ZTextBox.setEnabled(false);
            ETextBox.setEnabled(false);
        }        
    }
    
    private RheologyPhase currentPhase() {
        return (RheologyPhase)(rheologyPhases.get(rheologicalComponentsComboBox.getSelectedIndex()));
    }
    
    // ********* This function is now redundant ****************
    private void storeCurrentPhaseDetails() { // Take all the information about the currently selected phase from the window objects, and stores it in the appropriate phase.
        RheologyPhase selectedComponent=(RheologyPhase)(rheologyPhases.get(rheologicalComponentsComboBox.getSelectedIndex()));
        if (arrheniusButton.isSelected()) {
            selectedComponent.rheol_T_type=2;
        } else {
            selectedComponent.rheol_T_type=1;
        }
        selectedComponent.viscN0=Double.parseDouble(N0TextBox.getText());
        selectedComponent.viscT1=Double.parseDouble(T1TextBox.getText());
        //        N0TextBox.setText(""+viscN0);
        //            T1TextBox.setText(""+viscT1);
        //            T0TextBox.setText(""+viscT0);
        //          ZTextBox.setText(""+viscZ);
        //        ETextBox.setText(""+viscE);
        //      densityTextBox.setText(""+density);
        //    bulkViscosityTextBox.setText(""+bulkViscosity);
    }
    
    private void rheologicalComponentsComboBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_rheologicalComponentsComboBoxItemStateChanged
        int selectedPhaseIndex=rheologicalComponentsComboBox.getSelectedIndex();
        if (selectedPhaseIndex>rheologyPhases.size()-1) // If the selected phase doesn't exist anymore
            selectedPhaseIndex=rheologyPhases.size()-1; // Point to the last one which does exist.
        if (selectedPhaseIndex==-1) // If there is no selected phase, do nothing (this may happen when the combo box is emptied and then refilled of items by the program).
            return;
        ((RheologyPhase)rheologyPhases.get(selectedPhaseIndex)).updateRheologicalComponents();        // Update the window to correspond to the newly selected phase whenever the ComboBox's state is changed.
    }//GEN-LAST:event_rheologicalComponentsComboBoxItemStateChanged
    
    private void insertPhaseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_insertPhaseButtonActionPerformed
        if (rheologyPhases.size()==11) { // If we already have the maximum number of phases, just warn about this and return.
            JOptionPane.showMessageDialog(this, "Ellipsis does not allow more than eleven phases, so no more can be created.", "Error", JOptionPane.ERROR_MESSAGE);
        } else {
            int selectedPhase=rheologicalComponentsComboBox.getSelectedIndex();
            RheologyPhase newRheologyPhase=new RheologyPhase();
            rheologyPhases.add(selectedPhase+1, newRheologyPhase); // Add the new phase and it's label in the ComboBox after the currently selected phase.
            rheologicalComponentsComboBox.insertItemAt("Phase "+numberOfNextRheologyPhase, selectedPhase+1);
            ++numberOfNextRheologyPhase;
            rheologicalComponentsComboBox.setSelectedIndex(selectedPhase+1); // And set the ComboBox to point to the new phase.
            newRheologyPhase.updateRheologicalComponents();
        }
    }//GEN-LAST:event_insertPhaseButtonActionPerformed
    
    private void deletePhaseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deletePhaseButtonActionPerformed
        if (rheologyPhases.size()>=2) { // If there's only one phase we can't delete it - ellipsis requires at least one phase per material.
            int selectedPhase=rheologicalComponentsComboBox.getSelectedIndex(); // First find the currently selected phase.
            rheologyPhases.remove(selectedPhase);
            rheologicalComponentsComboBox.removeItemAt(selectedPhase);
//            if (selectedPhase==rheologyPhases.size()) { // I don't think I need this - java will see to it that if I delete the last and current phase then the combo box will point to something else.
//                rheologicalComponentsComboBox.setSelectedIndex(rheologyPhases.size()-1);
//            }
        } else {
            JOptionPane.showMessageDialog(this,"You cannot delete the only phase. At least one phase must be defined for each material.", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_deletePhaseButtonActionPerformed
    
    private void materialColourButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_materialColourButtonActionPerformed
        genericColourChangeListener.whatColourChooserIsBeingUsedFor=genericColourChangeListener.materialColour;
        genericColourChooser.theColourChooser.setColor(materialColourButton.getBackground());
        genericColourChooser.show();
    }//GEN-LAST:event_materialColourButtonActionPerformed
    
    private void materialRheologyWindowCloseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_materialRheologyWindowCloseButtonActionPerformed
        hide();
    }//GEN-LAST:event_materialRheologyWindowCloseButtonActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        //        dispose(); // Don't dispose - just hide the window.
    }//GEN-LAST:event_closeDialog
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new MaterialRheologyWindow(true, 1).show();
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel B0Label;
    private javax.swing.JTextField B0TextBox;
    private javax.swing.JTextField BcTextBox;
    private javax.swing.JTextField BpTextBox;
    private javax.swing.JTextField BzTextBox;
    private javax.swing.JLabel E0Label;
    private javax.swing.JTextField E0TextBox;
    private javax.swing.JLabel E0dtLabel;
    private javax.swing.JTextField E0dtTextBox;
    private javax.swing.JTextField EDot0TextBox;
    private javax.swing.JTextField EDotaTextBox;
    private javax.swing.JTextField EDotnTextBox;
    private javax.swing.JTextField ETTextBox;
    private javax.swing.JTextField ETextBox;
    private javax.swing.JTextField EaTextBox;
    private javax.swing.JLabel Edot0Label;
    private javax.swing.JTextField EnTextBox;
    private javax.swing.JTextField N0TextBox;
    private javax.swing.JTextField T0TextBox;
    private javax.swing.JTextField T1TextBox;
    private javax.swing.JTextField ZTextBox;
    private javax.swing.JRadioButton arrheniusButton;
    private javax.swing.JTextField blockingTemperatureTextBox;
    private javax.swing.JLabel bulkViscosityLabel;
    private javax.swing.JTextField bulkViscosityTextBox;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JButton coldColourButton;
    private NumberBoxBean coldOpacityNumberBeanBox;
    private javax.swing.JButton deletePhaseButton;
    private javax.swing.JTextField densityTextBox;
    private javax.swing.JRadioButton frankKamenetskiiButton;
    private javax.swing.JButton hotColourButton;
    private NumberBoxBean hotOpacityNumberBeanBox;
    private javax.swing.JButton insertPhaseButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel20;
    private javax.swing.JLabel jLabel21;
    private javax.swing.JLabel jLabel22;
    private javax.swing.JLabel jLabel24;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    javax.swing.JButton materialColourButton;
    javax.swing.JTextField materialNameTextBox;
    private javax.swing.JButton materialRheologyWindowCloseButton;
    private javax.swing.JButton ppmColoursButton;
    javax.swing.JComboBox ppmColoursComboBox;
    private javax.swing.JDialog ppmColoursDialog;
    private javax.swing.JButton ppmColoursDialogCloseButton;
    private javax.swing.JComboBox rheologicalComponentsComboBox;
    private javax.swing.ButtonGroup rheologyTypeButtonGroup;
    private javax.swing.JButton strainedColourButton;
    private NumberBoxBean strainedOpacityNumberBeanBox;
    javax.swing.JButton thermodynamicAndMeltPropertiesButton;
    private javax.swing.JCheckBox tracerReproductionCheckBox;
    private javax.swing.JLabel yieldStressLabel1;
    private javax.swing.JLabel yieldStressLabel10;
    private javax.swing.JLabel yieldStressLabel2;
    private javax.swing.JLabel yieldStressLabel3;
    private javax.swing.JLabel yieldStressLabel4;
    private javax.swing.JLabel yieldStressLabel5;
    private javax.swing.JLabel yieldStressLabel6;
    private javax.swing.JLabel yieldStressLabel7;
    private javax.swing.JLabel yieldStressLabel8;
    private javax.swing.JLabel yieldStressLabel9;
    private javax.swing.JTextField yieldStressMaximumTextBox;
    private javax.swing.JTextField yieldStressMinimumTextBox;
    // End of variables declaration//GEN-END:variables
    
    class PPMColours {
        Color coldColour;
        Color hotColour;
        Color strainedColour;
        
        double coldOpacity;
        double hotOpacity;
        double strainedOpacity;
        
        PPMColours() { // The default constructor simply takes the current window settings.
            getSettingsFromWindow();
        }
        
        void updateWindowObjects() { // Set the window objects to reflect the values of this PPMColours
            coldColourButton.setBackground(coldColour);
            coldColourButton.setForeground(Colours.contrast(coldColour));
            coldOpacityNumberBeanBox.setText(""+coldOpacity);
            
            hotColourButton.setBackground(hotColour);
            hotColourButton.setForeground(Colours.contrast(hotColour));
            hotOpacityNumberBeanBox.setText(""+hotOpacity);
            
            strainedColourButton.setBackground(strainedColour);
            strainedColourButton.setForeground(Colours.contrast(strainedColour));
            strainedOpacityNumberBeanBox.setText(""+strainedOpacity);
        }
        
        void getSettingsFromWindow() { // Set this objects parameters based on the window object's settings.
            coldColour=coldColourButton.getBackground();
            coldOpacity=Double.parseDouble(coldOpacityNumberBeanBox.getText());
            
            hotColour=hotColourButton.getBackground();
            hotOpacity=Double.parseDouble(hotOpacityNumberBeanBox.getText());
            
            strainedColour=strainedColourButton.getBackground();
            strainedOpacity=Double.parseDouble(strainedOpacityNumberBeanBox.getText());
        }
        
        
    } // End of class PPMColours
    
}
