/*
 * ScalingParametersTableModel.java
 *
 * Created on 07 June 2005, 09:51
 *
 * Copyright (C) 2005 The University of Sydney Institute of Marine Science
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Authors:
 *      James Clark <jclark@geosci.usyd.edu.au>
 *
 */

import java.util.List;
import javax.swing.table.*;


/**
 *  This class is used by the UI and the JTable in it to get and set data
 *  for an EllipsisUnitScaler
 *
 * @author jclark
 */
public class ScalingParametersTableModel extends AbstractTableModel {
    
    String[] columnNames = { "Parameter", "Value" };
    String[] parameterNames;    // the internal name
    String[] parameterLabels;   // what the user sees
    EllipsisUnitScaler elus;
    
    /** Create the table model based on an EllipsisUnitScaler */
    public ScalingParametersTableModel(EllipsisUnitScaler elus) {
        this.elus = elus;
        parameterNames = elus.getScalingParameters();
        parameterLabels = elus.getScalingParameterDescriptions();
    }
    
    public void setScaler(EllipsisUnitScaler elus) {
        this.elus = elus;
        parameterNames = elus.getScalingParameters();
        parameterLabels = elus.getScalingParameterDescriptions();
        fireTableDataChanged();
    }
    
    public int getColumnCount() {
        return columnNames.length;
    }

    public int getRowCount() {
        return parameterNames.length;
    }

    public String getColumnName(int col) {
        return columnNames[col];
    }

    public Object getValueAt(int row, int col) {
        if (col == 0)
        {
            return parameterLabels[row];
        }
        else
        {
            // Assuming this does the job correctly, we can pretty-print the double. JC
            return Useful.double2String(elus.getScalingParameter(parameterNames[row]));
        }
    }

    public java.lang.Class getColumnClass(int c) {
        // No thanks to Class.java, we have to specify this explicitly
        return String.class;   // all of them are String anyway.
    }

    public boolean isCellEditable(int row, int col) {
        // Note that the data/cell address is constant,
        // no matter where the cell appears onscreen.
        if (col <= 0)
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    public void setValueAt(Object value, int row, int col) {
        if (col == 1)       // since that's the only one we're supposed to be able to edit
        {
            try
            {
                double dvalue = Double.parseDouble((String)value);
                
                if (dvalue != 0)            // Zero for any scaling parameter makes no sense and is downright dangerous!
                {
                    elus.setScalingParameter(parameterNames[row], dvalue);
                }
                else
                {
                    MessageBoxes.warnAboutZero(null);
                }
            }
            catch (NumberFormatException nfe)
            {
                MessageBoxes.warnAboutBadDouble(null);
            }
        }
        fireTableCellUpdated(row, col);
    }
}
