/*
 * ThermodynamicAndMeltPropertiesDialog.java
 *
 * Created on October 3, 2003, 10:38 AM
 */

/**
 *
 * @author  mdowman
 */
public class ThermodynamicAndMeltPropertiesDialog extends javax.swing.JDialog {
    
    /** Creates new form ThermodynamicAndMeltPropertiesDialog */
    public ThermodynamicAndMeltPropertiesDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
    }
    
    String getEllipsisFileSettings(String materialParameterPrefix, MainWindow.Scaling scaling) {
        String settings="\n# Thermodynamic and Melt Properties Parameters\n\n";
        settings+=materialParameterPrefix+"therm_exp="+scaling.coefficientOfThermalExpansion2ellipsisUnits(thermodynamicExpansionTextField.getText())+"\n"; // Scaling is probably needed for at least some of these**********
        settings+=materialParameterPrefix+"therm_diff="+scaling.thermalDiffusivity2ellipsisUnits(thermalDiffusivityTextField.getText())+"\n";
        settings+=materialParameterPrefix+"Cp="+scaling.specificHeat2ellipsisUnits(cpTextBox.getText())+"\n";
        settings+=materialParameterPrefix+"Qt="+scaling.powerPerUnitMass2ellipsisUnits(qtTextBox.getText())+"\n\n";

        settings+=getMeltPropertiesFileSettings(materialParameterPrefix, scaling);
        
        return settings;
    }
    
    String getMeltPropertiesFileSettings(String materialParameterPrefix, MainWindow.Scaling scaling) {
        String settings="";
        
        if (pressureDepletionModelRadioButton.isSelected()) {
            settings+=materialParameterPrefix+"depl_T_type=1\n";
        } else {
            settings+=materialParameterPrefix+"depl_T_type=2\n";
        }
        
        settings+=materialParameterPrefix+"depl_exp="+scaling.density2EllipsisUnits(densityChangeNumberBoxBean.getText())+"\n"; 

        settings+=materialParameterPrefix+"sp0="+sp0NumberBoxBean.getText()+"\n"; // These 8 parameters are all dimensionless.
        settings+=materialParameterPrefix+"sp1="+sp1NumberBoxBean.getText()+"\n";
        settings+=materialParameterPrefix+"sp2="+sp2NumberBoxBean.getText()+"\n";
        settings+=materialParameterPrefix+"sp3="+sp3NumberBoxBean.getText()+"\n";
        
        settings+=materialParameterPrefix+"lp0="+lp0NumberBoxBean.getText()+"\n";
        settings+=materialParameterPrefix+"lp1="+lp1NumberBoxBean.getText()+"\n";
        settings+=materialParameterPrefix+"lp2="+lp2NumberBoxBean.getText()+"\n";
        settings+=materialParameterPrefix+"lp3="+lp3NumberBoxBean.getText()+"\n";
        
        return settings;
    }
 
    void parseAndRemoveMeltPropertySettings(String materialParameterPrefix, MainWindow.Files files, MainWindow.Scaling scaling) {
        try {
            int depletionModel=files.parseInteger(materialParameterPrefix+"depl_T_type");
            if (depletionModel==1)
                pressureDepletionModelRadioButton.setSelected(true);
            else
                depthDepletionModelRadioButton.setSelected(true);
        } catch (ParameterNotFoundException exception) {
            // Do nothing if the parameter is not found - just leave the radio button at its previous setting.
        } catch (ParameterSyntaxErrorException exception) {
            // If there is a syntax error with the parameter, it will already have been added to lines with errors, so just do nothing.
        }
        
        files.parseAndSetReal(materialParameterPrefix+"depl_exp", densityChangeNumberBoxBean, scaling.getEllipsisUnits2DensityScaler());
        
        files.parseAndSetReal(materialParameterPrefix+"sp0", sp0NumberBoxBean);
        files.parseAndSetReal(materialParameterPrefix+"sp1", sp1NumberBoxBean);
        files.parseAndSetReal(materialParameterPrefix+"sp2", sp2NumberBoxBean);
        files.parseAndSetReal(materialParameterPrefix+"sp3", sp3NumberBoxBean);
        
        files.parseAndSetReal(materialParameterPrefix+"lp0", lp0NumberBoxBean);
        files.parseAndSetReal(materialParameterPrefix+"lp1", lp1NumberBoxBean);
        files.parseAndSetReal(materialParameterPrefix+"lp2", lp2NumberBoxBean);
        files.parseAndSetReal(materialParameterPrefix+"lp3", lp3NumberBoxBean);
    }
    
    void parseAndRemoveRheologySettings(String materialParameterPrefix, MainWindow.Files files, MainWindow.Scaling scaling) {
        files.parseAndSetReal(materialParameterPrefix+"therm_exp", thermodynamicExpansionTextField, scaling.getEllipsisUnits2CoefficientOfThermalExpansionScaler()); // **** Scaling needed ???? ****
        files.parseAndSetReal(materialParameterPrefix+"therm_diff", thermalDiffusivityTextField, scaling.getEllipsisUnits2ThermalDiffusivityScaler());
        files.parseAndSetReal(materialParameterPrefix+"Cp", cpTextBox, scaling.getEllipsisUnits2SpecificHeatScaler());
        files.parseAndSetReal(materialParameterPrefix+"Qt", qtTextBox, scaling.getEllipsisUnits2powerPerUnitMassScaler());

        parseAndRemoveMeltPropertySettings(materialParameterPrefix, files, scaling);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        depletionModelButtonGroup = new javax.swing.ButtonGroup();
        jPanel10 = new javax.swing.JPanel();
        thermodynamicExpansionLabel = new javax.swing.JLabel();
        thermalDiffusivityLabel = new javax.swing.JLabel();
        jLabel41 = new javax.swing.JLabel();
        jLabel42 = new javax.swing.JLabel();
        thermodynamicExpansionTextField = new javax.swing.JTextField();
        thermalDiffusivityTextField = new javax.swing.JTextField();
        qtTextBox = new javax.swing.JTextField();
        cpTextBox = new javax.swing.JTextField();
        jPanel11 = new javax.swing.JPanel();
        pressureDepletionModelRadioButton = new javax.swing.JRadioButton();
        depletionModelButtonGroup.add(pressureDepletionModelRadioButton);
        depthDepletionModelRadioButton = new javax.swing.JRadioButton();
        depletionModelButtonGroup.add(depthDepletionModelRadioButton);
        solidusJPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        sp0NumberBoxBean = new NumberBoxBean();
        jLabel2 = new javax.swing.JLabel();
        sp1NumberBoxBean = new NumberBoxBean();
        jLabel3 = new javax.swing.JLabel();
        sp2NumberBoxBean = new NumberBoxBean();
        jLabel4 = new javax.swing.JLabel();
        sp3NumberBoxBean = new NumberBoxBean();
        liquidusJPanel = new javax.swing.JPanel();
        jLabel5 = new javax.swing.JLabel();
        lp0NumberBoxBean = new NumberBoxBean();
        jLabel6 = new javax.swing.JLabel();
        lp1NumberBoxBean = new NumberBoxBean();
        jLabel7 = new javax.swing.JLabel();
        lp2NumberBoxBean = new NumberBoxBean();
        jLabel8 = new javax.swing.JLabel();
        lp3NumberBoxBean = new NumberBoxBean();
        jLabel9 = new javax.swing.JLabel();
        densityChangeNumberBoxBean = new NumberBoxBean();
        thermodynamicAndMeltPropertiesDialogCloseButton = new javax.swing.JButton();
        materialNameLabel = new javax.swing.JLabel();

        getContentPane().setLayout(null);

        setTitle("Thermodynamic and Melt Properties");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jPanel10.setLayout(null);

        jPanel10.setBorder(new javax.swing.border.TitledBorder("Thermodynamics"));
        thermodynamicExpansionLabel.setText("Thermal Expansion ("+Greek.alpha+"):");
        thermodynamicExpansionLabel.setToolTipText("Set this to -1 to obtain an infinite viscosity");
        thermodynamicExpansionLabel.setMaximumSize(new java.awt.Dimension(170, 15));
        thermodynamicExpansionLabel.setMinimumSize(new java.awt.Dimension(170, 15));
        jPanel10.add(thermodynamicExpansionLabel);
        thermodynamicExpansionLabel.setBounds(10, 20, 170, 15);

        thermalDiffusivityLabel.setText("Thermal Diffusivivity ("+Greek.greekk+"):");
        thermalDiffusivityLabel.setToolTipText("Set this to -1 to obtain an infinite viscosity");
        thermalDiffusivityLabel.setMaximumSize(new java.awt.Dimension(170, 15));
        thermalDiffusivityLabel.setMinimumSize(new java.awt.Dimension(170, 15));
        jPanel10.add(thermalDiffusivityLabel);
        thermalDiffusivityLabel.setBounds(10, 40, 170, 15);

        jLabel41.setText("Qt:");
        jLabel41.setToolTipText("Set this to -1 to obtain an infinite viscosity");
        jPanel10.add(jLabel41);
        jLabel41.setBounds(10, 60, 170, 15);

        jLabel42.setText("Cp:");
        jLabel42.setToolTipText("Set this to -1 to obtain an infinite viscosity");
        jPanel10.add(jLabel42);
        jLabel42.setBounds(10, 80, 170, 15);

        thermodynamicExpansionTextField.setText("0");
        thermodynamicExpansionTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                thermodynamicExpansionTextFieldFocusLost(evt);
            }
        });

        jPanel10.add(thermodynamicExpansionTextField);
        thermodynamicExpansionTextField.setBounds(190, 20, 60, 19);

        thermalDiffusivityTextField.setText("0");
        thermalDiffusivityTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                thermalDiffusivityTextFieldFocusLost(evt);
            }
        });

        jPanel10.add(thermalDiffusivityTextField);
        thermalDiffusivityTextField.setBounds(190, 40, 60, 19);

        qtTextBox.setText("0");
        qtTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                qtTextBoxFocusLost(evt);
            }
        });

        jPanel10.add(qtTextBox);
        qtTextBox.setBounds(190, 60, 60, 19);

        cpTextBox.setText("1");
        cpTextBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                cpTextBoxFocusLost(evt);
            }
        });

        jPanel10.add(cpTextBox);
        cpTextBox.setBounds(190, 80, 60, 19);

        getContentPane().add(jPanel10);
        jPanel10.setBounds(0, 40, 260, 110);

        jPanel11.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel11.setBorder(new javax.swing.border.TitledBorder("Melt Properties"));
        pressureDepletionModelRadioButton.setSelected(true);
        pressureDepletionModelRadioButton.setText("Pressure depletion model");
        jPanel11.add(pressureDepletionModelRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        depthDepletionModelRadioButton.setText("Depth depletion model");
        jPanel11.add(depthDepletionModelRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        solidusJPanel.setLayout(new java.awt.GridLayout(4, 2, 0, 2));

        solidusJPanel.setBorder(new javax.swing.border.TitledBorder("Solidus"));
        jLabel1.setText("sp0");
        solidusJPanel.add(jLabel1);

        solidusJPanel.add(sp0NumberBoxBean);

        jLabel2.setText("sp1");
        solidusJPanel.add(jLabel2);

        solidusJPanel.add(sp1NumberBoxBean);

        jLabel3.setText("sp2");
        solidusJPanel.add(jLabel3);

        solidusJPanel.add(sp2NumberBoxBean);

        jLabel4.setText("sp3");
        solidusJPanel.add(jLabel4);

        solidusJPanel.add(sp3NumberBoxBean);

        jPanel11.add(solidusJPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 100, 130, 120));

        liquidusJPanel.setLayout(new java.awt.GridLayout(4, 2, 0, 2));

        liquidusJPanel.setBorder(new javax.swing.border.TitledBorder("Liquidus"));
        jLabel5.setText("sp0");
        liquidusJPanel.add(jLabel5);

        liquidusJPanel.add(lp0NumberBoxBean);

        jLabel6.setText("sp1");
        liquidusJPanel.add(jLabel6);

        liquidusJPanel.add(lp1NumberBoxBean);

        jLabel7.setText("sp2");
        liquidusJPanel.add(jLabel7);

        liquidusJPanel.add(lp2NumberBoxBean);

        jLabel8.setText("sp3");
        liquidusJPanel.add(jLabel8);

        liquidusJPanel.add(lp3NumberBoxBean);

        jPanel11.add(liquidusJPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 100, 130, 120));

        jLabel9.setText("Density change of depleted material:");
        jPanel11.add(jLabel9, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 70, -1, -1));

        jPanel11.add(densityChangeNumberBoxBean, new org.netbeans.lib.awtextra.AbsoluteConstraints(250, 70, 60, -1));

        getContentPane().add(jPanel11);
        jPanel11.setBounds(0, 150, 320, 230);

        thermodynamicAndMeltPropertiesDialogCloseButton.setText("Close");
        thermodynamicAndMeltPropertiesDialogCloseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                thermodynamicAndMeltPropertiesDialogCloseButtonActionPerformed(evt);
            }
        });

        getContentPane().add(thermodynamicAndMeltPropertiesDialogCloseButton);
        thermodynamicAndMeltPropertiesDialogCloseButton.setBounds(140, 390, 69, 25);

        materialNameLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        getContentPane().add(materialNameLabel);
        materialNameLabel.setBounds(30, 10, 210, 15);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-332)/2, (screenSize.height-452)/2, 332, 452);
    }//GEN-END:initComponents

    private void thermodynamicExpansionTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_thermodynamicExpansionTextFieldFocusLost
        try {
            Double.parseDouble(thermodynamicExpansionTextField.getText());
        } catch (NumberFormatException ex) {
            thermodynamicExpansionTextField.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_thermodynamicExpansionTextFieldFocusLost

    private void thermalDiffusivityTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_thermalDiffusivityTextFieldFocusLost
        try {
            Double.parseDouble(thermalDiffusivityTextField.getText());
        } catch (NumberFormatException ex) {
            thermalDiffusivityTextField.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_thermalDiffusivityTextFieldFocusLost

    private void cpTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_cpTextBoxFocusLost
        try {
            Double.parseDouble(cpTextBox.getText());
        } catch (NumberFormatException ex) {
            cpTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_cpTextBoxFocusLost

    private void qtTextBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_qtTextBoxFocusLost
        try {
            Double.parseDouble(qtTextBox.getText());
        } catch (NumberFormatException ex) {
            qtTextBox.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_qtTextBoxFocusLost

    private void thermodynamicAndMeltPropertiesDialogCloseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_thermodynamicAndMeltPropertiesDialogCloseButtonActionPerformed
        hide();
    }//GEN-LAST:event_thermodynamicAndMeltPropertiesDialogCloseButtonActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new ThermodynamicAndMeltPropertiesDialog(new javax.swing.JFrame(), true).show();
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField cpTextBox;
    private NumberBoxBean densityChangeNumberBoxBean;
    private javax.swing.ButtonGroup depletionModelButtonGroup;
    private javax.swing.JRadioButton depthDepletionModelRadioButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel41;
    private javax.swing.JLabel jLabel42;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel10;
    private javax.swing.JPanel jPanel11;
    private javax.swing.JPanel liquidusJPanel;
    private NumberBoxBean lp0NumberBoxBean;
    private NumberBoxBean lp1NumberBoxBean;
    private NumberBoxBean lp2NumberBoxBean;
    private NumberBoxBean lp3NumberBoxBean;
    javax.swing.JLabel materialNameLabel;
    private javax.swing.JRadioButton pressureDepletionModelRadioButton;
    private javax.swing.JTextField qtTextBox;
    private javax.swing.JPanel solidusJPanel;
    private NumberBoxBean sp0NumberBoxBean;
    private NumberBoxBean sp1NumberBoxBean;
    private NumberBoxBean sp2NumberBoxBean;
    private NumberBoxBean sp3NumberBoxBean;
    private javax.swing.JLabel thermalDiffusivityLabel;
    private javax.swing.JTextField thermalDiffusivityTextField;
    private javax.swing.JButton thermodynamicAndMeltPropertiesDialogCloseButton;
    private javax.swing.JLabel thermodynamicExpansionLabel;
    private javax.swing.JTextField thermodynamicExpansionTextField;
    // End of variables declaration//GEN-END:variables
    
}
