/*
 * MainWindow.java
 *
 * Created on September 26, 2003, 1:19 PM
 *
 *Copyright (C) 2003 The University of Sydney Institute of Marine Science
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Authors:
 *   Mike Dowman <mdowman@geosci.usyd.edu.au>
 *   Craig O'Neill <craig@geosci.usyd.edu.au>
 *
 * This program contains assert statements, and so must be compiled with the -source 1.4 option.
 * N.B. assert statements do not actually seem to work!
 * 
 */

/**
 *
 * @author  mdowman
 */


import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.io.File;
import javax.swing.filechooser.*;
import java.io.File;
import javax.swing.ImageIcon;
import java.awt.event.*;

public class MainWindow extends javax.swing.JFrame {
    
    javax.swing.Timer timer = new javax.swing.Timer(500, new ActionListener() { // This is the timer used for controlling the animation of selected regions.
        public void actionPerformed(ActionEvent evt) {
            if (backgroundImagePanel.theMouseListener.drawingInProgress==false && backgroundImagePanel.theMouseListener.triangleInProgress==false) { // We don't want to do the animation while a new region is being drawn.
                backgroundImagePanel.repaint(); // Request that the panel gets redrawn - will redraw the selected region differently to how it was last drawn.
            }
        }
    });
    
    JFileChooser fileChooser = new JFileChooser();

    GenericColourChooser generalColourChooser= new GenericColourChooser();
    GeneralColourChooserChangeListener generalColourChooserChangeListener=new GeneralColourChooserChangeListener();
    
    BackgroundImagePanel backgroundImagePanel;

    int numberOfNextMaterial; // Each material will initially be given a number (material 1, material 2, etc.). This keeps track of what the next material will be called.
    ArrayList materials=new ArrayList(); // Stores all the materials (including all their properties (inside their windows)).
    
    Files files; // Stores the input file, last saved file, filename etc. Includes all associated functions.
    Scaling scaling; // Stores the scaling parameters, and performs conversions between different units.
    
    static int numberOfOpenWindows=0;
    
    double windowDimensionX=20; // These are stored separately from their occurence in the textboxes for efficiency because they're frequently used in painting.
    double windowDimensionY=10; // N.B. it is important that these match the values initially displayed in the main window.
    
    LinkedList PPMFiles=new LinkedList(); // This stores all the PPM files defintitions (from output panel)
    LinkedList samplingTracers=new LinkedList(); // This strores all the definitions for sampling tracers.
    
    /** Creates new form MainWindow */
    public MainWindow() {
        initComponents();
        backgroundImagePanel = new BackgroundImagePanel(); // Create the panel for the background image (which contains the drawing panel). (Must be done after mainwindow and its contents is created.)
        
        ++numberOfOpenWindows;
        
        numberOfNextMaterial=1;
        materials.add(new Material());
        materialChooser.addItem( ((Material)materials.get(0)).getName() ); // Really we should set the materialNameLabel in the thermodynamicAndMeltPropertiesDialog, but that all seems to be taken care of (almost by accident, as the material name window must be focussed and then its focus lost before that window is opened).
        
        addNewPPMFile(); // Add an initial PPM file, with default parameter settings, to the list of PPM files, and update the window objects appropriately.
        
        generalColourChooser.setModal(true); // We want all other stuff blocked while the general colour chooser is in use.
        generalColourChooser.theColourChooser.getSelectionModel().addChangeListener(generalColourChooserChangeListener);

        scaling=new Scaling(); // Do this here, so all the window objects etc. are initialised first. N.B. Scaling is used in initialising files - so this line must occur before files are initialised.
        files=new Files(); // Do this here, so all the window objects etc. are initialised first.
        startTimer(); // Visual editor window is showing initially, so we need to begin with the timer started.
        
        initialiseEllipsisManual();
        initialiseNonDimensionalization();
    }

    void initialiseEllipsisManual() {
//        ellipsisManualEditorPane.setContentType("text/html");
//        java.io.FileReader ellipsisManualFile;
/*        try {
        ellipsisManualFile = new java.io.FileReader("NewEGui/ellipsisManual.html"); 
            try {
        ellipsisManualEditorPane.read(ellipsisManualFile, new javax.swing.text.html.HTMLDocument());
            } catch (Exception ex) {
                System.out.println("here");
            System.out.print(ex.getMessage());
            }
        } catch (Exception ex) {
            System.out.println("Error opening ellipsis manual");
            System.out.print(ex.getMessage());
            // If we can't read the manual, just ignore the problem.
        }
//        ellipsisManualEditorPane.setText(EllipsisManual.getText());*/
        try {
            ellipsisManualEditorPane.setPage("file:ellipsisManual.html");
//            ellipsisManualEditorPane.setPage("http://www.it.usyd.edu.au/~mike/");
        } catch (Exception ex) { // If we can't load the web page, just ignore this problem.
        }         
    }
    
    void initialiseNonDimensionalization() {
        try {
            nonDimensionalizationEditorPane.setPage("file:nonDimensionalization.html");
        } catch (Exception ex) { // If we can't load the web page, just ignore this problem.
        }         
    }
    
    void addNewSamplingTracer() {
        String newSamplingTracerName="Sampling tracer "+(samplingTracers.size()+1);
        
        if (samplingTracers.size()==0) { // If no sampling tracers exist just now.
            samplingTracersComboBox.removeAllItems(); // Remove the message saying no sampling tracer defined from the combo box.
        }
        
        samplingTracers.add(new SamplingTracer()); // Add a new sampling tracer, getting its properties from the window objects.
        samplingTracersComboBox.addItem(newSamplingTracerName);
        
        samplingTracersComboBox.setSelectedIndex(samplingTracers.size()-1);
        updateSelectedSamplingTracersWindowObjects();
    }
    
    void addNewPPMFile() {
        String newPPMName="PPM file "+(PPMFiles.size()+1); // The new ppm file will be numbered to reflect the number of ppm files already defined.
        PPMFiles.add(new PPM_file());
        PPMFilesComboBox.addItem(newPPMName); // This must follow the previous line, as it triggers a message, which will cause an exception if the PPM_file itself hasn't been created.
        
        PPMFilesComboBox.setSelectedIndex(PPMFiles.size()-1); // And select the PPM File we've just added (which will be the last one).
        updateSelectedPPMFilesWindowObjects(); // This is probably redundant, because when the item is added to the combo box, and when the selected item is chosen, this will probably be called anyway through ComboBox item state changed.
        
        updatePPMColours(); // And update the PPM Files, and the corresponding objects in the MaterialRheologyWindow.
    }
    
    void updatePPMColours() { // Here we need to go through each material, and for each one (1) set the PPM combo box to mirror the one in output window, (2) Add or remove PPM_Colours so there are the right number in the list.
        Iterator nextMaterial=materials.iterator();
        while (nextMaterial.hasNext()) {
            Material currentMaterial=(Material)nextMaterial.next();
            
            copyAllItems(PPMFilesComboBox, currentMaterial.materialRheologyWindow.ppmColoursComboBox); // Set the materials ppmColoursComboBox to be the same as the one in the output panel.
            
            currentMaterial.materialRheologyWindow.makeCorrectNumberOfPPMColours(); // And make the number of PPM Colours match the new setup of the combo box.
        }
        
    }
    
    void copyAllItems(JComboBox sourceBox, JComboBox destinationBox) { // Remove all items from destinationBox, and replace them with those in sourceBox.
        destinationBox.removeAllItems();
        
        for (int index=0; index<sourceBox.getItemCount(); ++index) {
            destinationBox.addItem(sourceBox.getItemAt(index));
        }
    }
    
    void addNewSamplingTracer(int initialSampling_lagrangian, int initialSampling_plot_num, double initialSampling_x, double initialSampling_z, int initialSampling_field, int initialSampling_dirn, int initialSampling_normalize, double initialSampling_R, double initialSampling_G, double initialSampling_B) {
        String newSamplingTracerName="Sampling tracer "+(samplingTracers.size()+1);
        
        if (samplingTracers.size()==0) { // If no sampling tracers exist just now.
            samplingTracersComboBox.removeAllItems(); // Remove the message saying no sampling tracer defined from the combo box.
        }
        
        samplingTracers.add(new SamplingTracer(initialSampling_lagrangian, initialSampling_plot_num, initialSampling_x, initialSampling_z, initialSampling_field, initialSampling_dirn, initialSampling_normalize, initialSampling_R, initialSampling_G, initialSampling_B)); // This time get the new sampling tracers properties from the function arguments, not the window objects. This function will be used when reading in from a file.
        samplingTracersComboBox.addItem(newSamplingTracerName);
        
        samplingTracersComboBox.setSelectedIndex(samplingTracers.size()-1);
        updateSelectedSamplingTracersWindowObjects();
    }
    
    void addNewPPMFile(int height, int coloring) { // This routine doesn't use window objects to get its settings, instead its given the parameters for the new PPM
        String newPPMName="PPM file "+(PPMFiles.size()+1); // The new ppm file will be numbered to reflect the number of ppm files already defined.
        PPMFiles.add(new PPM_file(height, coloring));
        PPMFilesComboBox.addItem(newPPMName); // This must follow the previous line, as it triggers a message, which will cause an exception if the PPM_file itself hasn't been created.
        
        PPMFilesComboBox.setSelectedIndex(PPMFiles.size()-1); // And select the PPM File we've just added (which will be the last one).
        updateSelectedPPMFilesWindowObjects(); // This is probably redundant, because when the item is added to the combo box, and when the selected item is chosen, this will probably be called anyway through ComboBox item state changed.
        
        updatePPMColours(); // And update the PPM Files, and the corresponding objects in the MaterialRheologyWindow.
    }
    
    void deleteSamplingTracer() { // Delete the currently selected PPM file. If it's the only one (or there aren't any), put a message saying so in the samplingTracerComboBox.
        if (samplingTracers.size()<=1) { // If there's one or no sampling tracers defined.
            samplingTracers=new LinkedList(); // Overwrite the existing linked list with an empty one.
            samplingTracersComboBox.removeAllItems(); // Remove everything from the combo box.
            samplingTracersComboBox.addItem("No tracers defined"); // Put in the message saying no tracers defined.
            return; // And return
        }
        
        samplingTracers.remove(samplingTracersComboBox.getSelectedIndex()); // First remove the selected sampling tracer from the list of SamplingTracers.
        samplingTracersComboBox.removeItemAt(samplingTracersComboBox.getItemCount()-1); // Remove the last item from the combo box (this is because it's the item with the highest number - and this will result in later items becoming matched with different numbers automatically.)
        
        updateSelectedSamplingTracersWindowObjects(); // Update the GUI, to show the newly selected sampling tracer's files parameters.
    }
    
    void deletePPMFile() { // Delete the currently selected PPM file (unless it's the only one).
        if (PPMFiles.size()<=1) { // If there's only one PPM file. (Or none - though that should never happen.)
            JOptionPane.showMessageDialog(this, "Only one PPM file is defined. You cannot delete the only PPM file.", "Error", JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        PPMFiles.remove(PPMFilesComboBox.getSelectedIndex()); // First remove the selected PPM_file from the list of PPM files.
        PPMFilesComboBox.removeItemAt(PPMFilesComboBox.getItemCount()-1); // Remove the last item from the combo box (this is because it's the item with the highest number - and this will result in later items becoming matched with different numbers automatically.)
        
        updateSelectedPPMFilesWindowObjects(); // Update the GUI, to show the newly selected PPM files parameters.
        
        updatePPMColours(); // And update the PPM Files, and the corresponding objects in the MaterialRheologyWindow.
    }
    
    void updateSelectedSamplingTracersWindowObjects() { // Update the appropriate window objects to reflect the settings for the current sampling Tracer
        if (samplingTracers.size()==0) { // If no sampling tracers are defined do nothing
            return;
        }
        
        if (!samplingTracersDialog.isShowing()) { // If the sampling tracers dialog isn't showing, there's no need to update the window objects. (We will have got here because an item was added to or removed from the combo box by something in the files class.)
            return;
        }
        
        if (samplingTracersComboBox.getItemCount()==0) { // If the combbox is empty, just do nothing.
            return;
        }
        
        if (samplingTracersComboBox.getSelectedIndex()==-1) { // If no item is selected, select the first one.
            samplingTracersComboBox.setSelectedIndex(0);
        }
        
        ((SamplingTracer)samplingTracers.get(samplingTracersComboBox.getSelectedIndex())).updateWindowObjects();
    }
    
    void updateSelectedPPMFilesWindowObjects() { // Update the appropriate window objects to reflect the current state of the PPM file.
        if (PPMFilesComboBox.getItemCount()==0) { // If the combo box is empty, just do nothing. We will get here when the combo box is emptied of all its items, and possibly at some other times.
            return;
        }
        
        if (PPMFilesComboBox.getSelectedIndex()==-1) { // If no item is currently selected, then select the first one.
            PPMFilesComboBox.setSelectedIndex(0);
        }
        
        ((PPM_file)PPMFiles.get(PPMFilesComboBox.getSelectedIndex())).updateWindowObjects(); // Retrieve the appropriate PPM files object from the list, and then update the window objects based on its values.
    }
    
    class GeneralColourChooserChangeListener implements javax.swing.event.ChangeListener {
        int whatColourChooserIsBeingUsedFor;
        final int drawing=0;
        final int tracer=1;
        final int boundary=2;
        final int samplingTracer=3;
        
        public void stateChanged(javax.swing.event.ChangeEvent evt) { // The same colour user is used for varios different things, so we need to work out which one it's being used for.
            if (whatColourChooserIsBeingUsedFor==drawing) {
                Color chosenColour=generalColourChooser.theColourChooser.getColor();
                drawingChangeColourButton.setBackground(chosenColour);
                if ((chosenColour.getRed()+chosenColour.getBlue()+chosenColour.getGreen())>382) { // If the chosen colour is light, use black text on the button, otherwise use white text.
                    drawingChangeColourButton.setForeground(java.awt.Color.black);
                } else {
                    drawingChangeColourButton.setForeground(java.awt.Color.white);
                }
            } else if (whatColourChooserIsBeingUsedFor==boundary) { // ********** Here we should also set the current boundary's colour (if any) and call repaint.
                Color chosenColour=generalColourChooser.theColourChooser.getColor();
                boundaryChangeColourButton.setBackground(chosenColour);
                if ((chosenColour.getRed()+chosenColour.getBlue()+chosenColour.getGreen())>382) { // If the chosen colour is light, use black text on the button, otherwise use white text.
                    boundaryChangeColourButton.setForeground(java.awt.Color.black);
                } else {
                    boundaryChangeColourButton.setForeground(java.awt.Color.white);
                }
            } else if (whatColourChooserIsBeingUsedFor==tracer) { // ********** Here we should also set the current tracers colour (if any) and call repaint.
                Color chosenColour=generalColourChooser.theColourChooser.getColor();
                tracerChangeColourButton.setBackground(chosenColour);
                if ((chosenColour.getRed()+chosenColour.getBlue()+chosenColour.getGreen())>382) { // If the chosen colour is light, use black text on the button, otherwise use white text.
                    tracerChangeColourButton.setForeground(java.awt.Color.black);
                } else {
                    tracerChangeColourButton.setForeground(java.awt.Color.white);
                }
            } else if (whatColourChooserIsBeingUsedFor==samplingTracer) {
                Color chosenColour=generalColourChooser.theColourChooser.getColor();
                profileColourButton.setBackground(chosenColour);
                profileColourButton.setForeground(Colours.contrast(chosenColour)); // Set the text colour to black or white, depending on what will show up best against the background.
            }
            
        }
    }
    
    private void recolourMaterialChooser() {
        int selectedIndex=materialChooser.getSelectedIndex();
        if (selectedIndex!=-1 && selectedIndex<materials.size()) { // If an item is selected, and the selected index doesn't point to a material that doesn't exist just now (can be a problem is one has just been deleted).
            Color selectedMaterialColour=((Material)(materials.get(selectedIndex))).materialRheologyWindow.materialColourButton.getBackground();
            materialChooser.setBackground(selectedMaterialColour);
            
            if ((selectedMaterialColour.getRed()+selectedMaterialColour.getBlue()+selectedMaterialColour.getGreen())>382) { // If the chosen colour is light, use black text on the chooser, otherwise use white text.
                materialChooser.setForeground(java.awt.Color.black);
            } else {
                materialChooser.setForeground(java.awt.Color.white);
            }
            
        }
        backgroundImagePanel.repaint(); // We also need to repaint the background image panel, because it might be displaying a region with the colour which has just changed.
    }
    
    private Material getSelectedMaterial() { // Returns a reference to the material that is currently selected.
        return (Material)materials.get(materialChooser.getSelectedIndex());
    }
    
    void disposeOfAllMaterialWindows() { // disposes of all the windows (rheology and thermodynamicaAndMelt) - so that when new materials are created the old windows aren't left hanging around.
        Iterator currentMaterial=materials.iterator();
        while (currentMaterial.hasNext()) {
            ((Material)currentMaterial.next()).disposeOfWindows();
        }
    }
    
    void blankRectangleControls() {
        rectangleX1TextField.setText("");
        rectangleX2TextField.setText("");
        rectangleY1TextField.setText("");
        rectangleY2TextField.setText("");
    }
    
    void blankTriangleControls() {
        triangleX1TextField.setText("");
        triangleX2TextField.setText("");
        triangleX3TextField.setText("");
        triangleY1TextField.setText("");
        triangleY2TextField.setText("");
        triangleY3TextField.setText("");
    }
    
    void blankLineControls() {
        lineX1TextField.setText("");
        lineX2TextField.setText("");
        lineZ1TextField.setText("");
        lineZ2TextField.setText("");
    }
    
    void blankCircleControls() {
        circleXTextField.setText("");
        circleZTextField.setText("");
        circleRadiusTextField.setText("");
    }
    
    void startTimer() { // Starts the timer for animating the selected region.
        
    }
    
    void enableAllYieldStressWindowObjects() {
        setAllYieldStressWindowObjects(true);
    }
    
    void disableAllYieldStressWindowObjects() {
        setAllYieldStressWindowObjects(false);
    }
    
    void setAllYieldStressWindowObjects(boolean set) { // Enable each material's yield stress parameters, if set is true, otherwise disable them.
        Iterator nextMaterial=materials.iterator();
        while (nextMaterial.hasNext()) {
            ((Material)nextMaterial.next()).setYieldStressWindowObjects(set);
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        drawingShapesButtonGroup = new javax.swing.ButtonGroup();
        grainSizeModelButtonGroup = new javax.swing.ButtonGroup();
        universalMaterialPropertiesDialogWindow = new javax.swing.JDialog();
        jLabel14 = new javax.swing.JLabel();
        growthNucleationButton = new javax.swing.JRadioButton();
        grainSizeModelButtonGroup.add(growthNucleationButton);
        growthNucleationButton.setSelected(true);
        empiricalButton = new javax.swing.JRadioButton();
        grainSizeModelButtonGroup.add(empiricalButton);
        useMaximumViscosityCheckBox = new javax.swing.JCheckBox();
        useMinimumViscosityCheckBox = new javax.swing.JCheckBox();
        temperatureDependentRheologicalParametersCheckBox = new javax.swing.JCheckBox();
        temperatureDependentRheologicalParametersCheckBox.setSelected(true);
        SDEPVCheckBox = new javax.swing.JCheckBox();
        YIELDCheckBox = new javax.swing.JCheckBox();
        GRDEPVCheckBox = new javax.swing.JCheckBox();
        maximumViscosityPanel = new javax.swing.JPanel();
        maximumViscosityLabel = new javax.swing.JLabel();
        maximumViscosityTextField = new javax.swing.JTextField();
        minimumViscosityPanel = new javax.swing.JPanel();
        minimumViscosityLabel = new javax.swing.JLabel();
        minimumViscosityTextField = new javax.swing.JTextField();
        universalMaterialPropertiesDialogWindowCloseButton = new javax.swing.JButton();
        aboutEllipsisGuiDialogWindow = new javax.swing.JDialog();
        jPanel1 = new javax.swing.JPanel();
        jTextArea5 = new javax.swing.JTextArea();
        usimsLogoLabel = new javax.swing.JLabel();
        aboutEllipsisGUIDialogWindowCloseButton = new javax.swing.JButton();
        boundaryAxisButtonGroup = new javax.swing.ButtonGroup();
        boundarySettingsDialog = new javax.swing.JDialog();
        periodicxCheckBox = new javax.swing.JCheckBox();
        periodic_rm_vxCheckBox = new javax.swing.JCheckBox();
        simpleBoundaryRadioButton = new javax.swing.JRadioButton();
        piecewiseBoundaryRadioButton = new javax.swing.JRadioButton();
        jPanel18 = new javax.swing.JPanel();
        jLabel22 = new javax.swing.JLabel();
        BCmoveX0vTextField = new javax.swing.JTextField();
        jLabel30 = new javax.swing.JLabel();
        BCmoveX1vTextField = new javax.swing.JTextField();
        jLabel31 = new javax.swing.JLabel();
        BCmoveX0v1TextField = new javax.swing.JTextField();
        jPanel19 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jLabel48 = new javax.swing.JLabel();
        boundarySettingsDialogCloseButton = new javax.swing.JButton();
        scalingOptionsButtonGroup = new javax.swing.ButtonGroup();
        boundaryMotionButtonGroup = new javax.swing.ButtonGroup();
        tracerTypeButtonGroup = new javax.swing.ButtonGroup();
        samplingTracersDialog = new javax.swing.JDialog();
        samplingTracersDialog.setBounds(100, 100, 550, 400);
        samplingTracersComboBox = new javax.swing.JComboBox();
        newSamplingTracerButton = new javax.swing.JButton();
        deleteSamplingTracerButton = new javax.swing.JButton();
        samplingTracerPropertiesJPanel = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        eulerianRadioButton = new javax.swing.JRadioButton();
        lagrangianRadioButton = new javax.swing.JRadioButton();
        tracerTypeButtonGroup.add(lagrangianRadioButton);
        jLabel36 = new javax.swing.JLabel();
        samplingPPMComboBox = new javax.swing.JComboBox();
        jLabel37 = new javax.swing.JLabel();
        jLabel38 = new javax.swing.JLabel();
        Sampling_fieldComboBox = new javax.swing.JComboBox();
        jLabel39 = new javax.swing.JLabel();
        jLabel40 = new javax.swing.JLabel();
        Sampling_dirnComboBox = new javax.swing.JComboBox();
        Sampling_normalizeCheckBox = new javax.swing.JCheckBox();
        profileColourButton = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        unrecognisedParametersDialog = new javax.swing.JDialog();
        jScrollPane1 = new javax.swing.JScrollPane();
        unrecognisedParametersDialogTextArea = new javax.swing.JTextArea();
        jPanel20 = new javax.swing.JPanel();
        unrecognisedParametersDialogCloseButton = new javax.swing.JButton();
        unrecognisedParametersDialog.setBounds(100,100,200,200);
        unitsAndScalingDialog = new javax.swing.JDialog();
        unitsAndScalingDialog.setBounds(100,100,300,300);
        jScrollPane2 = new javax.swing.JScrollPane();
        unitsAndScalingTextArea = new javax.swing.JTextArea();
        jPanel21 = new javax.swing.JPanel();
        unitsAndScalingDialogCloseButton = new javax.swing.JButton();
        unrecognisedParametersDialog.setBounds(100,100,200,200);
        ellipsisManualDialog = new javax.swing.JDialog();
        ellipsisManualDialog.setBounds(60, 60, 400, 500);
        jScrollPane3 = new javax.swing.JScrollPane();
        ellipsisManualEditorPane = new javax.swing.JEditorPane();
        jPanel22 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        thermalZonesButtonGroup = new javax.swing.ButtonGroup();
        nonDimensionalizationDialog = new javax.swing.JDialog();
        nonDimensionalizationDialog.setBounds(60, 60, 400, 500);
        jScrollPane4 = new javax.swing.JScrollPane();
        nonDimensionalizationEditorPane = new javax.swing.JEditorPane();
        jPanel25 = new javax.swing.JPanel();
        nonDimensionalizationDialogCloseButton = new javax.swing.JButton();
        mainPanes = new javax.swing.JTabbedPane();
        visualEditorPanel = new javax.swing.JPanel();
        visualControlsPanel = new javax.swing.JPanel();
        shapeChooser = new javax.swing.JComboBox();
        jPanel12 = new javax.swing.JPanel();
        drawMaterialsCheckBox = new javax.swing.JCheckBox();
        drawBoundariesCheckBox = new javax.swing.JCheckBox();
        drawTracersCheckBox = new javax.swing.JCheckBox();
        backgroundPictureCheckBox = new javax.swing.JCheckBox();
        drawThermalZonesCheckBox = new javax.swing.JCheckBox();
        drawDrawingCheckBox = new javax.swing.JCheckBox();
        controlsCardHolder = new javax.swing.JPanel();
        rectangleControls = new javax.swing.JPanel();
        rectangleX1TextField = new javax.swing.JTextField();
        rectangleX2TextField = new javax.swing.JTextField();
        rectangleY1TextField = new javax.swing.JTextField();
        rectangleY2TextField = new javax.swing.JTextField();
        jTextArea3 = new javax.swing.JTextArea();
        jTextArea4 = new javax.swing.JTextArea();
        jTextArea2 = new javax.swing.JTextArea();
        jTextArea1 = new javax.swing.JTextArea();
        addNewRectangleButton = new javax.swing.JButton();
        rectangleUpdateButton = new javax.swing.JButton();
        circleControls = new javax.swing.JPanel();
        jLabel18 = new javax.swing.JLabel();
        jLabel19 = new javax.swing.JLabel();
        jLabel20 = new javax.swing.JLabel();
        circleAddButton = new javax.swing.JButton();
        circleUpdateButton = new javax.swing.JButton();
        jLabel21 = new javax.swing.JLabel();
        circleRadiusTextField = new javax.swing.JTextField();
        circleXTextField = new javax.swing.JTextField();
        circleZTextField = new javax.swing.JTextField();
        lineControls = new javax.swing.JPanel();
        lineX2TextField = new javax.swing.JTextField();
        lineX1TextField = new javax.swing.JTextField();
        lineZ1TextField = new javax.swing.JTextField();
        lineZ2TextField = new javax.swing.JTextField();
        jTextArea6 = new javax.swing.JTextArea();
        jTextArea7 = new javax.swing.JTextArea();
        jTextArea8 = new javax.swing.JTextArea();
        jTextArea9 = new javax.swing.JTextArea();
        addNewLineButton = new javax.swing.JButton();
        lineUpdateButton = new javax.swing.JButton();
        harmonicControls = new javax.swing.JPanel();
        jTextArea10 = new javax.swing.JTextArea();
        polynomialControls = new javax.swing.JPanel();
        jTextArea11 = new javax.swing.JTextArea();
        triangleControls = new javax.swing.JPanel();
        triangleY2TextField = new javax.swing.JTextField();
        triangleY1TextField = new javax.swing.JTextField();
        triangleX1TextField = new javax.swing.JTextField();
        triangleX2TextField = new javax.swing.JTextField();
        jTextArea12 = new javax.swing.JTextArea();
        jTextArea13 = new javax.swing.JTextArea();
        jTextArea14 = new javax.swing.JTextArea();
        jTextArea15 = new javax.swing.JTextArea();
        addNewTriangleButton = new javax.swing.JButton();
        triangleUpdateButton = new javax.swing.JButton();
        jTextArea16 = new javax.swing.JTextArea();
        triangleY3TextField = new javax.swing.JTextField();
        triangleX3TextField = new javax.swing.JTextField();
        jTextArea17 = new javax.swing.JTextArea();
        drawSolidShapesCheckBox = new javax.swing.JCheckBox();
        animateSelectedRegionsCheckBox = new javax.swing.JCheckBox();
        materialsDrawingsBoundaryTracersPanel = new javax.swing.JPanel();
        materialsDrawingBoundaryTracersPanel = new javax.swing.JTabbedPane();
        materialsPanel = new javax.swing.JPanel();
        materialSettingsButton = new javax.swing.JButton();
        jPanel13 = new javax.swing.JPanel();
        materialChooser = new javax.swing.JComboBox();
        newMaterialButton = new javax.swing.JButton();
        deleteMaterialButton = new javax.swing.JButton();
        editRheologyButton = new javax.swing.JButton();
        jPanel14 = new javax.swing.JPanel();
        materialRegionForwardButton = new javax.swing.JButton();
        materialRegionBackButton = new javax.swing.JButton();
        materialRegionDeleteButton = new javax.swing.JButton();
        materialRegionListScrollPane = new javax.swing.JScrollPane();
        materialRegionList = new javax.swing.JList();
        materialRegionList.setModel(new DefaultListModel());

        drawingsPanel = new javax.swing.JPanel();
        drawingChangeColourButton = new javax.swing.JButton();
        jPanel15 = new javax.swing.JPanel();
        drawingRegionForwardButton = new javax.swing.JButton();
        drawingRegionBackButton = new javax.swing.JButton();
        drawingRegionDeleteButton = new javax.swing.JButton();
        drawingRegionListScrollPane = new javax.swing.JScrollPane();
        drawingRegionList = new javax.swing.JList();
        drawingRegionList.setModel(new DefaultListModel());

        boundaryConditionsPanel = new javax.swing.JPanel();
        boundaryConditionsSettingsButton = new javax.swing.JButton();
        boundaryRegionPropertiesJPanel = new javax.swing.JPanel();
        boundaryPropertyComboBox = new javax.swing.JComboBox();
        boundaryXAxisRadioButton = new javax.swing.JRadioButton();
        boundaryAxisButtonGroup.add(boundaryXAxisRadioButton);
        boundaryYAxisRadioButton = new javax.swing.JRadioButton();
        boundaryAxisButtonGroup.add(boundaryYAxisRadioButton);
        boundaryZAxisRadioButton = new javax.swing.JRadioButton();
        boundaryAxisButtonGroup.add(boundaryZAxisRadioButton);
        jLabel24 = new javax.swing.JLabel();
        jLabel25 = new javax.swing.JLabel();
        boundaryHalfWidthTextField = new javax.swing.JTextField();
        jLabel26 = new javax.swing.JLabel();
        jLabel27 = new javax.swing.JLabel();
        boundaryValueTextField = new javax.swing.JTextField();
        boundaryChangeColourButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jPanel17 = new javax.swing.JPanel();
        boundaryRegionForwardButton = new javax.swing.JButton();
        boundaryRegionBackButton = new javax.swing.JButton();
        boundaryRegionDeleteButton = new javax.swing.JButton();
        TracerRegionListScrollPane1 = new javax.swing.JScrollPane();
        boundaryRegionList = new javax.swing.JList();
        boundaryRegionList.setModel(new DefaultListModel());

        tracersPanel = new javax.swing.JPanel();
        currentNextTracerLabel = new javax.swing.JLabel();
        regionsDensityLabel = new javax.swing.JLabel();
        tracerDensityTextField = new javax.swing.JTextField();
        tracerChangeColourButton = new javax.swing.JButton();
        tracerRegionsPanel = new javax.swing.JPanel();
        tracerRegionForwardButton = new javax.swing.JButton();
        tracerRegionBackButton = new javax.swing.JButton();
        tracerRegionDeleteButton = new javax.swing.JButton();
        tracerRegionListScrollPane = new javax.swing.JScrollPane();
        tracerRegionList = new javax.swing.JList();
        tracerRegionList.setModel(new DefaultListModel());

        useDefaultTracerSetup = new javax.swing.JCheckBox();
        thermalZonesPanel = new javax.swing.JPanel();
        halfWidthLabel = new javax.swing.JLabel();
        jLabel47 = new javax.swing.JLabel();
        halfWidthLabel1 = new javax.swing.JLabel();
        addRadioButton = new javax.swing.JRadioButton();
        thermalZonesButtonGroup.add(addRadioButton);
        multiplyRadioButton = new javax.swing.JRadioButton();
        thermalZonesButtonGroup.add(multiplyRadioButton);
        replaceRadioButton = new javax.swing.JRadioButton();
        thermalZonesButtonGroup.add(replaceRadioButton);
        jPanel24 = new javax.swing.JPanel();
        thermalRegionForwardButton = new javax.swing.JButton();
        thermalRegionBackButton = new javax.swing.JButton();
        thermalRegionDeleteButton = new javax.swing.JButton();
        TracerRegionListScrollPane2 = new javax.swing.JScrollPane();
        thermalRegionList = new javax.swing.JList();
        thermalRegionList.setModel(new DefaultListModel());

        jPanel2 = new javax.swing.JPanel();
        jLabel15 = new javax.swing.JLabel();
        windowDimensionXTextField = new javax.swing.JTextField();
        jLabel16 = new javax.swing.JLabel();
        windowDimensionYTextField = new javax.swing.JTextField();
        jLabel17 = new javax.swing.JLabel();
        fixAspectRatioCheckBox = new javax.swing.JCheckBox();
        visualEditorCentralPanel = new javax.swing.JPanel();
        generalSettingsPanel = new javax.swing.JPanel();
        verbosityPanel = new javax.swing.JPanel();
        describeCheckBox = new javax.swing.JCheckBox();
        verboseInputValuesCheckBox = new javax.swing.JCheckBox();
        beginnerCheckBox = new javax.swing.JCheckBox();
        verboseCodeBehaviourCheckBox = new javax.swing.JCheckBox();
        multrigridPanel = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        timePanel = new javax.swing.JPanel();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jLabel42 = new javax.swing.JLabel();
        SolverTextField = new javax.swing.JTextField();
        initialConditionsPanel = new javax.swing.JPanel();
        environmentPanel = new javax.swing.JPanel();
        jLabel11 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        outputSettingsPanel = new javax.swing.JPanel();
        jPanel4 = new javax.swing.JPanel();
        jLabel13 = new javax.swing.JLabel();
        datafileTextField = new javax.swing.JTextField();
        COMPRESSCheckBox = new javax.swing.JCheckBox();
        gzipLabel = new javax.swing.JLabel();
        gzipTextField = new javax.swing.JTextField();
        jPanel5 = new javax.swing.JPanel();
        jLabel23 = new javax.swing.JLabel();
        jLabel28 = new javax.swing.JLabel();
        jPanel6 = new javax.swing.JPanel();
        VelxCheckBox = new javax.swing.JCheckBox();
        PstnCheckBox = new javax.swing.JCheckBox();
        PbdyCheckBox = new javax.swing.JCheckBox();
        StrfCheckBox = new javax.swing.JCheckBox();
        TempCheckBox = new javax.swing.JCheckBox();
        PresCheckBox = new javax.swing.JCheckBox();
        VelzCheckBox = new javax.swing.JCheckBox();
        jPanel7 = new javax.swing.JPanel();
        particleDataTempCheckBox = new javax.swing.JCheckBox();
        StrDCheckBox = new javax.swing.JCheckBox();
        StrTCheckBox = new javax.swing.JCheckBox();
        StrPCheckBox = new javax.swing.JCheckBox();
        EdotCheckBox = new javax.swing.JCheckBox();
        ViscCheckBox = new javax.swing.JCheckBox();
        particleDataPresCheckBox = new javax.swing.JCheckBox();
        GrszCheckBox = new javax.swing.JCheckBox();
        StrSCheckBox = new javax.swing.JCheckBox();
        jPanel10 = new javax.swing.JPanel();
        TpgxCheckBox = new javax.swing.JCheckBox();
        GrtxCheckBox = new javax.swing.JCheckBox();
        GrbxCheckBox = new javax.swing.JCheckBox();
        GrvxCheckBox = new javax.swing.JCheckBox();
        TpbkCheckBox = new javax.swing.JCheckBox();
        TpbxCheckBox = new javax.swing.JCheckBox();
        TpgkCheckBox = new javax.swing.JCheckBox();
        GrbkCheckBox = new javax.swing.JCheckBox();
        GrvkCheckBox = new javax.swing.JCheckBox();
        VzsfCheckBox = new javax.swing.JCheckBox();
        VxsfCheckBox = new javax.swing.JCheckBox();
        observablesBhflCheckBox = new javax.swing.JCheckBox();
        observablesShflCheckBox = new javax.swing.JCheckBox();
        GetkCheckBox = new javax.swing.JCheckBox();
        GebkCheckBox = new javax.swing.JCheckBox();
        GeokCheckBox = new javax.swing.JCheckBox();
        GetxCheckBox = new javax.swing.JCheckBox();
        GebxCheckBox = new javax.swing.JCheckBox();
        GeoxCheckBox = new javax.swing.JCheckBox();
        GrtkCheckBox = new javax.swing.JCheckBox();
        samplingTracersButton = new javax.swing.JButton();
        PPMFilesJPanel = new javax.swing.JPanel();
        PPMFilesComboBox = new javax.swing.JComboBox();
        jLabel29 = new javax.swing.JLabel();
        newPPMFileButton = new javax.swing.JButton();
        deletePPMFileButton = new javax.swing.JButton();
        PPMColoringComboBox = new javax.swing.JComboBox();
        jPanel8 = new javax.swing.JPanel();
        averagesTempCheckBox = new javax.swing.JCheckBox();
        WrmsCheckBox = new javax.swing.JCheckBox();
        VeloCheckBox = new javax.swing.JCheckBox();
        averagesViscCheckBox = new javax.swing.JCheckBox();
        UrmsCheckBox = new javax.swing.JCheckBox();
        jPanel9 = new javax.swing.JPanel();
        NussCheckBox = new javax.swing.JCheckBox();
        SvavCheckBox = new javax.swing.JCheckBox();
        StmxCheckBox = new javax.swing.JCheckBox();
        BhflCheckBox = new javax.swing.JCheckBox();
        ShflCheckBox = new javax.swing.JCheckBox();
        VzrmCheckBox = new javax.swing.JCheckBox();
        VxrmCheckBox = new javax.swing.JCheckBox();
        BvavCheckBox = new javax.swing.JCheckBox();
        VrmsCheckBox = new javax.swing.JCheckBox();
        scalingDetailsPanel = new javax.swing.JPanel();
        jPanel16 = new javax.swing.JPanel();
        automaticallyScaleParametersRadioButton = new javax.swing.JRadioButton();
        scalingOptionsButtonGroup.add(automaticallyScaleParametersRadioButton);
        manuallyScaleParametersRadioButton = new javax.swing.JRadioButton();
        scalingOptionsButtonGroup.add(manuallyScaleParametersRadioButton);
        doNotScaleParametersRadioButton = new javax.swing.JRadioButton();
        scalingOptionsButtonGroup.add(doNotScaleParametersRadioButton);
        jPanel11 = new javax.swing.JPanel();
        jLabel32 = new javax.swing.JLabel();
        jLabel35 = new javax.swing.JLabel();
        jLabel49 = new javax.swing.JLabel();
        jLabel45 = new javax.swing.JLabel();
        jLabel43 = new javax.swing.JLabel();
        jLabel50 = new javax.swing.JLabel();
        jPanel23 = new javax.swing.JPanel();
        jLabel44 = new javax.swing.JLabel();
        jLabel46 = new javax.swing.JLabel();
        jLabel34 = new javax.swing.JLabel();
        jLabel33 = new javax.swing.JLabel();
        inputFileEditorScrollPane = new javax.swing.JScrollPane();
        inputFileEditorTextArea = new javax.swing.JTextArea();
        jMenuBar1 = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        jMenuItem8 = new javax.swing.JMenuItem();
        openInputFileMenuItem = new javax.swing.JMenuItem();
        openPictureFileMenuItem = new javax.swing.JMenuItem();
        saveInputFileMenuItem = new javax.swing.JMenuItem();
        saveInputFileAsMenuItem = new javax.swing.JMenuItem();
        exitMenuItem = new javax.swing.JMenuItem();
        jMenu3 = new javax.swing.JMenu();
        showUnrecognisedParameters = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        ellipsisManualMenuItem = new javax.swing.JMenuItem();
        unitsAndScalingMenuItem = new javax.swing.JMenuItem();
        nonDimensionaliztionMenuItem = new javax.swing.JMenuItem();
        aboutEllipsisGuiMenuItem = new javax.swing.JMenuItem();

        universalMaterialPropertiesDialogWindow.getContentPane().setLayout(null);

        universalMaterialPropertiesDialogWindow.setTitle("Material Properties");
        universalMaterialPropertiesDialogWindow.setBounds(100, 100, 430, 285);
        jLabel14.setText("Grain size model:");
        jLabel14.setFocusable(false);
        universalMaterialPropertiesDialogWindow.getContentPane().add(jLabel14);
        jLabel14.setBounds(10, 10, 110, 14);

        growthNucleationButton.setText("Growth/nucleation");
        universalMaterialPropertiesDialogWindow.getContentPane().add(growthNucleationButton);
        growthNucleationButton.setBounds(120, 10, 150, 23);

        empiricalButton.setText("Empirical");
        universalMaterialPropertiesDialogWindow.getContentPane().add(empiricalButton);
        empiricalButton.setBounds(270, 10, 114, 23);

        useMaximumViscosityCheckBox.setText("Use maximum viscosity cut-off");
        useMaximumViscosityCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                useMaximumViscosityCheckBoxItemStateChanged(evt);
            }
        });

        universalMaterialPropertiesDialogWindow.getContentPane().add(useMaximumViscosityCheckBox);
        useMaximumViscosityCheckBox.setBounds(10, 40, 240, 23);

        useMinimumViscosityCheckBox.setText("Use minimum viscosity cut-off");
        useMinimumViscosityCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                useMinimumViscosityCheckBoxItemStateChanged(evt);
            }
        });

        universalMaterialPropertiesDialogWindow.getContentPane().add(useMinimumViscosityCheckBox);
        useMinimumViscosityCheckBox.setBounds(10, 70, 240, 23);

        temperatureDependentRheologicalParametersCheckBox.setText("Use temperature dependent rheological parameters");
        universalMaterialPropertiesDialogWindow.getContentPane().add(temperatureDependentRheologicalParametersCheckBox);
        temperatureDependentRheologicalParametersCheckBox.setBounds(10, 100, 370, 23);

        SDEPVCheckBox.setText("Use stress dependence of viscosity");
        universalMaterialPropertiesDialogWindow.getContentPane().add(SDEPVCheckBox);
        SDEPVCheckBox.setBounds(10, 130, 340, 23);

        YIELDCheckBox.setText("Use yield stress parameters");
        YIELDCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                YIELDCheckBoxItemStateChanged(evt);
            }
        });

        universalMaterialPropertiesDialogWindow.getContentPane().add(YIELDCheckBox);
        YIELDCheckBox.setBounds(10, 160, 330, 23);

        GRDEPVCheckBox.setText("Track grain growth");
        universalMaterialPropertiesDialogWindow.getContentPane().add(GRDEPVCheckBox);
        GRDEPVCheckBox.setBounds(10, 190, 330, 23);

        maximumViscosityPanel.setLayout(new javax.swing.BoxLayout(maximumViscosityPanel, javax.swing.BoxLayout.X_AXIS));

        maximumViscosityLabel.setText("Cut-off: ");
        maximumViscosityLabel.setToolTipText("The maximum viscosity cut-off");
        maximumViscosityLabel.setEnabled(false);
        maximumViscosityPanel.add(maximumViscosityLabel);

        maximumViscosityTextField.setText("0");
        maximumViscosityTextField.setToolTipText("The maximum viscosity cut-off");
        maximumViscosityTextField.setEnabled(false);
        maximumViscosityPanel.add(maximumViscosityTextField);

        universalMaterialPropertiesDialogWindow.getContentPane().add(maximumViscosityPanel);
        maximumViscosityPanel.setBounds(260, 40, 130, 20);

        minimumViscosityPanel.setLayout(new javax.swing.BoxLayout(minimumViscosityPanel, javax.swing.BoxLayout.X_AXIS));

        minimumViscosityLabel.setText("Cut-off: ");
        minimumViscosityLabel.setToolTipText("The minimum viscosity cut-off");
        minimumViscosityLabel.setEnabled(false);
        minimumViscosityPanel.add(minimumViscosityLabel);

        minimumViscosityTextField.setText("0");
        minimumViscosityTextField.setToolTipText("The minimum viscosity cut-off");
        minimumViscosityTextField.setEnabled(false);
        minimumViscosityPanel.add(minimumViscosityTextField);

        universalMaterialPropertiesDialogWindow.getContentPane().add(minimumViscosityPanel);
        minimumViscosityPanel.setBounds(260, 70, 130, 20);

        universalMaterialPropertiesDialogWindowCloseButton.setText("Close");
        universalMaterialPropertiesDialogWindowCloseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                universalMaterialPropertiesDialogWindowCloseButtonActionPerformed(evt);
            }
        });

        universalMaterialPropertiesDialogWindow.getContentPane().add(universalMaterialPropertiesDialogWindowCloseButton);
        universalMaterialPropertiesDialogWindowCloseButton.setBounds(160, 220, 61, 23);

        aboutEllipsisGuiDialogWindow.setTitle("About Ellipsis GUI");
        aboutEllipsisGuiDialogWindow.setBackground(new java.awt.Color(255, 255, 255));
        aboutEllipsisGuiDialogWindow.setFont(new java.awt.Font("Dialog", 1, 12));
        aboutEllipsisGuiDialogWindow.setLocationRelativeTo(this);
        aboutEllipsisGuiDialogWindow.setResizable(false);
        aboutEllipsisGuiDialogWindow.setBounds(60, 60, 450, 450);
        jPanel1.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel1.setBackground(new java.awt.Color(255, 255, 255));
        jTextArea5.setEditable(false);
        jTextArea5.setText("The University of Sydney Institute of Marine Science Ellipsis GUI\nby Mike Dowman and Craig O'Neill\nBased partly on the Ellipsis Layer Editor by David Symonds, and containg\nextracts from 'The ELLIPSIS Input Format', also by David Symonds.\nCopyright (C) 2003 The University of Sydney Institute of Marine Science\n \nThis program is free software; you can redistribute it and/or\nmodify it under the terms of the GNU General Public License,\nversion 2, as published by the Free Software Foundation.\n\nThis program is distributed in the hope that it will be useful,\nbut WITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\nGNU General Public License for more details.\n");
        jPanel1.add(jTextArea5, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 0, -1, 210));

        usimsLogoLabel.setIcon(new javax.swing.ImageIcon("usimsLogo.GIF"));
        usimsLogoLabel.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0)));
        usimsLogoLabel.setFocusable(false);
        jPanel1.add(usimsLogoLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(100, 220, -1, -1));

        aboutEllipsisGUIDialogWindowCloseButton.setText("Close");
        aboutEllipsisGUIDialogWindowCloseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutEllipsisGUIDialogWindowCloseButtonActionPerformed(evt);
            }
        });

        jPanel1.add(aboutEllipsisGUIDialogWindowCloseButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(160, 380, -1, -1));

        aboutEllipsisGuiDialogWindow.getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

        boundarySettingsDialog.getContentPane().setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        boundarySettingsDialog.setTitle("Boundary Condition Settings");
        boundarySettingsDialog.setBounds(100, 100, 420, 395);
        periodicxCheckBox.setText("Wrap-around boundary conditions");
        periodicxCheckBox.setToolTipText("Selecte whether or not boundary conditions should be wrapped around at the left and right borders");
        boundarySettingsDialog.getContentPane().add(periodicxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 10, -1, -1));

        periodic_rm_vxCheckBox.setText("Remove periodic boundary conditions up to a constant");
        boundarySettingsDialog.getContentPane().add(periodic_rm_vxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 30, -1, -1));

        simpleBoundaryRadioButton.setSelected(true);
        simpleBoundaryRadioButton.setText("Simple boundary condition motion");
        boundaryMotionButtonGroup.add(simpleBoundaryRadioButton);
        boundarySettingsDialog.getContentPane().add(simpleBoundaryRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        piecewiseBoundaryRadioButton.setText("Piecewise continuous boundary condition motion");
        boundaryMotionButtonGroup.add(piecewiseBoundaryRadioButton
        );
        boundarySettingsDialog.getContentPane().add(piecewiseBoundaryRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, -1, -1));

        jPanel18.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel18.setBorder(new javax.swing.border.TitledBorder("Simple boundary condition motion"));
        jPanel18.setFocusable(false);
        jLabel22.setText("Moving boundary velocity for left face:");
        jLabel22.setFocusable(false);
        jPanel18.add(jLabel22, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 25, -1, 20));

        BCmoveX0vTextField.setText("0");
        BCmoveX0vTextField.setPreferredSize(new java.awt.Dimension(60, 19));
        jPanel18.add(BCmoveX0vTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 30, -1, -1));

        jLabel30.setText("Moving boundary velocity for right face:");
        jLabel30.setFocusable(false);
        jPanel18.add(jLabel30, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        BCmoveX1vTextField.setText("0");
        BCmoveX1vTextField.setPreferredSize(new java.awt.Dimension(60, 19));
        jPanel18.add(BCmoveX1vTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 60, -1, -1));

        jLabel31.setText("BCmoveX0v1:");
        jLabel31.setFocusable(false);
        jPanel18.add(jLabel31, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 90, -1, -1));

        BCmoveX0v1TextField.setText("0");
        BCmoveX0v1TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        jPanel18.add(BCmoveX0v1TextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 90, -1, -1));

        boundarySettingsDialog.getContentPane().add(jPanel18, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 110, 340, 120));

        jPanel19.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel19.setBorder(new javax.swing.border.TitledBorder("Piecewise boundary condition motion"));
        jPanel19.setFocusable(false);
        jLabel2.setText("Please enter these parameters using the input file editor.");
        jLabel2.setFocusable(false);
        jPanel19.add(jLabel2, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 30, -1, -1));

        jLabel48.setText("They are not supported by the GUI");
        jLabel48.setFocusable(false);
        jPanel19.add(jLabel48, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 50, -1, -1));

        boundarySettingsDialog.getContentPane().add(jPanel19, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 240, 380, 80));

        boundarySettingsDialogCloseButton.setText("Close");
        boundarySettingsDialogCloseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                boundarySettingsDialogCloseButtonActionPerformed(evt);
            }
        });

        boundarySettingsDialog.getContentPane().add(boundarySettingsDialogCloseButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(170, 330, -1, -1));

        samplingTracersDialog.getContentPane().setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        samplingTracersDialog.setTitle("Sampling Tracers");
        samplingTracersDialog.setModal(true);
        samplingTracersDialog.setResizable(false);
        samplingTracersDialog.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                samplingTracersDialogComponentShown(evt);
            }
        });

        samplingTracersComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "No tracers defined" }));
        samplingTracersComboBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                samplingTracersComboBoxItemStateChanged(evt);
            }
        });

        samplingTracersDialog.getContentPane().add(samplingTracersComboBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 10, 160, -1));

        newSamplingTracerButton.setText("New Sampling Tracer");
        newSamplingTracerButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newSamplingTracerButtonActionPerformed(evt);
            }
        });

        samplingTracersDialog.getContentPane().add(newSamplingTracerButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(180, 10, -1, -1));

        deleteSamplingTracerButton.setText("Delete Sampling Tracer");
        deleteSamplingTracerButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deleteSamplingTracerButtonActionPerformed(evt);
            }
        });

        samplingTracersDialog.getContentPane().add(deleteSamplingTracerButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(350, 10, -1, -1));

        samplingTracerPropertiesJPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        samplingTracerPropertiesJPanel.setBorder(new javax.swing.border.TitledBorder("Sampling Tracer  Properties"));
        samplingTracerPropertiesJPanel.setFocusable(false);
        jPanel3.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel3.setBorder(new javax.swing.border.TitledBorder("Tracer type"));
        jPanel3.setFocusable(false);
        eulerianRadioButton.setSelected(true);
        eulerianRadioButton.setText("Eularian");
        tracerTypeButtonGroup.add(eulerianRadioButton);
        jPanel3.add(eulerianRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        lagrangianRadioButton.setText("Lagrangian");
        jPanel3.add(lagrangianRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        samplingTracerPropertiesJPanel.add(jPanel3, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 110, 70));

        jLabel36.setText("PPM file to store profile in:");
        jLabel36.setFocusable(false);
        samplingTracerPropertiesJPanel.add(jLabel36, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 100, -1, -1));

        samplingTracerPropertiesJPanel.add(samplingPPMComboBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(180, 100, 100, -1));

        jLabel37.setText("Initial x location of tracer");
        jLabel37.setFocusable(false);
        samplingTracerPropertiesJPanel.add(jLabel37, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 140, -1, -1));

        jLabel38.setText("Initial z location of tracer");
        jLabel38.setFocusable(false);
        samplingTracerPropertiesJPanel.add(jLabel38, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 160, -1, -1));

        Sampling_fieldComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Temperature", "X velocity", "Z velocity", "Nodal pressure", "Strain rate", "Stress", "Total stress", "Depletion", "Unrecognised" }));
        samplingTracerPropertiesJPanel.add(Sampling_fieldComboBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 190, -1, -1));

        jLabel39.setText("Field to sample:");
        jLabel39.setFocusable(false);
        samplingTracerPropertiesJPanel.add(jLabel39, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 190, -1, -1));

        jLabel40.setText("Direction in which to profile:");
        jLabel40.setFocusable(false);
        samplingTracerPropertiesJPanel.add(jLabel40, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 220, -1, -1));

        Sampling_dirnComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Parallel to X axis", "Parallel to Z axis", "Parallel to Y axis" }));
        samplingTracerPropertiesJPanel.add(Sampling_dirnComboBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(200, 220, -1, -1));

        Sampling_normalizeCheckBox.setText("Normalise data values");
        samplingTracerPropertiesJPanel.add(Sampling_normalizeCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 250, -1, -1));

        profileColourButton.setBackground(new java.awt.Color(0, 0, 0));
        profileColourButton.setForeground(new java.awt.Color(255, 255, 255));
        profileColourButton.setText("Profile Colour");
        profileColourButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                profileColourButtonActionPerformed(evt);
            }
        });

        samplingTracerPropertiesJPanel.add(profileColourButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(180, 40, -1, -1));

        samplingTracersDialog.getContentPane().add(samplingTracerPropertiesJPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, 350, 280));

        jButton2.setText("Close");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        samplingTracersDialog.getContentPane().add(jButton2, new org.netbeans.lib.awtextra.AbsoluteConstraints(250, 330, -1, -1));

        unrecognisedParametersDialog.setTitle("Unrecognised Parameters");
        unrecognisedParametersDialogTextArea.setEditable(false);
        jScrollPane1.setViewportView(unrecognisedParametersDialogTextArea);

        unrecognisedParametersDialog.getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

        unrecognisedParametersDialogCloseButton.setText("Close");
        unrecognisedParametersDialogCloseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                unrecognisedParametersDialogCloseButtonActionPerformed(evt);
            }
        });

        jPanel20.add(unrecognisedParametersDialogCloseButton);

        unrecognisedParametersDialog.getContentPane().add(jPanel20, java.awt.BorderLayout.SOUTH);

        unitsAndScalingDialog.setTitle("Units and Scaling");
        unitsAndScalingTextArea.setEditable(false);
        unitsAndScalingTextArea.setLineWrap(true);
        unitsAndScalingTextArea.setText("In the GUI, parameter values must always be entered in SI units, with the exception of length values, which must be entered in KM. Gravitational angle (gravacc) is entered in units such that when the value is multiplied by PI it will be a radian angle, measured anticlockwise from down.\n\nIf scaling factors have been entered in the scaling details pane, then these will be used to automatically scale the values written to, or read from, input files. This scaling is completely automatic, and should be invisible to the user. However, if editing input files manually, either with the text editor pane, or an external text editor, users must ensure that they scale all parameters correctly, so that their values are consistent with all the other parameters.\n\nHowever, gravitational acceleration (gravacc) is never scaled at all, and specific heat capacity (Cp) is scaled with a special scaling factor, that is not dependent on the other scaling factors. This makes the scaling inconsistent, but, because of the internals of how Ellipsis works, it tends to produce better results.\n\nThe list below specifies the units for all the ellipsis parameters recognised by the GUI. (Integer parameters do not appear in this list, because they are, without exception, dimensionless.) For an explanation of each parameter, see the 'The Ellipsis Input Format' by David Symonds.\n\nAbreviations used:\nm metre\ns second\nkg kilogram\nK kelvin\nPa pascals\nJ joule\n\ndimenx=km\ndimenz=km\nmaxadvtime=s\n\nBCmoveX0v=ms-1\nBCmoveX1v=ms-1\nBCmoveX0v1=ms-1\n\nBoundary conditions settings:\nicpt=km\naa1=km\naa2=km\nbb1=km\nbb2=km\naa=km\nbb=km\nrad=km\nhw=km\nmag is:-\n    K if Temp\n    Pa if Stress_x or Stress_z\n    ms-1 if Velocity_x or Velocity_z\n\ngravacc=ms-2 \ngravtheta=dimensionless (see above)\n\nTracer_rect_x1=km\nTracer_rect_x2=km\nTracer_rect_z1=km\nTracer_rect_z2=km\n\nvisc_max=kgm-1s-1\nvisc_min=kgm-1s-1\nT_block=K\nviscN0=kgm-1s-1\nviscT1=K\nviscT0=K                    This parameter is not scaled by the GUI\nviscZ=Jmole-1K-1    This parameter is not scaled by the GUI\nviscE=Jmole-1           This parameter is not scaled by the GUI\ndensity=kgm-3\nBulk_visc=dimensionless\n\ntherm_exp=K-1\ntherm_diff=m2s-1\nCp=m2s-2k-1\nQt=m2s-3\n\nminimum=kgs-2m-1\nmaximum=kgs-2m-1\nB0=Pa                           This parameter is not scaled by the GUI\nBz=Pam-1                    This parameter is not scaled by the GUI\nBp=dimensionless\nBc=Pa                           This parameter is not scaled by the GUI\nE0=dimensionless\nEdot0=s-1                   This parameter is not scaled by the GUI\nEa=dimensionless\nEdota=dimensionless\nEn=dimensionless\nEdotn=dimensionless\nET=K                            This parameter is not scaled by the GUI\nE0dt=s-1                    This parameter is not scaled by the GUI\n\ndepl_exp=kgm-3\nsp0=dimensionless\nsp1=dimensionless\nsp2=dimensionless\nsp3=dimensionless\nlp0=dimensionless\nlp1=dimensionless\nlp2=dimensionless\nlp3=dimensionless\n\nMaterial_rect_x1=km\nMaterial_rect_x2=km\nMaterial_rect_z1=km\nMaterial_rect_z2=km\n\nMaterial_circ_x1=km\nMaterial_circ_z1=km\nMaterial_circ_rad=km\n\nSampling_x=km\nSampling_z=km\n");
        unitsAndScalingTextArea.setWrapStyleWord(true);
        jScrollPane2.setViewportView(unitsAndScalingTextArea);

        unitsAndScalingDialog.getContentPane().add(jScrollPane2, java.awt.BorderLayout.CENTER);

        unitsAndScalingDialogCloseButton.setText("Close");
        unitsAndScalingDialogCloseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                unitsAndScalingDialogCloseButtonActionPerformed(evt);
            }
        });

        jPanel21.add(unitsAndScalingDialogCloseButton);

        unitsAndScalingDialog.getContentPane().add(jPanel21, java.awt.BorderLayout.SOUTH);

        ellipsisManualDialog.setTitle("Ellipsis Manual");
        ellipsisManualEditorPane.setEditable(false);
        jScrollPane3.setViewportView(ellipsisManualEditorPane);

        ellipsisManualDialog.getContentPane().add(jScrollPane3, java.awt.BorderLayout.CENTER);

        jButton1.setText("Close");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jPanel22.add(jButton1);

        ellipsisManualDialog.getContentPane().add(jPanel22, java.awt.BorderLayout.SOUTH);

        nonDimensionalizationDialog.setTitle("Ellipsis Manual");
        nonDimensionalizationEditorPane.setEditable(false);
        jScrollPane4.setViewportView(nonDimensionalizationEditorPane);

        nonDimensionalizationDialog.getContentPane().add(jScrollPane4, java.awt.BorderLayout.CENTER);

        nonDimensionalizationDialogCloseButton.setText("Close");
        nonDimensionalizationDialogCloseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nonDimensionalizationDialogCloseButtonActionPerformed(evt);
            }
        });

        jPanel25.add(nonDimensionalizationDialogCloseButton);

        nonDimensionalizationDialog.getContentPane().add(jPanel25, java.awt.BorderLayout.SOUTH);

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Ellipsis GUI");
        setBackground(new java.awt.Color(255, 255, 255));
        setFont(new java.awt.Font("Bitstream Charter", 1, 12));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        mainPanes.setToolTipText("");
        mainPanes.setPreferredSize(new java.awt.Dimension(32767, 32767));
        visualEditorPanel.setLayout(new java.awt.BorderLayout());

        visualEditorPanel.setMinimumSize(new java.awt.Dimension(600, 300));
        visualEditorPanel.setPreferredSize(new java.awt.Dimension(32767, 32767));
        visualEditorPanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentHidden(java.awt.event.ComponentEvent evt) {
                visualEditorPanelComponentHidden(evt);
            }
            public void componentShown(java.awt.event.ComponentEvent evt) {
                visualEditorPanelComponentShown(evt);
            }
        });

        visualControlsPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        shapeChooser.setToolTipText("");
        shapeChooser.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                shapeChooserItemStateChanged(evt);
            }
        });

        visualControlsPanel.add(shapeChooser, new org.netbeans.lib.awtextra.AbsoluteConstraints(40, 10, 100, -1));

        jPanel12.setLayout(null);

        jPanel12.setBorder(new javax.swing.border.TitledBorder("Show"));
        jPanel12.setFocusable(false);
        drawMaterialsCheckBox.setSelected(true);
        drawMaterialsCheckBox.setText("All Material Regions");
        drawMaterialsCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                drawMaterialsCheckBoxItemStateChanged(evt);
            }
        });

        jPanel12.add(drawMaterialsCheckBox);
        drawMaterialsCheckBox.setBounds(10, 40, 180, 23);

        drawBoundariesCheckBox.setSelected(true);
        drawBoundariesCheckBox.setText("All Boundary Conditions");
        drawBoundariesCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                drawBoundariesCheckBoxItemStateChanged(evt);
            }
        });

        jPanel12.add(drawBoundariesCheckBox);
        drawBoundariesCheckBox.setBounds(10, 60, 137, 23);

        drawTracersCheckBox.setText("All Tracers");
        drawTracersCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                drawTracersCheckBoxItemStateChanged(evt);
            }
        });

        jPanel12.add(drawTracersCheckBox);
        drawTracersCheckBox.setBounds(10, 80, 75, 23);

        backgroundPictureCheckBox.setText("Background Picture");
        backgroundPictureCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                backgroundPictureCheckBoxItemStateChanged(evt);
            }
        });

        jPanel12.add(backgroundPictureCheckBox);
        backgroundPictureCheckBox.setBounds(10, 20, 119, 23);

        drawThermalZonesCheckBox.setSelected(true);
        drawThermalZonesCheckBox.setText("All Thermal Zones");
        jPanel12.add(drawThermalZonesCheckBox);
        drawThermalZonesCheckBox.setBounds(10, 100, 180, 23);

        drawDrawingCheckBox.setSelected(true);
        drawDrawingCheckBox.setText("Drawings");
        drawDrawingCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                drawDrawingCheckBoxItemStateChanged(evt);
            }
        });

        jPanel12.add(drawDrawingCheckBox);
        drawDrawingCheckBox.setBounds(10, 120, 140, 23);

        visualControlsPanel.add(jPanel12, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 320, 200, 150));

        controlsCardHolder.setLayout(new java.awt.CardLayout());

        rectangleControls.setLayout(null);

        rectangleControls.setBorder(new javax.swing.border.EtchedBorder());
        rectangleControls.setFocusable(false);
        rectangleControls.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentHidden(java.awt.event.ComponentEvent evt) {
                rectangleControlsComponentHidden(evt);
            }
        });

        rectangleX1TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        rectangleControls.add(rectangleX1TextField);
        rectangleX1TextField.setBounds(20, 60, 60, 19);

        rectangleX2TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        rectangleControls.add(rectangleX2TextField);
        rectangleX2TextField.setBounds(110, 60, 60, 19);

        rectangleY1TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        rectangleControls.add(rectangleY1TextField);
        rectangleY1TextField.setBounds(20, 100, 60, 19);

        rectangleY2TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        rectangleControls.add(rectangleY2TextField);
        rectangleY2TextField.setBounds(110, 100, 60, 19);

        jTextArea3.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea3.setEditable(false);
        jTextArea3.setText("z1 (km)");
        jTextArea3.setFocusable(false);
        rectangleControls.add(jTextArea3);
        jTextArea3.setBounds(20, 80, 50, 18);

        jTextArea4.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea4.setEditable(false);
        jTextArea4.setText("z2 (km)");
        jTextArea4.setFocusable(false);
        rectangleControls.add(jTextArea4);
        jTextArea4.setBounds(110, 80, 50, 18);

        jTextArea2.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea2.setEditable(false);
        jTextArea2.setText("x2 (km)");
        jTextArea2.setFocusable(false);
        rectangleControls.add(jTextArea2);
        jTextArea2.setBounds(110, 40, 56, 18);

        jTextArea1.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea1.setEditable(false);
        jTextArea1.setText("x1 (km)");
        jTextArea1.setFocusable(false);
        rectangleControls.add(jTextArea1);
        jTextArea1.setBounds(25, 40, 50, 18);

        addNewRectangleButton.setText("Add new rectangle");
        addNewRectangleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addNewRectangleButtonActionPerformed(evt);
            }
        });

        rectangleControls.add(addNewRectangleButton);
        addNewRectangleButton.setBounds(8, 180, 170, 23);

        rectangleUpdateButton.setText("Update");
        rectangleUpdateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rectangleUpdateButtonActionPerformed(evt);
            }
        });

        rectangleControls.add(rectangleUpdateButton);
        rectangleUpdateButton.setBounds(10, 140, 170, 23);

        controlsCardHolder.add(rectangleControls, "Rectangle");

        circleControls.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        circleControls.setBorder(new javax.swing.border.EtchedBorder());
        circleControls.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentHidden(java.awt.event.ComponentEvent evt) {
                circleControlsComponentHidden(evt);
            }
        });

        jLabel18.setText("Centre:");
        circleControls.add(jLabel18, new org.netbeans.lib.awtextra.AbsoluteConstraints(70, 10, -1, -1));

        jLabel19.setText("x (km)");
        circleControls.add(jLabel19, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 30, -1, -1));

        jLabel20.setText("z (km)");
        jLabel20.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                jLabel20ComponentShown(evt);
            }
        });

        circleControls.add(jLabel20, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 30, -1, -1));

        circleAddButton.setText("Add new circle");
        circleAddButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                circleAddButtonActionPerformed(evt);
            }
        });

        circleControls.add(circleAddButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 180, -1, -1));

        circleUpdateButton.setText("Update");
        circleUpdateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                circleUpdateButtonActionPerformed(evt);
            }
        });

        circleControls.add(circleUpdateButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 140, -1, -1));

        jLabel21.setText("Radius (km)");
        circleControls.add(jLabel21, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 80, -1, -1));

        circleControls.add(circleRadiusTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(100, 80, 60, -1));

        circleControls.add(circleXTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 50, 60, -1));

        circleControls.add(circleZTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(110, 50, 60, -1));

        controlsCardHolder.add(circleControls, "Circle");

        lineControls.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        lineControls.setBorder(new javax.swing.border.EtchedBorder());
        lineX2TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        lineControls.add(lineX2TextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(100, 30, -1, -1));

        lineX1TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        lineControls.add(lineX1TextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 30, -1, -1));

        lineZ1TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        lineControls.add(lineZ1TextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 80, -1, -1));

        lineZ2TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        lineControls.add(lineZ2TextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(100, 80, -1, -1));

        jTextArea6.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea6.setEditable(false);
        jTextArea6.setText("z1 (km)");
        lineControls.add(jTextArea6, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 60, -1, -1));

        jTextArea7.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea7.setEditable(false);
        jTextArea7.setText("z2 (km)");
        lineControls.add(jTextArea7, new org.netbeans.lib.awtextra.AbsoluteConstraints(110, 60, -1, -1));

        jTextArea8.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea8.setEditable(false);
        jTextArea8.setText("x2 (km)");
        lineControls.add(jTextArea8, new org.netbeans.lib.awtextra.AbsoluteConstraints(110, 10, -1, -1));

        jTextArea9.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea9.setEditable(false);
        jTextArea9.setText("x1 (km)");
        lineControls.add(jTextArea9, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 10, -1, -1));

        addNewLineButton.setText("Add new line");
        lineControls.add(addNewLineButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 180, -1, -1));

        lineUpdateButton.setText("Update");
        lineControls.add(lineUpdateButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 140, -1, -1));

        controlsCardHolder.add(lineControls, "Line");

        harmonicControls.setBorder(new javax.swing.border.EtchedBorder());
        jTextArea10.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea10.setEditable(false);
        jTextArea10.setText("Ellipsis GUI does not yet \nsupport harmonics. Please \ncreate harmonic shapes by \nediting the input file directly.");
        harmonicControls.add(jTextArea10);

        controlsCardHolder.add(harmonicControls, "Harmonic");

        polynomialControls.setBorder(new javax.swing.border.EtchedBorder());
        jTextArea11.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea11.setEditable(false);
        jTextArea11.setText("Ellipsis GUI does not yet \nsupport polynomials. Please \ncreate polynomial shapes by \nediting the input file directly.");
        polynomialControls.add(jTextArea11);

        controlsCardHolder.add(polynomialControls, "Polynomial");

        triangleControls.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        triangleControls.setBorder(new javax.swing.border.EtchedBorder());
        triangleControls.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentHidden(java.awt.event.ComponentEvent evt) {
                triangleControlsComponentHidden(evt);
            }
        });

        triangleY2TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        triangleControls.add(triangleY2TextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(110, 70, -1, -1));

        triangleY1TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        triangleControls.add(triangleY1TextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(110, 30, -1, -1));

        triangleX1TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        triangleControls.add(triangleX1TextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 30, -1, -1));

        triangleX2TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        triangleControls.add(triangleX2TextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 70, -1, -1));

        jTextArea12.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea12.setEditable(false);
        jTextArea12.setText("z2 (km)");
        triangleControls.add(jTextArea12, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 50, -1, -1));

        jTextArea13.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea13.setEditable(false);
        jTextArea13.setText("z1 (km)");
        triangleControls.add(jTextArea13, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 10, -1, -1));

        jTextArea14.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea14.setEditable(false);
        jTextArea14.setText("x2 (km)");
        triangleControls.add(jTextArea14, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 50, -1, -1));

        jTextArea15.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea15.setEditable(false);
        jTextArea15.setText("x1 (km)");
        triangleControls.add(jTextArea15, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 10, -1, -1));

        addNewTriangleButton.setText("Add new triangle");
        addNewTriangleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addNewTriangleButtonActionPerformed(evt);
            }
        });

        triangleControls.add(addNewTriangleButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 180, 150, -1));

        triangleUpdateButton.setText("Update");
        triangleUpdateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                triangleUpdateButtonActionPerformed(evt);
            }
        });

        triangleControls.add(triangleUpdateButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 140, 150, -1));

        jTextArea16.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea16.setEditable(false);
        jTextArea16.setText("x3 (km)");
        triangleControls.add(jTextArea16, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 90, -1, -1));

        triangleY3TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        triangleControls.add(triangleY3TextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(110, 110, -1, -1));

        triangleX3TextField.setPreferredSize(new java.awt.Dimension(60, 19));
        triangleControls.add(triangleX3TextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 110, -1, -1));

        jTextArea17.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea17.setEditable(false);
        jTextArea17.setText("z3 (km)");
        triangleControls.add(jTextArea17, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 90, -1, -1));

        controlsCardHolder.add(triangleControls, "Triangle");

        visualControlsPanel.add(controlsCardHolder, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 50, 190, 220));

        drawSolidShapesCheckBox.setSelected(true);
        drawSolidShapesCheckBox.setText("Draw solid shapes");
        drawSolidShapesCheckBox.setToolTipText("Select whether to draw solid or outline shapes");
        visualControlsPanel.add(drawSolidShapesCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 280, -1, -1));

        animateSelectedRegionsCheckBox.setText("Animate Selected Regions");
        visualControlsPanel.add(animateSelectedRegionsCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 480, -1, -1));

        visualEditorPanel.add(visualControlsPanel, java.awt.BorderLayout.WEST);

        materialsDrawingsBoundaryTracersPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        materialsPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        materialsPanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                materialsPanelComponentShown(evt);
            }
        });

        materialSettingsButton.setText("Material Settings");
        materialSettingsButton.setToolTipText("Change the general settings affecting all materials");
        materialSettingsButton.setAlignmentX(0.5F);
        materialSettingsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                materialSettingsButtonActionPerformed(evt);
            }
        });

        materialsPanel.add(materialSettingsButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(50, 10, -1, -1));

        jPanel13.setLayout(null);

        jPanel13.setBorder(new javax.swing.border.TitledBorder("Materials"));
        jPanel13.setFocusable(false);
        jPanel13.setPreferredSize(new java.awt.Dimension(100, 180));
        materialChooser.setToolTipText("");
        materialChooser.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                materialChooserItemStateChanged(evt);
            }
        });

        jPanel13.add(materialChooser);
        materialChooser.setBounds(10, 20, 130, 30);

        newMaterialButton.setText("New Material");
        newMaterialButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newMaterialButtonActionPerformed(evt);
            }
        });

        jPanel13.add(newMaterialButton);
        newMaterialButton.setBounds(10, 60, 130, 23);

        deleteMaterialButton.setText("Delete Material");
        deleteMaterialButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deleteMaterialButtonActionPerformed(evt);
            }
        });

        jPanel13.add(deleteMaterialButton);
        deleteMaterialButton.setBounds(10, 90, 130, 23);

        editRheologyButton.setText("Edit Rheology");
        editRheologyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editRheologyButtonActionPerformed(evt);
            }
        });

        jPanel13.add(editRheologyButton);
        editRheologyButton.setBounds(10, 120, 130, 23);

        materialsPanel.add(jPanel13, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 40, 180, -1));

        jPanel14.setLayout(null);

        jPanel14.setBorder(new javax.swing.border.TitledBorder("Material Regions"));
        jPanel14.setFocusable(false);
        jPanel14.setPreferredSize(new java.awt.Dimension(200, 300));
        materialRegionForwardButton.setText("Bring Forward");
        materialRegionForwardButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                materialRegionForwardButtonActionPerformed(evt);
            }
        });

        jPanel14.add(materialRegionForwardButton);
        materialRegionForwardButton.setBounds(10, 130, 130, 23);

        materialRegionBackButton.setText("Put Back");
        materialRegionBackButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                materialRegionBackButtonActionPerformed(evt);
            }
        });

        jPanel14.add(materialRegionBackButton);
        materialRegionBackButton.setBounds(10, 160, 79, 23);

        materialRegionDeleteButton.setText("Delete");
        materialRegionDeleteButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                materialRegionDeleteButtonActionPerformed(evt);
            }
        });

        jPanel14.add(materialRegionDeleteButton);
        materialRegionDeleteButton.setBounds(110, 160, 65, 23);

        materialRegionList.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0)));
        materialRegionList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        materialRegionList.setToolTipText("Choose the region to edit or display");
        materialRegionList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                materialRegionListValueChanged(evt);
            }
        });

        materialRegionListScrollPane.setViewportView(materialRegionList);

        jPanel14.add(materialRegionListScrollPane);
        materialRegionListScrollPane.setBounds(10, 20, 190, 100);

        materialsPanel.add(jPanel14, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 230, 210, 200));

        materialsDrawingBoundaryTracersPanel.addTab("Materials", materialsPanel);

        drawingsPanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                drawingsPanelComponentShown(evt);
            }
        });

        drawingChangeColourButton.setText("Change Colour");
        drawingChangeColourButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                drawingChangeColourButtonActionPerformed(evt);
            }
        });

        drawingsPanel.add(drawingChangeColourButton);

        jPanel15.setLayout(null);

        jPanel15.setBorder(new javax.swing.border.TitledBorder("Drawing Regions"));
        jPanel15.setPreferredSize(new java.awt.Dimension(210, 300));
        drawingRegionForwardButton.setText("Bring Forward");
        drawingRegionForwardButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                drawingRegionForwardButtonActionPerformed(evt);
            }
        });

        jPanel15.add(drawingRegionForwardButton);
        drawingRegionForwardButton.setBounds(10, 130, 130, 23);

        drawingRegionBackButton.setText("Put Back");
        drawingRegionBackButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                drawingRegionBackButtonActionPerformed(evt);
            }
        });

        jPanel15.add(drawingRegionBackButton);
        drawingRegionBackButton.setBounds(10, 160, 79, 23);

        drawingRegionDeleteButton.setText("Delete");
        drawingRegionDeleteButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                drawingRegionDeleteButtonActionPerformed(evt);
            }
        });

        jPanel15.add(drawingRegionDeleteButton);
        drawingRegionDeleteButton.setBounds(110, 160, 65, 23);

        drawingRegionList.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0)));
        drawingRegionList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        drawingRegionList.setToolTipText("Choose the region to edit or display");
        drawingRegionList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                drawingRegionListValueChanged(evt);
            }
        });

        drawingRegionListScrollPane.setViewportView(drawingRegionList);

        jPanel15.add(drawingRegionListScrollPane);
        drawingRegionListScrollPane.setBounds(10, 20, 190, 100);

        drawingsPanel.add(jPanel15);

        materialsDrawingBoundaryTracersPanel.addTab("Drawings", drawingsPanel);

        boundaryConditionsPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        boundaryConditionsPanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                boundaryConditionsPanelComponentShown(evt);
            }
        });

        boundaryConditionsSettingsButton.setText("Boundary Settings");
        boundaryConditionsSettingsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                boundaryConditionsSettingsButtonActionPerformed(evt);
            }
        });

        boundaryConditionsPanel.add(boundaryConditionsSettingsButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(40, 10, -1, -1));

        boundaryRegionPropertiesJPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        boundaryRegionPropertiesJPanel.setBorder(new javax.swing.border.TitledBorder("Current/Next Region"));
        boundaryPropertyComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Temperature", "Stress (x axis)", "Stess (z axis)", "Velocity (x axis)", "Velocity (z axis)" }));
        boundaryRegionPropertiesJPanel.add(boundaryPropertyComboBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(51, 25, -1, -1));

        boundaryXAxisRadioButton.setText("x axis");
        boundaryXAxisRadioButton.setToolTipText("Select the axis which is normal to the plane of the boundary condition. When using Ellipsis 2D, the Y axis should normally be selected, and only regions normal to this axis are displayed by the GUI.");
        boundaryRegionPropertiesJPanel.add(boundaryXAxisRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        boundaryYAxisRadioButton.setSelected(true);
        boundaryYAxisRadioButton.setText("y axis");
        boundaryYAxisRadioButton.setToolTipText("Select the axis which is normal to the plane of the boundary condition. When using Ellipsis 2D, the Y axis should normally be selected, and only regions normal to this axis are displayed by the GUI.");
        boundaryRegionPropertiesJPanel.add(boundaryYAxisRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(70, 60, -1, -1));

        boundaryZAxisRadioButton.setText("z axis");
        boundaryZAxisRadioButton.setToolTipText("Select the axis which is normal to the plane of the boundary condition. When using Ellipsis 2D, the Y axis should normally be selected, and only regions normal to this axis are displayed by the GUI.");
        boundaryRegionPropertiesJPanel.add(boundaryZAxisRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(130, 60, -1, -1));

        jLabel24.setText("Half width of");
        boundaryRegionPropertiesJPanel.add(jLabel24, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 110, -1, -1));

        jLabel25.setText("smoothing edge");
        boundaryRegionPropertiesJPanel.add(jLabel25, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 130, -1, -1));

        boundaryHalfWidthTextField.setText("0");
        boundaryHalfWidthTextField.setPreferredSize(new java.awt.Dimension(60, 19));
        boundaryHalfWidthTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                boundaryHalfWidthTextFieldFocusLost(evt);
            }
        });

        boundaryRegionPropertiesJPanel.add(boundaryHalfWidthTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(130, 120, -1, -1));

        jLabel26.setText("Value of boundary");
        boundaryRegionPropertiesJPanel.add(jLabel26, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 150, -1, -1));

        jLabel27.setText("condition");
        boundaryRegionPropertiesJPanel.add(jLabel27, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 170, -1, -1));

        boundaryValueTextField.setText("0");
        boundaryValueTextField.setPreferredSize(new java.awt.Dimension(60, 19));
        boundaryValueTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                boundaryValueTextFieldFocusLost(evt);
            }
        });

        boundaryRegionPropertiesJPanel.add(boundaryValueTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(130, 160, -1, -1));

        boundaryChangeColourButton.setText("Change Colour");
        boundaryChangeColourButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                boundaryChangeColourButtonActionPerformed(evt);
            }
        });

        boundaryRegionPropertiesJPanel.add(boundaryChangeColourButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(50, 190, -1, -1));

        jLabel1.setText("Intercept");
        jLabel1.setToolTipText("The normal-axis intercept of the boundary condition plane");
        boundaryRegionPropertiesJPanel.add(jLabel1, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 90, -1, -1));

        boundaryConditionsPanel.add(boundaryRegionPropertiesJPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, 210, 230));

        jPanel17.setLayout(null);

        jPanel17.setBorder(new javax.swing.border.TitledBorder("Boundary Condition Regions"));
        jPanel17.setPreferredSize(new java.awt.Dimension(200, 300));
        boundaryRegionForwardButton.setText("Bring Forward");
        boundaryRegionForwardButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                boundaryRegionForwardButtonActionPerformed(evt);
            }
        });

        jPanel17.add(boundaryRegionForwardButton);
        boundaryRegionForwardButton.setBounds(10, 130, 130, 23);

        boundaryRegionBackButton.setText("Put Back");
        boundaryRegionBackButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                boundaryRegionBackButtonActionPerformed(evt);
            }
        });

        jPanel17.add(boundaryRegionBackButton);
        boundaryRegionBackButton.setBounds(10, 160, 79, 23);

        boundaryRegionDeleteButton.setText("Delete");
        boundaryRegionDeleteButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                boundaryRegionDeleteButtonActionPerformed(evt);
            }
        });

        jPanel17.add(boundaryRegionDeleteButton);
        boundaryRegionDeleteButton.setBounds(110, 160, 65, 23);

        boundaryRegionList.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0)));
        boundaryRegionList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        boundaryRegionList.setToolTipText("Choose the region to edit or display");
        boundaryRegionList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                boundaryRegionListValueChanged(evt);
            }
        });

        TracerRegionListScrollPane1.setViewportView(boundaryRegionList);

        jPanel17.add(TracerRegionListScrollPane1);
        TracerRegionListScrollPane1.setBounds(10, 20, 190, 100);

        boundaryConditionsPanel.add(jPanel17, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 280, 210, 200));

        materialsDrawingBoundaryTracersPanel.addTab("Boundary Conditions", boundaryConditionsPanel);

        tracersPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        tracersPanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                tracersPanelComponentShown(evt);
            }
        });

        currentNextTracerLabel.setText("Current/Next Tracer \n");
        currentNextTracerLabel.setToolTipText("The number of tracers per unit area");
        tracersPanel.add(currentNextTracerLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        regionsDensityLabel.setText("Region's Density");
        tracersPanel.add(regionsDensityLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        tracerDensityTextField.setText("4");
        tracerDensityTextField.setPreferredSize(new java.awt.Dimension(60, 19));
        tracerDensityTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                tracerDensityTextFieldFocusLost(evt);
            }
        });

        tracersPanel.add(tracerDensityTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 50, -1, -1));

        tracerChangeColourButton.setText("Change Colour");
        tracerChangeColourButton.setEnabled(false);
        tracerChangeColourButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tracerChangeColourButtonActionPerformed(evt);
            }
        });

        tracersPanel.add(tracerChangeColourButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(50, 90, -1, -1));

        tracerRegionsPanel.setLayout(null);

        tracerRegionsPanel.setBorder(new javax.swing.border.TitledBorder("Tracer Regions"));
        tracerRegionsPanel.setPreferredSize(new java.awt.Dimension(200, 300));
        tracerRegionsPanel.setEnabled(false);
        tracerRegionForwardButton.setText("Bring Forward");
        tracerRegionForwardButton.setEnabled(false);
        tracerRegionForwardButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tracerRegionForwardButtonActionPerformed(evt);
            }
        });

        tracerRegionsPanel.add(tracerRegionForwardButton);
        tracerRegionForwardButton.setBounds(10, 130, 130, 23);

        tracerRegionBackButton.setText("Put Back");
        tracerRegionBackButton.setEnabled(false);
        tracerRegionBackButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tracerRegionBackButtonActionPerformed(evt);
            }
        });

        tracerRegionsPanel.add(tracerRegionBackButton);
        tracerRegionBackButton.setBounds(10, 160, 79, 23);

        tracerRegionDeleteButton.setText("Delete");
        tracerRegionDeleteButton.setEnabled(false);
        tracerRegionDeleteButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tracerRegionDeleteButtonActionPerformed(evt);
            }
        });

        tracerRegionsPanel.add(tracerRegionDeleteButton);
        tracerRegionDeleteButton.setBounds(110, 160, 65, 23);

        tracerRegionListScrollPane.setEnabled(false);
        tracerRegionList.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0)));
        tracerRegionList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        tracerRegionList.setToolTipText("Choose the region to edit or display");
        tracerRegionList.setEnabled(false);
        tracerRegionList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                tracerRegionListValueChanged(evt);
            }
        });

        tracerRegionListScrollPane.setViewportView(tracerRegionList);

        tracerRegionsPanel.add(tracerRegionListScrollPane);
        tracerRegionListScrollPane.setBounds(10, 20, 190, 100);

        tracersPanel.add(tracerRegionsPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 130, 210, 200));

        useDefaultTracerSetup.setSelected(true);
        useDefaultTracerSetup.setText("Use default tracer setup");
        useDefaultTracerSetup.setToolTipText("Use only a single tracer, covering the whole simulation area, and with a density as specified in the box below.");
        useDefaultTracerSetup.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                useDefaultTracerSetupItemStateChanged(evt);
            }
        });

        tracersPanel.add(useDefaultTracerSetup, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 10, -1, -1));

        materialsDrawingBoundaryTracersPanel.addTab("Tracers", tracersPanel);

        thermalZonesPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        thermalZonesPanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                thermalZonesPanelComponentShown(evt);
            }
        });

        halfWidthLabel.setText("Half width of ");
        thermalZonesPanel.add(halfWidthLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 10, -1, -1));

        jLabel47.setText("of smoothed edge");
        thermalZonesPanel.add(jLabel47, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 30, -1, -1));

        halfWidthLabel1.setText("Temperature");
        thermalZonesPanel.add(halfWidthLabel1, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 70, -1, -1));

        addRadioButton.setSelected(true);
        addRadioButton.setText("Add");
        thermalZonesPanel.add(addRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 100, -1, -1));

        multiplyRadioButton.setText("Multiply");
        thermalZonesPanel.add(multiplyRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 120, -1, -1));

        replaceRadioButton.setText("Replace");
        thermalZonesPanel.add(replaceRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 140, -1, -1));

        jPanel24.setLayout(null);

        jPanel24.setBorder(new javax.swing.border.TitledBorder("Thermal Zone Regions"));
        jPanel24.setPreferredSize(new java.awt.Dimension(200, 300));
        thermalRegionForwardButton.setText("Bring Forward");
        thermalRegionForwardButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                thermalRegionForwardButtonActionPerformed(evt);
            }
        });

        jPanel24.add(thermalRegionForwardButton);
        thermalRegionForwardButton.setBounds(10, 130, 130, 23);

        thermalRegionBackButton.setText("Put Back");
        thermalRegionBackButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                thermalRegionBackButtonActionPerformed(evt);
            }
        });

        jPanel24.add(thermalRegionBackButton);
        thermalRegionBackButton.setBounds(10, 160, 79, 23);

        thermalRegionDeleteButton.setText("Delete");
        thermalRegionDeleteButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                thermalRegionDeleteButtonActionPerformed(evt);
            }
        });

        jPanel24.add(thermalRegionDeleteButton);
        thermalRegionDeleteButton.setBounds(110, 160, 65, 23);

        thermalRegionList.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0)));
        thermalRegionList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        thermalRegionList.setToolTipText("Choose the region to edit or display");
        thermalRegionList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                thermalRegionListValueChanged(evt);
            }
        });

        TracerRegionListScrollPane2.setViewportView(thermalRegionList);

        jPanel24.add(TracerRegionListScrollPane2);
        TracerRegionListScrollPane2.setBounds(10, 20, 190, 100);

        thermalZonesPanel.add(jPanel24, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 170, 210, 200));

        materialsDrawingBoundaryTracersPanel.addTab("Thermal Zones", thermalZonesPanel);

        materialsDrawingsBoundaryTracersPanel.add(materialsDrawingBoundaryTracersPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 0, 230, 550));

        visualEditorPanel.add(materialsDrawingsBoundaryTracersPanel, java.awt.BorderLayout.EAST);

        jPanel2.setToolTipText("Enter the real size of the area displayed in the window in kilometres.");
        jPanel2.setFocusable(false);
        jLabel15.setText("Window dimensions: x=");
        jLabel15.setFocusable(false);
        jPanel2.add(jLabel15);

        windowDimensionXTextField.setText("20");
        windowDimensionXTextField.setPreferredSize(new java.awt.Dimension(60, 19));
        windowDimensionXTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                windowDimensionXTextFieldFocusLost(evt);
            }
        });

        jPanel2.add(windowDimensionXTextField);

        jLabel16.setText("km   z=");
        jLabel16.setFocusable(false);
        jPanel2.add(jLabel16);

        windowDimensionYTextField.setText("10");
        windowDimensionYTextField.setPreferredSize(new java.awt.Dimension(60, 19));
        windowDimensionYTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                windowDimensionYTextFieldFocusLost(evt);
            }
        });

        jPanel2.add(windowDimensionYTextField);

        jLabel17.setText("km");
        jLabel17.setFocusable(false);
        jPanel2.add(jLabel17);

        fixAspectRatioCheckBox.setSelected(true);
        fixAspectRatioCheckBox.setText("Fix aspect ratio");
        fixAspectRatioCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                fixAspectRatioCheckBoxItemStateChanged(evt);
            }
        });

        jPanel2.add(fixAspectRatioCheckBox);

        visualEditorPanel.add(jPanel2, java.awt.BorderLayout.SOUTH);

        visualEditorCentralPanel.setLayout(null);

        visualEditorCentralPanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                visualEditorCentralPanelComponentResized(evt);
            }
        });

        visualEditorPanel.add(visualEditorCentralPanel, java.awt.BorderLayout.CENTER);

        mainPanes.addTab("Visual Editor", visualEditorPanel);

        generalSettingsPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        generalSettingsPanel.setFocusable(false);
        verbosityPanel.setLayout(null);

        verbosityPanel.setBorder(new javax.swing.border.TitledBorder("Verbosity"));
        verbosityPanel.setToolTipText("");
        verbosityPanel.setFocusable(false);
        describeCheckBox.setMnemonic('d');
        describeCheckBox.setText("Describe");
        describeCheckBox.setToolTipText("Controls whether Ellipsis will describe the search for parameters");
        verbosityPanel.add(describeCheckBox);
        describeCheckBox.setBounds(10, 20, 67, 23);

        verboseInputValuesCheckBox.setMnemonic('v');
        verboseInputValuesCheckBox.setText("Verbose (input values)");
        verboseInputValuesCheckBox.setToolTipText("Controls whether Ellipsis will print out the input parameters as they are read in");
        verboseInputValuesCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                verboseInputValuesCheckBoxItemStateChanged(evt);
            }
        });

        verbosityPanel.add(verboseInputValuesCheckBox);
        verboseInputValuesCheckBox.setBounds(10, 50, 170, 23);

        beginnerCheckBox.setMnemonic('b');
        beginnerCheckBox.setText("Beginner");
        beginnerCheckBox.setToolTipText("Controls whether to be verbose when parameters are missing");
        beginnerCheckBox.setEnabled(false);
        verbosityPanel.add(beginnerCheckBox);
        beginnerCheckBox.setBounds(10, 80, 67, 23);

        verboseCodeBehaviourCheckBox.setMnemonic('c');
        verboseCodeBehaviourCheckBox.setText("Verbose (code behaviour)");
        verboseCodeBehaviourCheckBox.setToolTipText("Controls whether the code should be verbose about its behaviour");
        verbosityPanel.add(verboseCodeBehaviourCheckBox);
        verboseCodeBehaviourCheckBox.setBounds(10, 110, 190, 23);

        generalSettingsPanel.add(verbosityPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(5, 5, 220, 150));

        multrigridPanel.setLayout(null);

        multrigridPanel.setBorder(new javax.swing.border.TitledBorder("Multigrid"));
        multrigridPanel.setToolTipText("");
        multrigridPanel.setFocusable(false);
        jLabel3.setText("Number of Multrigrid Elements, at multigrid level two:");
        jLabel3.setFocusable(false);
        multrigridPanel.add(jLabel3);
        jLabel3.setBounds(10, 20, 360, 20);

        jLabel4.setText("along x axis");
        jLabel4.setFocusable(false);
        multrigridPanel.add(jLabel4);
        jLabel4.setBounds(10, 50, 90, 20);

        jLabel5.setText("along z axis");
        jLabel5.setFocusable(false);
        multrigridPanel.add(jLabel5);
        jLabel5.setBounds(10, 80, 80, 20);

        multrigridPanel.add(jSeparator1);
        jSeparator1.setBounds(10, 110, 340, 10);

        jLabel6.setText("Number of multigrid levels (1-12)");
        jLabel6.setFocusable(false);
        multrigridPanel.add(jLabel6);
        jLabel6.setBounds(10, 120, 220, 20);

        jLabel7.setText("Number of pressure multigrid levels");
        jLabel7.setFocusable(false);
        multrigridPanel.add(jLabel7);
        jLabel7.setBounds(10, 150, 230, 20);

        generalSettingsPanel.add(multrigridPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(230, 5, 360, 180));

        timePanel.setLayout(null);

        timePanel.setBorder(new javax.swing.border.TitledBorder("Time"));
        timePanel.setToolTipText("");
        timePanel.setFocusable(false);
        jLabel8.setText("Maximum elapsed time to simulate");
        jLabel8.setFocusable(false);
        timePanel.add(jLabel8);
        jLabel8.setBounds(10, 20, 220, 20);

        jLabel9.setText("Minimum number of advection-diffusion time steps");
        jLabel9.setFocusable(false);
        timePanel.add(jLabel9);
        jLabel9.setBounds(10, 50, 330, 20);

        jLabel10.setText("Maximum number of advection-diffusion time steps");
        jLabel10.setFocusable(false);
        timePanel.add(jLabel10);
        jLabel10.setBounds(10, 80, 330, 20);

        generalSettingsPanel.add(timePanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 200, 420, 130));

        jLabel42.setText("Solver:");
        jLabel42.setFocusable(false);
        generalSettingsPanel.add(jLabel42, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 340, -1, -1));

        SolverTextField.setText("multigrid");
        generalSettingsPanel.add(SolverTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(60, 340, 60, -1));

        mainPanes.addTab("General Settings", generalSettingsPanel);

        initialConditionsPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        environmentPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        environmentPanel.setBorder(new javax.swing.border.TitledBorder("Environment"));
        environmentPanel.setFocusable(false);
        jLabel11.setText("Gravitational accleration");
        jLabel11.setFocusable(false);
        environmentPanel.add(jLabel11, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        jLabel12.setText("Gravitational angle");
        jLabel12.setToolTipText("The angle between the gravitational vector and the z axis. This parameter is multiplied by PI and then treated as a radian angle.");
        jLabel12.setFocusable(false);
        environmentPanel.add(jLabel12, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        initialConditionsPanel.add(environmentPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(5, 0, 240, 70));

        mainPanes.addTab("Initial Conditions", initialConditionsPanel);

        outputSettingsPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel4.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel4.setBorder(new javax.swing.border.TitledBorder("File settings"));
        jPanel4.setFocusable(false);
        jLabel13.setText("Root name of output files:");
        jLabel13.setFocusable(false);
        jPanel4.add(jLabel13, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        datafileTextField.setText("initialize");
        jPanel4.add(datafileTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(180, 20, 100, -1));

        COMPRESSCheckBox.setSelected(true);
        COMPRESSCheckBox.setText("Compress output files");
        jPanel4.add(COMPRESSCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        gzipLabel.setText("Path to file compression command");
        gzipLabel.setFocusable(false);
        jPanel4.add(gzipLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 70, -1, -1));

        gzipTextField.setText("/usr/local/bin/gzip");
        jPanel4.add(gzipTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(230, 70, 100, -1));

        outputSettingsPanel.add(jPanel4, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 0, 340, 100));

        jPanel5.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel5.setBorder(new javax.swing.border.TitledBorder("Frequency of Output"));
        jPanel5.setFocusable(false);
        jLabel23.setText("Timesteps between numerical data output");
        jLabel23.setToolTipText("Controls the number of timesteps between successive outputs of numerical data files (*.particles etc.).");
        jLabel23.setFocusable(false);
        jPanel5.add(jLabel23, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        jLabel28.setText("Timesteps between PPM output");
        jLabel28.setToolTipText("Controls the number of timesteps between successive outputs of PPM files.");
        jLabel28.setFocusable(false);
        jPanel5.add(jLabel28, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        outputSettingsPanel.add(jPanel5, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 100, 370, 70));

        jPanel6.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel6.setBorder(new javax.swing.border.TitledBorder("Nodal output variables"));
        jPanel6.setFocusable(false);
        VelxCheckBox.setText("Output velocity along the x axis");
        jPanel6.add(VelxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        PstnCheckBox.setSelected(true);
        PstnCheckBox.setText("Output plastic strain");
        jPanel6.add(PstnCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 140, -1, -1));

        PbdyCheckBox.setText("Output phase boundary");
        jPanel6.add(PbdyCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 120, -1, -1));

        StrfCheckBox.setText("Output stream function");
        jPanel6.add(StrfCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 100, -1, -1));

        TempCheckBox.setSelected(true);
        TempCheckBox.setText("Output temperature");
        jPanel6.add(TempCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, -1, -1));

        PresCheckBox.setText("Output nodal pressure");
        jPanel6.add(PresCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        VelzCheckBox.setText("Output velocity along the z axis");
        jPanel6.add(VelzCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        outputSettingsPanel.add(jPanel6, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 170, 240, 170));

        jPanel7.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel7.setBorder(new javax.swing.border.TitledBorder("Particle output variables"));
        jPanel7.setFocusable(false);
        particleDataTempCheckBox.setText("Output temperature");
        jPanel7.add(particleDataTempCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        StrDCheckBox.setText("Output deviatory stress");
        jPanel7.add(StrDCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 140, -1, -1));

        StrTCheckBox.setText("Output accumulated total strain");
        jPanel7.add(StrTCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 120, -1, -1));

        StrPCheckBox.setText("Output accumulated plastic strain");
        jPanel7.add(StrPCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 100, -1, -1));

        EdotCheckBox.setText("Output strain rate");
        jPanel7.add(EdotCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, -1, -1));

        ViscCheckBox.setText("Output viscosity");
        jPanel7.add(ViscCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        particleDataPresCheckBox.setText("Output pressure");
        jPanel7.add(particleDataPresCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        GrszCheckBox.setText("Output grain size");
        jPanel7.add(GrszCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 180, -1, -1));

        StrSCheckBox.setText("Output total stress");
        jPanel7.add(StrSCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 160, -1, -1));

        outputSettingsPanel.add(jPanel7, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 350, 250, 210));

        jPanel10.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel10.setBorder(new javax.swing.border.TitledBorder("Surface observables at z=0"));
        jPanel10.setFocusable(false);
        TpgxCheckBox.setText("Surface topography along x axis");
        jPanel10.add(TpgxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        GrtxCheckBox.setText("Grtx");
        jPanel10.add(GrtxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 140, -1, -1));

        GrbxCheckBox.setText("Grbx");
        jPanel10.add(GrbxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 120, -1, -1));

        GrvxCheckBox.setText("Gravity along x axis");
        jPanel10.add(GrvxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 100, -1, -1));

        TpbkCheckBox.setText("Tpbk");
        jPanel10.add(TpbkCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, -1, -1));

        TpbxCheckBox.setText("Tpbx");
        jPanel10.add(TpbxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        TpgkCheckBox.setText("Surface topography wave number");
        jPanel10.add(TpgkCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        GrbkCheckBox.setText("Gravity basal wave number");
        jPanel10.add(GrbkCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 180, -1, -1));

        GrvkCheckBox.setText("Grvk");
        jPanel10.add(GrvkCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 160, -1, -1));

        VzsfCheckBox.setText("Vzsf");
        jPanel10.add(VzsfCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 400, -1, -1));

        VxsfCheckBox.setText("Vxsf");
        jPanel10.add(VxsfCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 380, -1, -1));

        observablesBhflCheckBox.setText("Bottom heat flux");
        jPanel10.add(observablesBhflCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 360, -1, -1));

        observablesShflCheckBox.setText("Surface heat flux");
        jPanel10.add(observablesShflCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 340, -1, -1));

        GetkCheckBox.setText("Getk");
        jPanel10.add(GetkCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 320, -1, -1));

        GebkCheckBox.setText("Geoid basal wave number");
        jPanel10.add(GebkCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 300, -1, -1));

        GeokCheckBox.setText("Geoid wave number");
        jPanel10.add(GeokCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 280, -1, -1));

        GetxCheckBox.setText("Getx");
        jPanel10.add(GetxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 260, -1, -1));

        GebxCheckBox.setText("Gebx");
        jPanel10.add(GebxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 240, -1, -1));

        GeoxCheckBox.setText("Geoid along x axis");
        jPanel10.add(GeoxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 220, -1, -1));

        GrtkCheckBox.setText("Gravity topographic wave number");
        jPanel10.add(GrtkCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 200, -1, -1));

        outputSettingsPanel.add(jPanel10, new org.netbeans.lib.awtextra.AbsoluteConstraints(510, 0, 250, 430));

        samplingTracersButton.setText("Sampling Tracers");
        samplingTracersButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                samplingTracersButtonActionPerformed(evt);
            }
        });

        outputSettingsPanel.add(samplingTracersButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(300, 530, -1, -1));

        PPMFilesJPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        PPMFilesJPanel.setBorder(new javax.swing.border.TitledBorder("PPM files"));
        PPMFilesJPanel.setFocusable(false);
        PPMFilesComboBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                PPMFilesComboBoxItemStateChanged(evt);
            }
        });

        PPMFilesJPanel.add(PPMFilesComboBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 90, -1));

        jLabel29.setText("Height of PPM file (in pixels)");
        jLabel29.setFocusable(false);
        PPMFilesJPanel.add(jLabel29, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 50, -1, -1));

        newPPMFileButton.setText("New PPM file");
        newPPMFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newPPMFileButtonActionPerformed(evt);
            }
        });

        PPMFilesJPanel.add(newPPMFileButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(110, 20, -1, -1));

        deletePPMFileButton.setText("Delete PPM file");
        deletePPMFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deletePPMFileButtonActionPerformed(evt);
            }
        });

        PPMFilesJPanel.add(deletePPMFileButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(240, 20, -1, -1));

        PPMColoringComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Temperature", "Tracer weight", "Deviatoric stress", "Pressure", "Accumulated plastic strain", "Depletion", "Unrecognised option" }));
        PPMFilesJPanel.add(PPMColoringComboBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, -1, -1));

        outputSettingsPanel.add(PPMFilesJPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(510, 450, 380, 120));

        jPanel8.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel8.setBorder(new javax.swing.border.TitledBorder("Horizontally averaged values"));
        jPanel8.setFocusable(false);
        averagesTempCheckBox.setText("Output temperature");
        jPanel8.add(averagesTempCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        WrmsCheckBox.setText("Output RMS velocity along z axis");
        WrmsCheckBox.setToolTipText("Output the root mean square velocity along the z axis");
        jPanel8.add(WrmsCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 100, -1, -1));

        VeloCheckBox.setText("Output velocity");
        jPanel8.add(VeloCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        averagesViscCheckBox.setText("Output viscosity");
        jPanel8.add(averagesViscCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        UrmsCheckBox.setText("Output RMS velocity along x axis");
        UrmsCheckBox.setToolTipText("Output the root mean square velocity along the x axis");
        jPanel8.add(UrmsCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, -1, -1));

        outputSettingsPanel.add(jPanel8, new org.netbeans.lib.awtextra.AbsoluteConstraints(260, 180, 250, 130));

        jPanel9.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel9.setBorder(new javax.swing.border.TitledBorder("Large scale averages to time record"));
        jPanel9.setFocusable(false);
        NussCheckBox.setText("Nusselt number");
        jPanel9.add(NussCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        SvavCheckBox.setText("Surface average velocity");
        jPanel9.add(SvavCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 160, -1, -1));

        StmxCheckBox.setText("Sigma maximum");
        jPanel9.add(StmxCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 140, -1, -1));

        BhflCheckBox.setText("Bottom heat flux");
        jPanel9.add(BhflCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 120, -1, -1));

        ShflCheckBox.setText("Surface heat flux");
        jPanel9.add(ShflCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 100, -1, -1));

        VzrmCheckBox.setText("Linear RMS velocity along z axis");
        VzrmCheckBox.setToolTipText("Record the linear root mean square velocity along the z axis");
        jPanel9.add(VzrmCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, -1, -1));

        VxrmCheckBox.setText("Linear RMS velocity along x axis");
        VxrmCheckBox.setToolTipText("Record the linear root mean square velocity along the x axis");
        jPanel9.add(VxrmCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        BvavCheckBox.setText("Bottom average velocity");
        jPanel9.add(BvavCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 180, -1, -1));

        VrmsCheckBox.setText("Linear RMS velocity");
        VrmsCheckBox.setToolTipText("Record the linear root mean square velocity along the x axis");
        jPanel9.add(VrmsCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        outputSettingsPanel.add(jPanel9, new org.netbeans.lib.awtextra.AbsoluteConstraints(260, 310, 250, 210));

        mainPanes.addTab("Output Settings", outputSettingsPanel);

        scalingDetailsPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        scalingDetailsPanel.setToolTipText("");
        scalingDetailsPanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                scalingDetailsPanelComponentShown(evt);
            }
        });

        jPanel16.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel16.setBorder(new javax.swing.border.TitledBorder("Scaling options"));
        jPanel16.setFocusable(false);
        automaticallyScaleParametersRadioButton.setText("Automatically Scale Parameters");
        automaticallyScaleParametersRadioButton.setToolTipText("Ellipsis GUI will try to choose appropriate scaling factors automatically");
        automaticallyScaleParametersRadioButton.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                automaticallyScaleParametersRadioButtonItemStateChanged(evt);
            }
        });

        jPanel16.add(automaticallyScaleParametersRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        manuallyScaleParametersRadioButton.setSelected(true);
        manuallyScaleParametersRadioButton.setText("Manually Scale Parameters");
        manuallyScaleParametersRadioButton.setToolTipText("Ellipsis GUI will use the scaling factors specified below when outputting or reading in Ellipsis input files");
        manuallyScaleParametersRadioButton.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                manuallyScaleParametersRadioButtonItemStateChanged(evt);
            }
        });

        jPanel16.add(manuallyScaleParametersRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        doNotScaleParametersRadioButton.setText("Do not scale Parameters");
        doNotScaleParametersRadioButton.setToolTipText("Ellipsis GUI will not scale any parameters (except that length units will be converted from KM to metres before output, and from metres to KM when files are loaded)");
        doNotScaleParametersRadioButton.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                doNotScaleParametersRadioButtonItemStateChanged(evt);
            }
        });

        jPanel16.add(doNotScaleParametersRadioButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        scalingDetailsPanel.add(jPanel16, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 0, 330, 90));

        jPanel11.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel11.setBorder(new javax.swing.border.TitledBorder("Scaling Factors"));
        jPanel11.setFocusable(false);
        jLabel32.setText("One metre equals this many ellipsis length units:");
        jLabel32.setFocusable(false);
        jPanel11.add(jLabel32, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 30, -1, -1));

        jLabel35.setText("One kelvin equals this many ellipsis temperature units:");
        jLabel35.setFocusable(false);
        jPanel11.add(jLabel35, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 70, -1, -1));

        jLabel49.setText("One specific heat unit equals this many ellipsis specific heat units:");
        jLabel49.setToolTipText("Specific heat is scaled separately to the other parameters");
        jLabel49.setFocusable(false);
        jPanel11.add(jLabel49, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 110, -1, -1));

        jLabel45.setText("One gravity unit equals this many ellipsis gravity units:");
        jLabel45.setToolTipText("Gravity is always left unscaled");
        jLabel45.setFocusable(false);
        jPanel11.add(jLabel45, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 130, -1, -1));

        jLabel43.setText("One thermal diffuisivity unit equals this many ellipsis thermal diffusivity units:");
        jLabel43.setFocusable(false);
        jPanel11.add(jLabel43, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 50, -1, -1));

        jLabel50.setText("One viscosity unit equals this many viscosity units:");
        jLabel50.setFocusable(false);
        jPanel11.add(jLabel50, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 90, -1, -1));

        scalingDetailsPanel.add(jPanel11, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 90, 590, 160));

        jPanel23.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanel23.setBorder(new javax.swing.border.TitledBorder("Alternative Scaling Factors"));
        jPanel23.setFocusable(false);
        jLabel44.setText("One thermal expansion unit equals this many ellipsis thermal expansion units:");
        jLabel44.setFocusable(false);
        jPanel23.add(jLabel44, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, -1, -1));

        jLabel46.setText("One density unit equals this many ellipsis density units:");
        jLabel46.setFocusable(false);
        jPanel23.add(jLabel46, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        jLabel34.setText("One second equals this many ellipsis time units:");
        jLabel34.setFocusable(false);
        jPanel23.add(jLabel34, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        jLabel33.setText("One kilogram equals this many ellipsis mass units:");
        jLabel33.setFocusable(false);
        jPanel23.add(jLabel33, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, -1, -1));

        scalingDetailsPanel.add(jPanel23, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 250, 590, 110));

        mainPanes.addTab("Scaling Details", scalingDetailsPanel);

        inputFileEditorScrollPane.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                inputFileEditorScrollPaneComponentShown(evt);
            }
        });

        inputFileEditorTextArea.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                inputFileEditorTextAreaComponentShown(evt);
            }
        });
        inputFileEditorTextArea.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                inputFileEditorTextAreaFocusGained(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                inputFileEditorTextAreaFocusLost(evt);
            }
        });

        inputFileEditorScrollPane.setViewportView(inputFileEditorTextArea);

        mainPanes.addTab("Input File Editor", inputFileEditorScrollPane);

        getContentPane().add(mainPanes, java.awt.BorderLayout.CENTER);

        fileMenu.setMnemonic('f');
        fileMenu.setText("File");
        jMenuItem8.setText("New window");
        jMenuItem8.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem8ActionPerformed(evt);
            }
        });

        fileMenu.add(jMenuItem8);

        openInputFileMenuItem.setMnemonic('o');
        openInputFileMenuItem.setText("Open input file...");
        openInputFileMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openInputFileMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(openInputFileMenuItem);

        openPictureFileMenuItem.setMnemonic('p');
        openPictureFileMenuItem.setText("Open picture file...");
        openPictureFileMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openPictureFileMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(openPictureFileMenuItem);

        saveInputFileMenuItem.setMnemonic('s');
        saveInputFileMenuItem.setText("Save input file");
        saveInputFileMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveInputFileMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(saveInputFileMenuItem);

        saveInputFileAsMenuItem.setMnemonic('a');
        saveInputFileAsMenuItem.setText("Save input file as...");
        saveInputFileAsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveInputFileAsMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(saveInputFileAsMenuItem);

        exitMenuItem.setMnemonic('e');
        exitMenuItem.setText("Exit");
        fileMenu.add(exitMenuItem);

        jMenuBar1.add(fileMenu);

        jMenu3.setMnemonic('e');
        jMenu3.setText("Edit");
        showUnrecognisedParameters.setText("Unrecognised parameters...");
        showUnrecognisedParameters.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showUnrecognisedParametersActionPerformed(evt);
            }
        });

        jMenu3.add(showUnrecognisedParameters);

        jMenuBar1.add(jMenu3);

        helpMenu.setMnemonic('h');
        helpMenu.setText("Help");
        ellipsisManualMenuItem.setText("Ellipsis Manual...");
        ellipsisManualMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ellipsisManualMenuItemActionPerformed(evt);
            }
        });

        helpMenu.add(ellipsisManualMenuItem);

        unitsAndScalingMenuItem.setText("Units and scaling...");
        unitsAndScalingMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                unitsAndScalingMenuItemActionPerformed(evt);
            }
        });

        helpMenu.add(unitsAndScalingMenuItem);

        nonDimensionaliztionMenuItem.setMnemonic('N');
        nonDimensionaliztionMenuItem.setText("About non-dimensionalization...");
        nonDimensionaliztionMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nonDimensionaliztionMenuItemActionPerformed(evt);
            }
        });

        helpMenu.add(nonDimensionaliztionMenuItem);

        aboutEllipsisGuiMenuItem.setText("About Ellipsis Gui...");
        aboutEllipsisGuiMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutEllipsisGuiMenuItemActionPerformed(evt);
            }
        });

        helpMenu.add(aboutEllipsisGuiMenuItem);

        jMenuBar1.add(helpMenu);

        setJMenuBar(jMenuBar1);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-949)/2, (screenSize.height-680)/2, 949, 680);
    }//GEN-END:initComponents

    private void ellipsisUnitsPerSpecificHeatNumberBoxBeanFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_ellipsisUnitsPerSpecificHeatNumberBoxBeanFocusLost
        scaling.updateScalingFactors();
    }//GEN-LAST:event_ellipsisUnitsPerSpecificHeatNumberBoxBeanFocusLost

    private void viscosityScalingNumberBoxBeanFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_viscosityScalingNumberBoxBeanFocusLost
        // We are primarily scaling kg when we scale viscosity, so leave m and s unchanged. This won't affect any other basic scaling factors.
        ellipsisUnitsPerKGNumberBoxBean.setText(Useful.double2String(Double.parseDouble(viscosityScalingNumberBoxBean.getText())*Double.parseDouble(ellipsisUnitsPerMNumberBoxBean.getText())*Double.parseDouble(ellipsisUnitsPerSecondNumberBoxBean.getText())));
        scaling.updateScalingFactors();
    }//GEN-LAST:event_viscosityScalingNumberBoxBeanFocusLost

    private void nonDimensionaliztionMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nonDimensionaliztionMenuItemActionPerformed
        nonDimensionalizationDialog.show();
    }//GEN-LAST:event_nonDimensionaliztionMenuItemActionPerformed

    private void nonDimensionalizationDialogCloseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nonDimensionalizationDialogCloseButtonActionPerformed
        nonDimensionalizationDialog.hide();
    }//GEN-LAST:event_nonDimensionalizationDialogCloseButtonActionPerformed

    private void thermalRegionDeleteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_thermalRegionDeleteButtonActionPerformed
        backgroundImagePanel.thermalRegionContainer.deleteSelectedRegion();
    }//GEN-LAST:event_thermalRegionDeleteButtonActionPerformed

    private void thermalRegionBackButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_thermalRegionBackButtonActionPerformed
        backgroundImagePanel.thermalRegionContainer.moveSelectedRegionBackward();
    }//GEN-LAST:event_thermalRegionBackButtonActionPerformed

    private void thermalRegionForwardButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_thermalRegionForwardButtonActionPerformed
        backgroundImagePanel.thermalRegionContainer.moveSelectedRegionForward();
    }//GEN-LAST:event_thermalRegionForwardButtonActionPerformed

    private void thermalRegionListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_thermalRegionListValueChanged
        int theSelectedRegionIndex=thermalRegionList.getSelectedIndex();
        if (theSelectedRegionIndex!=-1) { // If no region is selected, then the list has been reset by the program, and we don't need to update the selection.
            backgroundImagePanel.thermalRegionContainer.setSelectedRegion(theSelectedRegionIndex);
        }
        backgroundImagePanel.repaint(); // We need to repaint so the selected region can be shown on the screen, and highlighted if necessary.
    }//GEN-LAST:event_thermalRegionListValueChanged

    private void thermalZonesPanelComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_thermalZonesPanelComponentShown
        shapeChooser.removeAllItems();
        shapeChooser.addItem("Rectangle");
        shapeChooser.addItem("Circle");
        shapeChooser.addItem("Triangle");
        
        backgroundImagePanel.thermalRegionContainer.updateControlsForSelectedItem(); // Update the controls to reflect the selected item (if any), else clear them.
        backgroundImagePanel.repaint(); // Now we've switched to materials, we need to update the window to show the appropriate regions.
    }//GEN-LAST:event_thermalZonesPanelComponentShown

    private void fixAspectRatioCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_fixAspectRatioCheckBoxItemStateChanged
        updateVisualEditorCentralPanel();
    }//GEN-LAST:event_fixAspectRatioCheckBoxItemStateChanged

    private void visualEditorCentralPanelComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_visualEditorCentralPanelComponentResized
        updateVisualEditorCentralPanel();
    }//GEN-LAST:event_visualEditorCentralPanelComponentResized

void updateVisualEditorCentralPanel() {
        int newHeightForBackgroundImagePanel;
        int newWidthForBackgroundImagePanel;

        if (fixAspectRatioCheckBox.isSelected()) { // If we need to try and keep the shape of the window about right.
            if (windowDimensionX/windowDimensionY > ((double)visualEditorCentralPanel.getWidth())/visualEditorCentralPanel.getHeight()) { // If the simualtion area is more elongated widthwise than the window...
                newWidthForBackgroundImagePanel=visualEditorCentralPanel.getWidth(); // Make the area as wide as possible.
                newHeightForBackgroundImagePanel=(int)Math.floor(windowDimensionY/windowDimensionX*visualEditorCentralPanel.getWidth()); // But trim it heightwise to keep things the right shape.
            
            } else { // And if it's more elongated heightwise...
                newHeightForBackgroundImagePanel=visualEditorCentralPanel.getHeight(); // Make the area as high as possible.
                newWidthForBackgroundImagePanel=(int)Math.floor(windowDimensionX/windowDimensionY*visualEditorCentralPanel.getHeight());
            }

        } else { // If we don't care about what shape the window ends up, just make it as big as possible.
            newWidthForBackgroundImagePanel=visualEditorCentralPanel.getWidth(); // Make the area as wide as possible.
            newHeightForBackgroundImagePanel=visualEditorCentralPanel.getHeight(); // Make the area as high as possible.
        }
        
        backgroundImagePanel.setBounds(0, 0, newWidthForBackgroundImagePanel, newHeightForBackgroundImagePanel);
//        System.out.println("new width: "+newWidthForBackgroundImagePanel+ " new height: "+ newHeightForBackgroundImagePanel);
    }
    
    private void boundarySettingsDialogCloseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_boundarySettingsDialogCloseButtonActionPerformed
        boundarySettingsDialog.hide();
    }//GEN-LAST:event_boundarySettingsDialogCloseButtonActionPerformed

    private void ellipsisUnitsPerKelvinNumberBoxBeanFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_ellipsisUnitsPerKelvinNumberBoxBeanFocusLost
        scaling.updateScalingFactors();
    }//GEN-LAST:event_ellipsisUnitsPerKelvinNumberBoxBeanFocusLost

    private void ellipsisUnitsPerKGNumberBoxBeanFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_ellipsisUnitsPerKGNumberBoxBeanFocusLost
        // KG is now not a basic scaling factor, so we must set viscosity and allow that to set kelvin
        viscosityScalingNumberBoxBean.setText(Useful.double2String(Double.parseDouble(ellipsisUnitsPerKGNumberBoxBean.getText())/scaling.ellipsisUnitsPerM/scaling.ellipsisUnitsPerSecond));
        scaling.updateScalingFactors();
    }//GEN-LAST:event_ellipsisUnitsPerKGNumberBoxBeanFocusLost

    private void ellipsisUnitsPerSecondNumberBoxBeanFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_ellipsisUnitsPerSecondNumberBoxBeanFocusLost
        // S is no longer a basic scaling factor, so we must set thermal dissusivity and allow that to set seconds.
        thermalDiffusivityScalingNumberBoxBean.setText(Useful.double2String(scaling.ellipsisUnitsPerM*scaling.ellipsisUnitsPerM/Double.parseDouble(ellipsisUnitsPerSecondNumberBoxBean.getText())));
        scaling.updateScalingFactors();
    }//GEN-LAST:event_ellipsisUnitsPerSecondNumberBoxBeanFocusLost

    private void ellipsisUnitsPerMNumberBoxBeanFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_ellipsisUnitsPerMNumberBoxBeanFocusLost
        // basic scaling factor, so no problem.
        scaling.updateScalingFactors();
    }//GEN-LAST:event_ellipsisUnitsPerMNumberBoxBeanFocusLost

    private void thermalExpansionScalingNumberBoxBeanFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_thermalExpansionScalingNumberBoxBeanFocusLost
        ellipsisUnitsPerKelvinNumberBoxBean.setText(Useful.double2String(1.0/Double.parseDouble(thermalExpansionScalingNumberBoxBean.getText())));
        scaling.updateScalingFactors();
    }//GEN-LAST:event_thermalExpansionScalingNumberBoxBeanFocusLost

    private void densityScalingNumberBoxBeanFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_densityScalingNumberBoxBeanFocusLost
        // We've got to set a basic scaling factor, not KG.
        viscosityScalingNumberBoxBean.setText(Useful.double2String(Double.parseDouble(densityScalingNumberBoxBean.getText())*scaling.ellipsisUnitsPerM*scaling.ellipsisUnitsPerM/scaling.ellipsisUnitsPerSecond));
//        ellipsisUnitsPerKGNumberBoxBean.setText(Useful.double2String(Double.parseDouble(densityScalingNumberBoxBean.getText())*scaling.ellipsisUnitsPerM*scaling.ellipsisUnitsPerM*scaling.ellipsisUnitsPerM));
        scaling.updateScalingFactors();
    }//GEN-LAST:event_densityScalingNumberBoxBeanFocusLost

    private void thermalDiffusivityScalingNumberBoxBeanFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_thermalDiffusivityScalingNumberBoxBeanFocusLost
        ellipsisUnitsPerSecondNumberBoxBean.setText(Useful.double2String(scaling.ellipsisUnitsPerM*scaling.ellipsisUnitsPerM/Double.parseDouble(thermalDiffusivityScalingNumberBoxBean.getText())));
        scaling.updateScalingFactors();
    }//GEN-LAST:event_thermalDiffusivityScalingNumberBoxBeanFocusLost

    private void gravityScalingNumberBoxBeanFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_gravityScalingNumberBoxBeanFocusLost
// Do nothing here - gravity is always left unscaled now, and the box is undeditable.
        //        ellipsisUnitsPerSecondNumberBoxBean.setText(Useful.double2String(Math.sqrt(scaling.ellipsisUnitsPerM/Double.parseDouble(gravityScalingNumberBoxBean.getText()))));
//        scaling.updateScalingFactors();
    }//GEN-LAST:event_gravityScalingNumberBoxBeanFocusLost

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        ellipsisManualDialog.hide();
    }//GEN-LAST:event_jButton1ActionPerformed

    private void ellipsisManualMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ellipsisManualMenuItemActionPerformed
        ellipsisManualDialog.show();
    }//GEN-LAST:event_ellipsisManualMenuItemActionPerformed

    private void unitsAndScalingDialogCloseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_unitsAndScalingDialogCloseButtonActionPerformed
        unitsAndScalingDialog.hide();
    }//GEN-LAST:event_unitsAndScalingDialogCloseButtonActionPerformed

    private void unitsAndScalingMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_unitsAndScalingMenuItemActionPerformed
        unitsAndScalingDialog.show();
    }//GEN-LAST:event_unitsAndScalingMenuItemActionPerformed

    private void showUnrecognisedParametersActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showUnrecognisedParametersActionPerformed
        unrecognisedParametersDialog.show();
    }//GEN-LAST:event_showUnrecognisedParametersActionPerformed

    private void unrecognisedParametersDialogCloseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_unrecognisedParametersDialogCloseButtonActionPerformed
        unrecognisedParametersDialog.hide();
    }//GEN-LAST:event_unrecognisedParametersDialogCloseButtonActionPerformed

    private void triangleControlsComponentHidden(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_triangleControlsComponentHidden
        blankTriangleControls(); // The controls will be blanked when hidden, so if they'return reselected they won't show the wrong data.
    }//GEN-LAST:event_triangleControlsComponentHidden

    private void addNewTriangleButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addNewTriangleButtonActionPerformed
        TriangleCoordinates coordinates=new TriangleCoordinates();
        if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
        
        if (materialsPanel.isShowing()) {
            backgroundImagePanel.materialRegionContainer.addTriangle(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2, coordinates.x3, coordinates.z3); // And add the triangle.
        }
    }//GEN-LAST:event_addNewTriangleButtonActionPerformed

    class TriangleCoordinates {
        double x1, x2, x3, z1, z2, z3;
        
        boolean getCoordinates() {
            try { // Get the values out of the text boxes.
                x1=Double.parseDouble(triangleX1TextField.getText());
                x2=Double.parseDouble(triangleX2TextField.getText());
                x3=Double.parseDouble(triangleX3TextField.getText());
                z1=Double.parseDouble(triangleY1TextField.getText());
                z2=Double.parseDouble(triangleY2TextField.getText());
                z3=Double.parseDouble(triangleY3TextField.getText());
            } catch (NumberFormatException ex) { // And if any of them are not valid doubles warn about this and return signalling an error done nothing.
                JOptionPane.showMessageDialog(MainWindow.this,"One or more of the coordinates is not a valid number. Please enter the coordinates again.", "Error", JOptionPane.ERROR_MESSAGE);
                return(false);
            }

            if (x1<0 || x2<0 || x3<0 || z1<0 || z2<0 || z3<0) {
                JOptionPane.showMessageDialog(MainWindow.this,"Triangle coordinates cannot be negative. Please enter the coordinates again.", "Error", JOptionPane.ERROR_MESSAGE);
                return false;
            }
            return true; // No errors detected.
        }
    }

    private void triangleUpdateButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_triangleUpdateButtonActionPerformed
        if (materialsPanel.isShowing()) {
            backgroundImagePanel.materialRegionContainer.updateSelectedTriangle();
        }
    }//GEN-LAST:event_triangleUpdateButtonActionPerformed

    private void profileColourButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_profileColourButtonActionPerformed
        generalColourChooserChangeListener.whatColourChooserIsBeingUsedFor=generalColourChooserChangeListener.samplingTracer; // Let the listener know that we're using the colour changer to colour the profileColour button.
        generalColourChooser.show(); // And show the colour listener.
    }//GEN-LAST:event_profileColourButtonActionPerformed

    private void samplingTracersComboBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_samplingTracersComboBoxItemStateChanged
        updateSelectedSamplingTracersWindowObjects();
    }//GEN-LAST:event_samplingTracersComboBoxItemStateChanged

    private void deleteSamplingTracerButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deleteSamplingTracerButtonActionPerformed
        deleteSamplingTracer();
    }//GEN-LAST:event_deleteSamplingTracerButtonActionPerformed

    private void newSamplingTracerButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newSamplingTracerButtonActionPerformed
        addNewSamplingTracer();
    }//GEN-LAST:event_newSamplingTracerButtonActionPerformed

    private void samplingTracersDialogComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_samplingTracersDialogComponentShown
        copyAllItems(PPMFilesComboBox, samplingPPMComboBox); // Update the combo box model to reflect the currently defined number of combo boxes every time the window is shown.
        updateSelectedSamplingTracersWindowObjects(); // And update the window objects, so that the right PPM file is shown in the sampling tracer window (this will also give a warning if the selected PPM doesn't exist).
    }//GEN-LAST:event_samplingTracersDialogComponentShown

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        samplingTracersDialog.hide();
    }//GEN-LAST:event_jButton2ActionPerformed

    private void samplingTracersButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_samplingTracersButtonActionPerformed
        samplingTracersDialog.show();
    }//GEN-LAST:event_samplingTracersButtonActionPerformed

    private void PPMFilesComboBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_PPMFilesComboBoxItemStateChanged
        updateSelectedPPMFilesWindowObjects(); // Just update appropriate window objects, based on which PPM is now selected.
    }//GEN-LAST:event_PPMFilesComboBoxItemStateChanged

    private void deletePPMFileButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deletePPMFileButtonActionPerformed
        deletePPMFile();
    }//GEN-LAST:event_deletePPMFileButtonActionPerformed

    private void newPPMFileButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newPPMFileButtonActionPerformed
        addNewPPMFile();
    }//GEN-LAST:event_newPPMFileButtonActionPerformed

    private void scalingDetailsPanelComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_scalingDetailsPanelComponentShown
        if (automaticallyScaleParametersRadioButton.isSelected()) { // If we've chosen autoscale, we need to update the scaling factors every time they are shown.
            scaling.autoscale();
        }
    }//GEN-LAST:event_scalingDetailsPanelComponentShown

    private void automaticallyScaleParametersRadioButtonItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_automaticallyScaleParametersRadioButtonItemStateChanged
        if (evt.getStateChange()==evt.SELECTED) { // If we've selected to automatically scale the parameters, set all the scaling factors boxes so they can't be edited, then autoscale.
            ellipsisUnitsPerMNumberBoxBean.setEditable(false);
            ellipsisUnitsPerKGNumberBoxBean.setEditable(false);
            ellipsisUnitsPerKelvinNumberBoxBean.setEditable(false);
            ellipsisUnitsPerSecondNumberBoxBean.setEditable(false);
            
//            gravityScalingNumberBoxBean.setEditable(false);
            thermalDiffusivityScalingNumberBoxBean.setEditable(false);
            densityScalingNumberBoxBean.setEditable(false);
            thermalExpansionScalingNumberBoxBean.setEditable(false);
            
            scaling.autoscale();
        }
    }//GEN-LAST:event_automaticallyScaleParametersRadioButtonItemStateChanged

    private void doNotScaleParametersRadioButtonItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_doNotScaleParametersRadioButtonItemStateChanged
        if (evt.getStateChange()==evt.SELECTED) { // If we've selected not to scale the parameters, set all the scaling factors boxes so they can't be edited, then set all there values to 1 (scaling ratio of one to one is no scaling).
            ellipsisUnitsPerMNumberBoxBean.setEditable(false);
            ellipsisUnitsPerKGNumberBoxBean.setEditable(false);
            ellipsisUnitsPerKelvinNumberBoxBean.setEditable(false);
            ellipsisUnitsPerSecondNumberBoxBean.setEditable(false);
            
            ellipsisUnitsPerMNumberBoxBean.setText("1");
            ellipsisUnitsPerKGNumberBoxBean.setText("1");
            ellipsisUnitsPerKelvinNumberBoxBean.setText("1");
            ellipsisUnitsPerSecondNumberBoxBean.setText("1");
            
//            gravityScalingNumberBoxBean.setEditable(false);
            thermalDiffusivityScalingNumberBoxBean.setEditable(false);
            densityScalingNumberBoxBean.setEditable(false);
            thermalExpansionScalingNumberBoxBean.setEditable(false);
            
//            gravityScalingNumberBoxBean.setText("1");
            thermalDiffusivityScalingNumberBoxBean.setText("1");
            densityScalingNumberBoxBean.setText("1");
            thermalExpansionScalingNumberBoxBean.setText("1");
        }
    }//GEN-LAST:event_doNotScaleParametersRadioButtonItemStateChanged

    private void manuallyScaleParametersRadioButtonItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_manuallyScaleParametersRadioButtonItemStateChanged
        if (evt.getStateChange()==evt.SELECTED) { // If we've selected to scale the parameters manually, set all the scaling factors boxes so they can be edited, but leaving their values at whatever they were last.
            ellipsisUnitsPerMNumberBoxBean.setEditable(true);
            ellipsisUnitsPerKGNumberBoxBean.setEditable(true);
            ellipsisUnitsPerKelvinNumberBoxBean.setEditable(true);
            ellipsisUnitsPerSecondNumberBoxBean.setEditable(true);
            
            //gravityScalingNumberBoxBean.setEditable(true);
            thermalDiffusivityScalingNumberBoxBean.setEditable(true);
            densityScalingNumberBoxBean.setEditable(true);
            thermalExpansionScalingNumberBoxBean.setEditable(true);
        }
    }//GEN-LAST:event_manuallyScaleParametersRadioButtonItemStateChanged

    private void shapeChooserItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_shapeChooserItemStateChanged
        if (materialsPanel.isShowing()) {
            backgroundImagePanel.materialRegionContainer.deselectRegion(); // The old region can't be selected any more, because now we're working on a different shape.
        } else if (drawingsPanel.isShowing()) {
            backgroundImagePanel.drawingRegionContainer.deselectRegion(); // The old region can't be selected any more, because now we're working on a different shape.
        } else if (tracersPanel.isShowing()) { // This option will be redundant as long as we can have only one shape of tracers.
            backgroundImagePanel.tracerRegionContainer.deselectRegion(); // The old region can't be selected any more, because now we're working on a different shape.
        } else if (boundaryConditionsPanel.isShowing()) {
            backgroundImagePanel.boundaryRegionContainer.deselectRegion(); // The old region can't be selected any more, because now we're working on a different shape.
        } else if (thermalZonesPanel.isShowing()) {
            backgroundImagePanel.thermalRegionContainer.deselectRegion();
        }
        
        String selectedShape=(String)shapeChooser.getSelectedItem(); // Get the selected shape in string format.
        ((CardLayout)controlsCardHolder.getLayout()).show(controlsCardHolder, selectedShape); // And switch to the right card.
    }//GEN-LAST:event_shapeChooserItemStateChanged

    private void drawMaterialsCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_drawMaterialsCheckBoxItemStateChanged
        backgroundImagePanel.repaint();
    }//GEN-LAST:event_drawMaterialsCheckBoxItemStateChanged

    private void drawBoundariesCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_drawBoundariesCheckBoxItemStateChanged
        backgroundImagePanel.repaint();
    }//GEN-LAST:event_drawBoundariesCheckBoxItemStateChanged

    private void drawTracersCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_drawTracersCheckBoxItemStateChanged
        backgroundImagePanel.repaint();
    }//GEN-LAST:event_drawTracersCheckBoxItemStateChanged

    private void backgroundPictureCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_backgroundPictureCheckBoxItemStateChanged
        backgroundImagePanel.repaint(); // If we've decided to show or hide the background image, then we need to repaint the backgroundImagePanel.
    }//GEN-LAST:event_backgroundPictureCheckBoxItemStateChanged

    private void drawDrawingCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_drawDrawingCheckBoxItemStateChanged
        backgroundImagePanel.repaint();
    }//GEN-LAST:event_drawDrawingCheckBoxItemStateChanged

    private void addNewRectangleButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addNewRectangleButtonActionPerformed
        RectangleCoordinates coordinates=new RectangleCoordinates();
        if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
        
        if (materialsPanel.isShowing()) {
            backgroundImagePanel.materialRegionContainer.addRectangle(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2); // And add the rectangle.
        } else if (drawingsPanel.isShowing()) {
            backgroundImagePanel.drawingRegionContainer.addRectangle(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2); // And add the rectangle.
        } else if (tracersPanel.isShowing()) {
            backgroundImagePanel.tracerRegionContainer.addRectangle(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2); // And add the rectangle.
        } else if (boundaryConditionsPanel.isShowing()) {
            backgroundImagePanel.boundaryRegionContainer.addRectangle(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2); // And add the rectangle.
        } else if (thermalZonesPanel.isShowing()) {
            backgroundImagePanel.thermalRegionContainer.addRectangle(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2);
        }
    }//GEN-LAST:event_addNewRectangleButtonActionPerformed

    private void rectangleUpdateButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rectangleUpdateButtonActionPerformed
        if (materialsPanel.isShowing()) {
            backgroundImagePanel.materialRegionContainer.updateSelectedRectangle();
        } else if (drawingsPanel.isShowing()) {
            backgroundImagePanel.drawingRegionContainer.updateSelectedRectangle();
        } else if (tracersPanel.isShowing()) {
            backgroundImagePanel.tracerRegionContainer.updateSelectedRectangle();
        } else if (boundaryConditionsPanel.isShowing()) {
            backgroundImagePanel.boundaryRegionContainer.updateSelectedRectangle();
        } else if (thermalZonesPanel.isShowing()) {
            backgroundImagePanel.thermalRegionContainer.updateSelectedRectangle();
        }
    }//GEN-LAST:event_rectangleUpdateButtonActionPerformed

    private void rectangleControlsComponentHidden(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_rectangleControlsComponentHidden
        blankRectangleControls(); // The controls will be blanked when hidden, so if they'return reselected they won't show the wrong data.
    }//GEN-LAST:event_rectangleControlsComponentHidden

    private void jLabel20ComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_jLabel20ComponentShown
        // Add your handling code here:
    }//GEN-LAST:event_jLabel20ComponentShown

    private void circleAddButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_circleAddButtonActionPerformed
        CircleCoordinates coordinates=new CircleCoordinates();
        if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
        
        if (materialsPanel.isShowing()) {
            backgroundImagePanel.materialRegionContainer.addCircle(coordinates.x, coordinates.z, coordinates.radius); // And add the circle.
        } else if (drawingsPanel.isShowing()) {
            backgroundImagePanel.drawingRegionContainer.addCircle(coordinates.x, coordinates.z, coordinates.radius); // And add the circle.
        } else if (boundaryConditionsPanel.isShowing()) {
            backgroundImagePanel.boundaryRegionContainer.addCircle(coordinates.x, coordinates.z, coordinates.radius); // And add the circle.
        } else if (thermalZonesPanel.isShowing()) {
            backgroundImagePanel.thermalRegionContainer.addCircle(coordinates.x, coordinates.z, coordinates.radius); // And add the circle.
        }
    }//GEN-LAST:event_circleAddButtonActionPerformed

    private void circleUpdateButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_circleUpdateButtonActionPerformed
        if (materialsPanel.isShowing()) {
            backgroundImagePanel.materialRegionContainer.updateSelectedCircle();
        } else if (drawingsPanel.isShowing()) {
            backgroundImagePanel.drawingRegionContainer.updateSelectedCircle();
        } else if (boundaryConditionsPanel.isShowing()) {
            backgroundImagePanel.boundaryRegionContainer.updateSelectedCircle();
        } else if (thermalZonesPanel.isShowing()) {
            backgroundImagePanel.thermalRegionContainer.updateSelectedCircle();
        }
    }//GEN-LAST:event_circleUpdateButtonActionPerformed

    private void circleControlsComponentHidden(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_circleControlsComponentHidden
        blankCircleControls(); // The controls will be blanked when hidden, so if they'return reselected they won't show the wrong data.
    }//GEN-LAST:event_circleControlsComponentHidden

    private void materialSettingsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_materialSettingsButtonActionPerformed
        universalMaterialPropertiesDialogWindow.show();
    }//GEN-LAST:event_materialSettingsButtonActionPerformed

    private void materialChooserItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_materialChooserItemStateChanged
        recolourMaterialChooser();
    }//GEN-LAST:event_materialChooserItemStateChanged

    private void newMaterialButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newMaterialButtonActionPerformed
        Material newMaterial=new Material(); // Create a new material.
        materialChooser.addItem( newMaterial.getName() ); // Add it's name to the materialChooser combo box.
        materials.add(newMaterial); // And add the material itself to the list of materials.
        materialChooser.setSelectedIndex(materialChooser.getItemCount()-1); // Now switch the combo box to show the new material (which will be the last material in the combo box).
        
        updatePPMColours(); // And the new material will have to have the correct number of PPMColours and have its ppmColoursComboBox set correctly.
    }//GEN-LAST:event_newMaterialButtonActionPerformed

    private void deleteMaterialButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deleteMaterialButtonActionPerformed
        int selectedMaterialIndex=materialChooser.getSelectedIndex();  // First find the currently selected material.
        Material selectedMaterial=((Material)materials.get(selectedMaterialIndex));
        
        if (backgroundImagePanel.materialRegionContainer.materialInUse(selectedMaterial)) { // We can't delete a material if its in use.
            JOptionPane.showMessageDialog(this,"You cannot delete this material because one or more regions using it currently exist.", "Error", JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        if (materials.size()>=2) { // If there's only one material we can't delete it - ellipsis requires at least one material.
            selectedMaterial.disposeOfWindows(); // Close and dispose of its windows.
            materials.remove(selectedMaterialIndex); // Remove it from the list of materials.
            materialChooser.removeItemAt(selectedMaterialIndex); // And remove its name from the Combo Box.
            recolourMaterialChooser();
        } else {
            JOptionPane.showMessageDialog(this,"You cannot delete the only material. Ellipsis requires at least one material to be defined.", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_deleteMaterialButtonActionPerformed

    private void editRheologyButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editRheologyButtonActionPerformed
        ((Material)materials.get(materialChooser.getSelectedIndex())).showRheologyWindow();
    }//GEN-LAST:event_editRheologyButtonActionPerformed

    private void materialRegionForwardButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_materialRegionForwardButtonActionPerformed
        backgroundImagePanel.materialRegionContainer.moveSelectedRegionForward();
    }//GEN-LAST:event_materialRegionForwardButtonActionPerformed

    private void materialRegionBackButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_materialRegionBackButtonActionPerformed
        backgroundImagePanel.materialRegionContainer.moveSelectedRegionBackward();
    }//GEN-LAST:event_materialRegionBackButtonActionPerformed

    private void materialRegionDeleteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_materialRegionDeleteButtonActionPerformed
        backgroundImagePanel.materialRegionContainer.deleteSelectedRegion();
    }//GEN-LAST:event_materialRegionDeleteButtonActionPerformed

    private void materialRegionListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_materialRegionListValueChanged
        int theSelectedRegionIndex=materialRegionList.getSelectedIndex();
        if (theSelectedRegionIndex!=-1) { // If no region is selected, then the list has been reset by the program, and we don't need to update the selection.
            backgroundImagePanel.materialRegionContainer.setSelectedRegion(theSelectedRegionIndex);
        }
        backgroundImagePanel.repaint(); // We need to repaint so the selected region can be shown on the screen, and highlighted if necessary.
    }//GEN-LAST:event_materialRegionListValueChanged

    private void materialsPanelComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_materialsPanelComponentShown
        shapeChooser.removeAllItems();
        shapeChooser.addItem("Rectangle");
        shapeChooser.addItem("Circle");
        shapeChooser.addItem("Triangle");
        
        backgroundImagePanel.materialRegionContainer.updateControlsForSelectedItem(); // Update the controls to reflect the selected item (if any), else clear them.
        backgroundImagePanel.repaint(); // Now we've switched to materials, we need to update the window to show the appropriate regions.
    }//GEN-LAST:event_materialsPanelComponentShown

    private void drawingChangeColourButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_drawingChangeColourButtonActionPerformed
        generalColourChooserChangeListener.whatColourChooserIsBeingUsedFor=generalColourChooserChangeListener.drawing; // Let the listener know that we're using the colour changer to colour the drawing button.
        generalColourChooser.show(); // And show the colour listener.
    }//GEN-LAST:event_drawingChangeColourButtonActionPerformed

    private void drawingRegionForwardButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_drawingRegionForwardButtonActionPerformed
        backgroundImagePanel.drawingRegionContainer.moveSelectedRegionForward();
    }//GEN-LAST:event_drawingRegionForwardButtonActionPerformed

    private void drawingRegionBackButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_drawingRegionBackButtonActionPerformed
        backgroundImagePanel.drawingRegionContainer.moveSelectedRegionBackward();
    }//GEN-LAST:event_drawingRegionBackButtonActionPerformed

    private void drawingRegionDeleteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_drawingRegionDeleteButtonActionPerformed
        backgroundImagePanel.drawingRegionContainer.deleteSelectedRegion();
    }//GEN-LAST:event_drawingRegionDeleteButtonActionPerformed

    private void drawingRegionListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_drawingRegionListValueChanged
        int theSelectedRegionIndex=drawingRegionList.getSelectedIndex();
        if (theSelectedRegionIndex!=-1) { // If no region is selected, then the list has been reset by the program, and we don't need to update the selection.
            backgroundImagePanel.drawingRegionContainer.setSelectedRegion(theSelectedRegionIndex);
        }
        backgroundImagePanel.repaint(); // We need to repaint so the selected region can be shown on the screen, and highlighted if necessary.
    }//GEN-LAST:event_drawingRegionListValueChanged

    private void drawingsPanelComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_drawingsPanelComponentShown
        shapeChooser.removeAllItems();
        shapeChooser.addItem("Rectangle");
        shapeChooser.addItem("Circle");
//        shapeChooser.addItem("Triangle");
        shapeChooser.addItem("Line");
        shapeChooser.addItem("Harmonic");
        shapeChooser.addItem("Polynomial");
        backgroundImagePanel.repaint(); // Now we've switched panes, we need to update the window to show the appropriate regions.
    }//GEN-LAST:event_drawingsPanelComponentShown

    private void boundaryConditionsSettingsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_boundaryConditionsSettingsButtonActionPerformed
        boundarySettingsDialog.show(); // If clicked, show the boundary settings dialog, in a non-modal way.
    }//GEN-LAST:event_boundaryConditionsSettingsButtonActionPerformed

    private void boundaryHalfWidthTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_boundaryHalfWidthTextFieldFocusLost
        try {
            Double.parseDouble(boundaryHalfWidthTextField.getText());
        } catch (NumberFormatException ex) {
            boundaryHalfWidthTextField.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_boundaryHalfWidthTextFieldFocusLost

    private void boundaryValueTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_boundaryValueTextFieldFocusLost
        try {
            Double.parseDouble(boundaryValueTextField.getText());
        } catch (NumberFormatException ex) {
            boundaryValueTextField.setText("0");
            MessageBoxes.warnAboutDoubleSetToZero(this);
        }
    }//GEN-LAST:event_boundaryValueTextFieldFocusLost

    private void boundaryChangeColourButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_boundaryChangeColourButtonActionPerformed
        generalColourChooserChangeListener.whatColourChooserIsBeingUsedFor=generalColourChooserChangeListener.boundary; // Let the listener know that we're using the colour changer to colour the drawing button.
        generalColourChooser.show(); // And show the colour listener.
    }//GEN-LAST:event_boundaryChangeColourButtonActionPerformed

    private void boundaryRegionForwardButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_boundaryRegionForwardButtonActionPerformed
        backgroundImagePanel.boundaryRegionContainer.moveSelectedRegionForward();
    }//GEN-LAST:event_boundaryRegionForwardButtonActionPerformed

    private void boundaryRegionBackButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_boundaryRegionBackButtonActionPerformed
        backgroundImagePanel.boundaryRegionContainer.moveSelectedRegionBackward();
    }//GEN-LAST:event_boundaryRegionBackButtonActionPerformed

    private void boundaryRegionDeleteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_boundaryRegionDeleteButtonActionPerformed
        backgroundImagePanel.boundaryRegionContainer.deleteSelectedRegion();
    }//GEN-LAST:event_boundaryRegionDeleteButtonActionPerformed

    private void boundaryRegionListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_boundaryRegionListValueChanged
        int theSelectedRegionIndex=boundaryRegionList.getSelectedIndex();
        if (theSelectedRegionIndex!=-1) { // If no region is selected, then the list has been reset by the program, and we don't need to update the selection.
            backgroundImagePanel.boundaryRegionContainer.setSelectedRegion(theSelectedRegionIndex);
        }
        backgroundImagePanel.repaint(); // We need to repaint so the selected region can be shown on the screen, and highlighted if necessary.
    }//GEN-LAST:event_boundaryRegionListValueChanged

    private void boundaryConditionsPanelComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_boundaryConditionsPanelComponentShown
        shapeChooser.removeAllItems();
        shapeChooser.addItem("Rectangle");
        shapeChooser.addItem("Circle");
        shapeChooser.addItem("Harmonic");
        shapeChooser.addItem("Polynomial");
    }//GEN-LAST:event_boundaryConditionsPanelComponentShown

    private void tracerDensityTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_tracerDensityTextFieldFocusLost
        try {
            Integer.parseInt(tracerDensityTextField.getText()); // Just check that the box's text is set to a valid integer.
        } catch (NumberFormatException ex) {
            tracerDensityTextField.setText("0");
            MessageBoxes.warnAboutIntegerSetToZero(this);
        }
    }//GEN-LAST:event_tracerDensityTextFieldFocusLost

    private void tracerChangeColourButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tracerChangeColourButtonActionPerformed
        generalColourChooserChangeListener.whatColourChooserIsBeingUsedFor=generalColourChooserChangeListener.tracer; // Let the listener know that we're using the colour changer to colour the drawing button.
        generalColourChooser.show(); // And show the colour listener.
    }//GEN-LAST:event_tracerChangeColourButtonActionPerformed

    private void tracerRegionForwardButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tracerRegionForwardButtonActionPerformed
        backgroundImagePanel.tracerRegionContainer.moveSelectedRegionForward();
    }//GEN-LAST:event_tracerRegionForwardButtonActionPerformed

    private void tracerRegionBackButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tracerRegionBackButtonActionPerformed
        backgroundImagePanel.tracerRegionContainer.moveSelectedRegionBackward();
    }//GEN-LAST:event_tracerRegionBackButtonActionPerformed

    private void tracerRegionDeleteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tracerRegionDeleteButtonActionPerformed
        backgroundImagePanel.tracerRegionContainer.deleteSelectedRegion();
    }//GEN-LAST:event_tracerRegionDeleteButtonActionPerformed

    private void tracerRegionListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_tracerRegionListValueChanged
        int theSelectedRegionIndex=tracerRegionList.getSelectedIndex();
        if (theSelectedRegionIndex!=-1) { // If no region is selected, then the list has been reset by the program, and we don't need to update the selection.
            backgroundImagePanel.tracerRegionContainer.setSelectedRegion(theSelectedRegionIndex);
        }
        backgroundImagePanel.repaint(); // We need to repaint so the selected region can be shown on the screen, and highlighted if necessary.
    }//GEN-LAST:event_tracerRegionListValueChanged

    private void useDefaultTracerSetupItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_useDefaultTracerSetupItemStateChanged
        if (evt.getStateChange()==evt.SELECTED) { // The tracer controls are enable iff we aren't just using the default tracer set up.
            //            currentNextTracerLabel.setEnabled(false);
            //            regionsDensityLabel.setEnabled(false);
            //            tracerDensityTextField.setEnabled(false);
            tracerChangeColourButton.setEnabled(false);
            tracerRegionsPanel.setEnabled(false);
            tracerRegionForwardButton.setEnabled(false);
            tracerRegionBackButton.setEnabled(false);
            tracerRegionDeleteButton.setEnabled(false);
            tracerRegionListScrollPane.setEnabled(false);
            tracerRegionList.setEnabled(false);
            
            backgroundImagePanel.tracerRegionContainer.deselectRegion();
            
        } else { // If it's been deselected.
            //            currentNextTracerLabel.setEnabled(true);
            //            regionsDensityLabel.setEnabled(true);
            //            tracerDensityTextField.setEnabled(true);
            tracerChangeColourButton.setEnabled(true);
            tracerRegionsPanel.setEnabled(true);
            tracerRegionForwardButton.setEnabled(true);
            tracerRegionBackButton.setEnabled(true);
            tracerRegionDeleteButton.setEnabled(true);
            tracerRegionListScrollPane.setEnabled(true);
            tracerRegionList.setEnabled(true);
        }
    }//GEN-LAST:event_useDefaultTracerSetupItemStateChanged

    private void tracersPanelComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_tracersPanelComponentShown
        shapeChooser.removeAllItems();
        shapeChooser.addItem("Rectangle");
    }//GEN-LAST:event_tracersPanelComponentShown

    private void windowDimensionXTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_windowDimensionXTextFieldFocusLost
        updateWindowDimensionXTextField();
    }//GEN-LAST:event_windowDimensionXTextFieldFocusLost

    private void windowDimensionYTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_windowDimensionYTextFieldFocusLost
        updateWindowDimensionYTextField(); // Call this, so that the variable storing the dimenx value gets updated if necessary. This parameter is exceptional in that it is one of the very few whoose values are stored in a separate variable.
    }//GEN-LAST:event_windowDimensionYTextFieldFocusLost

    private void visualEditorPanelComponentHidden(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_visualEditorPanelComponentHidden
        if (timer!=null) { // Check first that the timer is running.
            timer.stop(); // If the visual editor panel is hidden, we can stop animating the selected region.
        }
    }//GEN-LAST:event_visualEditorPanelComponentHidden

    private void visualEditorPanelComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_visualEditorPanelComponentShown
        timer.start(); // While the visual editor panel is shown, we want to animate the selected region.
    }//GEN-LAST:event_visualEditorPanelComponentShown

    private void verboseInputValuesCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_verboseInputValuesCheckBoxItemStateChanged
        if (evt.getStateChange() == java.awt.event.ItemEvent.DESELECTED) {
            beginnerCheckBox.setEnabled(false);
        } else {
            beginnerCheckBox.setEnabled(true);
        }
    }//GEN-LAST:event_verboseInputValuesCheckBoxItemStateChanged

    private void inputFileEditorTextAreaComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_inputFileEditorTextAreaComponentShown
        files.inputFileEditorPaneBecomesVisibleOrFocusGained();
    }//GEN-LAST:event_inputFileEditorTextAreaComponentShown

    private void inputFileEditorTextAreaFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_inputFileEditorTextAreaFocusGained
        files.inputFileEditorPaneBecomesVisibleOrFocusGained();
    }//GEN-LAST:event_inputFileEditorTextAreaFocusGained

    private void inputFileEditorTextAreaFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_inputFileEditorTextAreaFocusLost
        files.inputFileEditorPaneFocusLost();
    }//GEN-LAST:event_inputFileEditorTextAreaFocusLost

    private void inputFileEditorScrollPaneComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_inputFileEditorScrollPaneComponentShown
        files.inputFileEditorPaneBecomesVisibleOrFocusGained();
    }//GEN-LAST:event_inputFileEditorScrollPaneComponentShown

    private void YIELDCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_YIELDCheckBoxItemStateChanged
        if (evt.getStateChange()==evt.SELECTED) {
            enableAllYieldStressWindowObjects();
        } else {
            disableAllYieldStressWindowObjects();
        }
    }//GEN-LAST:event_YIELDCheckBoxItemStateChanged

    private void saveInputFileAsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveInputFileAsMenuItemActionPerformed
        files.menuSaveAs();
    }//GEN-LAST:event_saveInputFileAsMenuItemActionPerformed

    private void saveInputFileMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveInputFileMenuItemActionPerformed
        files.menuSave();
    }//GEN-LAST:event_saveInputFileMenuItemActionPerformed

    private void openInputFileMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openInputFileMenuItemActionPerformed
        files.menuLoad();
    }//GEN-LAST:event_openInputFileMenuItemActionPerformed
                                            
    class RectangleCoordinates {
        double x1, x2, z1, z2;
        
        boolean getCoordinates() {
            try { // Get the values out of the text boxes.
                x1=Double.parseDouble(rectangleX1TextField.getText());
                x2=Double.parseDouble(rectangleX2TextField.getText());
                z1=Double.parseDouble(rectangleY1TextField.getText());
                z2=Double.parseDouble(rectangleY2TextField.getText());
            } catch (NumberFormatException ex) { // And if any of them are not valid doubles warn about this and return signalling an error done nothing.
                JOptionPane.showMessageDialog(MainWindow.this,"One or more of the coordinates is not a valid number. Please enter the coordinates again.", "Error", JOptionPane.ERROR_MESSAGE);
                return(false);
            }
            if (x1>x2 || z1>z2) { // If the coordinates of the topleft of the rectangle are not above and left of the bottom right ones, we have an error - warn and return signalling an error.
                JOptionPane.showMessageDialog(MainWindow.this,"x1 must be less than or equal to x2, and z1 must be less than or equal to z2. Please enter the coordinates again.", "Error", JOptionPane.ERROR_MESSAGE);
                return false;
            }
            if (x1<0 || x2<0 || z1<0 || z2<0) {
                JOptionPane.showMessageDialog(MainWindow.this,"Rectangle coordinates cannot be negative. Please enter the coordinates again.", "Error", JOptionPane.ERROR_MESSAGE);
                return false;
            }
            return true; // No errors detected.
        }
    }
    
    class LineCoordinates {
        double x1, x2, z1, z2;
        
        boolean getCoordinates() {
            try { // Get the values out of the text boxes.
                x1=Double.parseDouble(lineX1TextField.getText());
                x2=Double.parseDouble(lineX2TextField.getText());
                z1=Double.parseDouble(lineZ1TextField.getText());
                z2=Double.parseDouble(lineZ2TextField.getText());
            } catch (NumberFormatException ex) { // And if any of them are not valid doubles warn about this and return signalling an error done nothing.
                JOptionPane.showMessageDialog(MainWindow.this,"One or more of the coordinates is not a valid number. Please enter the coordinates again.", "Error", JOptionPane.ERROR_MESSAGE);
                return(false);
            }
            if (x1>x2 || z1>z2) { // If the coordinates of the topleft of the line are not above and left of the bottom right ones, we have an error - warn and return signalling an error.
                JOptionPane.showMessageDialog(MainWindow.this,"x1 must be less than or equal to x2, and z1 must be less than or equal to z2. Please enter the coordinates again.", "Error", JOptionPane.ERROR_MESSAGE);
                return false;
            }
            if (x1<0 || x2<0 || z1<0 || z2<0) {
                JOptionPane.showMessageDialog(MainWindow.this,"Line coordinates cannot be negative. Please enter the coordinates again.", "Error", JOptionPane.ERROR_MESSAGE);
                return false;
            }
            return true; // No errors detected.
        }
    }
    
        class CircleCoordinates {
        double x, z, radius;
        
        boolean getCoordinates() {
            try { // Get the values out of the text boxes.
                x=Double.parseDouble(circleXTextField.getText());
                z=Double.parseDouble(circleZTextField.getText());
                radius=Double.parseDouble(circleRadiusTextField.getText());
            } catch (NumberFormatException ex) { // And if any of them are not valid doubles warn about this and return signalling an error done nothing.
                JOptionPane.showMessageDialog(MainWindow.this,"Either one or more of the coordinates or the radius is not a valid number. Please enter these values again.", "Error", JOptionPane.ERROR_MESSAGE);
                return(false);
            }
            if (x<0 || z<0 || radius<0) {
                JOptionPane.showMessageDialog(MainWindow.this,"Neither the coordinates of the centre of the circle nor its radius can be negative. Please enter the parameters again.", "Error", JOptionPane.ERROR_MESSAGE);
                return false;
            }
            return true; // No errors detected.
        }
    }
                    
    void updateWindowDimensionYTextField() {
        try {
            double newValue=Double.parseDouble(windowDimensionYTextField.getText());
            if (newValue==0) { // This value can't be set to zero - so if it is warn that it can't be changed and reset the textfield.
                MessageBoxes.warnAboutZero(this);
            } else {
                windowDimensionY=newValue;
                updateVisualEditorCentralPanel(); // We need to redraw the window too, at least if the aspect ratio is kept fixed.
                backgroundImagePanel.repaint(); // Changing this setting affects how the background image panel should be shown.
                return;
            }
        } catch (NumberFormatException ex) {
            MessageBoxes.warnAboutBadDouble(this);
        }
        windowDimensionYTextField.setText(""+windowDimensionY); // If we've got here, something has gone wrong, and we need to reset the text.
    }
        
    void updateWindowDimensionXTextField() {
        try {
            double newValue=Double.parseDouble(windowDimensionXTextField.getText());
            if (newValue==0) { // This value can't be set to zero - so if it is warn that it can't be changed and reset the textfield.
                MessageBoxes.warnAboutZero(this);
            } else {
                windowDimensionX=newValue;
                updateVisualEditorCentralPanel(); // We need to redraw the window too, at least if the aspect ratio is kept fixed.
                backgroundImagePanel.repaint(); // Changing this setting affects how the background image panel should be shown.
                return;
            }
        } catch (NumberFormatException ex) {
            MessageBoxes.warnAboutBadDouble(this);
        }
        windowDimensionXTextField.setText(""+windowDimensionX); // If we've got here, something has gone wrong, and we need to reset the text.
    }
    
    /* ImageFilter.java is a 1.4 example used by FileChooserDemo2.java. */
    class ImageFilter extends FileFilter {
        
        //Accept all directories and all gif, jpg, tiff, or png files.
        public boolean accept(File f) {
            if (f.isDirectory()) {
                return true;
            }
            
            String extension = Utils.getExtension(f);
            if (extension != null) {
                if (extension.equals(Utils.tiff) ||
                extension.equals(Utils.tif) ||
                extension.equals(Utils.gif) ||
                extension.equals(Utils.jpeg) ||
                extension.equals(Utils.jpg) ||
                extension.equals(Utils.png)) {
                    return true;
                } else {
                    return false;
                }
            }
            
            return false;
        }
        
        //The description of this filter
        public String getDescription() {
            return "Image Files";
        }
    }

    class EllipsisFileFilter extends FileFilter { 
        
        //Accept all directories and all .input files.
        public boolean accept(File f) {
            if (f.isDirectory()) {
                return true;
            }
            
            String extension = Utils.getExtension(f);
            if (extension != null) {
                if (extension.equals("input")) {
                    return true;
                } else {
                    return false;
                }
            }
            
            return false;
        }
        
        //The description of this filter
        public String getDescription() {
            return "Ellipsis Input Files";
        }
    }
    
    private void openPictureFileMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openPictureFileMenuItemActionPerformed
        fileChooser.resetChoosableFileFilters();
        //           fileChooser.addChoosableFileFilter(FileFilter filter);
        fileChooser.addChoosableFileFilter(new ImageFilter());
        
        int result = fileChooser.showOpenDialog(this);
        if(result == JFileChooser.APPROVE_OPTION) {
            java.io.File filename = fileChooser.getSelectedFile();
            if (filename.exists()) {
                backgroundImagePanel.setImage(filename); // Set this image in the background image panel, and display it.
            } else {
                JOptionPane.showMessageDialog(this,"The file does not exist.", "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
        
    }//GEN-LAST:event_openPictureFileMenuItemActionPerformed

    private void aboutEllipsisGUIDialogWindowCloseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutEllipsisGUIDialogWindowCloseButtonActionPerformed
        aboutEllipsisGuiDialogWindow.hide();
    }//GEN-LAST:event_aboutEllipsisGUIDialogWindowCloseButtonActionPerformed

    private void aboutEllipsisGuiMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutEllipsisGuiMenuItemActionPerformed
        aboutEllipsisGuiDialogWindow.show();
    }//GEN-LAST:event_aboutEllipsisGuiMenuItemActionPerformed

    private void jMenuItem8ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem8ActionPerformed
        MainWindow newWindow=new MainWindow(); // Create a new main window.
        String[] empty={}; // Call function as though no command line parameters had been specified.
        newWindow.show();
    }//GEN-LAST:event_jMenuItem8ActionPerformed
        
    private void universalMaterialPropertiesDialogWindowCloseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_universalMaterialPropertiesDialogWindowCloseButtonActionPerformed
        universalMaterialPropertiesDialogWindow.hide(); 
    }//GEN-LAST:event_universalMaterialPropertiesDialogWindowCloseButtonActionPerformed
                    
    // We only set a minimum viscosity if this option is enabled with the checkbox.
    private void useMinimumViscosityCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_useMinimumViscosityCheckBoxItemStateChanged
        if (evt.getStateChange() == java.awt.event.ItemEvent.DESELECTED) {
            minimumViscosityPanel.setEnabled(false);
            minimumViscosityTextField.setEnabled(false);
            minimumViscosityLabel.setEnabled(false);
        } else {
            minimumViscosityPanel.setEnabled(true);
            minimumViscosityTextField.setEnabled(true);
            minimumViscosityLabel.setEnabled(true);
        }        // Add your handling code here:
    }//GEN-LAST:event_useMinimumViscosityCheckBoxItemStateChanged
    
    // We only set a maximum viscosity if this option is enabled with the checkbox.
    private void useMaximumViscosityCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_useMaximumViscosityCheckBoxItemStateChanged
        if (evt.getStateChange() == java.awt.event.ItemEvent.DESELECTED) {
            maximumViscosityPanel.setEnabled(false); // N.B. Enabling or disabling the panel doesn't actually seem to do anything - hence the panels are pretty useless here.
            maximumViscosityTextField.setEnabled(false);
            maximumViscosityLabel.setEnabled(false);
        } else {
            maximumViscosityPanel.setEnabled(true);
            maximumViscosityTextField.setEnabled(true);
            maximumViscosityLabel.setEnabled(true);
        }        // Add your handling code here:
    }//GEN-LAST:event_useMaximumViscosityCheckBoxItemStateChanged
        
    // The beginner button should be enabled iff the Verbose (input values) checkbox is enabled.    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        if (files.inputFileChanged()) { // If a change has been made since the file was last loaded or saved....
            int optionChosen = JOptionPane.showConfirmDialog(this, // Check to see if we should save the file.
            "Changes have been made.\nDo you want to save the input file before you exit?");
            
            if (optionChosen==JOptionPane.CANCEL_OPTION) {
                return;
            }
            
            if (optionChosen==JOptionPane.YES_OPTION) {
                files.menuSave();
            }
        }
        
        --numberOfOpenWindows;
        if (numberOfOpenWindows==0) {
            System.exit(0);
        } else {
            dispose();
        }
    }//GEN-LAST:event_exitForm
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new MainWindow().show();
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField BCmoveX0v1TextField;
    private javax.swing.JTextField BCmoveX0vTextField;
    private javax.swing.JTextField BCmoveX1vTextField;
    private javax.swing.JCheckBox BhflCheckBox;
    private javax.swing.JCheckBox BvavCheckBox;
    private javax.swing.JCheckBox COMPRESSCheckBox;
    private javax.swing.JCheckBox EdotCheckBox;
    private javax.swing.JCheckBox GRDEPVCheckBox;
    private javax.swing.JCheckBox GebkCheckBox;
    private javax.swing.JCheckBox GebxCheckBox;
    private javax.swing.JCheckBox GeokCheckBox;
    private javax.swing.JCheckBox GeoxCheckBox;
    private javax.swing.JCheckBox GetkCheckBox;
    private javax.swing.JCheckBox GetxCheckBox;
    private javax.swing.JCheckBox GrbkCheckBox;
    private javax.swing.JCheckBox GrbxCheckBox;
    private javax.swing.JCheckBox GrszCheckBox;
    private javax.swing.JCheckBox GrtkCheckBox;
    private javax.swing.JCheckBox GrtxCheckBox;
    private javax.swing.JCheckBox GrvkCheckBox;
    private javax.swing.JCheckBox GrvxCheckBox;
    private javax.swing.JCheckBox NussCheckBox;
    private javax.swing.JComboBox PPMColoringComboBox;
    private javax.swing.JComboBox PPMFilesComboBox;
    private javax.swing.JPanel PPMFilesJPanel;
    private javax.swing.JCheckBox PbdyCheckBox;
    private javax.swing.JCheckBox PresCheckBox;
    private javax.swing.JCheckBox PstnCheckBox;
    private javax.swing.JCheckBox SDEPVCheckBox;
    private javax.swing.JComboBox Sampling_dirnComboBox;
    private javax.swing.JComboBox Sampling_fieldComboBox;
    private javax.swing.JCheckBox Sampling_normalizeCheckBox;
    private javax.swing.JCheckBox ShflCheckBox;
    private javax.swing.JTextField SolverTextField;
    private javax.swing.JCheckBox StmxCheckBox;
    private javax.swing.JCheckBox StrDCheckBox;
    private javax.swing.JCheckBox StrPCheckBox;
    private javax.swing.JCheckBox StrSCheckBox;
    private javax.swing.JCheckBox StrTCheckBox;
    private javax.swing.JCheckBox StrfCheckBox;
    private javax.swing.JCheckBox SvavCheckBox;
    private javax.swing.JCheckBox TempCheckBox;
    private javax.swing.JCheckBox TpbkCheckBox;
    private javax.swing.JCheckBox TpbxCheckBox;
    private javax.swing.JCheckBox TpgkCheckBox;
    private javax.swing.JCheckBox TpgxCheckBox;
    private javax.swing.JScrollPane TracerRegionListScrollPane1;
    private javax.swing.JScrollPane TracerRegionListScrollPane2;
    private javax.swing.JCheckBox UrmsCheckBox;
    private javax.swing.JCheckBox VeloCheckBox;
    private javax.swing.JCheckBox VelxCheckBox;
    private javax.swing.JCheckBox VelzCheckBox;
    private javax.swing.JCheckBox ViscCheckBox;
    private javax.swing.JCheckBox VrmsCheckBox;
    private javax.swing.JCheckBox VxrmCheckBox;
    private javax.swing.JCheckBox VxsfCheckBox;
    private javax.swing.JCheckBox VzrmCheckBox;
    private javax.swing.JCheckBox VzsfCheckBox;
    private javax.swing.JCheckBox WrmsCheckBox;
    private javax.swing.JCheckBox YIELDCheckBox;
    private javax.swing.JButton aboutEllipsisGUIDialogWindowCloseButton;
    private javax.swing.JDialog aboutEllipsisGuiDialogWindow;
    private javax.swing.JMenuItem aboutEllipsisGuiMenuItem;
    private javax.swing.JButton addNewLineButton;
    private javax.swing.JButton addNewRectangleButton;
    private javax.swing.JButton addNewTriangleButton;
    private javax.swing.JRadioButton addRadioButton;
    private javax.swing.JCheckBox animateSelectedRegionsCheckBox;
    private javax.swing.JRadioButton automaticallyScaleParametersRadioButton;
    private javax.swing.JCheckBox averagesTempCheckBox;
    private javax.swing.JCheckBox averagesViscCheckBox;
    private javax.swing.JCheckBox backgroundPictureCheckBox;
    private javax.swing.JCheckBox beginnerCheckBox;
    private javax.swing.ButtonGroup boundaryAxisButtonGroup;
    private javax.swing.JButton boundaryChangeColourButton;
    private javax.swing.JPanel boundaryConditionsPanel;
    private javax.swing.JButton boundaryConditionsSettingsButton;
    private javax.swing.JTextField boundaryHalfWidthTextField;
    private javax.swing.ButtonGroup boundaryMotionButtonGroup;
    private javax.swing.JComboBox boundaryPropertyComboBox;
    private javax.swing.JButton boundaryRegionBackButton;
    private javax.swing.JButton boundaryRegionDeleteButton;
    private javax.swing.JButton boundaryRegionForwardButton;
    private javax.swing.JList boundaryRegionList;
    private javax.swing.JPanel boundaryRegionPropertiesJPanel;
    private javax.swing.JDialog boundarySettingsDialog;
    private javax.swing.JButton boundarySettingsDialogCloseButton;
    private javax.swing.JTextField boundaryValueTextField;
    private javax.swing.JRadioButton boundaryXAxisRadioButton;
    private javax.swing.JRadioButton boundaryYAxisRadioButton;
    private javax.swing.JRadioButton boundaryZAxisRadioButton;
    private javax.swing.JButton circleAddButton;
    private javax.swing.JPanel circleControls;
    private javax.swing.JTextField circleRadiusTextField;
    private javax.swing.JButton circleUpdateButton;
    private javax.swing.JTextField circleXTextField;
    private javax.swing.JTextField circleZTextField;
    private javax.swing.JPanel controlsCardHolder;
    private javax.swing.JLabel currentNextTracerLabel;
    private javax.swing.JTextField datafileTextField;
    private javax.swing.JButton deleteMaterialButton;
    private javax.swing.JButton deletePPMFileButton;
    private javax.swing.JButton deleteSamplingTracerButton;
    private javax.swing.JCheckBox describeCheckBox;
    private javax.swing.JRadioButton doNotScaleParametersRadioButton;
    private javax.swing.JCheckBox drawBoundariesCheckBox;
    private javax.swing.JCheckBox drawDrawingCheckBox;
    private javax.swing.JCheckBox drawMaterialsCheckBox;
    private javax.swing.JCheckBox drawSolidShapesCheckBox;
    private javax.swing.JCheckBox drawThermalZonesCheckBox;
    private javax.swing.JCheckBox drawTracersCheckBox;
    private javax.swing.JButton drawingChangeColourButton;
    private javax.swing.JButton drawingRegionBackButton;
    private javax.swing.JButton drawingRegionDeleteButton;
    private javax.swing.JButton drawingRegionForwardButton;
    private javax.swing.JList drawingRegionList;
    private javax.swing.JScrollPane drawingRegionListScrollPane;
    private javax.swing.ButtonGroup drawingShapesButtonGroup;
    private javax.swing.JPanel drawingsPanel;
    private javax.swing.JButton editRheologyButton;
    private javax.swing.JDialog ellipsisManualDialog;
    private javax.swing.JEditorPane ellipsisManualEditorPane;
    private javax.swing.JMenuItem ellipsisManualMenuItem;
    private javax.swing.JRadioButton empiricalButton;
    private javax.swing.JPanel environmentPanel;
    private javax.swing.JRadioButton eulerianRadioButton;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JCheckBox fixAspectRatioCheckBox;
    private javax.swing.JPanel generalSettingsPanel;
    private javax.swing.ButtonGroup grainSizeModelButtonGroup;
    private javax.swing.JRadioButton growthNucleationButton;
    private javax.swing.JLabel gzipLabel;
    private javax.swing.JTextField gzipTextField;
    private javax.swing.JLabel halfWidthLabel;
    private javax.swing.JLabel halfWidthLabel1;
    private javax.swing.JPanel harmonicControls;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JPanel initialConditionsPanel;
    private javax.swing.JScrollPane inputFileEditorScrollPane;
    private javax.swing.JTextArea inputFileEditorTextArea;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel20;
    private javax.swing.JLabel jLabel21;
    private javax.swing.JLabel jLabel22;
    private javax.swing.JLabel jLabel23;
    private javax.swing.JLabel jLabel24;
    private javax.swing.JLabel jLabel25;
    private javax.swing.JLabel jLabel26;
    private javax.swing.JLabel jLabel27;
    private javax.swing.JLabel jLabel28;
    private javax.swing.JLabel jLabel29;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel30;
    private javax.swing.JLabel jLabel31;
    private javax.swing.JLabel jLabel32;
    private javax.swing.JLabel jLabel33;
    private javax.swing.JLabel jLabel34;
    private javax.swing.JLabel jLabel35;
    private javax.swing.JLabel jLabel36;
    private javax.swing.JLabel jLabel37;
    private javax.swing.JLabel jLabel38;
    private javax.swing.JLabel jLabel39;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel40;
    private javax.swing.JLabel jLabel42;
    private javax.swing.JLabel jLabel43;
    private javax.swing.JLabel jLabel44;
    private javax.swing.JLabel jLabel45;
    private javax.swing.JLabel jLabel46;
    private javax.swing.JLabel jLabel47;
    private javax.swing.JLabel jLabel48;
    private javax.swing.JLabel jLabel49;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel50;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JMenu jMenu3;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuItem jMenuItem8;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel10;
    private javax.swing.JPanel jPanel11;
    private javax.swing.JPanel jPanel12;
    private javax.swing.JPanel jPanel13;
    private javax.swing.JPanel jPanel14;
    private javax.swing.JPanel jPanel15;
    private javax.swing.JPanel jPanel16;
    private javax.swing.JPanel jPanel17;
    private javax.swing.JPanel jPanel18;
    private javax.swing.JPanel jPanel19;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel20;
    private javax.swing.JPanel jPanel21;
    private javax.swing.JPanel jPanel22;
    private javax.swing.JPanel jPanel23;
    private javax.swing.JPanel jPanel24;
    private javax.swing.JPanel jPanel25;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JTextArea jTextArea10;
    private javax.swing.JTextArea jTextArea11;
    private javax.swing.JTextArea jTextArea12;
    private javax.swing.JTextArea jTextArea13;
    private javax.swing.JTextArea jTextArea14;
    private javax.swing.JTextArea jTextArea15;
    private javax.swing.JTextArea jTextArea16;
    private javax.swing.JTextArea jTextArea17;
    private javax.swing.JTextArea jTextArea2;
    private javax.swing.JTextArea jTextArea3;
    private javax.swing.JTextArea jTextArea4;
    private javax.swing.JTextArea jTextArea5;
    private javax.swing.JTextArea jTextArea6;
    private javax.swing.JTextArea jTextArea7;
    private javax.swing.JTextArea jTextArea8;
    private javax.swing.JTextArea jTextArea9;
    private javax.swing.JRadioButton lagrangianRadioButton;
    private javax.swing.JPanel lineControls;
    private javax.swing.JButton lineUpdateButton;
    private javax.swing.JTextField lineX1TextField;
    private javax.swing.JTextField lineX2TextField;
    private javax.swing.JTextField lineZ1TextField;
    private javax.swing.JTextField lineZ2TextField;
    private javax.swing.JTabbedPane mainPanes;
    private javax.swing.JRadioButton manuallyScaleParametersRadioButton;
    private javax.swing.JComboBox materialChooser;
    private javax.swing.JButton materialRegionBackButton;
    private javax.swing.JButton materialRegionDeleteButton;
    private javax.swing.JButton materialRegionForwardButton;
    private javax.swing.JList materialRegionList;
    private javax.swing.JScrollPane materialRegionListScrollPane;
    private javax.swing.JButton materialSettingsButton;
    private javax.swing.JTabbedPane materialsDrawingBoundaryTracersPanel;
    private javax.swing.JPanel materialsDrawingsBoundaryTracersPanel;
    private javax.swing.JPanel materialsPanel;
    private javax.swing.JLabel maximumViscosityLabel;
    private javax.swing.JPanel maximumViscosityPanel;
    private javax.swing.JTextField maximumViscosityTextField;
    private javax.swing.JLabel minimumViscosityLabel;
    private javax.swing.JPanel minimumViscosityPanel;
    private javax.swing.JTextField minimumViscosityTextField;
    private javax.swing.JRadioButton multiplyRadioButton;
    private javax.swing.JPanel multrigridPanel;
    private javax.swing.JButton newMaterialButton;
    private javax.swing.JButton newPPMFileButton;
    private javax.swing.JButton newSamplingTracerButton;
    private javax.swing.JDialog nonDimensionalizationDialog;
    private javax.swing.JButton nonDimensionalizationDialogCloseButton;
    private javax.swing.JEditorPane nonDimensionalizationEditorPane;
    private javax.swing.JMenuItem nonDimensionaliztionMenuItem;
    private javax.swing.JCheckBox observablesBhflCheckBox;
    private javax.swing.JCheckBox observablesShflCheckBox;
    private javax.swing.JMenuItem openInputFileMenuItem;
    private javax.swing.JMenuItem openPictureFileMenuItem;
    private javax.swing.JPanel outputSettingsPanel;
    private javax.swing.JCheckBox particleDataPresCheckBox;
    private javax.swing.JCheckBox particleDataTempCheckBox;
    private javax.swing.JCheckBox periodic_rm_vxCheckBox;
    private javax.swing.JCheckBox periodicxCheckBox;
    private javax.swing.JRadioButton piecewiseBoundaryRadioButton;
    private javax.swing.JPanel polynomialControls;
    private javax.swing.JButton profileColourButton;
    private javax.swing.JPanel rectangleControls;
    private javax.swing.JButton rectangleUpdateButton;
    private javax.swing.JTextField rectangleX1TextField;
    private javax.swing.JTextField rectangleX2TextField;
    private javax.swing.JTextField rectangleY1TextField;
    private javax.swing.JTextField rectangleY2TextField;
    private javax.swing.JLabel regionsDensityLabel;
    private javax.swing.JRadioButton replaceRadioButton;
    private javax.swing.JComboBox samplingPPMComboBox;
    private javax.swing.JPanel samplingTracerPropertiesJPanel;
    private javax.swing.JButton samplingTracersButton;
    private javax.swing.JComboBox samplingTracersComboBox;
    private javax.swing.JDialog samplingTracersDialog;
    private javax.swing.JMenuItem saveInputFileAsMenuItem;
    private javax.swing.JMenuItem saveInputFileMenuItem;
    private javax.swing.JPanel scalingDetailsPanel;
    private javax.swing.ButtonGroup scalingOptionsButtonGroup;
    private javax.swing.JComboBox shapeChooser;
    private javax.swing.JMenuItem showUnrecognisedParameters;
    private javax.swing.JRadioButton simpleBoundaryRadioButton;
    private javax.swing.JCheckBox temperatureDependentRheologicalParametersCheckBox;
    private javax.swing.JButton thermalRegionBackButton;
    private javax.swing.JButton thermalRegionDeleteButton;
    private javax.swing.JButton thermalRegionForwardButton;
    private javax.swing.JList thermalRegionList;
    private javax.swing.ButtonGroup thermalZonesButtonGroup;
    private javax.swing.JPanel thermalZonesPanel;
    private javax.swing.JPanel timePanel;
    private javax.swing.JButton tracerChangeColourButton;
    private javax.swing.JTextField tracerDensityTextField;
    private javax.swing.JButton tracerRegionBackButton;
    private javax.swing.JButton tracerRegionDeleteButton;
    private javax.swing.JButton tracerRegionForwardButton;
    private javax.swing.JList tracerRegionList;
    private javax.swing.JScrollPane tracerRegionListScrollPane;
    private javax.swing.JPanel tracerRegionsPanel;
    private javax.swing.ButtonGroup tracerTypeButtonGroup;
    private javax.swing.JPanel tracersPanel;
    private javax.swing.JPanel triangleControls;
    private javax.swing.JButton triangleUpdateButton;
    private javax.swing.JTextField triangleX1TextField;
    private javax.swing.JTextField triangleX2TextField;
    private javax.swing.JTextField triangleX3TextField;
    private javax.swing.JTextField triangleY1TextField;
    private javax.swing.JTextField triangleY2TextField;
    private javax.swing.JTextField triangleY3TextField;
    private javax.swing.JDialog unitsAndScalingDialog;
    private javax.swing.JButton unitsAndScalingDialogCloseButton;
    private javax.swing.JMenuItem unitsAndScalingMenuItem;
    private javax.swing.JTextArea unitsAndScalingTextArea;
    private javax.swing.JDialog universalMaterialPropertiesDialogWindow;
    private javax.swing.JButton universalMaterialPropertiesDialogWindowCloseButton;
    private javax.swing.JDialog unrecognisedParametersDialog;
    private javax.swing.JButton unrecognisedParametersDialogCloseButton;
    private javax.swing.JTextArea unrecognisedParametersDialogTextArea;
    private javax.swing.JCheckBox useDefaultTracerSetup;
    private javax.swing.JCheckBox useMaximumViscosityCheckBox;
    private javax.swing.JCheckBox useMinimumViscosityCheckBox;
    private javax.swing.JLabel usimsLogoLabel;
    private javax.swing.JCheckBox verboseCodeBehaviourCheckBox;
    private javax.swing.JCheckBox verboseInputValuesCheckBox;
    private javax.swing.JPanel verbosityPanel;
    private javax.swing.JPanel visualControlsPanel;
    private javax.swing.JPanel visualEditorCentralPanel;
    private javax.swing.JPanel visualEditorPanel;
    private javax.swing.JTextField windowDimensionXTextField;
    private javax.swing.JTextField windowDimensionYTextField;
    // End of variables declaration//GEN-END:variables
    
    class Material {
        
        private MaterialRheologyWindow materialRheologyWindow;
        private ThermodynamicAndMeltPropertiesDialog thermodynamicAndMeltPropertiesDialog=new ThermodynamicAndMeltPropertiesDialog(MainWindow.this, false); // and also a ThermodynamicAndMeltPropertiesDialog
        
        void setYieldStressWindowObjects(boolean set) {
            materialRheologyWindow.setYieldStressWindowObjects(set);
        }
        
        String getEllipsisFileSettings() { // Return a string giving all the settings for this material in ellipsis input file format.
            String settings="";
            String materialParameterPrefix="Material_"+numberOfThisMaterial()+"_"; // This string is needed to prefix all the parameters for this material.

            settings+=materialRheologyWindow.getEllipsisFileSettings(materialParameterPrefix, scaling);
            settings+=thermodynamicAndMeltPropertiesDialog.getEllipsisFileSettings(materialParameterPrefix, scaling);
        
            return settings;
        }
        
        void parseAndRemoveRheologySettings() {
            String materialParameterPrefix="Material_"+numberOfThisMaterial()+"_"; // This string is needed to prefix all the parameters for this material.
            materialRheologyWindow.parseAndRemoveRheologySettings(materialParameterPrefix, files, scaling);
            thermodynamicAndMeltPropertiesDialog.parseAndRemoveRheologySettings(materialParameterPrefix, files, scaling);
        }
        
        int numberOfThisMaterial() { // Return the number of the material, which will be used to index the material in ellipsis input files. Materials are number from 0, in the order in which they occur in the materials list - N.B. ellipsis input file material numbers are completely unrelated to Ellipsis GUI material numbers or names.
            Iterator currentMaterial=materials.iterator();
            int numberOfCurrentMaterial=0;
            
            while (currentMaterial.hasNext()) {
                if (currentMaterial.next()==this) // If we've found this material...
                    return numberOfCurrentMaterial; // Then return its index.
                ++numberOfCurrentMaterial;
            }
            
            return -1; // We should never get here, because we should find every material for which a region is defined (we can't delete materials if they are in use).
        }
        
        String getName() {
            return materialRheologyWindow.materialNameTextBox.getText();
        }
        
        void showRheologyWindow() {
            materialRheologyWindow.show();
        }
        
        void disposeOfWindows() {
            materialRheologyWindow.dispose();
            materialRheologyWindow.disposeOfSubWindows();
            thermodynamicAndMeltPropertiesDialog.dispose();
        }
        
        Material() { // Constructor - needs to set up some event listeners for the windows.
            if (YIELDCheckBox!=null) { // Check if the YIELDCheckBox has been created yet.
                materialRheologyWindow=new MaterialRheologyWindow(YIELDCheckBox.isSelected(), numberOfNextMaterial); // Each material must have one MaterialRheologyWindow. Whether the yield stress objects are enabled depends on the setting of the YIELD checkbox. But the yield stress check box might not even have been created by now, so check that first.
            } else {
                materialRheologyWindow=new MaterialRheologyWindow(false, numberOfNextMaterial); // If the YIELD check box has not been created yet, initialise the window with yield stress turned off.
            }
            ++numberOfNextMaterial;
            
            materialRheologyWindow.materialNameTextBox.addFocusListener(new java.awt.event.FocusAdapter() { // Whenever someone has finished editing the textbox (even if they didn't change anything) update the material names in the materialChooser.
                public void focusLost(java.awt.event.FocusEvent evt) {
                    materialNameTextBoxFocusLost(evt);
                }
            });
            
            materialRheologyWindow.thermodynamicAndMeltPropertiesButton.addActionListener(new java.awt.event.ActionListener() { // Whenever the theromodynamicAndMeltPropertiesButton is pressed, we have to show the window.
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    thermodynamicAndMeltPropertiesButtonActionPerformed(evt);
                }
            });
            
            materialRheologyWindow.materialColourButton.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
                public void propertyChange(java.beans.PropertyChangeEvent evt) {
                    recolourMaterialChooser(); // Just calling this will ensure that the chooser is recoloured appropriately (i.e. if the selected material is the one the colour of which was just changed).
                }
            });
            
        }
        
        // This needs to update the materialChooser combo box in the main window.
        private void materialNameTextBoxFocusLost(java.awt.event.FocusEvent evt) {
            updateMaterialChooserAndMaterialName();
        }
        
        void updateMaterialChooserAndMaterialName() {
            // Problem: the material has no idea which item in the box it is - so we just reenter all the items into the box.
            int selectedIndex=materialChooser.getSelectedIndex(); // We want to keep the same material selected even when the name is changed, so record it here, and set the same one to be selected at the end of this function.
            materialChooser.removeAllItems();
            for (int i=0; i<materials.size(); ++i) { // Go through all the materials one by one.
                materialChooser.addItem(((Material)materials.get(i)).getName()); // And for each one get its name and put it in the materialChooser ComboBox.
            }
            materialChooser.setSelectedIndex(selectedIndex);
            
            thermodynamicAndMeltPropertiesDialog.materialNameLabel.setText(getName()); // Also update the material name in the thermodynamicAndMeltPropertiesDialog - but only for this material.
            
            backgroundImagePanel.materialRegionContainer.updateMaterialRegionList(); // We also need to update the list of material regions.
        }
        
        private void thermodynamicAndMeltPropertiesButtonActionPerformed(java.awt.event.ActionEvent evt) {
            thermodynamicAndMeltPropertiesDialog.show();
        }
        
    }  // End of class material.
    
    class Files {
        String lastSavedVersion;
        String inputFile; // This is the state of an input file as it will be when it's saved to disk. It will get out of date as changes are made in the GUI. This is what will be shown in the input file editor.
        String inputFileAsLoadedOrEnteredInTextArea; // This is a copy of inputFile that is kept, and reflects the state of the input file the last time it was read in.
        String unrecognisedParameters=""; // Any parameters found in a file that the GUI doesn't recognise (e.g. parameters for which it doesn't have any implementation).
        boolean unrecognisedParametersFoundLastTime=false; // Keep a record of whether any unrecognised parameters were found last time an input file was read in.
//        String comments=""; // Any comments found in a read in file that should be preserved. Not needed now that inputFile is being edited, not created from scratch
        String linesWithErrors=""; // Store any lines which have errors on them here.
        java.io.File filename; // The name with which the file was last opened or saved - if this is null no filename has been specified yet.
        
        final String ellipsisGUIParameterPrefix="# Ellipsis GUI parameter:";
        
        Files() { // N.B. all the window objects must be created and initialised before this is called.
            updateInputFile(); 
            lastSavedVersion=new String(inputFile);
        }

        String startOfFileMessage() {
            GregorianCalendar timeNow=new GregorianCalendar(); // Get a calendar occurence containing the present time.
            return "# Ellipsis Input File\n#\n# Created by the Ellipsis GUI V1.0 on "+timeNow.get(Calendar.DATE)+"/"+(timeNow.get(Calendar.MONTH)+1)+"/"+timeNow.get(Calendar.YEAR)+"\n#\n\n"; //Add any comments immediately below.\n#\n\n";
        }
        
        void updateInputFile() { // Create an input file based on all the settings in the gui, and any stored comments and unrecognised parameters.
            scaling.updateScalingFactors(); // Before we do any scaling, we have to make sure that the scaling factors are up to date.
            
            if (inputFileAsLoadedOrEnteredInTextArea==null) { // If we've not yet loaded a file, or edited one in the text area, then inputFile will be just as we have created it, so there's nothing more to do.
                createInputFile();
            } else {
                makeEditsToStoredInputFile();
            }
        }
        
        void createInputFile() { // Create an input file from scratch.
//            System.out.println("Using input file as created by GUI");

            inputFile=startOfFileMessage();

            inputFile+=getErroneousLines(); // As we are creating the input file from scratch, we won't have read in an input file, so presumably there will be no unrecognised parameters or lines with errors. However this code is kept here so that there can be the option of always creating an input file in the standard format if desired.
            inputFile+=unrecognisedParameters; // Start with all the previously stored comments and unrecognised parameters.
            
            inputFile+=getParameterSettings(); // Now add all the parameter settings, including scaling parameters.
        }
        
        String getParameterSettings() { // Get all the parameters stored in the GUI in ellipsis format.
            String parameters=getScalingParameters(); // Save the scaling parameters, so that they can be used for rescaling when the file is loaded in again.
            
            parameters+=getGeneralFileSettings();
            
            parameters+=backgroundImagePanel.boundaryRegionContainer.getFileSettings();

            parameters+=getInitialConditionsFileSettings();
            parameters+=backgroundImagePanel.tracerRegionContainer.getFileSettings();

            parameters+=getMaterialFileSettings();
            parameters+=backgroundImagePanel.materialRegionContainer.getFileSettings();
            
            parameters+=backgroundImagePanel.thermalRegionContainer.getFileSettings();
            
            parameters+=getOutputFileSettings();
            
            //parameters+=drawingRegionsContainer.getFileSettings(); // I may want to store any drawings in the input file as well, just so they'll be around if the file is reloaded.
            
            return parameters;
        }
        
        String getScalingParameters() {
            String settings="# SCALING PARAMETERS\n\n";
            
            settings+=ellipsisGUIParameterPrefix+"ellipsisUnitsPerM="+scaling.ellipsisUnitsPerM+"\n";
            settings+=ellipsisGUIParameterPrefix+"ellipsisUnitsPerSecond="+scaling.ellipsisUnitsPerSecond+"\n";
            settings+=ellipsisGUIParameterPrefix+"ellipsisUnitsPerKG="+scaling.ellipsisUnitsPerKG+"\n";
            settings+=ellipsisGUIParameterPrefix+"ellipsisUnitsPerKelvin="+scaling.ellipsisUnitsPerKelvin+"\n\n";
            
            return settings;
        }
        
        String getErroneousLines() {
            if (linesWithErrors.equals("")) // If there are no errors, return a blank string.
                return "";
            return "# The following lines contain errors which must be corrected before the input file can be used with Ellipsis\n\n"+linesWithErrors+"\n";
        }
        
        String getOutputFileSettings() { // All scaling is implemented (up to observables).
            String settings="# OUTPUT\n\n";
            settings+="datafile=\""+datafileTextField.getText()+"\"\n";
            settings+="COMPRESS="+Statics.boolean2OnOff(COMPRESSCheckBox.isSelected());
            settings+="gzip=\""+gzipTextField.getText()+"\"\n\n";
            
            settings+="storage_timesteps="+storage_timestepsTextField.getText();
            settings+="\ncheckpt_timesteps="+checkpt_timesteps.getText();
            
            settings+="\n\ndatatypes="+getRequiredDatatypes();
            settings+="particle_data="+getRequiredParticle_data();
            settings+="averages="+getRequiredAverages();
            settings+="timelog="+getRequiredTimelog();
            settings+="observables="+getRequiredObservables();
            
            settings+=getPPMFileSettings();
            settings+=getSamplingTracersFileSettings();
            
            return settings;
        }
        
        String getSamplingTracersFileSettings() {
            if (samplingTracers.size()==0) { // If there aren't any sampling tracers, just do nothing.
                return "";
            }
            
            String settings="\nSampling_tracers="+samplingTracers.size()+"\n"; 
            
            String Sampling_lagrangian="Sampling_lagrangian=";
            String Sampling_plot_num="Sampling_plot_num=";
            String Sampling_x="Sampling_x=";
            String Sampling_z="Sampling_z=";
            String Sampling_field="Sampling_field=";
            String Sampling_dirn="Sampling_dirn=";
            String Sampling_normalize="Sampling_normalize=";
            String Sampling_R="Sampling_R=";
            String Sampling_G="Sampling_G=";
            String Sampling_B="Sampling_B=";
            
            Iterator nextSamplingTracer=samplingTracers.iterator(); // Go through each SamplingTracer, one at a time.
            while (nextSamplingTracer.hasNext()) {
                SamplingTracer currentSamplingTracer=(SamplingTracer)nextSamplingTracer.next(); // Get a sampling tracer.
                
                Sampling_lagrangian+=currentSamplingTracer.Sampling_lagrangian+","; // And add each of its settigns to the appropriate list.
                Sampling_plot_num+=currentSamplingTracer.Sampling_plot_num+",";
                Sampling_x+=scaling.KM2EllipsisUnits(currentSamplingTracer.Sampling_x)+",";
                Sampling_z+=scaling.KM2EllipsisUnits(currentSamplingTracer.Sampling_z)+",";
                Sampling_field+=currentSamplingTracer.Sampling_field+",";
                Sampling_dirn+=currentSamplingTracer.Sampling_dirn+",";
                Sampling_normalize+=currentSamplingTracer.Sampling_normalize+",";
                Sampling_R+=Colours.javaFormat2R(currentSamplingTracer.profileColour)+",";
                Sampling_G+=Colours.javaFormat2G(currentSamplingTracer.profileColour)+",";
                Sampling_B+=Colours.javaFormat2B(currentSamplingTracer.profileColour)+",";
            }
            
            settings+=Useful.chopAnyTrailingComma(Sampling_lagrangian)+"\n"; // Now put the lists into settings (removing commas from the last items.
            settings+=Useful.chopAnyTrailingComma(Sampling_plot_num)+"\n";
            settings+=Useful.chopAnyTrailingComma(Sampling_x)+"\n";
            settings+=Useful.chopAnyTrailingComma(Sampling_z)+"\n";
            settings+=Useful.chopAnyTrailingComma(Sampling_field)+"\n";
            settings+=Useful.chopAnyTrailingComma(Sampling_dirn)+"\n";
            settings+=Useful.chopAnyTrailingComma(Sampling_normalize)+"\n";
            settings+=Useful.chopAnyTrailingComma(Sampling_R)+"\n";
            settings+=Useful.chopAnyTrailingComma(Sampling_G)+"\n";
            settings+=Useful.chopAnyTrailingComma(Sampling_B)+"\n";

            return settings;
        }
        
        String getPPMFileSettings() {
            String settings="\nPPM_files="+PPMFiles.size()+"\n"; // This parameter just records the number of PPM files to create at each step.
            
            String PPM_height="PPM_height=";
            String PPM_coloring="PPM_coloring=";
            Iterator nextPPM=PPMFiles.iterator();
            while (nextPPM.hasNext()) {
                PPM_file PPM=(PPM_file)nextPPM.next();
                PPM_height+=PPM.height+",";
                PPM_coloring+=PPM.coloring+",";
            }
            
            PPM_height=Useful.chopAnyTrailingComma(PPM_height);
            PPM_coloring=Useful.chopAnyTrailingComma(PPM_coloring);
            
            settings+=PPM_height+"\n";
            settings+=PPM_coloring+"\n";
            
            return settings;
        }

        String getRequiredObservables() {
            String list="\"";
            
            if (TpgxCheckBox.isSelected())
                list+="Tpgx,";
            if (TpgkCheckBox.isSelected())
                list+="Tpgk,";
            if (TpbxCheckBox.isSelected())
                list+="Tpbx,";
            if (TpbkCheckBox.isSelected())
                list+="Tpbk,";
            if (GrvxCheckBox.isSelected())
                list+="Grvs,";
            if (GrbxCheckBox.isSelected())
                list+="Grbx,";
            if (GrtxCheckBox.isSelected())
                list+="Grtx,";
            if (GrvkCheckBox.isSelected())
                list+="Grvk,";
            if (GrbkCheckBox.isSelected())
                list+="Grbk,";
            if (GrtkCheckBox.isSelected())
                list+="Grtk,";
            if (GeoxCheckBox.isSelected())
                list+="Geox,";
            if (GebxCheckBox.isSelected())
                list+="Gebx,";
            if (GetxCheckBox.isSelected())
                list+="Getx,";
            if (GeokCheckBox.isSelected())
                list+="Geok,";
            if (GebkCheckBox.isSelected())
                list+="Gebk,";
            if (GetkCheckBox.isSelected())
                list+="Getk,";
            if (observablesShflCheckBox.isSelected())
                list+="Shfl,";
            if (observablesBhflCheckBox.isSelected())
                list+="Bhfl,";
            if (VxsfCheckBox.isSelected())
                list+="Vxsf,";
            if (VzsfCheckBox.isSelected())
                list+="Vzsf";
            
            list=Useful.chopAnyTrailingComma(list);
            list+="\"\n";
            return list;
        }
        
        String getRequiredTimelog() {
            String list="\"";
            
            if (NussCheckBox.isSelected())
                list+="Nuss,";
            if (VrmsCheckBox.isSelected())
                list+="Vrms,";
            if (VxrmCheckBox.isSelected())
                list+="Vxrm,";
            if (VzrmCheckBox.isSelected())
                list+="Vzrm,";
            if (ShflCheckBox.isSelected())
                list+="Shfl,";
            if (BhflCheckBox.isSelected())
                list+="Bhfl,";
            if (StmxCheckBox.isSelected())
                list+="Stmx,";
            if (SvavCheckBox.isSelected())
                list+="Svav,";
            if (BvavCheckBox.isSelected())
                list+="Bvav";
            
            list=Useful.chopAnyTrailingComma(list);
            list+="\"\n";
            return list;
        }

        String getRequiredAverages() {
            String list="\"";

            if (averagesTempCheckBox.isSelected())
                list+="Temp,";
            if (averagesViscCheckBox.isSelected())
                list+="Visc,";
            if (VeloCheckBox.isSelected())
                list+="Velo,";
            if (UrmsCheckBox.isSelected())
                list+="Urms,";
            if (WrmsCheckBox.isSelected())
                list+="Wrms";
            
            list=Useful.chopAnyTrailingComma(list);
            list+="\"\n";
            return list;
        }
        
        String getRequiredParticle_data() {
            String list="\"";
            if (particleDataTempCheckBox.isSelected()) 
                list+="Temp,";
            if (particleDataPresCheckBox.isSelected())
                list+="Pres,";
            if (ViscCheckBox.isSelected())
                list+="Visc,";
            if (EdotCheckBox.isSelected())
                list+="Edot,";
            if (StrPCheckBox.isSelected())
                list+="StrP,";
            if (StrTCheckBox.isSelected())
                list+="StrT,";
            if (StrDCheckBox.isSelected())
                list+="StrD,";
            if (StrDCheckBox.isSelected())
                list+="StrS,";
            if (GrszCheckBox.isSelected())
                list+="Grsz";
            
            list=Useful.chopAnyTrailingComma(list);
            list+="\"\n";
            return list;
        }
        
        String getRequiredDatatypes() {
            String list="\"";
            if (VelxCheckBox.isSelected())
                list+="Velx,";
            if (VelzCheckBox.isSelected())
                list+="Velz,";
            if (PresCheckBox.isSelected())
                list+="Pres,";
            if (TempCheckBox.isSelected())
                list+="Temp,";
            if (StrfCheckBox.isSelected())
                list+="Strf,";
            if (PbdyCheckBox.isSelected())
                list+="Pbdy,";
            if (PstnCheckBox.isSelected())
                list+="Pstn";
            
            list=Useful.chopAnyTrailingComma(list);
            list+="\"\n";
            return list;
        }
        
        String getInitialConditionsFileSettings() {
            String settings="# INITIAL CONDITIONS\n\n# Environment\n";
            settings+="gravacc="+scaling.acceleration2EllipsisUnits(gravityNumberBoxBean.getText()); // N.B. Gravity is never scaled! *********** BUT I HAVE SCALED GRAVITY **********
            settings+="\ngravtheta="+gravityAngleNumberBoxBean.getText()+"\n\n"; // This is dimensionless.
            
            return settings;
        }
                
        String getGeneralFileSettings() { // Create a string giving all the general settings in ellipsis input file format.
            // Verbosity
            String settings="# GENERAL SETUP\n\n# Verbosity\n";
            settings+="DESCRIBE="+Statics.boolean2OnOff(describeCheckBox.isSelected());
            settings+="VERBOSE="+Statics.boolean2OnOff(verboseInputValuesCheckBox.isSelected());
            if (verboseInputValuesCheckBox.isSelected()) // Beginner parameter is only relevant when verbose mode is on.
                settings+="BEGINNER="+Statics.boolean2OnOff(beginnerCheckBox.isSelected());
            settings+="verbose="+Statics.boolean2OnOff(verboseCodeBehaviourCheckBox.isSelected());
            
            // Geometry
            settings+="\n# Geomerty\n";
            settings+="Geometry=cart2d\n"; // The GUI is only for the 2D version of Ellipsis.
            settings+="dimenx="+scaling.KM2EllipsisUnits(windowDimensionX); // These are taken from the size the window is set to in KM.
            settings+="\ndimenz="+scaling.KM2EllipsisUnits(windowDimensionY); // N.B. is Y - really it should be Z.
            
            // Multigrid -- All dimensionless.
            settings+="\n\n# Multigrid\n";
            settings+="mgunitx="+mgunitxNumberBoxBean.getText();
            settings+="\nmgunitz="+mgunitzNumberBoxBean.getText();
            settings+="\nlevels="+levelsNumberBoxBean.getText();
            settings+="\npmg_levels="+pmg_levelsNumberBoxBean.getText();
            
            // Time
            settings+="\n\n# Time\n";
            settings+="maxadvtime="+scaling.S2EllipsisUnits(maxadvtimeNumberBoxBean.getText()); 
            settings+="\nminstep="+minstepNumberBoxBean.getText(); // Dimenionless
            settings+="\nmaxstep="+maxstepNumberBoxBean.getText(); // Dimenionless
            
            settings+="\nSolver="+SolverTextField.getText()+"\n\n";
            
            return settings;
        }
                
        String getMaterialFileSettings() {
            String settings="# MATERIALS\n";
            
            settings+=getGeneralMaterialSettings(); // First do general settings that apply to all materials.
            
            settings+=getMaterialDefinitionFileSettings();
            
            return settings;
        }
        
        String getMaterialDefinitionFileSettings() { // Get the ellipsis input file parameters defining all the materials.
            String settings="\n# Material Definitions\n\n";
            Iterator currentMaterial=materials.iterator(); // Go through the materials in order.
            while (currentMaterial.hasNext()) {
                settings+=((Material)(currentMaterial.next())).getEllipsisFileSettings(); // Add on the settings for this material, and move on to the next material.
            }
            
            return settings;
        }

        String getGeneralMaterialSettings() {
            String settings="\n# General Material Properties\n\n";
            settings+="Different_materials="+materials.size();
            
            if (growthNucleationButton.isSelected()) {
                settings+="\nGrain_size_model=1\n";
            } else {
                settings+="\nGrain_size_model=2\n";
            }
            
            settings+="VMAX="+Statics.boolean2OnOff(useMaximumViscosityCheckBox.isSelected());
            if (useMaximumViscosityCheckBox.isSelected())
                settings+="visc_max="+scaling.viscosity2EllipsisUnits(maximumViscosityTextField.getText())+"\n";
            
            settings+="VMIN="+Statics.boolean2OnOff(useMinimumViscosityCheckBox.isSelected());
            if (useMinimumViscosityCheckBox.isSelected())
                settings+="visc_min="+scaling.viscosity2EllipsisUnits(minimumViscosityTextField.getText())+"\n";
            
            settings+="TDEPV="+Statics.boolean2OnOff(temperatureDependentRheologicalParametersCheckBox.isSelected());
            settings+="SDEPV="+Statics.boolean2OnOff(SDEPVCheckBox.isSelected());
            settings+="YIELD="+Statics.boolean2OnOff(YIELDCheckBox.isSelected());
            settings+="GRDEPV="+Statics.boolean2OnOff(GRDEPVCheckBox.isSelected());
            
            return settings;
        }

        void updateGUIetc() { // Take input file and load its setting into the GUI - storing unrecognised parameters/comments etc. This function works by looking for each recognised parameter in turn, and storing it in the GUI, and removing it from the input file. Then the left over bits of the input file will be comments etc. The functions always work on the member variable inputFile.
//            System.out.println("Entering update GUI");
            
            scaling.updateScalingFactors(); // Before we do any scaling, we have to make sure that the scaling factors are up to date. Although we might in fact load new ones from the input file.

            inputFile=inputFileEditorTextArea.getText(); // Get the text from the input file editor.
            inputFileAsLoadedOrEnteredInTextArea=inputFile; // Make a copy of the input file, as we're going to change the other one.
            
            linesWithErrors=""; // Start off with a blank string for lines with errors on them. User will get the option of whether to keep or delete them.

            parseAndRemoveScalingParameters(); // N.B. It is very important that this is done first, because the scaling factors will be used when reading in the real ellipsis parameters.
            
            parseAndRemoveGeneralFileSettings();
            parseAndRemoveBoundaryFileSettings();
            parseAndRemoveInitialConditionFileSettings();
            parseAndRemoveOutputFileSettings(); // This must come before parseAndRemoveMaterialFileSettings, because we have to have read in the PPM file parameters before we can read in the PPM colours (which are defined as part of each material).
            parseAndRemoveMaterialFileSettings();
            parseAndRemoveThermalZoneFileSettings();
            
//            removeDuplicateComments(); // This is so that any comments added by the GUI don't get left in, because they'll be added again when the file is output. THIS LINE IS REDUNDANT, BECAUSE NOW WE'LL ALWAYS BE EDITING THE INPUT FILE, so any comments added by the gui will be ignored. == new parameters will just be added at the bottom of the file (probably without any comments, except for EllipsisGUIParameters).
            
            backgroundImagePanel.repaint(); // We will probably have changed some regions, so they need repaining.
            
            reportAndAskAboutErroneousParameters(); // Report if there are any parameters with syntax errors, or values out of range. Ask whether they should be deleted, or put at start of input file. Take appropriate action.

            reportIfUnrecognisedParameters(); // If there are any parameters left in the file we want to warn the user of this, and tell them to edit the file by hand.
            
            //comments=inputFile; // Anything left now must just be a comment, so put them in the input file too. *** Just now don't keep any comments - this may be a bit of a problem - also remove duplicate comments is not implemented yet. ********
        }
        
        void reportAndAskAboutErroneousParameters() {
            if (linesWithErrors.equals("")) // If there are no lines with errors, do nothing.
                return;
            
            int answer=JOptionPane.showConfirmDialog(MainWindow.this, "One or more of the lines in the input file contained errors.\nDo you want to delete to delete those lines?", "", JOptionPane.YES_NO_OPTION);
            if (answer==JOptionPane.YES_OPTION) { // If the user chooser to delete lines with errors, do so, then return.
                if (inputFileAsLoadedOrEnteredInTextArea!=null) {
                    inputFileAsLoadedOrEnteredInTextArea=deleteTheseLines(linesWithErrors, inputFileAsLoadedOrEnteredInTextArea);
                }
                linesWithErrors="";
                return;
            }

            // If we're not deleting the lines with errors, tell the user whre to find them and what to do with them.
            JOptionPane.showMessageDialog(MainWindow.this, "The lines containing errors have been placed near the top of the input file.\nPlease edit them yourself using the input file editor.", "", JOptionPane.INFORMATION_MESSAGE);
        }
        
        void reportIfUnrecognisedParameters() { // Put any unrecognised parameters in unrecognisedParameters, and report if there are any.
            removeErroneousParametersFromInputFile(); // If parameters are in linesWithErrors, then they aren't unrecognised, so remove them.
            int index=0; // Start looking for them from the start of the file.
            unrecognisedParameters="";
            while (index<inputFile.length()) {
//                System.out.println("Index="+index+" Data is "+inputFile.substring(index,Math.min(index+10, inputFile.length())));
                if (!everythingElseOnLineIsWhiteSpaceOrComment(index)) { // If we find a parameter...
                    unrecognisedParameters+=removeLine(index); // put it in unrecognised parameters.
                    // index will now point at the start of the next line (because we removed the present one) or the end of the file (if we just removed the last line).
                } else {
                    index=startOfNextLine(index); // May point at first character after end of the string.
                }
            }
            
            if (unrecognisedParameters.equals("")) {
                unrecognisedParametersDialogTextArea.setText("");
                unrecognisedParametersFoundLastTime=false;
                return; // If there aren't any unrecognised parameters just return.
            }
            
            unrecognisedParameters="# The following parameters were not recognised by Ellipsis GUI.\n# Please edit them yourself.\n\n"+unrecognisedParameters+"\n";
            unrecognisedParametersDialogTextArea.setText(unrecognisedParameters); // Put the unrecognised paramters inthe text area so the user can see them if they want.
            
            if (unrecognisedParametersFoundLastTime==false) { // Only warn about unrecognised parameters if they are a new occurence - so that the user doesn't constantly get warned about them again and again.
                int answer=JOptionPane.showConfirmDialog(MainWindow.this, "The input file contained one or more unrecognised parameters. Please edit them yourself using the input file editor.\nWould you like to see the unrecognised parameters now?", "Unrecognised Parameters", JOptionPane.YES_NO_OPTION);
                if (answer==JOptionPane.YES_OPTION) {
                    displayUnrecognisedParameters();
                }
            }
            unrecognisedParametersFoundLastTime=true;
        }
        
        void displayUnrecognisedParameters() {
            unrecognisedParametersDialog.show();
        }
        
        void removeErroneousParametersFromInputFile() { // Removes all the parameters that occur in linesWithErrors from inputFile.
            int index=0; // Start looking from the beginning of linesWithErrors.
            while (true) {
                index=findStartOfNextParameter(index, linesWithErrors); // Looks in linesWithErrors for the location of the first parameter name at or after index.
                if (index==-1) // Stop looking if the function signals that there are no more parameters in the input file.
                    break;
                
                String parameterName=parseParameterName(index, linesWithErrors); // Gets the parameter name from lines with Errors.
                
                removeAllOccurences(parameterName, 0); // Remove the entire lines containing any occurences of parameterName from inputFile. (Start looking at 0, the beginning of the file.)
                
                ++index; // Move on, so we don't just find the same parameter next time.
            }
        }
                        
        void makeEditsToStoredInputFile() { // Here, we use the parameter settings to edit an input file we loaded or got from the text area before. This ensures that the structure of that text area is maintained.
//            System.out.println("Making edits to stored input file");
            String parameters=getParameterSettings(); // First get an up to date record of all the parameters set by the GUI, in ellipsis format.
            inputFile=inputFileAsLoadedOrEnteredInTextArea; // And get the input file as it was last loaded, ready to be edited.

            int indexInInputFile=0; // Start searching from the beginning of inputFile
            indexInInputFile=findStartOfNextParameter(indexInInputFile, inputFile); // Move forwards to point at the first parameter.
            while (indexInInputFile!=-1) { // Keep going as long as we can find parameters.
//                System.out.println("Index in input file ="+indexInInputFile);
                String parameterName=parseParameterName(indexInInputFile, inputFile); // Find the name of the parameter.
                if (parameterIsDefined(parameterName, parameters)) {  // See if its in parameters.
//                    System.out.println(parameterName+" is being replaced");
                    parameters=replaceSettingAndDelete(parameterName, indexInInputFile, parameters); // Replace the version of the parameter in inputFile with the one in parameters, and delete any extra occurences of the parameter. This changes input file, so be careful about indices to that file (it won't change anything forward of indexInInputFile). Also delete the old version from parameters, so we won't add it again as though it were a new parameter. Parameters is returned from this function, so that it wil be updated with the appropriate parameter removed.
                    indexInInputFile=findStartOfNextParameter(indexInInputFile+1, inputFile); // Start looking for the next parameter from one point further on.   
                } else if (parameterIsDefined(parameterName, unrecognisedParameters)) {
//                    System.out.println(parameterName+" is unrecognised");
                    // If it's an unrecognised parameter, just do nothing with it - its not the business of the GUI to edit it.
                    indexInInputFile=findStartOfNextParameter(indexInInputFile+1, inputFile); // Start looking for the next parameter from one point further on.   
                } else { // If it's not been created as part of parameters, and it's not an unrecognised parameter, then it must be an optional parameter that isn't defined just now, so delete it.
//                    System.out.println(parameterName+" is being deleted");
                    removeLine(indexInInputFile); 
                    indexInInputFile=findStartOfNextParameter(indexInInputFile, inputFile); // N.B. it's very important that after removing a line we don't move on indexInInputFile before looking for the next parameter, as it could now start from the same location as teh old one started at (which will be the case if they are on subsequent lines).
                }
            }

            inputFile+=removeAllCommentsAndBlankLines(parameters); //  Now add anything left over in input file - these are parameters known by the GUI, but which werne't defined in the original inputFile. -- but we need to remove the comments, or else we'll just get all the left over comments being added. N.B. There can't be any ellipsis GUI parameters that aren't recognised by the GUI, so we don't have to worry about not deleting them.
            
            inputFile=getErroneousLines()+inputFile; // Put the erroneous lines we stored when reading in the file at the start of input file. (Together with a message explaining that they are erroneous.)
        }

        String replaceSettingAndDelete(String parameterName, int indexInInputFile, String parameters) { // Replace the value assigned to parameterName in inputFile with the one it is assigned in parameters.
            replaceSetting(parameterName, indexInInputFile, parameters); // Do the replacement here.
            
            int indexOfSubsequentOccurenceOfParameter=findParameter(parameterName, indexInInputFile+1); // Now we need to remove any subsequent occurences of parameterNameFrom inputFile -- starting looking from just after the original location of the parameter.
            while (indexOfSubsequentOccurenceOfParameter!=-1) { // Keep going as long as we can find subsequent occurence of the parameter.
                removeLine(indexOfSubsequentOccurenceOfParameter);
                indexOfSubsequentOccurenceOfParameter=findParameter(parameterName, indexOfSubsequentOccurenceOfParameter); // N.B. We don't move on the index before looking for the next parameter, because we've just deleted a line, which will bring the rest of the file forward, possibly bringing a new occurence of the parameter to be exactly at the old index. This can't cause an infinite loop, becauae remove line must always delete at least one character.
            }
            
            parameters=removeLine(findParameter(parameterName, 0, parameters), parameters)[0]; // And remove the single occurence of the parameter from parameters; (the function retruns an array of two string, the first of which will be the new version of parameters).
            
            return parameters;
        }

        void replaceSetting(String parameterName, int indexInInputFile, String parameters) { // Actually do the parameter swap, replacing the parameter enty in input file with that in parameters.
            int indexOfEndOfSettingInInputFile=indexOfEndOfSetting(indexInInputFile, parameterName, inputFile); // This returns an index to the first character after the end of the setting.
            int indexOfStartOfParameterInParameters=findParameter(parameterName, 0, parameters); // Find the parameter name in parameters (there must be exactly one, as parameters will never contain duplicate parameters).
            int indexOfEndOfSettingInParameters=indexOfEndOfSetting(indexOfStartOfParameterInParameters, parameterName, parameters);
            
//            System.out.println("Replacing "+indexInInputFile+" to "+indexOfEndOfSettingInInputFile+" with "+indexOfStartOfParameterInParameters+" to "+indexOfEndOfSettingInParameters);
            inputFile=inputFile.substring(0, indexInInputFile)+parameters.substring(indexOfStartOfParameterInParameters, indexOfEndOfSettingInParameters)+inputFile.substring(indexOfEndOfSettingInInputFile); // Actually make the swap.
        }
        
        int indexOfEndOfSetting(int indexOfStartOfParameterName, String parameterName, String string) { // Find the end of the definition of the parameter, or whatever will be taken to be that. N.B. indexOfEndOfSettingMust be robust enought that it is able to cope with syntax errors in parameter definitions. And it must also cope with Ellisps GUI parameters.
            int index=indexOfStartOfParameterName;
            
            if (string.charAt(indexOfStartOfParameterName)=='#') { // If this is an ellispsi GUI Parameter.
                index+=ellipsisGUIParameterPrefix.length(); // Move forward to point to the start of the parameter definition proper.
            }

            index=skipToCommentOrEndOfLine(index, string); // To find the end of the parameter, search forwards to the end of the line, or the start of a comment, then search backwards to the end of the parameter definition. This method is good in that it will not cause any problems when there are errors in the parameter.
            index=skipWhiteSpaceBackwards(index, string);
            
            return index+1; // We want to return the first character after the end of the string.
        }

        int skipWhiteSpaceBackwards(int index, String string) { // Skip backwards, and return the location of the first non-white-space character (may be the end of a previous line).
            --index;
            while (index>=0) { // Don't go past the beginning of the string.
                if (string.charAt(index)!='\t' && string.charAt(index)!=' ') { // If we've found a non-white space character.
                    return index;
                }
                --index;
            }
            
            return index; // If we get here, we return the location before the start of the string (-1).
        }
        
        int skipToCommentOrEndOfLine(int index, String string) { // This routine must skip over any quoted strings # can be in a string but \n cannot be - if it is it's an error, and it will terminate the string.
            while (index<string.length()) { // We need to stop if we've passed the end of the string.
                if (string.charAt(index)=='\'' || string.charAt(index)=='\"') {
                    index=skipQuotedString(index, string);
                } else if (string.charAt(index)=='\n' || string.charAt(index)=='#') {
                    return index;
                } else {
                    ++index; // If we've not found the end of the string, move on to the next character.
                }
            }
            
            return index;// If we get here, its because we've passed the end of the string.
        }
                
        int skipQuotedString(int index, String stringToSearch) {
            char quoteSymbol=stringToSearch.charAt(index); // Record whether the string is quoted by ' or "
            ++index; // And start searching from the character after the opening quote.
            
            while (index<stringToSearch.length()) {
                if (stringToSearch.charAt(index)==quoteSymbol) { // If we've found the end of the string, return the location of the first character after it.
                    return index+1;
                } if (stringToSearch.charAt(index)=='\n') { // But if we've found the end of the line (in which case there must be an error somewhere), return the location of the new line character.
                    return index;
                }
                ++index;
            }
            return index; // If we get here, we've reached the end of the string.
        }
        
            /*            int indexOfEndOfParameterName=indexOfStartOfParameterName+=parameterName.length();
            
            index=skipWhiteSpace(index, string);
            if (string.charAt(index)!='=') { // If we've not got to an equals sign, this is a syntax error, so just return the index of the start of the comment, or the end of the line.
                return skipToCommentOrEndOfLine(indexOfEndOfParametername, string);
            }
            
            index=skipWhite(index+1, string); // Now skip over any white space immediately after the equals sign. (While ellipsis doesn't allow such white space to exist, the GUI does.)

            if (string.charAt(index)=='\'' || string.charAt(index)=='\"') { // If we've found a quoted string, skip it.
                index=skipString(index);
                
        }
  */
        int findStartOfNextParameter(int index, String string) { // Find the location of the next parameter in string, starting at the loaction pointed to by index. This could be an ellipsis gui parameter.
            while (index<string.length()) { // While we haven't passed the end of the file.
                if (thisIsTheLocationOfTheFirstItemOnLine(index, string)) { // If we've potentially found the start of a new parameter.
                    index=skipWhiteSpace(index, string); // Move forward to the parameter's start.
                    if (string.charAt(index)!='\n' && string.charAt(index)!='=' && string.charAt(index)!='#') { // Check that this is the real start of a parameter.
                        return index;
                    } else if (string.length()>=(index+ellipsisGUIParameterPrefix.length()) && string.substring(index, index+ellipsisGUIParameterPrefix.length()).equals(ellipsisGUIParameterPrefix)) { // If it's not, check if it's the start of an ellipsis GUI parameter.
                       // System.out.println("Ellipsis GUI parameter:"+string.substring(index, index+ellipsisGUIParameterPrefix.length()+20)+" found at: "+index);
                        return index;
                    }
                }
                ++index; // If we've not found the start of a parameter, move index on to point at the next location.
            }
            
            return -1; // We only get here if we can't find any parameter, so signal that.
        }
        
        String deleteTheseLines(String linesToDelete, String stringToDeleteFrom) { // Delete each line in linesToDelete from stringToDeleteFrom. If a line doesn't exist in stringToDeleteFrom, just omit that line.
            while (linesToDelete.length()>0) { // Keep going as long as linesToDelete isn't empty.
                String[] newStringAndRemovedLine=removeLine(0, linesToDelete);
                linesToDelete=newStringAndRemovedLine[0]; // Update linesToDelete with the version of it with the line chopped out.
                
                stringToDeleteFrom=removeThisLine(newStringAndRemovedLine[1], stringToDeleteFrom); // And chop the line out of the other string (if it exists).
            }
            
            return stringToDeleteFrom; // Now return the version of the string with all the lines removed.
        }
        
        String removeThisLine(String line, String stringToDeleteFrom) { // Remove any and all occurences of the line from stringToDeleteFrom.
            int index=stringToDeleteFrom.indexOf(line); // Start searching from the beginning of the string.
            while (index!=-1) { // Keep going as long as we can find occurences of the line.
                if (index==0 || stringToDeleteFrom.charAt(index-1)=='\n') { // To be at the begining of a line, we must be at the start of the file or just after the end of a line.
                    stringToDeleteFrom=removeLine(index, stringToDeleteFrom)[0]; // Actually remove the line.
                    index=stringToDeleteFrom.indexOf(line, index); // Start looking from the point where the line we just removed started.
                } else {
                    index=stringToDeleteFrom.indexOf(line, index+1); // If we didn't remove a line, start lookng from a point one charactar further on, os we don't just find the same line again.
                }
            }
            
            return stringToDeleteFrom;
        }
        
        int startOfNextLine(int index) { // Move index to point to the start of the next line, if there is one, otherwise the character after the end of inputFile.
            return startOfNextLine(index, inputFile);
        }
        
        int startOfNextLine(int index, String string) {
            while (index<string.length()) {
                if (string.charAt(index)=='\n')
                    return index+1;
                ++index;
            }
            return index;
        }

        String removeAllCommentsAndBlankLines(String string) { // Remove every line which starts with the comment symbol, or which is just completely blank, from string. But be sure to leave behind any ellipsis Gui parameters.
            int index=0;
            while (index<string.length()) { // Keep going till we've passed the end of the string.
                if (index+ellipsisGUIParameterPrefix.length()<=string.length() && string.substring(index, index+ellipsisGUIParameterPrefix.length()).equals(ellipsisGUIParameterPrefix)) { // If the line starts with an ellipsis GUI parameter.
                    index=startOfNextLine(index, string);
                } else if (string.charAt(index)=='#' || string.charAt(index)=='\n') {
                    string=removeLine(index, string)[0]; // Remove line returns two arguements, we want the first one - the new state of string.
                } else {
                    index=startOfNextLine(index, string);
                }
            }
            
            return string;
        }
        
        void removeDuplicateComments() { // ******** This is redundant.
            
        }
        
        void parseAndRemoveMaterialFileSettings() {
            parseAndRemoveUniversalMaterialParameterSettings();
            parseAndRemoveEachMaterialsRheologySettings();
            backgroundImagePanel.materialRegionContainer.parseAndRemoveMaterialRegions();
        }
        
        void parseAndRemoveEachMaterialsRheologySettings() {
            int numberOfMaterials;
            numberOfNextMaterial=1;
            try {
                numberOfMaterials=parseInteger("Different_materials");
            } catch (ParameterNotFoundException exception) {
                // If no materials are found, just create a default one and return.
                createSingleMaterial();
                return;
            } catch (ParameterSyntaxErrorException exception) {
                createSingleMaterial(); // If there is a syntax error with the parameter, it will already have been added to lines with errors, so again just create a default material and return.
                return;
            }
            
            // From here on deals with case in which at least one material should be defined.
            disposeOfAllMaterialWindows();
            materials=new ArrayList();  // First get rid of all the old materials.
            Material newMaterial;
            do { // Make sure at least one material is always created.
                newMaterial=new Material();
                materials.add(newMaterial);
                updatePPMColours(); // This must be called after every time a new material has been created and added to the materials list (it can't be in a constructor, because we have to add a material to the materials list before we call it)
                newMaterial.parseAndRemoveRheologySettings();
                
                --numberOfMaterials;
            } while (numberOfMaterials>0);
            

            materialChooser.setSelectedIndex(0); // Select the first material in the combo box.
            newMaterial.updateMaterialChooserAndMaterialName(); // And update the combo box. - it doesn't matter which material this gets called through.
        }
        
        void createSingleMaterial() { // Set the materials list so there's only a single material.
            disposeOfAllMaterialWindows();
            materials=new ArrayList();
            Material onlyMaterial=new Material();
            materials.add(onlyMaterial);
            materialChooser.setSelectedIndex(0); // Select the material in the combo box.
            onlyMaterial.updateMaterialChooserAndMaterialName();
            
            updatePPMColours(); // This must be called after every time a new material has been created and added to the materials list (it can't be in a constructor, because we have to add a material to the materials list before we call it)
        }
        
        void parseAndRemoveUniversalMaterialParameterSettings() { // This is for the parameters which affext all materials
            try {
                int grainSizeModel=parseInteger("Grain_size_model");
                if (grainSizeModel==2)
                    empiricalButton.setSelected(true);
                else
                    growthNucleationButton.setSelected(true);
            } catch (ParameterNotFoundException exception) {
                // Do nothing if the parameter is not found.
            } catch (ParameterSyntaxErrorException exception) {
                // If there is a syntax error with the parameter, it will already have been added to lines with errors, so just do nothing.
            }
            
            parseAndSetBoolean("VMAX", useMaximumViscosityCheckBox);
            parseAndSetReal("visc_max", maximumViscosityTextField, scaling.getEllipsisUnits2ViscosityScaler());
            parseAndSetBoolean("VMIN", useMinimumViscosityCheckBox);
            parseAndSetReal("visc_min", minimumViscosityTextField, scaling.getEllipsisUnits2ViscosityScaler());
            
            parseAndSetBoolean("TDEPV", temperatureDependentRheologicalParametersCheckBox);
            parseAndSetBoolean("SDEPV", SDEPVCheckBox);
            parseAndSetBoolean("YIELD", YIELDCheckBox);
            parseAndSetBoolean("GRDEPV", GRDEPVCheckBox);
        }
        
        void parseAndRemoveScalingParameters() {
            boolean autoscaleIsSelected=automaticallyScaleParametersRadioButton.isSelected(); // Remember whether we're using autoscaling.
            manuallyScaleParametersRadioButton.setSelected(true); // Switch to manual scaling so we can input the values read from the file.

            parseAndSetReal(ellipsisGUIParameterPrefix+"ellipsisUnitsPerM", ellipsisUnitsPerMNumberBoxBean);
            parseAndSetReal(ellipsisGUIParameterPrefix+"ellipsisUnitsPerSecond", ellipsisUnitsPerSecondNumberBoxBean);
            parseAndSetReal(ellipsisGUIParameterPrefix+"ellipsisUnitsPerKG", ellipsisUnitsPerKGNumberBoxBean);
            parseAndSetReal(ellipsisGUIParameterPrefix+"ellipsisUnitsPerKelvin", ellipsisUnitsPerKelvinNumberBoxBean);
            
            scaling.updateScalingFactors(); // Store the new values in the scaling object.
            if (autoscaleIsSelected) { // If necessary, reselect autoscale.
                automaticallyScaleParametersRadioButton.setSelected(true);
            } else if (scaling.ellipsisUnitsPerM==1 && scaling.ellipsisUnitsPerKG==1 && scaling.ellipsisUnitsPerSecond==1 && scaling.ellipsisUnitsPerKelvin==1)
                doNotScaleParametersRadioButton.setSelected(true);
        }

        void parseAndRemoveOutputFileSettings() { 
            parseAndRemoveFileSettings();
            parseAndRemoveFrequencyOfOutputSettings();
            
            parseAndRemoveDatatypes();
            parseAndRemoveParticle_data();
            parseAndRemoveAverages();
            parseAndRemoveTimelog();
            parseAndRemoveObservables();
            
            parseAndRemovePPMFiles();
            parseAndRemoveSamplingTracers();
        }
        
        void parseAndRemoveSamplingTracers() {
            samplingTracersComboBox.removeAllItems(); // First of all, delete all the existing Sampling tracers, and remve them from the combo box.
            samplingTracers=new LinkedList();
            
            String SamplingTracersSettings=files.getAllLinesForSettingsWithThisPrefix("Sampling_"); // Store all the SamplingTracer settings, so we can report them if there is an error.
            
            int Sampling_tracers;
            try {
                try { // This code wil cause problems when there are errors, because it may leave one or two parameters in inputFile.
                    Sampling_tracers=parseInteger("Sampling_tracers"); // Try to find out the number of Sampling tracers.
                    if (Sampling_tracers==0) { // If no sampling tracers are defined, this is no problem, but we just need to set the message in the combo box to say so.
                        deleteAllSamplingTracers();
                        linesWithErrors+=SamplingTracersSettings; // And if any other samplingTracers parameters are defined, this will be an error.
                        return;
                    }
                } catch (ParameterNotFoundException ex) {
                    deleteAllSamplingTracers(); // This is to make sure the appropriate message gets put in the combo box if there aren't any sampling tracers.
                    linesWithErrors+=SamplingTracersSettings; // If we don't find the parameter, that's OK, unless another SamplingTracer parameter is defined, in which case we have an error.
                    return;
                } // Don't catch anything else - if there was a syntax error - that can just be caught using the same catches as below. (Although maybe that would work for ParameterNotFoundException too).
                
                String Sampling_lagrangian=parseAndRemoveList("Sampling_lagrangian"); // Get all the SamplingTracers parameters (each a list) checking that there are Sampling_tracers elements in each list. No - allow there to be the wrong number of sampling tracers in each list - this way the program can read craig's input files.
                String Sampling_plot_num=parseAndRemoveList("Sampling_plot_num");
                String Sampling_x=parseAndRemoveList("Sampling_x");
                String Sampling_z=parseAndRemoveList("Sampling_z");
                String Sampling_field=parseAndRemoveList("Sampling_field");
                String Sampling_dirn=parseAndRemoveList("Sampling_dirn");
                String Sampling_normalize=parseAndRemoveList("Sampling_normalize");
                String Sampling_R=parseAndRemoveList("Sampling_R");
                String Sampling_G=parseAndRemoveList("Sampling_G");
                String Sampling_B=parseAndRemoveList("Sampling_B");
                
                try { // Then add each samplingTracer one at a time.
                    for (int numberOfSamplingTracer=1; numberOfSamplingTracer<=Sampling_tracers; ++numberOfSamplingTracer) {
                        addNewSamplingTracer(Statics.nthInteger(Sampling_lagrangian, numberOfSamplingTracer), Statics.nthInteger(Sampling_plot_num, numberOfSamplingTracer), scaling.ellipsisUnits2KM(Statics.nthReal(Sampling_x, numberOfSamplingTracer)), scaling.ellipsisUnits2KM(Statics.nthReal(Sampling_z, numberOfSamplingTracer)), Statics.nthInteger(Sampling_field, numberOfSamplingTracer), Statics.nthInteger(Sampling_dirn, numberOfSamplingTracer), Statics.nthInteger(Sampling_normalize, numberOfSamplingTracer), Statics.nthReal(Sampling_R, numberOfSamplingTracer), Statics.nthReal(Sampling_G, numberOfSamplingTracer), Statics.nthReal(Sampling_B, numberOfSamplingTracer));
                    }
                } catch (ParameterSyntaxErrorException exception) { // If there's a problem with one of the lists (e.g. one of the elements isn't an integer or real, we'll get this exception.
                    linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                    throw exception; // And then re-throw the exception
                }
                
            } catch (ParameterNotFoundException exception) { // If any of the parameters are missing, or there is a syntax error with one, make sure there are no SamplingTracers defined, and the message saying this is put in the combo box (required by the GUI). And put all the SamplingTracer settings down as errors. (This ensures that they are eventually removed from input file.)
                deleteAllSamplingTracers();
                linesWithErrors+=SamplingTracersSettings;
            } catch (ParameterSyntaxErrorException exception) {
                deleteAllSamplingTracers();
                linesWithErrors=removeLastLineFromString(linesWithErrors)+SamplingTracersSettings; // The parameter which caused the error will already have been added to lines with errors, so now we need to take it off before adding on all the SamplingTracers settings.
            } catch (ParameterIncompatibilityException exception) {
                deleteAllSamplingTracers();
                linesWithErrors+=SamplingTracersSettings;
            }
        }
        
        void deleteAllSamplingTracers() {
            samplingTracersComboBox.removeAllItems(); 
            samplingTracers=new LinkedList();
            samplingTracersComboBox.addItem("No tracers defined");
        }
            
        void parseAndRemovePPMFiles() {
            PPMFilesComboBox.removeAllItems(); // First take all the items out of the combo box.
            PPMFiles=new LinkedList(); // And start a new list to hold the new PPMFiles().

            String PPMSettings=files.getAllLinesForSettingsWithThisPrefix("PPM_files"); // Store all the PPM settings in case there is an error.
            PPMSettings+=files.getAllLinesForSettingsWithThisPrefix("PPM_height");
            PPMSettings+=files.getAllLinesForSettingsWithThisPrefix("PPM_coloring");

            try { // This code wil cause problems when there are errors, because it may leave one or two parameters in inputFile. I think this is incorrect, because if a parameter is added to lines with errors, it gets removed no problem. And this code seems to work just fine.
                int PPM_files=parseInteger("PPM_files"); // Try to find out the number of PPM files.
                String PPM_height=parseAndRemoveList("PPM_height", PPM_files); // And all the other PPM parameters (we should have PPM_files items in each list.
                String PPM_coloring=parseAndRemoveList("PPM_coloring", PPM_files);

                try {
                    for (int numberOfPPMFile=1; numberOfPPMFile<=PPM_files; ++numberOfPPMFile) {
                        addNewPPMFile(Statics.nthInteger(PPM_height, numberOfPPMFile), Statics.nthInteger(PPM_coloring, numberOfPPMFile));
                    }
                } catch (ParameterSyntaxErrorException exception) {
                    linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                    throw exception; // And then re-throw the exception
                }
                
            } catch (ParameterNotFoundException exception) { // If any of the parameters are missing, or there is a syntax error with one, just create a single PPM File (required by the GUI). But put all the PPM settings down as errors. (This ensures that they are eventually removed from input file.)
                createSinglePPMFile();
                linesWithErrors+=PPMSettings;
            } catch (ParameterSyntaxErrorException exception) {
                linesWithErrors=removeLastLineFromString(linesWithErrors)+PPMSettings; // The parameter which caused the error will already have been added to lines with errors, so now we need to take it off before adding on all the PPM files.
                createSinglePPMFile(); 
            } catch (ParameterIncompatibilityException exception) {
                createSinglePPMFile();
                linesWithErrors+=PPMSettings;
            }
        }

        void createSinglePPMFile() {
            PPMFilesComboBox.removeAllItems(); // First take all the items out of the combo box.
            PPMFiles=new LinkedList(); // And start a new list to hold the new PPMFiles().
            addNewPPMFile(256, 1); // create a ppm file with default settings.
        }
            
        void parseAndRemoveParticle_data() { // N.B. None of these functions do anything about invalid settings, they just ignore them.
            String particle_data=parseAndRemoveListCatchingExceptions("particle_data");
            setCheckBoxIfPresent(particle_data, "Temp", particleDataTempCheckBox);
            setCheckBoxIfPresent(particle_data, "Pres", particleDataPresCheckBox);
            setCheckBoxIfPresent(particle_data, "Visc", ViscCheckBox);
            setCheckBoxIfPresent(particle_data, "Edot", EdotCheckBox);
            setCheckBoxIfPresent(particle_data, "StrP", StrPCheckBox);
            setCheckBoxIfPresent(particle_data, "StrT", StrTCheckBox);
            setCheckBoxIfPresent(particle_data, "StrD", StrDCheckBox);
            setCheckBoxIfPresent(particle_data, "StrS", StrSCheckBox);
            setCheckBoxIfPresent(particle_data, "Grsz", GrszCheckBox);
        }
        
        void parseAndRemoveAverages() {
            String averages=parseAndRemoveListCatchingExceptions("averages");
            setCheckBoxIfPresent(averages, "Temp", averagesTempCheckBox);
            setCheckBoxIfPresent(averages, "Visc", averagesViscCheckBox);
            setCheckBoxIfPresent(averages, "Velo", VeloCheckBox);
            setCheckBoxIfPresent(averages, "Urms", UrmsCheckBox);
            setCheckBoxIfPresent(averages, "Wrms", WrmsCheckBox);
        }
        
        void parseAndRemoveTimelog() {
            String timelog=parseAndRemoveListCatchingExceptions("timelog");
            setCheckBoxIfPresent(timelog, "Nuss", NussCheckBox);
            setCheckBoxIfPresent(timelog, "Vrms", VrmsCheckBox);
            setCheckBoxIfPresent(timelog, "Vxrm", VxrmCheckBox);
            setCheckBoxIfPresent(timelog, "Vzrm", VzrmCheckBox);
            setCheckBoxIfPresent(timelog, "Shfl", ShflCheckBox);
            setCheckBoxIfPresent(timelog, "Bhfl", BhflCheckBox);
            setCheckBoxIfPresent(timelog, "Stmx", StmxCheckBox);
            setCheckBoxIfPresent(timelog, "Svav", SvavCheckBox);
            setCheckBoxIfPresent(timelog, "Bvav", BvavCheckBox);
        }
        
        void parseAndRemoveObservables() {
            String observables=parseAndRemoveListCatchingExceptions("observables");
            setCheckBoxIfPresent(observables, "Tpgx", TpgxCheckBox);
            setCheckBoxIfPresent(observables, "Tpgk", TpgkCheckBox);
            setCheckBoxIfPresent(observables, "Tpbx", TpbxCheckBox);
            setCheckBoxIfPresent(observables, "Tpbk", TpbkCheckBox);
            setCheckBoxIfPresent(observables, "Grvx", GrvxCheckBox);
            setCheckBoxIfPresent(observables, "Grbx", GrbxCheckBox);
            setCheckBoxIfPresent(observables, "Grtx", GrtxCheckBox);
            setCheckBoxIfPresent(observables, "Grvk", GrvkCheckBox);
            setCheckBoxIfPresent(observables, "Grbk", GrbkCheckBox);
            setCheckBoxIfPresent(observables, "Grtk", GrtkCheckBox);
            setCheckBoxIfPresent(observables, "Geox", GeoxCheckBox);
            setCheckBoxIfPresent(observables, "Gebx", GebxCheckBox);
            setCheckBoxIfPresent(observables, "Getx", GetxCheckBox);
            setCheckBoxIfPresent(observables, "Geok", GeokCheckBox);
            setCheckBoxIfPresent(observables, "Gebk", GebkCheckBox);
            setCheckBoxIfPresent(observables, "Getk", GetkCheckBox);
            setCheckBoxIfPresent(observables, "Shfl", observablesShflCheckBox);
            setCheckBoxIfPresent(observables, "Bhfl", observablesBhflCheckBox);
            setCheckBoxIfPresent(observables, "Vxsf", VxsfCheckBox);
            setCheckBoxIfPresent(observables, "Vzsf", VzsfCheckBox);
        }
        
        void parseAndRemoveDatatypes() {
            String datatypes=parseAndRemoveListCatchingExceptions("datatypes"); // Get the list of settings for datatypes.
            setCheckBoxIfPresent(datatypes, "Velx", VelxCheckBox);
            setCheckBoxIfPresent(datatypes, "Velz", VelzCheckBox);
            setCheckBoxIfPresent(datatypes, "Pres", PresCheckBox);
            setCheckBoxIfPresent(datatypes, "Temp", TempCheckBox);
            setCheckBoxIfPresent(datatypes, "Strf", StrfCheckBox);
            setCheckBoxIfPresent(datatypes, "Pbdy", PbdyCheckBox);
            setCheckBoxIfPresent(datatypes, "Pstn", PstnCheckBox);
        }
        
        void setCheckBoxIfPresent(String list, String keyword, JCheckBox checkBox) { // If string string contains keyword as an item in its list set JCheckBox, otherwise reset it.
            checkBox.setSelected(contains(list, keyword));
        }

        boolean contains(String list, String keyword) { // Return true iff list contains keyword.
            try {
                int itemToCheck=1;
                while (true) { // Go through the list one item at a time, and each time check whether the item found matches the keyword.
                    if (Statics.nthItem(list, itemToCheck).equals(keyword)) {
                        return true; // If We've found the item, return true.
                    }
                    ++itemToCheck; // Otherwise look at the next item in the list
                }
            } catch (ParameterIncompatibilityException exception) { // We get here if we've gone past the end of the list, and hence it doesn't contain keyword - so return false.
                return false;
            }
        }
        
        void parseAndRemoveFileSettings() {
            parseAndSetString("datafile", datafileTextField);
            parseAndSetBoolean("COMPRESS", COMPRESSCheckBox);
            parseAndSetString("gzip", gzipTextField);
        }
        
        String parseAndRemoveListCatchingExceptions(String parameterName) {
            try {
                return parseAndRemoveList(parameterName);
            } catch (ParameterSyntaxErrorException exception) {
//                linesWithErrors+=parameterName+"=\n"; // Because of the way the program is written, we've lost the actual parameter value, but can put in the value itself.
                return "";
            } catch (ParameterNotFoundException exception) { // This is no problem, just return an empty string.
                return "";
            }
        }
        
        String parseAndRemoveListCatchingExceptions(String parameterName, int numberOfItems) {
            try {
                return parseAndRemoveList(parameterName, numberOfItems);
            } catch (ParameterSyntaxErrorException exception) {
//                linesWithErrors+=parameterName+"=\n"; // Because of the way the program is written, we've lost the actual parameter value, but can put in the value itself.
                return "";
            } catch (ParameterNotFoundException exception) { // This is no problem, just return an empty string.
                return "";
            } catch (ParameterIncompatibilityException exception) {
                linesWithErrors+=parameterName+"\n"; // We've lost the line itself, but this will show that there was an error with this parameter.
                return "";
            }
        }
        
        void parseAndSetString(String parameterName, JTextField parameterTextField) { // Iff the parameter exists, parse it, remove it from the file, and put its value in the text field. Cope with exceptions in the process.
            try {
                String parameterValue=parseAndRemoveList(parameterName, 1); // Parse string by treating them as a list of strings with only one item.
                parameterTextField.setText(parameterValue);
            } catch (ParameterSyntaxErrorException exception) {
//                linesWithErrors+=parameterName+"=\n"; // Because of the way the program is written, we've lost the actual parameter value, but can put in the value itself.
            } catch (ParameterNotFoundException exception) { // This is no problem, just do nothing.
            } catch (ParameterIncompatibilityException exception) {
//                linesWithErrors+=parameterName+"=\n"; // Because of the way the program is written, we've lost the actual parameter value, but can put in the value itself.
            }
        }
        
        void parseAndRemoveFrequencyOfOutputSettings() {
            parseAndSetInteger("storage_timesteps",  storage_timestepsTextField);
            parseAndSetInteger("checkpt_timesteps", checkpt_timesteps);
        }
            
        void parseAndRemoveInitialConditionFileSettings() {
            parseAndSetReal("gravacc", gravityNumberBoxBean, scaling.getEllipsisUnits2AccelerationScaler()); // **** Scaling required - although Craig says its best to leave this unscaled I don't understand how it can be **** SO SCALING HAS NOW BEEN IMPLEMENTED
            parseAndSetReal("gravtheta", gravityAngleNumberBoxBean); // No scaling required (dimensionless)
            
            backgroundImagePanel.tracerRegionContainer.parseAndRemoveTracerRegions();
        }
        
        void parseAndRemoveThermalZoneFileSettings() {
            backgroundImagePanel.thermalRegionContainer.parseAndRemoveThermalRegions();
        }
        
        void parseAndRemoveBoundaryFileSettings() {
            parseAndSetBoolean("periodicx", periodicxCheckBox);
            parseAndSetBoolean("periodic_rm_vx", periodic_rm_vxCheckBox);
            
/*  This section has been commented out, because the version of ellipsis I have does not even seem to recognise piecwise boundary movement parameters - hence there seems to be little point in implementing them.
 boolean BCmove_with_time_interval=parseBoolean("BCmove_with_time_interval");
            BCmove_with_time_intervalCheckBox.setSelected(BCmove_with_time_interval);
            if (BCmove_with_time_interval) { // Set the radio button choosing which kind of boundary condition mostion to use, based on the setting of this parameter.
                simpleBoundaryRadioButton.setSelected(true);
            } else {
                piecewiseBoundaryRadioButton.setSelected(true);
            }
  */          
            parseAndSetReal("BCmoveX0v", BCmoveX0vTextField, scaling.getEllipsisUnits2VelocityScaler()); // N.B. Search for both these parameters (for simple boundary condition motion), and the ones for more complex boundary condition motion. Problem --> setting the radio button in the GUI. **********
            parseAndSetReal("BCmoveX1v", BCmoveX1vTextField, scaling.getEllipsisUnits2VelocityScaler());
            parseAndSetReal("BCmoveX0v1", BCmoveX0v1TextField, scaling.getEllipsisUnits2VelocityScaler()); // *** Is scaling correct?
        
            backgroundImagePanel.boundaryRegionContainer.parseAndRemoveBoundaryRegions(); // This must be a member of this class, not of the container, so that all the parsing functions can be used. --> or else it can call back to here? The function must start by getting rid of all existing boundaries. --> make new boundary region container.
            
        }
        
        void parseAndRemoveGeneralFileSettings() {
            parseAndSetBoolean("DESCRIBE", describeCheckBox);
            parseAndSetBoolean("VERBOSE", verboseInputValuesCheckBox);
            parseAndSetBoolean("BEGINNER", beginnerCheckBox); // We always do this, even though the beginner parameter is only applicable if verbose is set. However, making the check seems to give little benefit.
            parseAndSetBoolean("verbose", verboseCodeBehaviourCheckBox);
            
            parseStringAndConfirm("Geometry", "cart2d"); // Check that if there is a setting for Geometry, it is cart2d

            parseAndSetReal("dimenx", windowDimensionXTextField, scaling.getEllipsisUnits2KMScaler());
            updateWindowDimensionXTextField(); // Call this, so that the variable storing the dimenx value gets updated if necessary. This parameter is exceptional in that it is one of the very few whoose values are stored in a separate variable.
            parseAndSetReal("dimenz", windowDimensionYTextField, scaling.getEllipsisUnits2KMScaler());
            updateWindowDimensionYTextField();
            
            parseAndSetInteger("mgunitx", mgunitxNumberBoxBean);
            parseAndSetInteger("mgunitz", mgunitzNumberBoxBean);
            parseAndSetInteger("levels", levelsNumberBoxBean);
            parseAndSetInteger("pmg_levels", pmg_levelsNumberBoxBean);
            
            parseAndSetReal("maxadvtime", maxadvtimeNumberBoxBean, scaling.getEllipsisUnits2SScaler());
            parseAndSetInteger("minstep", minstepNumberBoxBean);
            parseAndSetInteger("maxstep", maxstepNumberBoxBean);
            
            parseAndSetString("Solver", SolverTextField);
        }
        
        void parseStringAndConfirm(String parameterName, String expectedParameterValue) {
            try {
                String actualParameterValue=parseString(parameterName); // Check that the parameter is set correctly, and remove it from the input file. If it is set correctly, we don't have to do anyting, otherwise we just have to give a warning that it will be changed.
                
                if ( ! actualParameterValue.equals(expectedParameterValue)) { // If the value the parameter is set to doesn't match the expected one.
                    JOptionPane.showMessageDialog(MainWindow.this, "The parameter "+parameterName+" must be set to \n\""+expectedParameterValue+"\", but it was set to \""+actualParameterValue+"\".\n This parameter has been reset.", "Warning", JOptionPane.WARNING_MESSAGE);
                }
            } catch (ParameterNotFoundException exception) {
//                System.out.println("Parameter not found: "+parameterName+"\n"); // Debugging code only
                // If the parameter isn't found, no problem, just do nothing - it will keep its default or previous setting.
            } catch (ParameterSyntaxErrorException exception) { 
//                System.out.println("Parameter syntax error: "+parameterName+"\n");
                //                MessageBoxes.warnAboutParameterSyntaxError(MainWindow.this, parameterName); // No  - don't warn about it here - all the errors are collected, so they can be reported at the end if necessary.
            }
        }
        
        void parseAndSetReal(String parameterName, JTextField textField) { // Overloaded version of function for parameters that don't need scaling.
            parseAndSetReal(parameterName, textField, new UnitScaler() { // Creates an anonymous inner class, whose scale method just returns the same value as it is given.
                public double scale(double value) {
                    return value;
                }
            });
        }
        
        void parseAndSetInteger(String parameterName, JTextField textField) {
            try {
                int parameterValue=parseInteger(parameterName);
                textField.setText(""+parameterValue);
            } catch (ParameterNotFoundException exception) {
                // If the parameter isn't found, no problem, just do nothing - it will keep its default or previous setting.
            } catch (ParameterSyntaxErrorException exception) {
                // If there is a syntax error with the parameter, it will already have been added to lines with errors, so just do nothing.
            }
        }
        
        void parseAndSetReal(String parameterName, JTextField textField, UnitScaler scaler) {
            try {
                double parameterValue=scaler.scale(parseReal(parameterName)); // Find the parameter, remove the appropriate line from the input line (checking for multiple occurences etc.), get its value, and scale it.
                textField.setText(Useful.double2String(parameterValue));
            } catch (ParameterNotFoundException exception) {
//                System.out.println("Parameter not found: "+parameterName+"\n"); // Debugging code only
                // If the parameter isn't found, no problem, just do nothing - it will keep its default or previous setting.
            } catch (ParameterSyntaxErrorException exception) { // If there is a syntax error with the parameter, it will already have been added to lines with errors, so just do nothing.
//                System.out.println("Parameter syntax error: "+parameterName+"\n");
            }
        }

        void parseAndSetBoolean(String parameterName, JCheckBox checkBox) { // Search in input file for the parameter. If it exists set the checkbox to selected (if setting is 1/on) or to deselected (if setting is 0/off). Remove the parameter from the input file.
            try {
                boolean onOrOff=parseBoolean(parameterName); // This not only finds the setting for parameterName, it also removes that line from input file.
                checkBox.setSelected(onOrOff);

//                System.out.println("Setting parameter: "+parameterName+" to: "+onOrOff+"\n"); // Debugging code only.
            } catch (ParameterNotFoundException exception) {
//                System.out.println("Parameter not found: "+parameterName+"\n"); // Debugging code only
                // If the parameter isn't found, no problem, just do nothing - it will keep its default or previous setting.
            } catch (ParameterSyntaxErrorException exception) { // If there is a syntax error with the parameter, just warn about it, and then do nothing. 
//                System.out.println("Parameter syntax error: "+parameterName+"\n");
//                MessageBoxes.warnAboutParameterSyntaxError(MainWindow.this, parameterName); // No  - don't warn about it here - all the errors are collected, so they can be reported at the end if necessary.
            }
//            } catch (DuplicateParameterException exception) { // This is no good - because the parameter still needs to be set.
//                MessageBoxes.warnAboutDuplicateParameter(MainWindow.this, parameterName);
//            }
        }

        String parseAndRemoveList(String parameterName, int numberOfItems) throws ParameterSyntaxErrorException, ParameterNotFoundException, ParameterIncompatibilityException {
            String list=parseAndRemoveList(parameterName);
            verifyThisManyItems(list, numberOfItems); // Throws an exception if there aren't the correct number of items in the list.
            
            return list;
        }
        
        void verifyThisManyItems(String list, int numberOfItems) throws ParameterIncompatibilityException {
            Statics.nthItem(list, numberOfItems); // If there aren't enough items, this will throw a parameter incompaibility exception.
            
            try {
                Statics.nthItem(list, numberOfItems+1); // If an item exists after what should be the last item, we have an error.
            } catch (ParameterIncompatibilityException exception) {
                return; // But if there is no extra item, and a parameter incompatibility exception is thrown, just return, having caught the exception.
            }
            throw new ParameterIncompatibilityException();
        }

        String parseAndRemoveList(String parameterName) throws ParameterSyntaxErrorException, ParameterNotFoundException {
            int settingIndex=findStartOfValue(parameterName);
            String list=parseStringValueAt(settingIndex);
            
            return list;
        }
        
        char checkThatCharacterIsXorYorZ(char character) throws ParameterSyntaxErrorException {
            if (character=='x' || character=='X')
                return 'X';
            if (character=='z' || character=='Z')
                return 'Z';
            if (character=='y' || character=='Y')
                return 'Y';
            
            throw new ParameterSyntaxErrorException();
        }
        
        String parseAndRemoveEllipsisGUIList(String parameterName, int numberOfItems) throws ParameterSyntaxErrorException, ParameterNotFoundException, ParameterIncompatibilityException {
            return parseAndRemoveList(ellipsisGUIParameterPrefix+parameterName, numberOfItems);
        }
        
        String parseAndRemoveEllipsisGUIList(String parameterName) throws ParameterSyntaxErrorException, ParameterNotFoundException { // Same as the previous routine - but doesn't check that the list contains a particular number of items.
            return parseAndRemoveList(ellipsisGUIParameterPrefix+parameterName);
        }
        
        String parseString(String parameterName) throws ParameterSyntaxErrorException, ParameterNotFoundException {
            int settingIndex=findStartOfValue(parameterName); // Return the index of the first character after the equals sign.
            String parameterValue=parseStringValueAt(settingIndex); // Just get the value for this string, removing quotes if necessary, and remove the line containing it.
            
            return parameterValue;
        }
        
        int parseInteger(String parameterName) throws ParameterSyntaxErrorException, ParameterNotFoundException {
            int settingIndex=findStartOfValue(parameterName);
            int parameterValue=parseIntegerValueAt(settingIndex);
            
            return parameterValue;
        }
        
        double parseReal(String parameterName) throws ParameterSyntaxErrorException, ParameterNotFoundException {
            int settingIndex=findStartOfValue(parameterName); // Return the index of the first character after the equals sign.
            double parameterValue=parseRealValueAt(settingIndex); // Just get the value for this real, and remove the line containing it.
            
            return parameterValue;
        }
        
        boolean parseBoolean(String parameterName) throws ParameterSyntaxErrorException, ParameterNotFoundException {
//            System.out.println("1");
            int settingIndex=findStartOfValue(parameterName); // Return the index of the first character after the equals sign.
//            System.out.println("1");
            boolean onOrOff=parseBooleanValueAt(settingIndex); // Just get the value for this boolean and remove the line containing it.
//            System.out.println("1");

            return onOrOff;
        }
        
        String parseStringValueAt(int indexOfStartOfValue) throws ParameterSyntaxErrorException {
            if (inputFile.charAt(indexOfStartOfValue)=='\"') // Work out whether the string is quoted, and if so what kind of quotation marks it uses, and parse it appropriately.
                return parseQuotedStringValueAt(indexOfStartOfValue, '\"');
            
            if (inputFile.charAt(indexOfStartOfValue)=='\'')
                return parseQuotedStringValueAt(indexOfStartOfValue, '\'');
            
            return parseUnquotedStringValueAt(indexOfStartOfValue);
        }

        String parseUnquotedStringValueAt(int indexOfStartOfValue) throws ParameterSyntaxErrorException {
            String value=parseWord(indexOfStartOfValue);
            if (value.equals("")) { // Blank unquoted strings are regarded as syntax errors.
                linesWithErrors+=removeLine(indexOfStartOfValue);
                throw new ParameterSyntaxErrorException();
            }
            
            checkThatEverythingElseOnLineIsWhiteSpaceOrComment(skipWord(indexOfStartOfValue));
            
            removeLine(indexOfStartOfValue);
            return value;
        }

        String parseQuotedStringValueAt(int indexOfStartOfValue, char quoteSymbol) throws ParameterSyntaxErrorException {
            ++indexOfStartOfValue; // Move the index forward, so that we don't include the opening quote mark.
            String value="";
            
            while (true) { // Keep searching until we find either the matching quote, or the end of the line (in which case we've got an error).
                if (inputFile.charAt(indexOfStartOfValue)==quoteSymbol)
                    break;
                    
                if (inputFile.charAt(indexOfStartOfValue)=='\n') {
                    linesWithErrors+=removeLine(indexOfStartOfValue);
                    throw new ParameterSyntaxErrorException();
                }
         
                value+=inputFile.charAt(indexOfStartOfValue);
                ++indexOfStartOfValue;
            }
            
            checkThatEverythingElseOnLineIsWhiteSpaceOrComment(indexOfStartOfValue+1); // There should really be a lot more calls to this to really check the syntax properly.
            
            removeLine(indexOfStartOfValue);
            return value;
        }
           
        void checkThatEverythingElseOnLineIsWhiteSpaceOrComment(int index) throws ParameterSyntaxErrorException { // Throws ParameterSyntaxErrorException if everything from index to the end of the line isn't just blank space or a comment.
            if (everythingElseOnLineIsWhiteSpaceOrComment(index))
                return;
            
            linesWithErrors+=removeLine(index);
            throw new ParameterSyntaxErrorException();
        }
        
        boolean everythingElseOnLineIsWhiteSpaceOrComment(int index) { // returns false if everything from index to the end of the line isn't just blank space or a comment, else true.
            index=skipWhiteSpace(index);
            if (index==inputFile.length()) // If we've reached the end of the file that's OK.
                return true;
            
            if (inputFile.charAt(index)=='#' || inputFile.charAt(index)=='\n') // Or if we've found the start of a comment or the end of the line, that's good too.
                return true;
            
            return false; // If we haven't found a comment, end of line or end of file, something is on the line.
        }
            
        
        double parseRealValueAt(int indexOfStartOfValue) throws ParameterSyntaxErrorException {
            String value=parseWord(indexOfStartOfValue);
            double valueInDoubleFormat;
            
            try {
                valueInDoubleFormat=Double.parseDouble(value);
            } catch (NumberFormatException exception) { // If the parameter is set to something other than a valid double.
                linesWithErrors+=removeLine(indexOfStartOfValue); // Get the line with the error in it, remove it from input file, and store it in lines with errors in them.
                throw new ParameterSyntaxErrorException();
            }
            
            checkThatEverythingElseOnLineIsWhiteSpaceOrComment(skipWord(indexOfStartOfValue)); // Check that there's nothing except a comment on the line after the value.
            
            removeLine(indexOfStartOfValue);
            return valueInDoubleFormat;
        }
        
        int parseIntegerValueAt(int indexOfStartOfValue) throws ParameterSyntaxErrorException {
            String value=parseWord(indexOfStartOfValue);
            int valueInIntegerFormat;
            
            try {
                valueInIntegerFormat=Integer.parseInt(value);
            } catch (NumberFormatException exception) { // If the parameter is set to something other than a valid integer.
                linesWithErrors+=removeLine(indexOfStartOfValue); // Get the line with the error in it, remove it from input file, and store it in lines with errors in them.
                throw new ParameterSyntaxErrorException();
            }
            
            checkThatEverythingElseOnLineIsWhiteSpaceOrComment(skipWord(indexOfStartOfValue)); // Check that there's nothing except a comment on the line after the value.
            
            removeLine(indexOfStartOfValue);
            return valueInIntegerFormat;
        }
        
        boolean parseBooleanValueAt(int indexOfStartOfValue) throws ParameterSyntaxErrorException {
            String value=parseWord(indexOfStartOfValue); // Get the value in input file.
            boolean onOrOff;
            
//            System.out.println("Boolean value has been parsed. It is :\""+value+"\"");
            
            if (value.equals("on"))
                onOrOff=true;
            else if (value.equals("off"))
                onOrOff=false;
            else {
                try {
                    double numericValue=Double.parseDouble(value);
                    if (numericValue==0)
                        onOrOff=false;
                    else
                        onOrOff=true;
                } catch (NumberFormatException exception) { // If the parameter isn't set to on, off, or a valid number.
//                    System.out.println("Error with boolean parameter syntax. Parameter value is not valid.");
                    linesWithErrors+=removeLine(indexOfStartOfValue); // Get the line with the error in it, remove it from input file, and store it in lines with errors in them.
                    throw new ParameterSyntaxErrorException();
                }
            }
            
            checkThatEverythingElseOnLineIsWhiteSpaceOrComment(skipWord(indexOfStartOfValue)); // Check that there's nothing except a comment on the line after the value.
            
            removeLine(indexOfStartOfValue); // And remove the line containing it.
            return onOrOff; // Finally return the boolean setting.
        }
        
        int skipWord(int indexOfStartOfWord) { // Find the index of the first character after the end of the word pointed to by indexOfStartOfWord.
            int indexOfEndOfWord=indexOfStartOfWord; // Start looking for the end of the word starting at the position of its start.
            while (indexOfEndOfWord<inputFile.length() && inputFile.charAt(indexOfEndOfWord)!=' ' && inputFile.charAt(indexOfEndOfWord)!='\t' && inputFile.charAt(indexOfEndOfWord)!='\n')
                ++indexOfEndOfWord;
           
            return indexOfEndOfWord;
        }
            
        String parseWord(int indexOfStartOfWord) { // Always operates on inputFile.
            return parseWord(indexOfStartOfWord, inputFile);
        }
        
        String parseWord(int indexOfStartOfWord, String string) { // Parse all the characters from this point until the end of line, end of the file, or white space is found. This may be no characers, in which case a blank string is returned. Will search in String string
            int indexOfEndOfWord=indexOfStartOfWord; // Start looking for the end of the word starting at the position of its start.
            while (indexOfEndOfWord<string.length() && string.charAt(indexOfEndOfWord)!=' ' && string.charAt(indexOfEndOfWord)!='\t' && string.charAt(indexOfEndOfWord)!='\n')
                ++indexOfEndOfWord;
            
            return string.substring(indexOfStartOfWord, indexOfEndOfWord); // 1 need not be added to end of word index because index already point to the character after the end of the word, unless the word is just the empty string.
        }
        
        String parseParameterName(int indexOfStartOfParameterName, String string) { // Like parseWord, but treats = as the end of the parameter.
            int indexOfEndOfParameterName=indexOfStartOfParameterName; // Start looking for the end of the word starting at the position of its start.

            if (string.charAt(indexOfStartOfParameterName)=='#') { // If the parameter starts in # it must be a GUI one, so move on to skip over any white space.
                indexOfEndOfParameterName+=ellipsisGUIParameterPrefix.length();
            }
            
            while (indexOfEndOfParameterName<string.length() && string.charAt(indexOfEndOfParameterName)!=' ' && string.charAt(indexOfEndOfParameterName)!='\t' && string.charAt(indexOfEndOfParameterName)!='\n' && string.charAt(indexOfEndOfParameterName)!='=')
                ++indexOfEndOfParameterName;
            
            return string.substring(indexOfStartOfParameterName, indexOfEndOfParameterName); // 1 need not be added to end of word index because index already point to the character after the end of the word, unless the word is just the empty string.
        }
        
        String removeLastLineFromString(String string) {
            if (string.equals("")) { // Debugging code only
//                System.out.println("***ERROR string is of size 0 in removeLastLineFromString***");
            }
            
            int index=string.length()-2; // Start looking at the second last character (if the last one is a newline, its still part of the last line, so, as we're looking for the start of the second last line we should start there).
            while (index>=0 && string.charAt(index)!='\n') // We'll have passed the start of the line when we find either the start of the previous line, or go past the start of the file.
                --index;
            
            return string.substring(0, index+1); // Return the string minus the last line.
        }
        
        String extractLine(int indexOfPointInLine) { // Get the line pointed to by index from input file, but don't remove it.
            return findLineWithOptionalRemove(indexOfPointInLine, false);
        }
        
        String removeLine(int indexOfPointInLine) { // Remove the line from the file which index points to a point in. Return that line. This always works on iputFile
            return findLineWithOptionalRemove(indexOfPointInLine, true);
        }
        
        String[] removeLine(int indexOfPointInLine, String string) {
            return findLineWithOptionalRemove(indexOfPointInLine, true, string);
        }
        
        String findLineWithOptionalRemove(int indexOfPointInLine, boolean removeLine) { // This version works just on input file.
            int indexOfStartOfLine=indexOfPointInLine;
            int indexOfEndOfLine=indexOfPointInLine;
            
            while (indexOfStartOfLine>=1 && (inputFile.charAt(indexOfStartOfLine-1)!='\n')) // Search backwards for the start of the line - its either the first character in the file, or the first character after a newline.
                --indexOfStartOfLine;
            
            while (indexOfEndOfLine<inputFile.length()-1 && inputFile.charAt(indexOfEndOfLine)!='\n') // Search forwards for the end of the file, or the first newline encountered (which can be the character the index initially points to.
                ++indexOfEndOfLine;
            
//            System.out.println("Start :"+indexOfStartOfLine+" End :"+indexOfEndOfLine+" Length of string: "+inputFile.length()+" Index of point in line :"+indexOfPointInLine);
//            System.out.flush();
//            if (indexOfEndOfLine+1>inputFile.length()) { // *** More debugging code
//                JOptionPane.showMessageDialog(MainWindow.this, "The maximum *************value for this parameter isparameter's value has therefore not been changed.", "Error", JOptionPane.ERROR_MESSAGE);
 //           }
            String lineToBeRemoved=inputFile.substring(indexOfStartOfLine, indexOfEndOfLine+1);

            if (removeLine)
                inputFile=inputFile.substring(0, indexOfStartOfLine)+inputFile.substring(indexOfEndOfLine+1); // Chop the line out of input file.
 
            return lineToBeRemoved; // And return the line itself.
        }
        
        String[] findLineWithOptionalRemove(int indexOfPointInLine, boolean removeLine, String string) { // This version works on any string. It returns an array containing two references, the first of which is the string in its new form, and the second of which is the line which has been removed.
            int indexOfStartOfLine=indexOfPointInLine;
            int indexOfEndOfLine=indexOfPointInLine;
            
            while (indexOfStartOfLine>=1 && (string.charAt(indexOfStartOfLine-1)!='\n')) // Search backwards for the start of the line - its either the first character in the file, or the first character after a newline.
                --indexOfStartOfLine;
            
            while (indexOfEndOfLine<string.length()-1 && string.charAt(indexOfEndOfLine)!='\n') // Search forwards for the end of the file, or the first newline encountered (which can be the character the index initially points to.
                ++indexOfEndOfLine;
               
            String lineToBeRemoved=string.substring(indexOfStartOfLine, indexOfEndOfLine+1);

            if (removeLine)
                string=string.substring(0, indexOfStartOfLine)+string.substring(indexOfEndOfLine+1); // Chop the line out of input file.
 
            String[] stringAndRemovedLine={string, lineToBeRemoved}; // And return the possibly modeifed version of the string togheher with the removed line.
            
            return stringAndRemovedLine;
        }
        
        int findStartOfValue(String parameterName) throws ParameterNotFoundException, ParameterSyntaxErrorException {
            int index=findParameter(parameterName, 0); // point index at the start of the first occurence of parameter name in input file (only finding proper occurences of the parameter, not ones in comments etc.).
            if (index==-1)
                throw new ParameterNotFoundException();
            
            index+=parameterName.length(); // Change the index so that it points to the first character after the start of the parameter name.
            
                        
            if (findParameter(parameterName, index)!=-1) { // See if there are any further occurences of the parameter.
                MessageBoxes.warnAboutDuplicateParameter(MainWindow.this, parameterName); // If so, warn about this and remove the lines with the duplicates, but don't do anything else.
                removeAllOccurences(parameterName,index); // This will alter inputFile, but that is OK, because changes will be to point after the parameter we are interested in, so its index will not be changed.
            }
            
            index=skipWhiteSpace(index); // Move index forward to the start of the first non-whitespace character.
            
            if (inputFile.charAt(index)!='=') {// If the first non-whitespace character after the end of the parameter isn't an = sign, we've got an error.
//                System.out.println("Error with parameter "+parameterName+" syntax. First non-whitespace characater after parameter name is not =");
                linesWithErrors+=removeLine(index); // Get the line with the error in it, remove it from input file, and store it in lines with errors in them.
                throw new ParameterSyntaxErrorException();
            }

            ++index; // Move index on to point at the first character after the equals sign.
            index=skipWhiteSpace(index); // Skip any white space after the equals sign.
            
            return index; // Now we've found what should be the start of the value.
        }
        
        void removeAllOccurences(String parameterName, int indexToSearchFrom) { // Remove all lines containing a setting for parameterNameFrom the input file, starting at index to search from. When writing this function be careful about indexes chaning when inputFile is changed.
            while (true) {
                indexToSearchFrom=findParameter(parameterName, indexToSearchFrom);
                if (indexToSearchFrom==-1) // Return if there are no more occurences of the parameter.
                    return;
                
                removeLine(indexToSearchFrom); // Remove the occurence of the parameter we just found, and throw it away.
            }
        }
        
        int skipWhiteSpace(int index) { // This always works on input file.
            return skipWhiteSpace(index, inputFile);
        }
        
        int skipWhiteSpace(int index, String string) { // This works on any string.
            while (string.charAt(index)==' ' || string.charAt(index)=='\t') // skip over any white space.
                ++index;
            
            return index;
        }

        boolean parameterIsDefined(String parameterName, String stringToSearch) {
            if (findParameter(parameterName, 0, stringToSearch)==-1)
                return false;
            return true;
        }
            
        boolean checkThatParameterIsDefined(String parameterName) { // This function is like parameterIsDefined, but only works on inputFile.
            return parameterIsDefined(parameterName, inputFile);
        }
            
        String getAllLinesForSettingsWithThisPrefix(String parameterPrefix) { // This function also gets equivalent lines preceded by Ellipsis GUI parameter
            return getAllLinesForSettingsWithJustThisPrefix(parameterPrefix)+getAllLinesForSettingsWithJustThisPrefix(ellipsisGUIParameterPrefix+parameterPrefix);
        }
        
        String getAllLinesForSettingsWithJustThisPrefix(String parameterPrefix) {
            String lines="";
            int index=0;
            
            while (index<inputFile.length()) {
                index=findNextOccurenceOfPrefix(parameterPrefix, index); // Find the start of the next occurence of the prefix.
                if (index==-1) // If we can't find any more occurences of the prefix.
                    break;
                lines+=extractLine(index); // Extract the line a point in which is pointed to by index.
                ++index; // Now move index on, so we don't just find the same the prefix next time.
            }
            
            return lines;
        }
        
        
        
        int findNextOccurenceOfPrefix(String parameterPrefix, int index) {
            while (index<inputFile.length()) {
                index=inputFile.indexOf(parameterPrefix, index);
                if (index==-1) // If there is no occurence of the paramter we're finished.
                    break;
                if (thisIsTheLocationOfTheFirstItemOnLine(index)) // And if we've found the parameter we're finished.
                    break;
                
                ++index; // Move index on, so we don't just find the same string next time.
            }
            return index;
        }
            
        int findParameter(String parameterName, int index) { // Start looking from the point in the file indicated by index, and always look in inputFile.
            return findParameter(parameterName, index, inputFile);
        }
        
        int findParameter(String parameterName, int index, String stringToSearch) {
            while (true) { // Keep going as long as we don't get to a stage where we can't find the parameter at all.
                if (index>=inputFile.length()) // If we've now gone past the end of the file, give up looking.
                    break;
                index=stringToSearch.indexOf(parameterName, index); // Find the first occurence of Parameter name at or after the point pointed to by index.
                if (index==-1) {// If we can't find the parameter, give up looking.
                    break;
                }
                if (stringToSearch.charAt(index+parameterName.length())=='=' || stringToSearch.charAt(index+parameterName.length())==' ' || stringToSearch.charAt(index+parameterName.length())=='\n' || stringToSearch.charAt(index+parameterName.length())=='\t') // If we've found the end of the parameter. (N.B., this parameter name could be a subset of another, or part of a comment etc.).
                    if (thisIsTheLocationOfTheFirstItemOnLine(index, stringToSearch)) // Check that there is nothing before the parameter name (other than tabs, white space etc.) (e.g. a comment symbol, or some text that might be the first part of another parameters name).
                        return index; // If we've found the parameter, just return its index.
                
                ++index; // Now start looking from one character further on, in another attempt to find a real occurence of this parameter (so that we don't just find the same substring again).
            } // If we haven't found the parameter this time, go back and look again.
            
//            System.out.println("Signalling that "+parameterName+" was not found");
            return -1; // Signal parameter not found.
        }

        boolean thisIsTheLocationOfTheFirstItemOnLine(int index) { // Check that this location is not preceeded by anything on the same line other than white space, tabs etc. This function always works on input file.
            return thisIsTheLocationOfTheFirstItemOnLine(index, inputFile);
        }
        
        boolean thisIsTheLocationOfTheFirstItemOnLine(int index, String string) { // Allows the file to be specified.
            --index; // Start at the first location to the left of the locatoin.
            while (index>=0 && string.charAt(index)!='\n') { // Keep going until we reach the beginining of the file, or the end of the previous line.
                if (string.charAt(index)!=' ' && string.charAt(index)!='\t') // If we find anything other than white space then this isn't the first item on the line.
                    return false;
                --index; // Look at the previous character next time.
            }
            return true; // Nothing has been found on the line, so it is the location of the first item on the line.
        }
        
        String makeListOfIntegers(int integer, int lengthOfList) {
            return makeListOfStrings(""+integer, lengthOfList);
        }
        
        String makeListOfStrings(String string, int lengthOfList) {
            String list="";
            for (int i=1; i<=lengthOfList; ++i) {
                list+=string+",";
            }
            list=Useful.chopAnyTrailingComma(list);
            return list;
        }
        
        boolean inputFileChanged() { // Returns true iff the InputFile (once updated) is not identical to the LastSavedVersion.
            updateInputFile();
            return !inputFile.equals(lastSavedVersion); 
        }
        
        void menuLoad() {
            if (inputFileChanged()) {
                int optionChosen = JOptionPane.showConfirmDialog(MainWindow.this, // Check to see if we should save the file. In reality, we should compare the file as it would be output if saved to the one read in (or the one that would be produced if no settings were changed), and only ask if the user wants to save if changes have been made.
                "Changes have been made which will be lost when you load a new file.\nDo you want to save the current input file before you load a new one?");
                
                if (optionChosen==JOptionPane.CANCEL_OPTION) {
                    return;
                }
                
                if (optionChosen==JOptionPane.YES_OPTION) {
                    menuSave(); // If YES - save the changes, then come back here and load another file.
                }
                
                // If NO, or after saving a file following YES, just go ahead and load the file.
            }
            
            java.io.File newFilename=chooseEllipsisFile(OPEN);
            if (newFilename==null) // If cancel chosen
                return; // Do nothing
            
            if (!newFilename.exists()) { // If the chosen file doesn't exist
                JOptionPane.showMessageDialog(MainWindow.this,"No such file.", "Error", JOptionPane.ERROR_MESSAGE); // Warn about this
                return; // And do nothing.
            }
            
            filename=newFilename; // Otherwise go ahead and use the new filename to load the file.
            doLoad(); // Do actual load here - into input file - set all parameters specified by the file - warn about any bad lines in the file - print them to a box along with line numbers.
        }
        
        void doLoad() {
            try {
                inputFile=""; // Start by overwriting any existing input file.
                unrecognisedParametersFoundLastTime=false; // And it's a new file, so if we get any unrecognised parameters we need to be warned about them.
                
                java.io.FileReader inFile = new java.io.FileReader(filename); // Create a filereader object for the chosen file.
                java.io.BufferedReader bufferedInFile= new java.io.BufferedReader(inFile);

                while (bufferedInFile.ready()) { // Read the whole file into inputFile
                    inputFile+=bufferedInFile.readLine()+"\n";
                }
                
                inputFileEditorTextArea.setText(inputFile); // Put the input file in the text area (this is where updateguiwill get it from). 
                updateGUIetc(); // Now parse the file, setting GUI objects etc as necessary.
                
                lastSavedVersion=new String(inputFile); // Now we haven't made any changes from the version on the disk. N.B. We must put this here, because the update input file might make some changes which we will want to be able to save, even if we haven't actually changed any setting in the GUI.
                
                updateInputFile(); // Set all parameters not specified by the file to whatever they are set to in the GUI. This is done now so that if the inputFileEditor text area is visible, it will be up to date immediately - any parameters missing from it will be added.
//                System.out.println("***");
                inputFileEditorTextArea.setText(inputFile);
//                System.out.println("***");
            } catch(Exception ex) {
                JOptionPane.showMessageDialog(MainWindow.this,"Unexpected error when loading a file: "+ex.getMessage()+"\nPlease try again.", "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
        
        void doSave() {
            try {
                updateInputFile(); // Create an input file reflecting the current state of the gui objects, including any unrecognised parameters etc.
                
//                System.out.println(filename);
                java.io.FileWriter outFile = new java.io.FileWriter(filename);
                outFile.write(inputFile); // Save the whole inputFile.
                outFile.close();
                
                lastSavedVersion=new String(inputFile);
            } catch (Exception ex) {
                JOptionPane.showMessageDialog(MainWindow.this,"Unexpected error when saving a file.", "Error", JOptionPane.ERROR_MESSAGE);
            }      
        }
        
        int locationOfCaretInTextArea=0;
        void inputFileEditorPaneBecomesVisibleOrFocusGained() { // When we show, or start typing in the text area, update it to reflect the current state of the GUI.
            updateInputFile();
            
            inputFileEditorTextArea.setText(inputFile);
            inputFileEditorTextArea.setCaretPosition(Math.min(locationOfCaretInTextArea, inputFile.length()-1)); // Set the caret position to where it was before, or to the end of the text area if the text area has shrunk. This will prevent the caret position from jumping around all the time, every time the user switches to or from the text area.
        }
        
        void inputFileEditorPaneFocusLost() { // When we stop typing on the text area, update the input file to reflect the changes, and update the GUI, and comments, unregognised parameters etc. as appropriate.
            inputFile=inputFileEditorTextArea.getText();
            locationOfCaretInTextArea=inputFileEditorTextArea.getCaretPosition(); // Store this, so that the edit point won't move every time the user switches to or from the text area.
            
            updateGUIetc();
        }

        boolean seeAndReportIfSaveNeeded() { // Returns true iff the input file has changed.
            if (filename==null) // If we haven't loaded a file yet, but are just saving the original state of the GUI, or the original state with changes, then we will always do save.
                return true;
            if (!inputFileChanged()) {
                JOptionPane.showMessageDialog(MainWindow.this,"No changes need to be saved.", "", JOptionPane.INFORMATION_MESSAGE); // Inform the user that no changes need to be saved.
                return false;
            }
            return true;
        }
        
        void menuSave() {
            if (filename==null) { // If we haven't got a filename stored yet, we need to ask for one, so do save as instead.
                menuSaveAs();
            } else {
                if (!seeAndReportIfSaveNeeded()) // If no changes need to be saved just return.
                    return;
                
                if (fileHasErrorsAndUserChoosesNotToSave()) // If the file contains errors, ask the user if they want to save the file. If they don't, just do nothing.
                    return;

                doSave(); // Do actual save here using stored filename, if any.
            }
        }
        
        java.io.File addEllipsisFileSuffixIfNecessary(java.io.File file) {
            String filename=file.getName();
            
//            System.out.println(filename);
            if (filename.length()<6 || !filename.substring(filename.length()-6).equals(".input")) { // If the filename doesn't end in .input (either it's too short to have this suffix, or the last five characters of the string don't match it).
                filename+=".input"; // then add the suffix.
                file= new java.io.File(file.getParent(), filename); // And create a new file object based on the old path, but with the filename changed.
            }
            return file; // And return either the original file, or the new one.
        }

        boolean fileHasErrorsAndUserChoosesNotToSave() {
            if (linesWithErrors.equals("")) // If there are no lines with errors just return false - there will be no option of not saving.
                return false;
            
            int answer=JOptionPane.showConfirmDialog(MainWindow.this, "One or more of the lines in the input file contains errors.\nEllipsis will not be able to load the file in its current state.\nDo you want to save the file anyway?", "Warning", JOptionPane.YES_NO_OPTION);
            if (answer==JOptionPane.YES_OPTION) { // If the user chooser to delete lines with errors, do so, then return.
                return false; // User chooses to save anyway.
            } else {
                return true; // If user chooses not to save.
            }
        }
        
        void menuSaveAs() {
//            if (!seeAndReportIfSaveNeeded()) // If no changes need to be saved just return. --> Not applicable to Save As - we will always do the save.
//                return;
            if (fileHasErrorsAndUserChoosesNotToSave()) // If the file contains errors, ask the user if they want to save the file. If they don't, just do nothing.
                return;
            
            java.io.File newFilename=chooseEllipsisFile(SAVE);
            if (newFilename==null) // If cancel was pressed, just return.
                return;
            
            newFilename=addEllipsisFileSuffixIfNecessary(newFilename);
            
            if (newFilename.exists()) {
                int option=JOptionPane.showConfirmDialog(MainWindow.this, "The selected file already exits. Do you want to overwrite it?", "Warning", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
                
                if (option==JOptionPane.NO_OPTION) // Just abandon save if user chooses no.
                    return;
            }
            
            filename=newFilename; // So now the filename used is the new one selected.
            doSave(); // And actually do the save.
            
        }
        
        static final boolean OPEN=true;
        static final boolean SAVE=false;
        java.io.File chooseEllipsisFile(boolean openOrClose) { // Open a file chooser and get the user to choose a filename for an ellipsis file.
            fileChooser.resetChoosableFileFilters(); // Remove any file filters, in case one we don't want is already set.
            fileChooser.addChoosableFileFilter(new EllipsisFileFilter()); // Set a filter to show just ellipsis files.
            
            int result;
            if (openOrClose==OPEN) {
                result=fileChooser.showOpenDialog(MainWindow.this); // Open the file chooser (in a modal way), and return the result.
            } else {
                result=fileChooser.showSaveDialog(MainWindow.this); // The same as for OPEN, but different labels in the filechooser this time.
            }
            if(result == JFileChooser.APPROVE_OPTION) {
                return fileChooser.getSelectedFile();
            } else {
                return null; // If the cancel button was pressed, signal this.
            }
        }
        
    } // End of Class Files
    
    class BackgroundImagePanel extends javax.swing.JPanel {
        private JPanel drawingPanel = new javax.swing.JPanel();
        private Image theBackgroundImage;
        
        MaterialRegionContainer materialRegionContainer=new MaterialRegionContainer();
        DrawingRegionContainer drawingRegionContainer=new DrawingRegionContainer();
        TracerRegionContainer tracerRegionContainer=new TracerRegionContainer();
        BoundaryRegionContainer boundaryRegionContainer=new BoundaryRegionContainer();
        ThermalRegionContainer thermalRegionContainer=new ThermalRegionContainer();
        
        MyMouseListener theMouseListener;
        
        BackgroundImagePanel() { // Set up the panel to its original (blank state), and put the drawing panel inside it.
            setLayout(new java.awt.GridLayout(1, 1));
            setBackground(java.awt.Color.white);
//            setPreferredSize(new java.awt.Dimension(200, 300));
            setCursor(new java.awt.Cursor(java.awt.Cursor.CROSSHAIR_CURSOR));
            setMinimumSize(new java.awt.Dimension(10, 10)); // This component must have a minimum size, because if it's shrunk to zero we could get divide by zero exceptions when painting. N.B. This doesn't seem to do anything at all - but for some reason no exception get thrown.
            setToolTipText("Click and drag to draw a shape");
            
            drawingPanel.setLayout(null); // Drawing panel is probablly fairly redundant - the drawing could be done directly on the background image panel.
            drawingPanel.setOpaque(false);
            add(drawingPanel);
            
            theMouseListener=new MyMouseListener(); // We want to listen for both mouse button and movement events.
            drawingPanel.addMouseListener(theMouseListener);
            drawingPanel.addMouseMotionListener(theMouseListener);
            
//            visualEditorCentralPanel.add(this, java.awt.BorderLayout.CENTER); // Add the panel to the main window's visual editor's central panel.
            visualEditorCentralPanel.add(this); // Add the panel to the main window's visual editor's central panel - don't give it a layout - I don't want it to get resized automatically.
        }
        
        void setImage(java.io.File imageFileName) {
            backgroundPictureCheckBox.setSelected(true); // If we've just loaded an image, presumably we want it to be displayed.
            Toolkit toolkit = Toolkit.getDefaultToolkit();
            theBackgroundImage = toolkit.getImage(imageFileName.getPath());
            paintComponent(getGraphics()); // Update the panel on the screen - this method requires a graphics component for the panel.
        }
        
//        int temp=0; // Debuggin code only
        protected void paintComponent(Graphics graphics) { // This must be protected because the super class's one is.
//            System.out.println("painting"+temp);
//            ++temp;
            
            super.paintComponent(graphics); // paint the background.
            if (theBackgroundImage!=null && backgroundPictureCheckBox.isSelected()) { // Draw on the background image (if any) so long as one's been loaded, and the display image checkbox is set.
                graphics.drawImage(theBackgroundImage, 0, 0, getWidth(), getHeight(), this);
            }
            
            if (drawTracersCheckBox.isSelected() && !useDefaultTracerSetup.isSelected()) { // If we're using the defulat tracer setup, we don't want to draw any tracer regions that might have been created.
                tracerRegionContainer.paintAllRegions(graphics);
            }
            if (drawMaterialsCheckBox.isSelected()) {
                materialRegionContainer.paintAllRegions(graphics);
            }
            if (drawDrawingCheckBox.isSelected()) {
                drawingRegionContainer.paintAllRegions(graphics);
            }
            if (drawBoundariesCheckBox.isSelected()) {
                boundaryRegionContainer.paintAllRegions(graphics);
            }
            if (drawThermalZonesCheckBox.isSelected()) {
                thermalRegionContainer.paintAllRegions(graphics);
            }
            
            // Only one (or zero) selected region will ever be painted at a time.
            if (materialsPanel.isShowing()) {
                materialRegionContainer.paintSelectedRegion(graphics); // Finally draw the selected region/drawing (if any).
            } else if (drawingsPanel.isShowing()) {
                drawingRegionContainer.paintSelectedRegion(graphics);
            } else if (tracersPanel.isShowing()) {
                tracerRegionContainer.paintSelectedRegion(graphics);
            } else if (boundaryConditionsPanel.isShowing()) {
                boundaryRegionContainer.paintSelectedRegion(graphics);
            } else if (thermalZonesPanel.isShowing()) {
                thermalRegionContainer.paintSelectedRegion(graphics);
            }
        }
        
        // These four functions convert between coordinates in km, and the coordinates of things on the screen.
        double xPixel2km(int xCoordinate) {
            return windowDimensionX*(xCoordinate+0.5)/getWidth(); // N.B. 0.5 is added so that we are pointing to the middle of the pixel, not the left edge.
        }
        
        double yPixel2km(int yCoordinate) {
            return windowDimensionY*(yCoordinate+0.5)/getHeight();
        }
        
        int xKm2Pixel(double xCoordinate) {
            if (xCoordinate==windowDimensionX) { // If we're right at the right edge of the window, just return the coordinate of the last pixel. (This prevents and overflow - otherwise we'd return the coordinates of one more than the last pixel.)
                return getWidth()-1;
            } else {
                return (int)Math.round(xCoordinate/windowDimensionX*getWidth()-0.5); // N.B. 0.5 must be deducted to point at the middle of the pixel, not the right edge.
            }
        }
        
        int yKm2Pixel(double yCoordinate) {
            if (yCoordinate==windowDimensionY) { // If we're right at the right edge of the window, just return the coordinate of the last pixel. (This prevents and overflow - otherwise we'd return the coordinates of one more than the last pixel.)
                return getHeight()-1;
            } else {
                return (int)Math.round(yCoordinate/windowDimensionY*getHeight()-0.5); // N.B. 0.5 must be deducted to point at the middle of the pixel, not the right edge.
            }
        }
        
        class MyMouseListener extends javax.swing.event.MouseInputAdapter { // Handles all the mouse events on the BackgroundImagePanel.
            boolean drawingInProgress=false;
            boolean triangleInProgress=false;
            Point pointWhereMouseWasPressed;
            Point pointWhereMouseWasReleased; // This is only used when drawing triangles.
            Point lastPointWhereMouseWas;
            
            public void mousePressed(MouseEvent mouseEvent) { // Note that we've started to draw a shape, and store the coordinates of where the mouse was pressed.
                if (triangleInProgress) { // If we're already in the process of drawing a triangle and then click, this must be to put the third point in place.
                    triangleInProgress=false; // We've finished the triangle, so signal that.
                    Point finalMouseLocation=getCoordinatesInsideWindow(mouseEvent);
                    if (materialsPanel.isShowing()) { // At present we've only got material triangles.
                        materialRegionContainer.addTriangle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), xPixel2km(pointWhereMouseWasReleased.x), yPixel2km(pointWhereMouseWasReleased.y), xPixel2km(finalMouseLocation.x), yPixel2km(finalMouseLocation.y));
                    } else if (thermalZonesPanel.isShowing()) { // But now we've got triangular thermal zones too.
                        thermalRegionContainer.addTriangle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), xPixel2km(pointWhereMouseWasReleased.x), yPixel2km(pointWhereMouseWasReleased.y), xPixel2km(finalMouseLocation.x), yPixel2km(finalMouseLocation.y));
                    }
                    
                } else { // If we're not in the process of drawing a triangle, we'll get here.
                    drawingInProgress=true;
                    pointWhereMouseWasPressed=new Point(mouseEvent.getX(), mouseEvent.getY());
                    lastPointWhereMouseWas=pointWhereMouseWasPressed;
                    if (rectangleControls.isShowing()) { // If we're drawing a rectangle
                        new MyRectangle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), false).updateControls(); // Create a temporary rectangle object, and use it to update the controls.
                    } else if (circleControls.isShowing()) { // If we're doing a circle, immediately draw it to the screen so it'll be xored off later. (This isn't necessary for rectangle - presumably because a rectangel of width zero doesn't get drawn at all, while a circle of width zero seems to.
                        xorDrawCircle();
                        new Circle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), getRadiusInkm(pointWhereMouseWasPressed.x, pointWhereMouseWasPressed.y,  lastPointWhereMouseWas.x, lastPointWhereMouseWas.y), false).updateControls(); // Create a temporary circle region and update the controls card based upon it's properties.
                    } else if (triangleControls.isShowing()) {
                        new Triangle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), false).updateControls(); // Create a temporary triangle object, and use it to update the controls.
                        triangleInProgress=true;
                    }
                }
            }

            public void mouseReleased(MouseEvent mouseEvent) {
                Point newMouseLocation=getCoordinatesInsideWindow(mouseEvent);
                drawingInProgress=false; // We've finished the drawing.
                if (rectangleControls.isShowing()) { // But what we do depends on what type of shape we're drawing.
                    if (materialsPanel.isShowing()) { // And whether we're doing materials, tracers, boundaries or drawings.
                        materialRegionContainer.addRectangle(xPixel2km(Math.min(pointWhereMouseWasPressed.x, newMouseLocation.x)), yPixel2km(Math.min(pointWhereMouseWasPressed.y, newMouseLocation.y)), xPixel2km(Math.max(pointWhereMouseWasPressed.x, newMouseLocation.x)), yPixel2km(Math.max(pointWhereMouseWasPressed.y, newMouseLocation.y))); // The container takes care of choosing the right material and updating the list in the window. The smallest two coordiantes will always be top left, and largest two bottom right.
                    } else if (drawingsPanel.isShowing()) {
                        drawingRegionContainer.addRectangle(xPixel2km(Math.min(pointWhereMouseWasPressed.x, newMouseLocation.x)), yPixel2km(Math.min(pointWhereMouseWasPressed.y, newMouseLocation.y)), xPixel2km(Math.max(pointWhereMouseWasPressed.x, newMouseLocation.x)), yPixel2km(Math.max(pointWhereMouseWasPressed.y, newMouseLocation.y)));
                    } else if (tracersPanel.isShowing()) {
                        tracerRegionContainer.addRectangle(xPixel2km(Math.min(pointWhereMouseWasPressed.x, newMouseLocation.x)), yPixel2km(Math.min(pointWhereMouseWasPressed.y, newMouseLocation.y)), xPixel2km(Math.max(pointWhereMouseWasPressed.x, newMouseLocation.x)), yPixel2km(Math.max(pointWhereMouseWasPressed.y, newMouseLocation.y)));
                    } else if (boundaryConditionsPanel.isShowing()) {
                        boundaryRegionContainer.addRectangle(xPixel2km(Math.min(pointWhereMouseWasPressed.x, newMouseLocation.x)), yPixel2km(Math.min(pointWhereMouseWasPressed.y, newMouseLocation.y)), xPixel2km(Math.max(pointWhereMouseWasPressed.x, newMouseLocation.x)), yPixel2km(Math.max(pointWhereMouseWasPressed.y, newMouseLocation.y)));
                    } else if (thermalZonesPanel.isShowing()) {
                        thermalRegionContainer.addRectangle(xPixel2km(Math.min(pointWhereMouseWasPressed.x, newMouseLocation.x)), yPixel2km(Math.min(pointWhereMouseWasPressed.y, newMouseLocation.y)), xPixel2km(Math.max(pointWhereMouseWasPressed.x, newMouseLocation.x)), yPixel2km(Math.max(pointWhereMouseWasPressed.y, newMouseLocation.y)));
                    }
                } else if (circleControls.isShowing()) { // If we're drawing a circle.
                    if (materialsPanel.isShowing()) { // And it's a material circle.
                        materialRegionContainer.addCircle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), getRadiusInkm(pointWhereMouseWasPressed.x, pointWhereMouseWasPressed.y, newMouseLocation.x, newMouseLocation.y));
                    } else if (drawingsPanel.isShowing()) {
                        drawingRegionContainer.addCircle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), getRadiusInkm(pointWhereMouseWasPressed.x, pointWhereMouseWasPressed.y, newMouseLocation.x, newMouseLocation.y));
                    } else if (boundaryConditionsPanel.isShowing()) {
                        boundaryRegionContainer.addCircle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), getRadiusInkm(pointWhereMouseWasPressed.x, pointWhereMouseWasPressed.y, newMouseLocation.x, newMouseLocation.y));
                    } else if (thermalZonesPanel.isShowing()) {
                        thermalRegionContainer.addCircle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), getRadiusInkm(pointWhereMouseWasPressed.x, pointWhereMouseWasPressed.y, newMouseLocation.x, newMouseLocation.y));
                    }
                } else if (lineControls.isShowing()) { // If we'return drawing a line.
                    if (drawingsPanel.isShowing()) { // If we're doing a drawing.
                        drawingRegionContainer.addLine(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), xPixel2km(newMouseLocation.x), yPixel2km(newMouseLocation.y));
                    }
                } else if (triangleControls.isShowing()) {
                    if (materialsPanel.isShowing()) {
                        xorDrawLine(); // We need to erase the line, so that it won't be a problem when we start drawing the triangle.
                        pointWhereMouseWasReleased=newMouseLocation; // We still aren't finished with the triangle,because to do it we need to define a third point by moving the mouse again and clicking somewhere.
//                        xorDrawTriangle(); // And draw the initial triangle. This is redundant - a flat XORed triangle will overwrite itself, and so be completely blank.
                    } else if (thermalZonesPanel.isShowing()) {
                        xorDrawLine();
                        pointWhereMouseWasReleased=newMouseLocation;
                    }
                }
            }
            
            Point getCoordinatesInsideWindow(MouseEvent mouseEvent) { // If the coordinates of the mouse event come outside of the window, move them to just inside its edge. Return the points.
                return new Point(Math.max (Math.min(mouseEvent.getX(), BackgroundImagePanel.this.getWidth()-1), 0), Math.max (Math.min(mouseEvent.getY(), BackgroundImagePanel.this.getHeight()-1), 0));
            }
            
            public void mouseDragged(MouseEvent mouseEvent) { // This needs to update the onscreen drawing of the shape as the mouse is moved. N.B. we don't need to bother cleaning up after this, because the window will always be redrawn.
                //                System.out.println("dragging"+temp);
                Point newMouseLocation=getCoordinatesInsideWindow(mouseEvent);
                if (rectangleControls.isShowing()) { // But what we do depends on what type of shape we're drawing.
                    xorDrawRectangle(); // Overwrite the old region so we can't see it anymore.
                    lastPointWhereMouseWas=newMouseLocation; // Update our record of the region based on where the mouse is now.
                    xorDrawRectangle(); // Draw the new region.
                    new MyRectangle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), xPixel2km(lastPointWhereMouseWas.x), yPixel2km(lastPointWhereMouseWas.y), false).updateControls(); // Create a temporary rectangle object, and use it to update the controls.
                } else if (circleControls.isShowing()) {
                    xorDrawCircle();
                    lastPointWhereMouseWas=newMouseLocation; // Update our record of the region based on where the mouse is now.
                    xorDrawCircle();
                    new Circle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), getRadiusInkm(pointWhereMouseWasPressed.x, pointWhereMouseWasPressed.y,  lastPointWhereMouseWas.x, lastPointWhereMouseWas.y), false).updateControls(); // Create a temporary circle region and update the controls card based upon it's properties.
                } else if (lineControls.isShowing() || triangleControls.isShowing()) { // N.B. when drawing the first side of a triangle this is just like drawing a line, as far as that is just what is displayed on the screen.
                    xorDrawLine();
                    lastPointWhereMouseWas=newMouseLocation; // Update our record of the region based on where the mouse is now.
                    xorDrawLine();
                    if (triangleControls.isShowing()) { // If we're drawing a triangle, we want to update the triangle controls as we go along.
                        new Triangle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), xPixel2km(lastPointWhereMouseWas.x), yPixel2km(lastPointWhereMouseWas.y), xPixel2km(lastPointWhereMouseWas.x), yPixel2km(lastPointWhereMouseWas.y), false).updateControls(); // Create a temporary triangle object, and use it to update the controls.
                    }
                }
            }
            
            public void mouseMoved(MouseEvent mouseEvent) { // When drawing the last part of a triangle, we need to monitor mouse movements even if the mouse button isn't pressed.
//                System.out.println("moving"+temp);
                Point newMouseLocation=getCoordinatesInsideWindow(mouseEvent);
                if (triangleInProgress==true) {
                    xorDrawTriangle(); // Overwrite the old region so we can't see it anymore.
                    lastPointWhereMouseWas=newMouseLocation; // Update our record of the region based on where the mouse is now.
                    xorDrawTriangle(); // Draw the new region.
                    new Triangle(xPixel2km(pointWhereMouseWasPressed.x), yPixel2km(pointWhereMouseWasPressed.y), xPixel2km(pointWhereMouseWasReleased.x), yPixel2km(pointWhereMouseWasReleased.y), xPixel2km(lastPointWhereMouseWas.x), yPixel2km(lastPointWhereMouseWas.y), false).updateControls(); // Create a temporary triangle object, and use it to update the controls.
                }
            }
            
            private void xorDrawCircle() {
                Graphics graphicsContext=drawingPanel.getGraphics();
                graphicsContext.setXORMode(Colours.white);
                double radius=getRadiusInkm(pointWhereMouseWasPressed.x, pointWhereMouseWasPressed.y, lastPointWhereMouseWas.x, lastPointWhereMouseWas.y); // Get the radius of the circle in km.
                graphicsContext.drawOval(pointWhereMouseWasPressed.x-xKm2Pixel(radius), pointWhereMouseWasPressed.y-yKm2Pixel(radius), xKm2Pixel(radius*2), yKm2Pixel(radius*2));
            }
            
            private void xorDrawRectangle() {
                Graphics graphicsContext=drawingPanel.getGraphics();
                graphicsContext.setXORMode(Colours.white);
                graphicsContext.drawRect(Math.min(pointWhereMouseWasPressed.x, lastPointWhereMouseWas.x), Math.min(pointWhereMouseWasPressed.y, lastPointWhereMouseWas.y), Math.abs(pointWhereMouseWasPressed.x-lastPointWhereMouseWas.x), Math.abs(pointWhereMouseWasPressed.y-lastPointWhereMouseWas.y));
            }

            private void xorDrawTriangle() { 
                Graphics graphicsContext=drawingPanel.getGraphics();
                graphicsContext.setXORMode(Colours.white);
                
                int[] xCoordinates=new int[3];
                int[] yCoordinates=new int[3];
                
                xCoordinates[0]=pointWhereMouseWasPressed.x;
                xCoordinates[1]=pointWhereMouseWasReleased.x;
                xCoordinates[2]=lastPointWhereMouseWas.x;
                
                yCoordinates[0]=pointWhereMouseWasPressed.y;
                yCoordinates[1]=pointWhereMouseWasReleased.y;
                yCoordinates[2]=lastPointWhereMouseWas.y;

                graphicsContext.drawPolygon(xCoordinates, yCoordinates, 3);
            }
            
            private void xorDrawLine() {
                Graphics graphicsContext=drawingPanel.getGraphics();
                graphicsContext.setXORMode(Colours.white);
                graphicsContext.drawLine(pointWhereMouseWasPressed.x, pointWhereMouseWasPressed.y, lastPointWhereMouseWas.x, lastPointWhereMouseWas.y);
            }
            
            private double getRadiusInkm(int centreX, int centreY, int edgeX, int edgeY) { // Take pixel coordinates for the centre and edge of a circle, and return its radius in km. N.B. the radius must not put the edge of the circle outside of the window.
                double radius=Math.pow(Math.pow(xPixel2km(Math.abs(centreX-edgeX)), 2.0)+Math.pow(yPixel2km(centreY-edgeY),2.0), 0.5); // First convert the horizontal and vertical distances into km, and then convert these to the radius using pythagoruses rule.
                
                radius=Math.min(xPixel2km(centreX),  radius); // Trim the radius, so that it doesn't extend outside of the window.
                radius=Math.min(windowDimensionX-xPixel2km(centreX), radius);
                
                radius=Math.min(yPixel2km(centreY),  radius);
                radius=Math.min(windowDimensionY-yPixel2km(centreY), radius);

                return radius;
            }
            
        } // End of MyMouseListener
        
        class MaterialRegionContainer {
            private LinkedList materialRegions=new LinkedList();
            private int selectedMaterialRegionIndex=-1; // This will be used to keep track of the index of which region is selected.
            
            void parseAndRemoveMaterialRegions() { // Take the material regions from files.inputFile, and put them in the material region container.
                materialRegions= new LinkedList(); // Start by getting rid of all the old regions (if any).
                selectedMaterialRegionIndex=-1; // And no region will be selected initially.
                
                parseAndRemoveRectangles();
                parseAndRemoveTriangles();
                parseAndRemoveCircles();
                
                updateMaterialRegionList(); // Update the JList to reflect the new regions.

            }
            
            void parseAndRemoveTriangles() {
                
                String parameterPrefix="Material_trgl";
                if (files.checkThatParameterIsDefined(parameterPrefix) || files.checkThatParameterIsDefined(parameterPrefix+"_property") || files.checkThatParameterIsDefined(parameterPrefix+"_x1") || files.checkThatParameterIsDefined(parameterPrefix+"_x2") || files.checkThatParameterIsDefined(parameterPrefix+"_z1") || files.checkThatParameterIsDefined(parameterPrefix+"_z2")) { // If no parameters are defined for triangles, just ignore it completely. But if at least one is, then we will continue.
                    
                    String triangleSettings=files.getAllLinesForSettingsWithThisPrefix(parameterPrefix); // Store all the settings incase we encounter an error.
                    
                    try {
                        int numberOfTriangles=files.parseInteger(parameterPrefix);
                        parseAndRemoveThisManyTriangles(parameterPrefix, numberOfTriangles);
                        
                    } catch (ParameterNotFoundException exception) {
                        files.linesWithErrors+=triangleSettings; // If one of the parameters isn't found, add all the settings for this property to linesWithErrors.
                    } catch (ParameterSyntaxErrorException exception) {
                        files.linesWithErrors=files.removeLastLineFromString(files.linesWithErrors)+triangleSettings; // If there is a syntax error with one or more parameters, it will already have been added to lines with errors, but we'll now add all of the lines in, so just delete the last line.
                    } catch (ParameterIncompatibilityException exception) {
                        files.linesWithErrors+=triangleSettings;
                    }
                    
                }
                
            }
            
            void parseAndRemoveThisManyTriangles(String parameterPrefix, int numberOfTriangles) throws ParameterNotFoundException, ParameterSyntaxErrorException, ParameterIncompatibilityException {
                // This code no longer checks that each list contains the right number of elements.
                String property=files.parseAndRemoveList(parameterPrefix+"_property");
                String x1=files.parseAndRemoveList(parameterPrefix+"_x1");
                String x2=files.parseAndRemoveList(parameterPrefix+"_x2");
                String x3=files.parseAndRemoveList(parameterPrefix+"_x3");
                String z1=files.parseAndRemoveList(parameterPrefix+"_z1");
                String z2=files.parseAndRemoveList(parameterPrefix+"_z2");
                String z3=files.parseAndRemoveList(parameterPrefix+"_z3");
                
                String filled;
                try {
                    filled=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_filled");
                } catch (ParameterNotFoundException exception) {
                    filled=files.makeListOfStrings("unfilled", numberOfTriangles);
                }
                
                for (int regionIndex=1; regionIndex<=numberOfTriangles; ++regionIndex) {
                    try {
                        materialRegions.add(new MaterialTriangle(scaling.ellipsisUnits2KM(Statics.nthReal(x1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(x2, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z2, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(x3, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z3, regionIndex)), Statics.filledOrUnfilled2Boolean(Statics.nthItem(filled, regionIndex)), materialWithThisNumber(Statics.nthInteger(property, regionIndex)))); // Add a region with it's boundaries specified in km, either solid or not solid, with the right colour, and with the right material (=property) according to the files settings.
                    } catch (ParameterSyntaxErrorException exception) {
                        files.linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                        throw exception; // And then re-throw the exception
                    }
                }
                
            }
            
            void parseAndRemoveRectangles() {
                
                String parameterPrefix="Material_rect";
                if (files.checkThatParameterIsDefined(parameterPrefix) || files.checkThatParameterIsDefined(parameterPrefix+"_property") || files.checkThatParameterIsDefined(parameterPrefix+"_x1") || files.checkThatParameterIsDefined(parameterPrefix+"_x2") || files.checkThatParameterIsDefined(parameterPrefix+"_z1") || files.checkThatParameterIsDefined(parameterPrefix+"_z2")) { // If no parameters are defined for rectangles, just ignore it completely. But if at least one is, then we will continue.
                    
                    String rectangleSettings=files.getAllLinesForSettingsWithThisPrefix(parameterPrefix); // Store all the settings incase we encounter an error.
                    
                    try {
                        int numberOfRectangles=files.parseInteger(parameterPrefix);
                        parseAndRemoveThisManyRectangles(parameterPrefix, numberOfRectangles);
                        
                    } catch (ParameterNotFoundException exception) {
                        files.linesWithErrors+=rectangleSettings; // If one of the parameters isn't found, add all the settings for this property to linesWithErrors.
                    } catch (ParameterSyntaxErrorException exception) {
                        files.linesWithErrors=files.removeLastLineFromString(files.linesWithErrors)+rectangleSettings; // If there is a syntax error with one or more parameters, it will already have been added to lines with errors, but we'll now add all of the lines in, so just delete the last line.
                    } catch (ParameterIncompatibilityException exception) {
                        files.linesWithErrors+=rectangleSettings;
                    }
                    
                }
                
            }
            
            void parseAndRemoveThisManyRectangles(String parameterPrefix, int numberOfRectangles) throws ParameterNotFoundException, ParameterSyntaxErrorException, ParameterIncompatibilityException {
/*                String property=files.parseAndRemoveList(parameterPrefix+"_property", numberOfRectangles);
                String x1=files.parseAndRemoveList(parameterPrefix+"_x1", numberOfRectangles);
                String x2=files.parseAndRemoveList(parameterPrefix+"_x2", numberOfRectangles);
                String z1=files.parseAndRemoveList(parameterPrefix+"_z1", numberOfRectangles);
                String z2=files.parseAndRemoveList(parameterPrefix+"_z2", numberOfRectangles);*/
                
                // New version of code - this allows lists to contain more items than the number of rectangles 
                // - excess items will just be ignored. This makes the GUI consistent with Ellipsis
                // and allows it to load Craig's files.
                String property=files.parseAndRemoveList(parameterPrefix+"_property");
                String x1=files.parseAndRemoveList(parameterPrefix+"_x1");
                String x2=files.parseAndRemoveList(parameterPrefix+"_x2");
                String z1=files.parseAndRemoveList(parameterPrefix+"_z1");
                String z2=files.parseAndRemoveList(parameterPrefix+"_z2");
                
                String filled;
                try {
                    filled=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_filled");
                } catch (ParameterNotFoundException exception) {
                    filled=files.makeListOfStrings("filled", numberOfRectangles);
                }
                
                for (int regionIndex=1; regionIndex<=numberOfRectangles; ++regionIndex) {
                    try {
                        materialRegions.add(new MaterialRectangle(scaling.ellipsisUnits2KM(Statics.nthReal(x1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(x2, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z2, regionIndex)), Statics.filledOrUnfilled2Boolean(Statics.nthItem(filled, regionIndex)), materialWithThisNumber(Statics.nthInteger(property, regionIndex)))); // Add a region with it's boundaries specified in km, either solid or not solid, with the right colour, and with the right material (=property) according to the files settings.
                    } catch (ParameterSyntaxErrorException exception) {
                        files.linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                        throw exception; // And then re-throw the exception
                    }
                }
                
            }
            
            void parseAndRemoveCircles() {
                
                String parameterPrefix="Material_circ";
                if (files.checkThatParameterIsDefined(parameterPrefix) || files.checkThatParameterIsDefined(parameterPrefix+"_property") || files.checkThatParameterIsDefined(parameterPrefix+"_x1") || files.checkThatParameterIsDefined(parameterPrefix+"_z1") || files.checkThatParameterIsDefined(parameterPrefix+"rad")) { // If no parameters are defined for rectangles, just ignore it completely. But if at least one is, then we will continue.
                    
                    String circleSettings=files.getAllLinesForSettingsWithThisPrefix(parameterPrefix); // Store all the settings incase we encounter an error.
                    
                    try {
                        int numberOfCircles=files.parseInteger(parameterPrefix);
                        parseAndRemoveThisManyCircles(parameterPrefix, numberOfCircles);
                        
                    } catch (ParameterNotFoundException exception) {
                        files.linesWithErrors+=circleSettings; // If one of the parameters isn't found, add all the settings for this property to linesWithErrors.
                    } catch (ParameterSyntaxErrorException exception) {
                        files.linesWithErrors=files.removeLastLineFromString(files.linesWithErrors)+circleSettings; // If there is a syntax error with one or more parameters, it will already have been added to lines with errors, but we'll now add all of the lines in, so just delete the last line.
                    } catch (ParameterIncompatibilityException exception) {
                        files.linesWithErrors+=circleSettings;
                    }
                    
                }
                
            }
            
            void parseAndRemoveThisManyCircles(String parameterPrefix, int numberOfCircles) throws ParameterNotFoundException, ParameterSyntaxErrorException, ParameterIncompatibilityException {
                // This code no longer checks that each list contains the right number of elements.
                String property=files.parseAndRemoveList(parameterPrefix+"_property");
                String x1=files.parseAndRemoveList(parameterPrefix+"_x");
                String z1=files.parseAndRemoveList(parameterPrefix+"_z");
                String rad=files.parseAndRemoveList(parameterPrefix+"_rad");
                
                String filled;
                try {
                    filled=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_filled");
                } catch (ParameterNotFoundException exception) {
                    filled=files.makeListOfStrings("unfilled", numberOfCircles);
                }
                
                for (int regionIndex=1; regionIndex<=numberOfCircles; ++regionIndex) {
                    try {
                        materialRegions.add(new MaterialCircle(scaling.ellipsisUnits2KM(Statics.nthReal(x1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(rad, regionIndex)), Statics.filledOrUnfilled2Boolean(Statics.nthItem(filled, regionIndex)), materialWithThisNumber(Statics.nthInteger(property, regionIndex)))); // Add a region with it's boundaries specified in km, either solid or not solid, with the right colour, and with the right material (=property) according to the files settings.
                    } catch (ParameterSyntaxErrorException exception) {
                        files.linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                        throw exception; // And then re-throw the exception
                    }
                }
                
            }
            
            Material materialWithThisNumber(int materialNumber) throws ParameterIncompatibilityException { // Get the material which has this number in ellipsis input file.
                if (materialNumber>=materials.size() || materialNumber<0) // If there is no material with the number specified, we have an error. Also if the material number is negative, that's an error too.
                    throw new ParameterIncompatibilityException();
                
                return (Material)materials.get(materialNumber); // Just find it in the ArrayList of materials in the main window.
            }
            
            String getFileSettings() { // Returns a string specifying all the boundary region settings in ellipsis input file format.
                String settings="\n# MATERIAL REGIONS\n\n";
                
                settings+=getRectangleFileSettings();
                settings+=getTriangleFileSettings();
                settings+=getCircleFileSettings();
                
                return settings;
            }
            
            private String getTriangleFileSettings() {
                String settings="# Material Triangles\n\n";
                
                if (numberOfTriangles()!=0) {
                    settings+="Material_trgl="+numberOfTriangles()+"\n\n";
                    
                    String property="";
                    String x1="";
                    String x2="";
                    String x3="";
                    String z1="";
                    String z2="";
                    String z3="";
                    
                    String filled=""; // Thiw is whether the region is drawn filled in the Ellipsis Gui.
                    
                    Iterator currentRegion=materialRegions.iterator();
                    while (currentRegion.hasNext()) {
                        Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                        if (theRegion instanceof MaterialTriangle) { // If it's a rectangle,
                            MaterialTriangle triangle=(MaterialTriangle)theRegion;
                            
                            property+=triangle.triangleMaterial.numberOfThisMaterial()+",";
                            x1+=scaling.KM2EllipsisUnits(triangle.x1)+",";
                            x2+=scaling.KM2EllipsisUnits(triangle.x2)+",";
                            x3+=scaling.KM2EllipsisUnits(triangle.x3)+",";
                            z1+=scaling.KM2EllipsisUnits(triangle.z1)+",";
                            z2+=scaling.KM2EllipsisUnits(triangle.z2)+",";
                            z3+=scaling.KM2EllipsisUnits(triangle.z3)+",";
                            
                            filled+=Statics.boolean2filledOrUnfilled(triangle.filled)+",";
                        }
                    }
                    
                    settings+="Material_trgl_property="+Useful.chopAnyTrailingComma(property)+"\n";
                    settings+="Material_trgl_x1="+Useful.chopAnyTrailingComma(x1)+"\n";
                    settings+="Material_trgl_x2="+Useful.chopAnyTrailingComma(x2)+"\n";
                    settings+="Material_trgl_x3="+Useful.chopAnyTrailingComma(x3)+"\n";
                    settings+="Material_trgl_z1="+Useful.chopAnyTrailingComma(z1)+"\n";
                    settings+="Material_trgl_z2="+Useful.chopAnyTrailingComma(z2)+"\n";
                    settings+="Material_trgl_z3="+Useful.chopAnyTrailingComma(z3)+"\n";
                    
                    settings+=files.ellipsisGUIParameterPrefix+"Material_trgl_filled="+Useful.chopAnyTrailingComma(filled)+"\n\n";
                }
                
                return settings;
            }
                               
            private String getRectangleFileSettings() {
                String settings="# Material Rectangles\n\n";
                
                if (numberOfRectangles()!=0) { // Only do the settings if at least one rectangle exists.
                    settings+="Material_rect="+numberOfRectangles()+"\n\n";
                    
                    String property="";
                    String x1="";
                    String x2="";
                    String z1="";
                    String z2="";
                    
                    String filled=""; // Thiw is whether the region is drawn filled in the Ellipsis Gui.
                    
                    Iterator currentRegion=materialRegions.iterator();
                    while (currentRegion.hasNext()) {
                        Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                        if (theRegion instanceof MaterialRectangle) { // If it's a rectangle,
                            MaterialRectangle rectangle=(MaterialRectangle)theRegion;
                            
                            property+=rectangle.rectangleMaterial.numberOfThisMaterial()+",";
                            x1+=scaling.KM2EllipsisUnits(rectangle.x1)+",";
                            x2+=scaling.KM2EllipsisUnits(rectangle.x2)+",";
                            z1+=scaling.KM2EllipsisUnits(rectangle.z1)+",";
                            z2+=scaling.KM2EllipsisUnits(rectangle.z2)+",";
                            
                            filled+=Statics.boolean2filledOrUnfilled(rectangle.filled)+",";
                        }
                    }
                    
                    settings+="Material_rect_property="+Useful.chopAnyTrailingComma(property)+"\n";
                    settings+="Material_rect_x1="+Useful.chopAnyTrailingComma(x1)+"\n";
                    settings+="Material_rect_x2="+Useful.chopAnyTrailingComma(x2)+"\n";
                    settings+="Material_rect_z1="+Useful.chopAnyTrailingComma(z1)+"\n";
                    settings+="Material_rect_z2="+Useful.chopAnyTrailingComma(z2)+"\n";
                    
                    settings+=files.ellipsisGUIParameterPrefix+"Material_rect_filled="+Useful.chopAnyTrailingComma(filled)+"\n\n";
                }
                
                return settings;
            }
            
            private String getCircleFileSettings() {
                String settings="# Material Circles\n\n";
                
                if (numberOfCircles()!=0) { // Only do the settings if at least one rectangle exists.
                    settings+="Material_circ="+numberOfCircles()+"\n\n";
                    
                    String property="";
                    String x="";
                    String z="";
                    String radius="";
                    
                    String filled=""; // Thiw is whether the region is drawn filled in the Ellipsis Gui.
                    
                    Iterator currentRegion=materialRegions.iterator();
                    while (currentRegion.hasNext()) {
                        Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                        if (theRegion instanceof MaterialCircle) { // If it's a rectangle,
                            MaterialCircle circle=(MaterialCircle)theRegion;
                            
                            property+=circle.circleMaterial.numberOfThisMaterial()+",";
                            x+=scaling.KM2EllipsisUnits(circle.x)+",";
                            z+=scaling.KM2EllipsisUnits(circle.z)+",";
                            radius+=scaling.KM2EllipsisUnits(circle.radius)+",";
                            
                            filled+=Statics.boolean2filledOrUnfilled(circle.filled)+",";
                        }
                    }
                    
                    settings+="Material_circ_property="+Useful.chopAnyTrailingComma(property)+"\n";
                    settings+="Material_circ_x="+Useful.chopAnyTrailingComma(x)+"\n";
                    settings+="Material_circ_z="+Useful.chopAnyTrailingComma(z)+"\n";
                    settings+="Material_circ_rad="+Useful.chopAnyTrailingComma(radius)+"\n";
                    
                    settings+=files.ellipsisGUIParameterPrefix+"Material_circ_filled="+Useful.chopAnyTrailingComma(filled)+"\n\n";
                }
                
                return settings;
            }
            
            private int numberOfThisMaterial(Material material) { // Return the number of the material, which will be used to index the material in ellipsis input files. Materials are number from 0, in the order in which they occur in the materials list - N.B. ellipsis input file material numbers are completely unrelated to Ellipsis GUI material numbers or names.
                Iterator currentMaterial=materials.iterator();
                int numberOfCurrentMaterial=0;
                
                while (currentMaterial.hasNext()) {
                    if (currentMaterial.next()==material) // If we've found the material we're looking for...
                        return numberOfCurrentMaterial; // Then return its index.
                    ++numberOfCurrentMaterial;
                }
                
                return -1; // We should never get here, because we should find every material for which a region is defined (we can't delete materials if they are in use).
            }
            
            private int numberOfRectangles() { // Return the number of material rectangle regions.
                int numberOfRectangles=0;
                Iterator currentRegion=materialRegions.iterator();
                while (currentRegion.hasNext()) {
                    Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                    if (theRegion instanceof MaterialRectangle) { // If it's a rectangle,
                        ++numberOfRectangles; // Then count it.
                    }
                }
                
                return numberOfRectangles;
            }
            
            private int numberOfTriangles() { // Return the number of material triangle regions.
                int numberOfTriangles=0;
                Iterator currentRegion=materialRegions.iterator();
                while (currentRegion.hasNext()) {
                    Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                    if (theRegion instanceof MaterialTriangle) { // If it's a rectangle,
                        ++numberOfTriangles; // Then count it.
                    }
                }
                
                return numberOfTriangles;
            }
            
            private int numberOfCircles() { // Return the number of material circle regions.
                int numberOfCirles=0;
                Iterator currentRegion=materialRegions.iterator();
                while (currentRegion.hasNext()) {
                    Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                    if (theRegion instanceof MaterialCircle) { // If it's a circle,
                        ++numberOfCirles; // Then count it.
                    }
                }
                
                return numberOfCirles;
            }
            
            boolean materialInUse(Material theMaterial) { // Returns true iff one or more regions of theMaterial currently exist.
                Iterator currentRegion=materialRegions.iterator();  // Do this by going through all the regions and comparing their materials to theMaterial.
                while (currentRegion.hasNext()) {
                    if (((Region)currentRegion.next()).getMaterial()==theMaterial) { // If the current region's material matches the one in use.
                        return true;
                    }
                }
                return false; // No matching material has been found.
            }
            
            void updateSelectedRectangle() {
                if (selectedMaterialRegionIndex==-1) return; // If no region is selected, do nothing.
                
                RectangleCoordinates coordinates=new RectangleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((MyRectangle)materialRegions.get(selectedMaterialRegionIndex)).update(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                repaint();
            }
            
            void updateSelectedTriangle() {
                if (selectedMaterialRegionIndex==-1) return; // If no region is selected, do nothing.
                
                TriangleCoordinates coordinates=new TriangleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((Triangle)materialRegions.get(selectedMaterialRegionIndex)).update(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2, coordinates.x3, coordinates.z3, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                repaint();
            }
            
            void updateSelectedCircle() {
                if (selectedMaterialRegionIndex==-1) return; // If no region is selected, do nothing.
                
                CircleCoordinates coordinates=new CircleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((Circle)materialRegions.get(selectedMaterialRegionIndex)).update(coordinates.x, coordinates.z, coordinates.radius, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                repaint();
            }
            
            void deleteSelectedRegion() {
                if (selectedMaterialRegionIndex==-1) return; // If no region is selected, do nothing.
                materialRegions.remove(selectedMaterialRegionIndex); // Remove the region from the list of regions.
                selectedMaterialRegionIndex=-1;
                
                updateMaterialRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted without this region.
            }
            
            void moveSelectedRegionForward() {
                if (selectedMaterialRegionIndex==-1 || selectedMaterialRegionIndex==materialRegions.size()-1) return; // If no region is selected, or the region is already at the front, do nothing.
                Region theRegion=(Region)materialRegions.remove(selectedMaterialRegionIndex); // Take the region out of the list
                materialRegions.add(selectedMaterialRegionIndex+1, theRegion); // And put it back one further down.
                ++selectedMaterialRegionIndex; // Keep the same region selected.
                
                updateMaterialRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted with this region in its new position.
            }
            
            void moveSelectedRegionBackward() {
                if (selectedMaterialRegionIndex==-1 || selectedMaterialRegionIndex==0) return; // If no region is selected, or the region is already at the back, do nothing.
                Region theRegion=(Region)materialRegions.remove(selectedMaterialRegionIndex); // Take the region out of the list
                materialRegions.add(selectedMaterialRegionIndex-1, theRegion); // And put it back one further up.
                --selectedMaterialRegionIndex; // Keep the same region selected.
                
                updateMaterialRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted with this region in its new position.
            }
            
            void setSelectedRegion(int newSelectedRegionIndex) { // If we've changed regions, then we need to update the drawing controls to reflect that.
                //                System.out.println("["+newSelectedRegionIndex);
                selectedMaterialRegionIndex=newSelectedRegionIndex; // Do this twice, because if the controls are switched to those for a different colour the first time, then the shape will become deselected.
                updateControlsForSelectedItem();
                selectedMaterialRegionIndex=newSelectedRegionIndex;
                updateControlsForSelectedItem();
                
                materialRegionList.setSelectedIndex(newSelectedRegionIndex); // And update the JList
            }
            
            void updateMaterialRegionList() {
                DefaultListModel theListModel=(DefaultListModel)materialRegionList.getModel(); // Get the list model.
                int theSelectedMaterialRegionIndex=selectedMaterialRegionIndex; // We need to cache this here because the value will get upset when we remove all the items from the list - that will cause messages to be sent about the selection having changed.
                theListModel.removeAllElements(); // Remove all the items from the list.
                Iterator currentRegion=materialRegions.iterator();
                while (currentRegion.hasNext()) {
                    Region theRegion=(Region)currentRegion.next(); // Get the current Region.
                    String listItem=theRegion.getName()+" ("+theRegion.getShapeDescription()+")"; // Create a string describing the region, made up from the material's name and the type of shape.
                    theListModel.addElement(listItem); // And add it to the list.
                }
                
                selectedMaterialRegionIndex=theSelectedMaterialRegionIndex; // Now we can restore this because the list is back to its proper state.
                
                if (selectedMaterialRegionIndex!=-1) { // If a region is selected then set it to be selected in the Jlist.
                    materialRegionList.setSelectedIndex(selectedMaterialRegionIndex);
                }
                
                materialRegionList.scrollRectToVisible(new java.awt.Rectangle(0, materialRegionList.getHeight()+50, 1, 1)); // Make sure that the new item is visible in the scroll pane by scrolling to the bottom. (And scroll to the left of the pane.) The +50 ensures that we get right to the bottom of the scrollpane - I think the lists size hasn't been updated by this point so this is necessary.
            }
            
            void deselectRegion() { // Set the container so that no region is selected.
                selectedMaterialRegionIndex=-1;
                materialRegionList.clearSelection(); // And deselect the region in the JList
            }
            
            void addRectangle(double x1, double y1, double x2, double y2) { // We're adding a material region with these coordinates in KM.
                materialRegions.add(new MaterialRectangle(x1, y1, x2, y2, drawSolidShapesCheckBox.isSelected(), getSelectedMaterial())); // Add a region with it's boundaries specified in km, either solid or not accoding to checkbox setting, and of the current material type.
                selectedMaterialRegionIndex=materialRegions.size()-1; // Make the new region be the selected one.
                updateMaterialRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint(); // We've added a region - so now we need to get the on screen view updated.
            }
            
            void addTriangle(double x1, double y1, double x2, double y2, double x3, double y3) { // We're adding a material region with these coordinates in KM.
                materialRegions.add(new MaterialTriangle(x1, y1, x2, y2, x3, y3, drawSolidShapesCheckBox.isSelected(), getSelectedMaterial())); // Add a triangle with it's boundaries specified in km, either solid or not accoding to checkbox setting, and of the current material type.
                selectedMaterialRegionIndex=materialRegions.size()-1; // Make the new region be the selected one.
                updateMaterialRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint(); // We've added a region - so now we need to get the on screen view updated.
            }
            
            void addCircle(double x, double z, double radius) {
                materialRegions.add(new MaterialCircle(x, z, radius, drawSolidShapesCheckBox.isSelected(), getSelectedMaterial()));
                selectedMaterialRegionIndex=materialRegions.size()-1; // Make the new region be the selected one.
                updateMaterialRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint();
            }
            
            void paintAllRegions(Graphics graphics) { // Draw all material regions in the window pointed to by graphics.
                Iterator currentRegion=materialRegions.iterator();
                while (currentRegion.hasNext()) {
                    ((Region)currentRegion.next()).draw(graphics); // Call the base class draw function to draw each region - it's polymorphic, so can be used to draw all types of region.
                }
            }
            
            boolean paintSelectedRegionNextTime=true;
            void paintSelectedRegion(Graphics graphicsContext) { // Paints the selected region (if there is one) with a blue XOR border.
                if (selectedMaterialRegionIndex==-1) { // If there's no selected region there's nothing to do.
                    return;
                } else {
                    if (paintSelectedRegionNextTime || !animateSelectedRegionsCheckBox.isSelected()) { // Every second time, draw the selected region in XOR birghtpink. Unless we're not doing animations, in which case always highlight the selected region.
                        Graphics copyOfGraphicsContext=graphicsContext.create(); // Make a copy of the context, so the changes we make won't affect any other drawing operations.
                        
                        copyOfGraphicsContext.setXORMode(Colours.brightPink); // Draw shape by XORing between black and white.
                        
                        ((Region)materialRegions.get(selectedMaterialRegionIndex)).draw(copyOfGraphicsContext);
                    }
                }
                paintSelectedRegionNextTime= ! paintSelectedRegionNextTime;
            }
            
            void updateControlsForSelectedItem() { // Update the controls to reflect the selected item (if any), else clear them.
                if (materialRegions.size()==0 || selectedMaterialRegionIndex==-1) { // If there aren't any regions, or none is selected, blank out the controls.
                    if (rectangleControls.isShowing()) {
                        blankRectangleControls();
                    } else if (circleControls.isShowing()) {
                        blankCircleControls();
                    }
                } else { // Otherwise update the controls based on the selected region, and make sure the correct controls are showing.
                    Region selectedRegion=(Region)materialRegions.get(selectedMaterialRegionIndex);
                    selectedRegion.showControls(); // Polymorphic code to show the appropriate kind of controls.
                    selectedRegion.updateControls(); // And to update them.
                }
            }
            
        } // End of MaterialRegionContainer
        
        class TracerRegionContainer {
            private LinkedList tracerRegions=new LinkedList();
            private int selectedTracerRegionIndex=-1; // This will be used to keep track of the index of which region is selected.
            
            void parseAndRemoveTracerRegions() { // Take the tracer regions from files.inputFile, and put them in the tracer region container.
                tracerRegions= new LinkedList(); // Start by getting rid of all the old regions (if any).
                selectedTracerRegionIndex=-1; // And no region will be selected initially.
                
                useDefaultTracerSetup.setSelected(false); // And unset the use default tracer checkbox, because we're going to use the tracer regions we load in (possible none), not the default (the user can always switch it on again if they want).
                
                parseAndRemoveRectangles();
                
                updateTracerRegionList(); // Update the JList to reflect the new regions.

            }
            
            void parseAndRemoveRectangles() {
                
                String parameterPrefix="Tracer_rect";
                if (files.checkThatParameterIsDefined(parameterPrefix) || files.checkThatParameterIsDefined(parameterPrefix+"_x1") || files.checkThatParameterIsDefined(parameterPrefix+"_x2") || files.checkThatParameterIsDefined(parameterPrefix+"_z1") || files.checkThatParameterIsDefined(parameterPrefix+"_z2") || files.checkThatParameterIsDefined(parameterPrefix+"_density")) { // If no parameters are defined for this tracer shape, just ignore it completely. But if at least one is, then we will continue.
                    
                    String rectangleSettings=files.getAllLinesForSettingsWithThisPrefix(parameterPrefix); // Store all the settings incase we encounter an error.
                    
                    try {
                        int numberOfRectangles=files.parseInteger(parameterPrefix);
                        parseAndRemoveThisManyRectangles(parameterPrefix, numberOfRectangles);
                        
                    } catch (ParameterNotFoundException exception) {
                        files.linesWithErrors+=rectangleSettings; // If one of the parameters isn't found, add all the settings for this property to linesWithErrors.
                    } catch (ParameterSyntaxErrorException exception) {
                        files.linesWithErrors=files.removeLastLineFromString(files.linesWithErrors)+rectangleSettings; // If there is a syntax error with one or more parameters, it will already have been added to lines with errors, but we'll now add all of the lines in, so just delete the last line.
                    } catch (ParameterIncompatibilityException exception) {
                        files.linesWithErrors+=rectangleSettings;
                    }
                }
                
            }

            void parseAndRemoveThisManyRectangles(String parameterPrefix, int numberOfRectangles) throws ParameterNotFoundException, ParameterSyntaxErrorException, ParameterIncompatibilityException {
                    String density=files.parseAndRemoveList(parameterPrefix+"_density");
                    String x1=files.parseAndRemoveList(parameterPrefix+"_x1");
                    String x2=files.parseAndRemoveList(parameterPrefix+"_x2");
                    String z1=files.parseAndRemoveList(parameterPrefix+"_z1");
                    String z2=files.parseAndRemoveList(parameterPrefix+"_z2");
                    
                    String colours;
                    String filled;
                    try {
                        colours=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_colours");
                        filled=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_filled");
                    } catch (ParameterNotFoundException exception) {
                        colours=files.makeListOfIntegers(Colours.GRAY.getRGB(), numberOfRectangles); // If these parameters aren't found, that's not an error - they will only exist if the file was originally created by the GUI, but we have to make lists for them to compensate for their absence.
                        filled=files.makeListOfStrings("unfilled", numberOfRectangles);
                    }
                    
                    for (int regionIndex=1; regionIndex<=numberOfRectangles; ++regionIndex) {
                        try {
                            tracerRegions.add(new TracerRectangle(scaling.ellipsisUnits2KM(Statics.nthReal(x1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(x2, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z2, regionIndex)), Statics.filledOrUnfilled2Boolean(Statics.nthItem(filled, regionIndex)), Color.decode(Statics.nthItem(colours, regionIndex)), Statics.nthInteger(density, regionIndex))); // Add a region with it's boundaries specified in km, either solid or not solid, with the right colour, and with the right density, according to the files settings.
                        } catch (ParameterSyntaxErrorException exception) {
                            files.linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                            throw exception; // And then re-throw the exception
                        }
                    }
                    
            }
            
            String getFileSettings() { // Returns a string specifying all the boundary region settings in ellipsis input file format.
                String settings="# TRACERS\n";
                
                if (useDefaultTracerSetup.isSelected()) { // If user has selected just to use a default tracer, covering the whole window.
                    settings+=getTracerFileSettingsForDefault();
                } else { // Otherwise output all the defined regions (if any).
                    settings+=getTracerRegionFileSettings();
                }
                
                return settings;
            }
            
            private String getTracerFileSettingsForDefault() {
                String settings="\n# This defines a single tracer which covers the whole simulation area.\n";
                settings+="Tracer_rect=1\n";
                settings+="Tracer_rect_density="+tracerDensityTextField.getText(); // But still get the density set by the user.
                settings+="\nTracer_rect_x1=0\n"; // And the rectangle covers the whole simulation area.
                settings+="Tracer_rect_x2="+scaling.KM2EllipsisUnits(windowDimensionX);
                settings+="\nTracer_rect_z1=0\n";
                settings+="Tracer_rect_z2="+scaling.KM2EllipsisUnits(windowDimensionY)+"\n\n";
                
                return settings;
            }
            
            private String getTracerRegionFileSettings() {
                String settings="\n# Tracer Regions\n\n";
                
                if (tracerRegions.size()>0) { // Only add tracer regions if there is at least one. (Though perhaps ellipsis requires at least one?).
                    
                    settings+="Tracer_rect="+tracerRegions.size()+"\n";
                    
                    String density="";
                    
                    String x1="";
                    String x2="";
                    String z1="";
                    String z2="";
                    
                    String colours=""; // This is the colour with which the boundary is drawn in the Ellipsis Gui.
                    String filled=""; // Thiw is whether the region is drawn filled in the Ellipsis Gui.
                    
                    Iterator currentRegion=tracerRegions.iterator();
                    while (currentRegion.hasNext()) {
                        TracerRectangle rectangle=(TracerRectangle)currentRegion.next(); // Get the current rectangle.
                        
                        density+=rectangle.tracerDensity+",";
                        x1+=scaling.KM2EllipsisUnits(rectangle.x1)+",";
                        x2+=scaling.KM2EllipsisUnits(rectangle.x2)+",";
                        z1+=scaling.KM2EllipsisUnits(rectangle.z1)+",";
                        z2+=scaling.KM2EllipsisUnits(rectangle.z2)+",";
                        
                        colours+=rectangle.rectangleColour.getRGB()+","; // Output the circles colour in RGB format (is a kind of int).
                        filled+=Statics.boolean2filledOrUnfilled(rectangle.filled)+",";
                    }
                    
                    settings+="Tracer_rect_density="+Useful.chopAnyTrailingComma(density)+"\n"; // Now add the fields to the settings string.
                    settings+="Tracer_rect_x1="+Useful.chopAnyTrailingComma(x1)+"\n"; // Now add the fields to the settings string.
                    settings+="Tracer_rect_x2="+Useful.chopAnyTrailingComma(x2)+"\n"; // Now add the fields to the settings string.
                    settings+="Tracer_rect_z1="+Useful.chopAnyTrailingComma(z1)+"\n"; // Now add the fields to the settings string.
                    settings+="Tracer_rect_z2="+Useful.chopAnyTrailingComma(z2)+"\n"; // Now add the fields to the settings string.
                    
                    settings+=files.ellipsisGUIParameterPrefix+"Tracer_rect_colours="+Useful.chopAnyTrailingComma(colours)+"\n";
                    settings+=files.ellipsisGUIParameterPrefix+"Tracer_rect_filled="+Useful.chopAnyTrailingComma(filled)+"\n\n";
                }
                
                return settings;
            }
            
            void updateSelectedRectangle() {
                if (selectedTracerRegionIndex==-1) return; // If no region is selected, do nothing.
                if (useDefaultTracerSetup.isSelected()) return; // And if we'return not using tracer regions do nothing.
                
                RectangleCoordinates coordinates=new RectangleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((MyRectangle)tracerRegions.get(selectedTracerRegionIndex)).update(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                repaint();
            }
            
            void updateSelectedCircle() {
                if (selectedTracerRegionIndex==-1) return; // If no region is selected, do nothing.
                if (useDefaultTracerSetup.isSelected()) return; // And if we'return not using tracer regions do nothing.
                
                CircleCoordinates coordinates=new CircleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((Circle)tracerRegions.get(selectedTracerRegionIndex)).update(coordinates.x, coordinates.z, coordinates.radius, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                repaint();
            }
            
            void deleteSelectedRegion() {
                if (selectedTracerRegionIndex==-1) return; // If no region is selected, do nothing.
                tracerRegions.remove(selectedTracerRegionIndex); // Remove the region from the list of regions.
                selectedTracerRegionIndex=-1;
                
                updateTracerRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted without this region.
            }
            
            void moveSelectedRegionForward() {
                if (selectedTracerRegionIndex==-1 || selectedTracerRegionIndex==tracerRegions.size()-1) return; // If no region is selected, or the region is already at the front, do nothing.
                Region theRegion=(Region)tracerRegions.remove(selectedTracerRegionIndex); // Take the region out of the list
                tracerRegions.add(selectedTracerRegionIndex+1, theRegion); // And put it back one further down.
                ++selectedTracerRegionIndex; // Keep the same region selected.
                
                updateTracerRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted with this region in its new position.
            }
            
            void moveSelectedRegionBackward() {
                if (selectedTracerRegionIndex==-1 || selectedTracerRegionIndex==0) return; // If no region is selected, or the region is already at the back, do nothing.
                Region theRegion=(Region)tracerRegions.remove(selectedTracerRegionIndex); // Take the region out of the list
                tracerRegions.add(selectedTracerRegionIndex-1, theRegion); // And put it back one further up.
                --selectedTracerRegionIndex; // Keep the same region selected.
                
                updateTracerRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted with this region in its new position.
            }
            
            void setSelectedRegion(int newSelectedRegionIndex) { // If we've changed regions, then we need to update the drawing controls to reflect that.
                selectedTracerRegionIndex=newSelectedRegionIndex; // Do this twice, because if the controls are switched to those for a different colour the first time, then the shape will become deselected.
                updateControlsForSelectedItem();
                selectedTracerRegionIndex=newSelectedRegionIndex;
                updateControlsForSelectedItem();
                
                tracerRegionList.setSelectedIndex(newSelectedRegionIndex); // And update the JList
            }
            
            void updateTracerRegionList() {
                DefaultListModel theListModel=(DefaultListModel)tracerRegionList.getModel(); // Get the list model.
                int theSelectedTracerRegionIndex=selectedTracerRegionIndex; // We need to cache this here because the value will get upset when we remove all the items from the list - that will cause messages to be sent about the selection having changed.
                theListModel.removeAllElements(); // Remove all the items from the list.
                Iterator currentRegion=tracerRegions.iterator();
                while (currentRegion.hasNext()) {
                    Region theRegion=(Region)currentRegion.next(); // Get the current Region.
                    String listItem=theRegion.getShapeDescription(); // Create a string describing the region, made up from the tracer's name and the type of shape.
                    theListModel.addElement(listItem); // And add it to the list.
                }
                
                selectedTracerRegionIndex=theSelectedTracerRegionIndex; // Now we can restore this because the list is back to its proper state.
                
                if (selectedTracerRegionIndex!=-1) { // If a region is selected then set it to be selected in the Jlist.
                    tracerRegionList.setSelectedIndex(selectedTracerRegionIndex);
                }
                
                tracerRegionList.scrollRectToVisible(new java.awt.Rectangle(0, tracerRegionList.getHeight()+50, 1, 1)); // Make sure that the new item is visible in the scroll pane by scrolling to the bottom. (And scroll to the left of the pane.) The +50 ensures that we get right to the bottom of the scrollpane - I think the lists size hasn't been updated by this point so this is necessary.
            }
            
            void deselectRegion() { // Set the container so that no region is selected.
                selectedTracerRegionIndex=-1;
                tracerRegionList.clearSelection(); // And deselect the region in the JList
            }
            
            void addRectangle(double x1, double y1, double x2, double y2) { // We're adding a tracer region with these coordinates in KM.
                if (useDefaultTracerSetup.isSelected()) return; // And if we'return not using tracer regions do nothing.

                tracerRegions.add(new TracerRectangle(x1, y1, x2, y2, drawSolidShapesCheckBox.isSelected(), tracerChangeColourButton.getBackground(), Integer.parseInt(tracerDensityTextField.getText()))); // Add a region with it's boundaries specified in km, either solid or not accoding to checkbox setting, and with the currently selected tracer properties.
                selectedTracerRegionIndex=tracerRegions.size()-1; // Make the new region be the selected one.
                updateTracerRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint(); // We've added a region - so now we need to get the on screen view updated.
            }
                        
            void paintAllRegions(Graphics graphics) { // Draw all tracer regions in the window pointed to by graphics.
                Iterator currentRegion=tracerRegions.iterator();
                while (currentRegion.hasNext()) {
                    ((Region)currentRegion.next()).draw(graphics); // Call the base class draw function to draw each region - it's polymorphic, so can be used to draw all types of region.
                }
            }
            
            boolean paintSelectedRegionNextTime=true;
            void paintSelectedRegion(Graphics graphicsContext) { // Paints the selected region (if there is one) with a blue XOR border.
                if (selectedTracerRegionIndex==-1) { // If there's no selected region there's nothing to do.
                    return;
                } else {
                    if (paintSelectedRegionNextTime || !animateSelectedRegionsCheckBox.isSelected()) { // Every second time, draw the selected region in XOR birghtpink.
                        Graphics copyOfGraphicsContext=graphicsContext.create(); // Make a copy of the context, so the changes we make won't affect any other drawing operations.
                        
                        copyOfGraphicsContext.setXORMode(Colours.brightPink); // Draw shape by XORing between black and white.
                        
                        ((Region)tracerRegions.get(selectedTracerRegionIndex)).draw(copyOfGraphicsContext);
                    }
                }
                paintSelectedRegionNextTime= ! paintSelectedRegionNextTime;
            }
            
            void updateControlsForSelectedItem() { // Update the controls to reflect the selected item (if any), else clear them.
                if (tracerRegions.size()==0 || selectedTracerRegionIndex==-1) { // If there aren't any regions, or none is selected, blank out the controls.
                    if (rectangleControls.isShowing()) {
                        blankRectangleControls();
                    } else if (circleControls.isShowing()) {
                        blankCircleControls();
                    }
                } else { // Otherwise update the controls based on the selected region, and make sure the correct controls are showing.
                    Region selectedRegion=(Region)tracerRegions.get(selectedTracerRegionIndex);
                    selectedRegion.showControls(); // Polymorphic code to show the appropriate kind of controls.
                    selectedRegion.updateControls(); // And to update them.
                }
            }
            
        } // End of TracerRegionContainer

        
        class BoundaryRegionContainer {
            private LinkedList boundaryRegions=new LinkedList();
            private int selectedBoundaryRegionIndex=-1; // This will be used to keep track of the index of which region is selected.
                   
            void parseAndRemoveBoundaryRegions() { // Take the boundary regions from files.inputFile, and put them in the boundary region container.
                boundaryRegions= new LinkedList(); // Start by getting rid of all the old regions (if any).
                selectedBoundaryRegionIndex=-1; // And no region will be selected initially.
                
                parseAndRemoveRectangles();
                
                parseAndRemoveCircles();
                
                updateBoundaryRegionList(); // Update the JList to reflect the new regions.

            }
            
            void parseAndRemoveRectangles() {
                
                for (int property=Boundary.temperature; property<=Boundary.velocity_z; ++property) {
                    String parameterPrefix=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect";
                    if (files.checkThatParameterIsDefined(parameterPrefix) || files.checkThatParameterIsDefined(parameterPrefix+"_norm") || files.checkThatParameterIsDefined(parameterPrefix+"_icpt") || files.checkThatParameterIsDefined(parameterPrefix+"_aa1") || files.checkThatParameterIsDefined(parameterPrefix+"_aa2") || files.checkThatParameterIsDefined(parameterPrefix+"_bb1") || files.checkThatParameterIsDefined(parameterPrefix+"_bb2") || files.checkThatParameterIsDefined(parameterPrefix+"_hw") || files.checkThatParameterIsDefined(parameterPrefix+"_mag")) { // If no parameters are defined for this boundary property, just ignore it completely. But if at least one is, then we will continue.
                        
                        String propertySettings=files.getAllLinesForSettingsWithThisPrefix(parameterPrefix); // Store all the settings incase we encounter an error.
                        
                        try {
                            int numberOfRectangles=files.parseInteger(parameterPrefix);
                            parseAndRemoveThisManyRectangles(parameterPrefix, numberOfRectangles, property);
                            
                        } catch (ParameterNotFoundException exception) {
                            files.linesWithErrors+=propertySettings; // If one of the parameters isn't found, add all the settings for this property to linesWithErrors.
                        } catch (ParameterSyntaxErrorException exception) {
                            files.linesWithErrors=files.removeLastLineFromString(files.linesWithErrors)+propertySettings; // If there is a syntax error with one or more parameters, it will already have been added to lines with errors, but we'll now add all of the lines in, so just delete the last line.
                        } catch (ParameterIncompatibilityException exception) {
                            files.linesWithErrors+=propertySettings;
                        }
                    }
                    
                }
                
            }

            void parseAndRemoveThisManyRectangles(String parameterPrefix, int numberOfRectangles, int property) throws ParameterNotFoundException, ParameterSyntaxErrorException, ParameterIncompatibilityException {
                    String norm=files.parseAndRemoveList(parameterPrefix+"_norm");
                    String icpt=files.parseAndRemoveList(parameterPrefix+"_icpt");
                    String aa1=files.parseAndRemoveList(parameterPrefix+"_aa1");
                    String aa2=files.parseAndRemoveList(parameterPrefix+"_aa2");
//                    String bb1=files.parseAndRemoveList(parameterPrefix+"_bb1");
//                    String bb2=files.parseAndRemoveList(parameterPrefix+"_bb2");
                    String hw=files.parseAndRemoveList(parameterPrefix+"_hw");
                    String mag=files.parseAndRemoveList(parameterPrefix+"_mag");

                    String bb1;
                    try { // If either bb1 or bb2 are missing that's OK - just set them to their default values of 0 - this is because it is normal to just set boundaries at the top of the simulation area, and many existing files leave bb1 and bb2 unspecified.
                        bb1=files.parseAndRemoveList(parameterPrefix+"_bb1");
                    } catch (ParameterNotFoundException exception) {
                        bb1=files.makeListOfIntegers(0, numberOfRectangles);
                    }
                    
                    String bb2;
                    try {
                        bb2=files.parseAndRemoveList(parameterPrefix+"_bb2");
                    } catch (ParameterNotFoundException exception) {
                        bb2=files.makeListOfIntegers(0, numberOfRectangles);
                    }
                    
                    String colours;
                    String filled;
                    try {
                        colours=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_colours");
                        filled=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_filled");
                    } catch (ParameterNotFoundException exception) {
                        colours=files.makeListOfIntegers(Colours.GRAY.getRGB(), numberOfRectangles); // If these parameters aren't found, that's not an error - they will only exist if the file was originally created by the GUI, but we have to make lists for them to compensate for their absence.
                        filled=files.makeListOfStrings("unfilled", numberOfRectangles);
                    }
                    
                    for (int regionIndex=1; regionIndex<=numberOfRectangles; ++regionIndex) {
                        Boundary aBoundary=new Boundary(); // This is a dummy boundary that has to be created just because the compiler doesn't allow Boundary to have static members.
                        try {
                            Boundary theBoundary=new Boundary(files.checkThatCharacterIsXorYorZ(Statics.nthItem(norm, regionIndex).charAt(0)), scaling.ellipsisUnits2KM(Statics.nthReal(icpt, regionIndex)), property, scaling.ellipsisUnits2KM(Statics.nthReal(hw, regionIndex)), aBoundary.scaleValue(Statics.nthReal(mag, regionIndex), property)); // **** Should half width be scaled ???? The property will be scaled appropriately by scaleValue.
                            boundaryRegions.add(new BoundaryRectangle(scaling.ellipsisUnits2KM(Statics.nthReal(aa1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(bb1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(aa2, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(bb2, regionIndex)), Statics.filledOrUnfilled2Boolean(Statics.nthItem(filled, regionIndex)), Color.decode(Statics.nthItem(colours, regionIndex)), theBoundary)); // Add a region with it's boundaries specified in km, either solid or not solid, with the right colour, and with the boundary we made before, according to the files settings.
                        } catch (ParameterSyntaxErrorException exception) {
                            files.linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                            throw exception; // And then re-throw the exception
                        }
                    }
                    
//                } catch (ParameterNotFoundException exception) { // This is an error - because if we have one for number of rectangles, we must have all the ones defining rectangles.
//                    JOptionPane.showMessageDialog(MainWindow.this,"There is a setting for "+parameterPrefix+", but one of the parameters defining the rectangles\nthemselves is missing. No rectangles for this boundary property have been created.", "Error", JOptionPane.ERROR_MESSAGE);
//                } catch (ParameterSyntaxErrorException exception) {
//                    // If we get this error, do nothing - syntax errors are reported anyway -- and if we are doing a merge of files to maintain the original file structure, then the original lines will be left in anyway.
//                } catch (ParameterIncompatibilityException exception) {
//                    // Also do nothing?? Treat like a syntax error exception??
//                }
            }
            
            void parseAndRemoveCircles() {
                
                for (int property=Boundary.temperature; property<=Boundary.velocity_z; ++property) {
                    String parameterPrefix=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_circ";
                    if (files.checkThatParameterIsDefined(parameterPrefix) || files.checkThatParameterIsDefined(parameterPrefix+"_norm") || files.checkThatParameterIsDefined(parameterPrefix+"_icpt") || files.checkThatParameterIsDefined(parameterPrefix+"_aa") || files.checkThatParameterIsDefined(parameterPrefix+"_bb1") || files.checkThatParameterIsDefined(parameterPrefix+"_rad") || files.checkThatParameterIsDefined(parameterPrefix+"_hw") || files.checkThatParameterIsDefined(parameterPrefix+"_mag")) { // If no parameters are defined for this boundary property, just ignore it completely. But if at least one is, then we will continue.
                        
                        String propertySettings=files.getAllLinesForSettingsWithThisPrefix(parameterPrefix); // Store all the settings incase we encounter an error.
                        
                        try {
                            int numberOfCircles=files.parseInteger(parameterPrefix);
                            parseAndRemoveThisManyCircles(parameterPrefix, numberOfCircles, property);
                            
                        } catch (ParameterNotFoundException exception) {
                            files.linesWithErrors+=propertySettings; // If one of the parameters isn't found, add all the settings for this property to linesWithErrors.
                        } catch (ParameterSyntaxErrorException exception) {
                            files.linesWithErrors=files.removeLastLineFromString(files.linesWithErrors)+propertySettings; // If there is a syntax error with one or more parameters, it will already have been added to lines with errors, but we'll now add all of the lines in, so just delete the last line.
                        } catch (ParameterIncompatibilityException exception) {
                            files.linesWithErrors+=propertySettings;
                        }
                    }
                    
                }
                
            }

            void parseAndRemoveThisManyCircles(String parameterPrefix, int numberOfCircles, int property) throws ParameterNotFoundException, ParameterSyntaxErrorException, ParameterIncompatibilityException {
//                try {
                    String norm=files.parseAndRemoveList(parameterPrefix+"_norm");
                    String icpt=files.parseAndRemoveList(parameterPrefix+"_icpt");
                    String aa=files.parseAndRemoveList(parameterPrefix+"_aa");
                    String bb=files.parseAndRemoveList(parameterPrefix+"_bb");
                    String rad=files.parseAndRemoveList(parameterPrefix+"_rad");
                    String hw=files.parseAndRemoveList(parameterPrefix+"_hw");
                    String mag=files.parseAndRemoveList(parameterPrefix+"_mag");
                    
                    String colours;
                    String filled;
                    try {
                        colours=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_colours");
                        filled=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_filled");
                    } catch (ParameterNotFoundException exception) {
                        colours=files.makeListOfIntegers(Colours.GRAY.getRGB(), numberOfCircles); // If these parameters aren't found, that's not an error - they will only exist if the file was originally created by the GUI, but we have to make lists for them to compensate for their absence.
                        filled=files.makeListOfStrings("unfilled", numberOfCircles);
                    }
                    
                    for (int regionIndex=1; regionIndex<=numberOfCircles; ++regionIndex) {
                        Boundary aBoundary=new Boundary(); // This is a dummy boundary that has to be created just because the compiler doesn't allow Boundary to have static members.
                        try {
                            Boundary theBoundary=new Boundary(files.checkThatCharacterIsXorYorZ(Statics.nthItem(norm, regionIndex).charAt(0)), scaling.ellipsisUnits2KM(Statics.nthReal(icpt, regionIndex)), property, scaling.ellipsisUnits2KM(Statics.nthReal(hw, regionIndex)), aBoundary.scaleValue(Statics.nthReal(mag, regionIndex), property)); // The property will be scaled appropriately by scaleValue.
                            boundaryRegions.add(new BoundaryCircle(scaling.ellipsisUnits2KM(Statics.nthReal(aa, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(bb, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(rad, regionIndex)), Statics.filledOrUnfilled2Boolean(Statics.nthItem(filled, regionIndex)), Color.decode(Statics.nthItem(colours, regionIndex)), theBoundary)); // Add a region with it's boundaries specified in km, either solid or not solid, with the right colour, and with the boundary we made before, according to the files settings.
                        } catch (ParameterSyntaxErrorException exception) {
                            files.linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                            throw exception; // And then re-throw the exception
                        }
                    }
                    
            }
            
            String getFileSettings() { // Returns a string specifying all the boundary region settings in ellipsis input file format.
                String settings="# BOUNDARY CONDITIONS\n\n# General Boundary Settings\n\n";

                settings+=getGeneralBoundarySettings();
                
                settings+=getRectangleFileSettings();
                
                settings+=getCircleFileSettings();
                
                return settings;
            }
            
            String getGeneralBoundarySettings() {
                String settings="periodicx="+Statics.boolean2OnOff(periodicxCheckBox.isSelected());
                settings+="periodic_rm_vx="+Statics.boolean2OnOff(periodic_rm_vxCheckBox.isSelected());
                
                if (simpleBoundaryRadioButton.isSelected()) {
                    settings+="BCmoveX0v="+scaling.velocity2EllipsisUnits(BCmoveX0vTextField.getText());
                    settings+="\nBCmoveX1v="+scaling.velocity2EllipsisUnits(BCmoveX1vTextField.getText());
                    settings+="\nBCmoveX0v1="+scaling.velocity2EllipsisUnits(BCmoveX0v1TextField.getText())+"\n";
                }
                
                return settings; 
            }
            
            private String getRectangleFileSettings() {
                String settings="\n# Boundary Rectangles\n\n";
                
                for (int property=Boundary.temperature; property<=Boundary.velocity_z; ++property) {
                    if (numberOfRectangles(property)!=0) { // Only do the settings if at least one boundary with this property exists.
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect="+numberOfRectangles(property)+"\n\n";
                        
                        String norm="";
                        String icpt="";
                        String aa1="";
                        String aa2="";
                        String bb1="";
                        String bb2="";
                        String hw="";
                        String mag="";
                        
                        String colours=""; // This is the colour with which the boundary is drawn in the Ellipsis Gui.
                        String filled=""; // Thiw is whether the region is drawn filled in the Ellipsis Gui.
                        
                        Iterator currentRegion=boundaryRegions.iterator();
                        while (currentRegion.hasNext()) {
                            Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                            if (theRegion instanceof BoundaryRectangle) { // If it's a rectangle,
                                BoundaryRectangle rectangle=(BoundaryRectangle)theRegion;
                                if (rectangle.boundary.property==property) { // and has the right property then add all the right value to the list for each Field.
                                    norm+=rectangle.boundary.axis+","; 
                                    icpt+=scaling.KM2EllipsisUnits(rectangle.boundary.intercept)+",";
                                    aa1+=scaling.KM2EllipsisUnits(rectangle.x1)+",";
                                    aa2+=scaling.KM2EllipsisUnits(rectangle.x2)+",";
                                    bb1+=scaling.KM2EllipsisUnits(rectangle.z1)+",";
                                    bb2+=scaling.KM2EllipsisUnits(rectangle.z2)+",";
                                    hw+=scaling.KM2EllipsisUnits(rectangle.boundary.halfWidth)+",";
                                    mag+=rectangle.boundary.scaleValueToEllipsisUnits(rectangle.boundary.value, property)+","; // *** Value should be scaled***

                                    colours+=rectangle.rectangleColour.getRGB()+","; // Output the circles colour in RGB format (is a kind of int).
                                    filled+=Statics.boolean2filledOrUnfilled(rectangle.filled)+",";
                                }
                            }
                        }
                        
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect_norm="+Useful.chopAnyTrailingComma(norm)+"\n"; // Now add the fields to the settings string.
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect_icpt="+Useful.chopAnyTrailingComma(icpt)+"\n";
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect_aa1="+Useful.chopAnyTrailingComma(aa1)+"\n";
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect_aa2="+Useful.chopAnyTrailingComma(aa2)+"\n";
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect_bb1="+Useful.chopAnyTrailingComma(bb1)+"\n";
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect_bb2="+Useful.chopAnyTrailingComma(bb2)+"\n";
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect_hw="+Useful.chopAnyTrailingComma(hw)+"\n";
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect_mag="+Useful.chopAnyTrailingComma(mag)+"\n";

                        settings+=files.ellipsisGUIParameterPrefix+Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect_colours="+Useful.chopAnyTrailingComma(colours)+"\n";
                        settings+=files.ellipsisGUIParameterPrefix+Statics.getBoundaryPropertyEllipsisName(property)+"_bc_rect_filled="+Useful.chopAnyTrailingComma(filled)+"\n\n";
                    }
                } // Do the same forthe next property.
                
                return settings;
            }
            
            
            private String getCircleFileSettings() {
                String settings="# Boundary Circles\n\n";
                
                for (int property=Boundary.temperature; property<=Boundary.velocity_z; ++property) {
                    if (numberOfCircles(property)!=0) { 
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_circ="+numberOfCircles(property)+"\n\n";
                        
                        String norm="";
                        String icpt="";
                        String aa="";
                        String bb="";
                        String rad="";
                        String hw="";
                        String mag="";
                        
                        String colours=""; // This is the colour with which the boundary is drawn in the Ellipsis Gui.
                        String filled=""; // Thiw is whether the region is drawn filled in the Ellipsis Gui.

                        Iterator currentRegion=boundaryRegions.iterator();
                        while (currentRegion.hasNext()) {
                            Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                            if (theRegion instanceof BoundaryCircle) { // If it's a circle,
                                BoundaryCircle circle=(BoundaryCircle)theRegion;
                                if (circle.boundary.property==property) { // and has the right property then add all the right value to the list for each Field.
                                    norm+=circle.boundary.axis+","; 
                                    icpt+=scaling.KM2EllipsisUnits(circle.boundary.intercept)+","; 
                                    aa+=scaling.KM2EllipsisUnits(circle.x)+",";
                                    bb+=scaling.KM2EllipsisUnits(circle.z)+",";
                                    rad+=scaling.KM2EllipsisUnits(circle.radius)+",";
                                    hw+=scaling.KM2EllipsisUnits(circle.boundary.halfWidth)+","; 
                                    mag+=circle.boundary.scaleValueToEllipsisUnits(circle.boundary.value, property)+",";
                                    
                                    colours+=circle.circleColour.getRGB()+","; // Output the circles colour in RGB format (is a kind of int).
                                    filled+=Statics.boolean2filledOrUnfilled(circle.filled)+",";
                                }
                            }
                        }
                        
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_circ_norm="+Useful.chopAnyTrailingComma(norm)+"\n"; // Now add the fields to the settings string.
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_circ_icpt="+Useful.chopAnyTrailingComma(icpt)+"\n";
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_circ_aa="+Useful.chopAnyTrailingComma(aa)+"\n";
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_circ_bb="+Useful.chopAnyTrailingComma(bb)+"\n";
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_circ_rad="+Useful.chopAnyTrailingComma(rad)+"\n";
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_circ_hw="+Useful.chopAnyTrailingComma(hw)+"\n";
                        settings+=Statics.getBoundaryPropertyEllipsisName(property)+"_bc_circ_mag="+Useful.chopAnyTrailingComma(mag)+"\n";

                        settings+=files.ellipsisGUIParameterPrefix+Statics.getBoundaryPropertyEllipsisName(property)+"_bc_circ_colours="+Useful.chopAnyTrailingComma(colours)+"\n";
                        settings+=files.ellipsisGUIParameterPrefix+Statics.getBoundaryPropertyEllipsisName(property)+"_bc_circ_filled="+Useful.chopAnyTrailingComma(filled)+"\n\n";
                    }
                } // Do the same for the next property.
                
                return settings;
            }
            
            private int numberOfRectangles(int property) { // Return the number of boundary rectangle regions with the given property.
                int numberOfRectangles=0;
                Iterator currentRegion=boundaryRegions.iterator();
                while (currentRegion.hasNext()) {
                    Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                    if (theRegion instanceof BoundaryRectangle) { // If it's a rectangle, 
                        BoundaryRectangle rectangle=(BoundaryRectangle)theRegion;
                        if (rectangle.boundary.property==property) { // and has the right property.
                            ++numberOfRectangles; // Then count it.
                        }
                    }
                }
                
                return numberOfRectangles;
            }
 
            
            private int numberOfCircles(int property) { // Return the number of boundary rectangle regions with the given property.
                int numberOfCircles=0;
                Iterator currentRegion=boundaryRegions.iterator();
                while (currentRegion.hasNext()) {
                    Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                    if (theRegion instanceof BoundaryCircle) { // If it's a rectangle, 
                        BoundaryCircle circle=(BoundaryCircle)theRegion;
                        if (circle.boundary.property==property) { // and has the right property.
                            ++numberOfCircles; // Then count it.
                        }
                    }
                }
                
                return numberOfCircles;
            }
 
            void updateSelectedRectangle() {
                if (selectedBoundaryRegionIndex==-1) return; // If no region is selected, do nothing.
                
                RectangleCoordinates coordinates=new RectangleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((MyRectangle)boundaryRegions.get(selectedBoundaryRegionIndex)).update(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                updateBoundaryRegionList(); // We need to do this incase the boundary property has been changed.
                repaint();
            }
            
            void updateSelectedCircle() {
                if (selectedBoundaryRegionIndex==-1) return; // If no region is selected, do nothing.
                
                CircleCoordinates coordinates=new CircleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((Circle)boundaryRegions.get(selectedBoundaryRegionIndex)).update(coordinates.x, coordinates.z, coordinates.radius, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                updateBoundaryRegionList(); // We need to do this incase the boundary property has been changed.
                repaint();
            }
            
            void deleteSelectedRegion() {
                if (selectedBoundaryRegionIndex==-1) return; // If no region is selected, do nothing.
                boundaryRegions.remove(selectedBoundaryRegionIndex); // Remove the region from the list of regions.
                selectedBoundaryRegionIndex=-1;
                
                updateBoundaryRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted without this region.
            }
            
            void moveSelectedRegionForward() {
                if (selectedBoundaryRegionIndex==-1 || selectedBoundaryRegionIndex==boundaryRegions.size()-1) return; // If no region is selected, or the region is already at the front, do nothing.
                Region theRegion=(Region)boundaryRegions.remove(selectedBoundaryRegionIndex); // Take the region out of the list
                boundaryRegions.add(selectedBoundaryRegionIndex+1, theRegion); // And put it back one further down.
                ++selectedBoundaryRegionIndex; // Keep the same region selected.
                
                updateBoundaryRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted with this region in its new position.
            }
            
            void moveSelectedRegionBackward() {
                if (selectedBoundaryRegionIndex==-1 || selectedBoundaryRegionIndex==0) return; // If no region is selected, or the region is already at the back, do nothing.
                Region theRegion=(Region)boundaryRegions.remove(selectedBoundaryRegionIndex); // Take the region out of the list
                boundaryRegions.add(selectedBoundaryRegionIndex-1, theRegion); // And put it back one further up.
                --selectedBoundaryRegionIndex; // Keep the same region selected.
                
                updateBoundaryRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted with this region in its new position.
            }
            
            void setSelectedRegion(int newSelectedRegionIndex) { // If we've changed regions, then we need to update the drawing controls to reflect that.
                selectedBoundaryRegionIndex=newSelectedRegionIndex; // Do this twice, because if the controls are switched to those for a different colour the first time, then the shape will become deselected.
                updateControlsForSelectedItem();
                selectedBoundaryRegionIndex=newSelectedRegionIndex;
                updateControlsForSelectedItem();
                
                boundaryRegionList.setSelectedIndex(newSelectedRegionIndex); // And update the JList
            }
            
            void updateBoundaryRegionList() {
                DefaultListModel theListModel=(DefaultListModel)boundaryRegionList.getModel(); // Get the list model.
                int theSelectedBoundaryRegionIndex=selectedBoundaryRegionIndex; // We need to cache this here because the value will get upset when we remove all the items from the list - that will cause messages to be sent about the selection having changed.
                theListModel.removeAllElements(); // Remove all the items from the list.
                Iterator currentRegion=boundaryRegions.iterator();
                while (currentRegion.hasNext()) {
                    Region theRegion=(Region)currentRegion.next(); // Get the current Region.
                    String listItem=theRegion.getName()+" ("+theRegion.getShapeDescription()+")"; // Create a string describing the region, made up from the boundary's property and the type of shape.
                    theListModel.addElement(listItem); // And add it to the list.
                }
                
                selectedBoundaryRegionIndex=theSelectedBoundaryRegionIndex; // Now we can restore this because the list is back to its proper state.
                
                if (selectedBoundaryRegionIndex!=-1) { // If a region is selected then set it to be selected in the Jlist.
                    boundaryRegionList.setSelectedIndex(selectedBoundaryRegionIndex);
                }
                
                boundaryRegionList.scrollRectToVisible(new java.awt.Rectangle(0, boundaryRegionList.getHeight()+50, 1, 1)); // Make sure that the new item is visible in the scroll pane by scrolling to the bottom. (And scroll to the left of the pane.) The +50 ensures that we get right to the bottom of the scrollpane - I think the lists size hasn't been updated by this point so this is necessary.
            }
            
            void deselectRegion() { // Set the container so that no region is selected.
                selectedBoundaryRegionIndex=-1;
                boundaryRegionList.clearSelection(); // And deselect the region in the JList
            }
            
            void addRectangle(double x1, double y1, double x2, double y2) { // We're adding a boundary region with these coordinates in KM.
                boundaryRegions.add(new BoundaryRectangle(x1, y1, x2, y2, drawSolidShapesCheckBox.isSelected(), boundaryChangeColourButton.getBackground(), new Boundary())); // Add a region with it's boundaries specified in km, either solid or not accoding to checkbox setting, and with the currently selected boundary properties.
                selectedBoundaryRegionIndex=boundaryRegions.size()-1; // Make the new region be the selected one.
                updateBoundaryRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint(); // We've added a region - so now we need to get the on screen view updated.
            }
            
            void addCircle(double x, double z, double radius) {
                boundaryRegions.add(new BoundaryCircle(x, z, radius, drawSolidShapesCheckBox.isSelected(), boundaryChangeColourButton.getBackground(), new Boundary()));
                selectedBoundaryRegionIndex=boundaryRegions.size()-1; // Make the new region be the selected one.
                updateBoundaryRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint();
            }
            
            void paintAllRegions(Graphics graphics) { // Draw all boundary regions in the window pointed to by graphics. N.B> Actually only draw those regions which are normal to the y axis (ie in the x z plane). The other ones won't be displayed graphically at all.
                Iterator currentRegion=boundaryRegions.iterator();
                while (currentRegion.hasNext()) {
                    Region theRegion=(Region)currentRegion.next(); // Get the region.
                    if (theRegion instanceof BoundaryCircle) {
                        if (((BoundaryCircle)theRegion).boundary.axis=='Y') { // Only draw regionss that are in the XZ plane.
                            theRegion.draw(graphics); // Call the base class draw function to draw each region - it's polymorphic, so can be used to draw all types of region.
                        }
                    } else if (theRegion instanceof BoundaryRectangle) {
                        if (((BoundaryRectangle)theRegion).boundary.axis=='Y') { // Only draw regionss that are in the XZ plane.
                            theRegion.draw(graphics); // Call the base class draw function to draw each region - it's polymorphic, so can be used to draw all types of region.
                        }
                    }
                }
            }

            boolean paintSelectedRegionNextTime=true;
            void paintSelectedRegion(Graphics graphicsContext) { // Paints the selected region (if there is one) with a blue XOR border.
                if (selectedBoundaryRegionIndex==-1) { // If there's no selected region there's nothing to do.
                    return;
                } else {
                    if (paintSelectedRegionNextTime || !animateSelectedRegionsCheckBox.isSelected()) { // Every second time, draw the selected region in XOR birghtpink.
                        Graphics copyOfGraphicsContext=graphicsContext.create(); // Make a copy of the context, so the changes we make won't affect any other drawing operations.
                        
                        copyOfGraphicsContext.setXORMode(Colours.brightPink); // Draw shape by XORing between black and white.
                        
                        Region theRegion=((Region)boundaryRegions.get(selectedBoundaryRegionIndex));
                        if (theRegion instanceof BoundaryCircle) {
                            if (((BoundaryCircle)theRegion).boundary.axis=='Y') { // Only draw regionss that are in the XZ plane.
                                theRegion.draw(copyOfGraphicsContext); // Call the base class draw function to draw each region - it's polymorphic, so can be used to draw all types of region.
                            }
                        } else if (theRegion instanceof BoundaryRectangle) {
                            if (((BoundaryRectangle)theRegion).boundary.axis=='Y') { // Only draw regionss that are in the XZ plane.
                                theRegion.draw(copyOfGraphicsContext); // Call the base class draw function to draw each region - it's polymorphic, so can be used to draw all types of region.
                            }
                        }                        
                    }
                }
                paintSelectedRegionNextTime= ! paintSelectedRegionNextTime;
            }
            
            void updateControlsForSelectedItem() { // Update the controls to reflect the selected item (if any), else clear them.
                if (boundaryRegions.size()==0 || selectedBoundaryRegionIndex==-1) { // If there aren't any regions, or none is selected, blank out the controls.
                    if (rectangleControls.isShowing()) {
                        blankRectangleControls();
                    } else if (circleControls.isShowing()) {
                        blankCircleControls();
                    }
                } else { // Otherwise update the controls based on the selected region, and make sure the correct controls are showing.
                    Region selectedRegion=(Region)boundaryRegions.get(selectedBoundaryRegionIndex);
                    selectedRegion.showControls(); // Polymorphic code to show the appropriate kind of controls.
                    selectedRegion.updateControls(); // And to update them.
                }
            }
            
        } // End of BoundaryRegionContainer

        class ThermalRegionContainer {
            private LinkedList thermalRegions=new LinkedList();
            private int selectedThermalRegionIndex=-1; // This will be used to keep track of the index of which region is selected.
                   
            void parseAndRemoveThermalRegions() { // Take the thermal regions from files.inputFile, and put them in the thermal region container.
                thermalRegions= new LinkedList(); // Start by getting rid of all the old regions (if any).
                selectedThermalRegionIndex=-1; // And no region will be selected initially.
                
                parseAndRemoveRectangles();
                
                parseAndRemoveCircles();
                
                parseAndRemoveTriangles();
                
                updateThermalRegionList(); // Update the JList to reflect the new regions.

            }
            
            void parseAndRemoveRectangles() {
                
                String parameterPrefix="Temp_rect";
                if (files.checkThatParameterIsDefined(parameterPrefix) || files.checkThatParameterIsDefined(parameterPrefix+"_hw") || files.checkThatParameterIsDefined(parameterPrefix+"_mag") || files.checkThatParameterIsDefined(parameterPrefix+"_ovl") || files.checkThatParameterIsDefined(parameterPrefix+"_x1") || files.checkThatParameterIsDefined(parameterPrefix+"_x2") || files.checkThatParameterIsDefined(parameterPrefix+"_z1") || files.checkThatParameterIsDefined(parameterPrefix+"_z2")); // If no parameters are defined for this thermal property, just ignore it completely. But if at least one is, then we will continue.
                
                String thermalSettings=files.getAllLinesForSettingsWithThisPrefix(parameterPrefix); // Store all the settings incase we encounter an error.
                
                try {
                    int numberOfRectangles=files.parseInteger(parameterPrefix);
                    parseAndRemoveThisManyRectangles(parameterPrefix, numberOfRectangles);
                    
                } catch (ParameterNotFoundException exception) {
                    files.linesWithErrors+=thermalSettings; // If one of the parameters isn't found, add all the settings for this property to linesWithErrors.
                } catch (ParameterSyntaxErrorException exception) {
                    files.linesWithErrors=files.removeLastLineFromString(files.linesWithErrors)+thermalSettings; // If there is a syntax error with one or more parameters, it will already have been added to lines with errors, but we'll now add all of the lines in, so just delete the last line.
                } catch (ParameterIncompatibilityException exception) {
                    files.linesWithErrors+=thermalSettings;
                }
            }

            void parseAndRemoveThisManyRectangles(String parameterPrefix, int numberOfRectangles) throws ParameterNotFoundException, ParameterSyntaxErrorException, ParameterIncompatibilityException {
                    String hw=files.parseAndRemoveList(parameterPrefix+"_hw");
                    String mag=files.parseAndRemoveList(parameterPrefix+"_mag");
                    String ovl=files.parseAndRemoveList(parameterPrefix+"_ovl");
                    String x1=files.parseAndRemoveList(parameterPrefix+"_x1");
                    String x2=files.parseAndRemoveList(parameterPrefix+"_x2");
                    String z1=files.parseAndRemoveList(parameterPrefix+"_z1");
                    String z2=files.parseAndRemoveList(parameterPrefix+"_z2");
                    
                    String filled;
                    try {
                        filled=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_filled");
                    } catch (ParameterNotFoundException exception) {
                        filled=files.makeListOfStrings("filled", numberOfRectangles);
                    }
                    
                    for (int regionIndex=1; regionIndex<=numberOfRectangles; ++regionIndex) {
                        try {
                            thermalRegions.add(new ThermalRectangle(scaling.ellipsisUnits2KM(Statics.nthReal(x1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(x2, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z2, regionIndex)), Statics.filledOrUnfilled2Boolean(Statics.nthItem(filled, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(hw, regionIndex)), scaling.ellipsisUnits2kelvin(Statics.nthReal(mag, regionIndex)), Statics.string2Char(Statics.nthItem(ovl, regionIndex)))); 
                        } catch (ParameterSyntaxErrorException exception) {
                            files.linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                            throw exception; // And then re-throw the exception
                        }
                    }
                    
//                } catch (ParameterNotFoundException exception) { // This is an error - because if we have one for number of rectangles, we must have all the ones defining rectangles.
//                    JOptionPane.showMessageDialog(MainWindow.this,"There is a setting for "+parameterPrefix+", but one of the parameters defining the rectangles\nthemselves is missing. No rectangles for this thermal property have been created.", "Error", JOptionPane.ERROR_MESSAGE);
//                } catch (ParameterSyntaxErrorException exception) {
//                    // If we get this error, do nothing - syntax errors are reported anyway -- and if we are doing a merge of files to maintain the original file structure, then the original lines will be left in anyway.
//                } catch (ParameterIncompatibilityException exception) {
//                    // Also do nothing?? Treat like a syntax error exception??
//                }
            }
            
            void parseAndRemoveTriangles() {
                
                String parameterPrefix="Temp_trgl";
                if (files.checkThatParameterIsDefined(parameterPrefix) || files.checkThatParameterIsDefined(parameterPrefix+"_hw") || files.checkThatParameterIsDefined(parameterPrefix+"_mag") || files.checkThatParameterIsDefined(parameterPrefix+"_ovl") || files.checkThatParameterIsDefined(parameterPrefix+"_x1") || files.checkThatParameterIsDefined(parameterPrefix+"_x2") || files.checkThatParameterIsDefined(parameterPrefix+"_x3") || files.checkThatParameterIsDefined(parameterPrefix+"_z1") || files.checkThatParameterIsDefined(parameterPrefix+"_z2") || files.checkThatParameterIsDefined(parameterPrefix+"_z3")); // If no parameters are defined for this thermal property, just ignore it completely. But if at least one is, then we will continue.
                
                String thermalSettings=files.getAllLinesForSettingsWithThisPrefix(parameterPrefix); // Store all the settings incase we encounter an error.
                
                try {
                    int numberOfTriangles=files.parseInteger(parameterPrefix);
                    parseAndRemoveThisManyTriangles(parameterPrefix, numberOfTriangles);
                    
                } catch (ParameterNotFoundException exception) {
                    files.linesWithErrors+=thermalSettings; // If one of the parameters isn't found, add all the settings for this property to linesWithErrors.
                } catch (ParameterSyntaxErrorException exception) {
                    files.linesWithErrors=files.removeLastLineFromString(files.linesWithErrors)+thermalSettings; // If there is a syntax error with one or more parameters, it will already have been added to lines with errors, but we'll now add all of the lines in, so just delete the last line.
                } catch (ParameterIncompatibilityException exception) {
                    files.linesWithErrors+=thermalSettings;
                }
            }

            void parseAndRemoveThisManyTriangles(String parameterPrefix, int numberOfTriangles) throws ParameterNotFoundException, ParameterSyntaxErrorException, ParameterIncompatibilityException {
                    String hw=files.parseAndRemoveList(parameterPrefix+"_hw");
                    String mag=files.parseAndRemoveList(parameterPrefix+"_mag");
                    String ovl=files.parseAndRemoveList(parameterPrefix+"_ovl");
                    String x1=files.parseAndRemoveList(parameterPrefix+"_x1");
                    String x2=files.parseAndRemoveList(parameterPrefix+"_x2");
                    String z1=files.parseAndRemoveList(parameterPrefix+"_z1");
                    String z2=files.parseAndRemoveList(parameterPrefix+"_z2");
                    String x3=files.parseAndRemoveList(parameterPrefix+"_x3");
                    String z3=files.parseAndRemoveList(parameterPrefix+"_z3");
                    
                    String filled;
                    try {
                        filled=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_filled");
                    } catch (ParameterNotFoundException exception) {
                        filled=files.makeListOfStrings("filled", numberOfTriangles);
                    }
                    
                    for (int regionIndex=1; regionIndex<=numberOfTriangles; ++regionIndex) {
                        try {
                            thermalRegions.add(new ThermalTriangle(scaling.ellipsisUnits2KM(Statics.nthReal(x1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(x2, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z2, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(x3, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z3, regionIndex)), Statics.filledOrUnfilled2Boolean(Statics.nthItem(filled, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(hw, regionIndex)), scaling.ellipsisUnits2kelvin(Statics.nthReal(mag, regionIndex)), Statics.string2Char(Statics.nthItem(ovl, regionIndex)))); 
                        } catch (ParameterSyntaxErrorException exception) {
                            files.linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                            throw exception; // And then re-throw the exception
                        }
                    }
                    
//                } catch (ParameterNotFoundException exception) { // This is an error - because if we have one for number of rectangles, we must have all the ones defining rectangles.
//                    JOptionPane.showMessageDialog(MainWindow.this,"There is a setting for "+parameterPrefix+", but one of the parameters defining the rectangles\nthemselves is missing. No rectangles for this thermal property have been created.", "Error", JOptionPane.ERROR_MESSAGE);
//                } catch (ParameterSyntaxErrorException exception) {
//                    // If we get this error, do nothing - syntax errors are reported anyway -- and if we are doing a merge of files to maintain the original file structure, then the original lines will be left in anyway.
//                } catch (ParameterIncompatibilityException exception) {
//                    // Also do nothing?? Treat like a syntax error exception??
//                }
            }

            void parseAndRemoveCircles() {
                
                String parameterPrefix="Temp_circ";
                if (files.checkThatParameterIsDefined(parameterPrefix) || files.checkThatParameterIsDefined(parameterPrefix+"_hw") || files.checkThatParameterIsDefined(parameterPrefix+"_mag") || files.checkThatParameterIsDefined(parameterPrefix+"_ovl") || files.checkThatParameterIsDefined(parameterPrefix+"_x1") || files.checkThatParameterIsDefined(parameterPrefix+"_z1") || files.checkThatParameterIsDefined(parameterPrefix+"_rad")) { // If no parameters are defined for this thermal property, just ignore it completely. But if at least one is, then we will continue.
                    
                    String thermalSettings=files.getAllLinesForSettingsWithThisPrefix(parameterPrefix); // Store all the settings incase we encounter an error.
                    
                    try {
                        int numberOfCircles=files.parseInteger(parameterPrefix);
                        parseAndRemoveThisManyCircles(parameterPrefix, numberOfCircles);
                        
                    } catch (ParameterNotFoundException exception) {
                        files.linesWithErrors+=thermalSettings; // If one of the parameters isn't found, add all the settings for this property to linesWithErrors.
                    } catch (ParameterSyntaxErrorException exception) {
                        files.linesWithErrors=files.removeLastLineFromString(files.linesWithErrors)+thermalSettings; // If there is a syntax error with one or more parameters, it will already have been added to lines with errors, but we'll now add all of the lines in, so just delete the last line.
                    } catch (ParameterIncompatibilityException exception) {
                        files.linesWithErrors+=thermalSettings;
                    }
                }
                
            }

            void parseAndRemoveThisManyCircles(String parameterPrefix, int numberOfCircles) throws ParameterNotFoundException, ParameterSyntaxErrorException, ParameterIncompatibilityException {
//                try {
                    String hw=files.parseAndRemoveList(parameterPrefix+"_hw");
                    String mag=files.parseAndRemoveList(parameterPrefix+"_mag");
                    String ovl=files.parseAndRemoveList(parameterPrefix+"_ovl");
                    String x1=files.parseAndRemoveList(parameterPrefix+"_x1");
                    String z1=files.parseAndRemoveList(parameterPrefix+"_z1");
                    String rad=files.parseAndRemoveList(parameterPrefix+"_rad");
                    
                    String colours;
                    String filled;
                    try {
                        filled=files.parseAndRemoveEllipsisGUIList(parameterPrefix+"_filled");
                    } catch (ParameterNotFoundException exception) {
                        filled=files.makeListOfStrings("filled", numberOfCircles);
                    }
                    
                    for (int regionIndex=1; regionIndex<=numberOfCircles; ++regionIndex) {
                        try {
                            thermalRegions.add(new ThermalCircle(scaling.ellipsisUnits2KM(Statics.nthReal(x1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(z1, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(rad, regionIndex)), Statics.filledOrUnfilled2Boolean(Statics.nthItem(filled, regionIndex)), scaling.ellipsisUnits2KM(Statics.nthReal(hw, regionIndex)), scaling.ellipsisUnits2kelvin(Statics.nthReal(mag, regionIndex)), Statics.string2Char(Statics.nthItem(ovl, regionIndex)))); 
                        } catch (ParameterSyntaxErrorException exception) {
                            files.linesWithErrors+="Temporary line which should never be seen\n"; // Normally when a syntax error ecveption is encountered, a line is added to the string lines with errors, so, for consistency we need to add this line here, so it can be removed later without a problem (nthReal etc. do not add a line to linesWithErrors).
                            throw exception; // And then re-throw the exception
                        }
                    }
                    
            }
            
            String getFileSettings() { // Returns a string specifying all the thermal region settings in ellipsis input file format.
                String settings="# THERMAL REGIONS\n\n";

                settings+=getRectangleFileSettings();
                
                settings+=getTriangleFileSettings();
                
                settings+=getCircleFileSettings();
                
                return settings;
            }
            
            private String getRectangleFileSettings() {
                String settings="\n# Thermal Rectangles\n\n";
                
                    if (numberOfRectangles()!=0) { // Only do the settings if at least one thermal with this property exists.
                        settings+="Temp_rect="+numberOfRectangles()+"\n\n";
                        
                        String hw="";
                        String mag="";
                        String ovl="";
                        String x1="";
                        String z1="";
                        String x2="";
                        String z2="";
                        
                        String filled=""; // Thiw is whether the region is drawn filled in the Ellipsis Gui.
                        
                        Iterator currentRegion=thermalRegions.iterator();
                        while (currentRegion.hasNext()) {
                            Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                            if (theRegion instanceof ThermalRectangle) { // If it's a rectangle,
                                ThermalRectangle rectangle=(ThermalRectangle)theRegion;
                                    hw+=scaling.KM2EllipsisUnits(rectangle.halfWidth)+","; 
                                    mag+=scaling.kelvin2EllipsisUnits(rectangle.temperature)+",";
                                    ovl+=rectangle.overlaps+",";
                                    x1+=scaling.KM2EllipsisUnits(rectangle.x1)+",";
                                    x2+=scaling.KM2EllipsisUnits(rectangle.x2)+",";
                                    z1+=scaling.KM2EllipsisUnits(rectangle.z1)+",";
                                    z2+=scaling.KM2EllipsisUnits(rectangle.z2)+",";
                                    
                                    filled+=Statics.boolean2filledOrUnfilled(rectangle.filled)+",";
                            }
                        }
                        
                        settings+="Temp_rect_hw="+Useful.chopAnyTrailingComma(hw)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_rect_mag="+Useful.chopAnyTrailingComma(mag)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_rect_ovl="+Useful.chopAnyTrailingComma(ovl)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_rect_x1="+Useful.chopAnyTrailingComma(x1)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_rect_x2="+Useful.chopAnyTrailingComma(x2)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_rect_z1="+Useful.chopAnyTrailingComma(z1)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_rect_z2="+Useful.chopAnyTrailingComma(z2)+"\n"; // Now add the fields to the settings string.
                        
                        settings+=files.ellipsisGUIParameterPrefix+"Temp_rect_filled="+Useful.chopAnyTrailingComma(filled)+"\n\n";
                    }
                
                return settings;
            }
            
            private String getCircleFileSettings() {
                String settings="\n# Thermal Circles\n\n";
                
                if (numberOfCircles()!=0) { // Only do the settings if at least one thermal with this property exists.
                    settings+="Temp_circ="+numberOfCircles()+"\n\n";
                    
                    String hw="";
                    String mag="";
                    String ovl="";
                    String x1="";
                    String z1="";
                    String rad="";
                    
                    String filled=""; // Thiw is whether the region is drawn filled in the Ellipsis Gui.
                    
                    Iterator currentRegion=thermalRegions.iterator();
                    while (currentRegion.hasNext()) {
                        Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                        if (theRegion instanceof ThermalCircle) { // If it's a rectangle,
                            ThermalCircle circle=(ThermalCircle)theRegion;
                            hw+=scaling.KM2EllipsisUnits(circle.halfWidth)+",";
                            mag+=scaling.kelvin2EllipsisUnits(circle.temperature)+",";
                            ovl+=circle.overlaps+",";
                            x1+=scaling.KM2EllipsisUnits(circle.x)+",";
                            z1+=scaling.KM2EllipsisUnits(circle.z)+",";
                            rad+=scaling.KM2EllipsisUnits(circle.radius)+",";
                            
                            filled+=Statics.boolean2filledOrUnfilled(circle.filled)+",";
                        }
                    }
                    
                    settings+="Temp_circ_hw="+Useful.chopAnyTrailingComma(hw)+"\n"; // Now add the fields to the settings string.
                    settings+="Temp_circ_mag="+Useful.chopAnyTrailingComma(mag)+"\n"; // Now add the fields to the settings string.
                    settings+="Temp_circ_ovl="+Useful.chopAnyTrailingComma(ovl)+"\n"; // Now add the fields to the settings string.
                    settings+="Temp_circ_x1="+Useful.chopAnyTrailingComma(x1)+"\n"; // Now add the fields to the settings string.
                    settings+="Temp_circ_z1="+Useful.chopAnyTrailingComma(z1)+"\n"; // Now add the fields to the settings string.
                    settings+="Temp_circ_rad="+Useful.chopAnyTrailingComma(rad)+"\n"; // Now add the fields to the settings string.
                    
                    settings+=files.ellipsisGUIParameterPrefix+"Temp_circ_filled="+Useful.chopAnyTrailingComma(filled)+"\n\n";
                }
                
                return settings;
            }

            private String getTriangleFileSettings() {
                String settings="\n# Thermal Triangles\n\n";
                
                    if (numberOfTriangles()!=0) { // Only do the settings if at least one thermal with this property exists.
                        settings+="Temp_trgl="+numberOfTriangles()+"\n\n";
                        
                        String hw="";
                        String mag="";
                        String ovl="";
                        String x1="";
                        String z1="";
                        String x2="";
                        String z2="";
                        String x3="";
                        String z3="";
                        
                        String filled=""; // Thiw is whether the region is drawn filled in the Ellipsis Gui.
                        
                        Iterator currentRegion=thermalRegions.iterator();
                        while (currentRegion.hasNext()) {
                            Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                            if (theRegion instanceof ThermalTriangle) { // If it's a rectangle,
                                ThermalTriangle triangle=(ThermalTriangle)theRegion;
                                    hw+=scaling.KM2EllipsisUnits(triangle.halfWidth)+","; 
                                    mag+=scaling.kelvin2EllipsisUnits(triangle.temperature)+",";
                                    ovl+=triangle.overlaps+",";
                                    x1+=scaling.KM2EllipsisUnits(triangle.x1)+",";
                                    x2+=scaling.KM2EllipsisUnits(triangle.x2)+",";
                                    x3+=scaling.KM2EllipsisUnits(triangle.x3)+",";
                                    z1+=scaling.KM2EllipsisUnits(triangle.z1)+",";
                                    z2+=scaling.KM2EllipsisUnits(triangle.z2)+",";
                                    z3+=scaling.KM2EllipsisUnits(triangle.z3)+",";                                    

                                    filled+=Statics.boolean2filledOrUnfilled(triangle.filled)+",";
                            }
                        }
                        
                        settings+="Temp_trgl_hw="+Useful.chopAnyTrailingComma(hw)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_trgl_mag="+Useful.chopAnyTrailingComma(mag)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_trgl_ovl="+Useful.chopAnyTrailingComma(ovl)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_trgl_x1="+Useful.chopAnyTrailingComma(x1)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_trgl_x2="+Useful.chopAnyTrailingComma(x2)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_trgl_z1="+Useful.chopAnyTrailingComma(z1)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_trgl_z2="+Useful.chopAnyTrailingComma(z2)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_trgl_x3="+Useful.chopAnyTrailingComma(x3)+"\n"; // Now add the fields to the settings string.
                        settings+="Temp_trgl_z3="+Useful.chopAnyTrailingComma(z3)+"\n"; // Now add the fields to the settings string.

                        settings+=files.ellipsisGUIParameterPrefix+"Temp_trgl_filled="+Useful.chopAnyTrailingComma(filled)+"\n\n";
                    }
                
                return settings;
            }                        

            private int numberOfRectangles() { // Return the number of thermal rectangle regions.
                int numberOfRectangles=0;
                Iterator currentRegion=thermalRegions.iterator();
                while (currentRegion.hasNext()) {
                    Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                    if (theRegion instanceof ThermalRectangle) { // If it's a rectangle, 
                            ++numberOfRectangles; // Then count it.
                    }
                }
                
                return numberOfRectangles;
            }
 
            
            private int numberOfCircles() { // Return the number of thermal circle regions 
                int numberOfCircles=0;
                Iterator currentRegion=thermalRegions.iterator();
                while (currentRegion.hasNext()) {
                    Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                    if (theRegion instanceof ThermalCircle) { // If it's a rectangle, 
                            ++numberOfCircles; // Then count it.
                    }
                }
                
                return numberOfCircles;
            }
 
            private int numberOfTriangles() { // Return the number of thermal triangle regions 
                int numberOfTriangles=0;
                Iterator currentRegion=thermalRegions.iterator();
                while (currentRegion.hasNext()) {
                    Object theRegion=(Region)currentRegion.next(); // Get the current Region.
                    if (theRegion instanceof ThermalTriangle) { // If it's a rectangle, 
                            ++numberOfTriangles; // Then count it.
                    }
                }
                
                return numberOfTriangles;
            }

            void updateSelectedRectangle() {
                if (selectedThermalRegionIndex==-1) return; // If no region is selected, do nothing.
                
                RectangleCoordinates coordinates=new RectangleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((MyRectangle)thermalRegions.get(selectedThermalRegionIndex)).update(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                updateThermalRegionList(); // We need to do this incase the thermal property has been changed.
                repaint();
            }
            
            void updateSelectedTriangle() {
                if (selectedThermalRegionIndex==-1) return; // If no region is selected, do nothing.
                
                TriangleCoordinates coordinates=new TriangleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((Triangle)thermalRegions.get(selectedThermalRegionIndex)).update(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2, coordinates.x3, coordinates.z3, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                updateThermalRegionList(); // We need to do this incase the thermal property has been changed.
                repaint();
            }
            
            void updateSelectedCircle() {
                if (selectedThermalRegionIndex==-1) return; // If no region is selected, do nothing.
                
                CircleCoordinates coordinates=new CircleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((Circle)thermalRegions.get(selectedThermalRegionIndex)).update(coordinates.x, coordinates.z, coordinates.radius, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                updateThermalRegionList(); // We need to do this incase the thermal property has been changed.
                repaint();
            }
            
            void deleteSelectedRegion() {
                if (selectedThermalRegionIndex==-1) return; // If no region is selected, do nothing.
                thermalRegions.remove(selectedThermalRegionIndex); // Remove the region from the list of regions.
                selectedThermalRegionIndex=-1;
                
                updateThermalRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted without this region.
            }
            
            void moveSelectedRegionForward() {
                if (selectedThermalRegionIndex==-1 || selectedThermalRegionIndex==thermalRegions.size()-1) return; // If no region is selected, or the region is already at the front, do nothing.
                Region theRegion=(Region)thermalRegions.remove(selectedThermalRegionIndex); // Take the region out of the list
                thermalRegions.add(selectedThermalRegionIndex+1, theRegion); // And put it back one further down.
                ++selectedThermalRegionIndex; // Keep the same region selected.
                
                updateThermalRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted with this region in its new position.
            }
            
            void moveSelectedRegionBackward() {
                if (selectedThermalRegionIndex==-1 || selectedThermalRegionIndex==0) return; // If no region is selected, or the region is already at the back, do nothing.
                Region theRegion=(Region)thermalRegions.remove(selectedThermalRegionIndex); // Take the region out of the list
                thermalRegions.add(selectedThermalRegionIndex-1, theRegion); // And put it back one further up.
                --selectedThermalRegionIndex; // Keep the same region selected.
                
                updateThermalRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted with this region in its new position.
            }
            
            void setSelectedRegion(int newSelectedRegionIndex) { // If we've changed regions, then we need to update the drawing controls to reflect that.
                selectedThermalRegionIndex=newSelectedRegionIndex; // Do this twice, because if the controls are switched to those for a different colour the first time, then the shape will become deselected.
                updateControlsForSelectedItem();
                selectedThermalRegionIndex=newSelectedRegionIndex;
                updateControlsForSelectedItem();
                
                thermalRegionList.setSelectedIndex(newSelectedRegionIndex); // And update the JList
            }
            
            void updateThermalRegionList() {
                DefaultListModel theListModel=(DefaultListModel)thermalRegionList.getModel(); // Get the list model.
                int theSelectedThermalRegionIndex=selectedThermalRegionIndex; // We need to cache this here because the value will get upset when we remove all the items from the list - that will cause messages to be sent about the selection having changed.
                theListModel.removeAllElements(); // Remove all the items from the list.
                Iterator currentRegion=thermalRegions.iterator();
                while (currentRegion.hasNext()) {
                    Region theRegion=(Region)currentRegion.next(); // Get the current Region.
                    String listItem=theRegion.getName()+" ("+theRegion.getShapeDescription()+")"; // Create a string describing the region, made up from the thermal's property and the type of shape.
                    theListModel.addElement(listItem); // And add it to the list.
                }
                
                selectedThermalRegionIndex=theSelectedThermalRegionIndex; // Now we can restore this because the list is back to its proper state.
                
                if (selectedThermalRegionIndex!=-1) { // If a region is selected then set it to be selected in the Jlist.
                    thermalRegionList.setSelectedIndex(selectedThermalRegionIndex);
                }
                
                thermalRegionList.scrollRectToVisible(new java.awt.Rectangle(0, thermalRegionList.getHeight()+50, 1, 1)); // Make sure that the new item is visible in the scroll pane by scrolling to the bottom. (And scroll to the left of the pane.) The +50 ensures that we get right to the bottom of the scrollpane - I think the lists size hasn't been updated by this point so this is necessary.
            }
            
            void deselectRegion() { // Set the container so that no region is selected.
                selectedThermalRegionIndex=-1;
                thermalRegionList.clearSelection(); // And deselect the region in the JList
            }
            
            char currentlySelectedOverlap() {
                if (addRadioButton.isSelected())
                    return 'A';
                if (multiplyRadioButton.isSelected())
                    return 'M';
                
                return 'R';
            }
            
            void addRectangle(double x1, double y1, double x2, double y2) { // We're adding a thermal region with these coordinates in KM.
                char theOverlap='A';
                thermalRegions.add(new ThermalRectangle(x1, y1, x2, y2, drawSolidShapesCheckBox.isSelected(), Double.parseDouble(halfWidthNumberBoxBean.getText()), Double.parseDouble(temperatureNumberBoxBean.getText()), currentlySelectedOverlap())); // Add a region with it's boundaries specified in km, either solid or not accoding to checkbox setting, and with the currently selected thermal properties.
                selectedThermalRegionIndex=thermalRegions.size()-1; // Make the new region be the selected one.
                updateThermalRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint(); // We've added a region - so now we need to get the on screen view updated.
            }
            
            void addTriangle(double x1, double y1, double x2, double y2, double x3, double y3) { // We're adding a thermal region with these coordinates in KM.
                thermalRegions.add(new ThermalTriangle(x1, y1, x2, y2, x3, y3, drawSolidShapesCheckBox.isSelected(), Double.parseDouble(halfWidthNumberBoxBean.getText()), Double.parseDouble(temperatureNumberBoxBean.getText()), currentlySelectedOverlap())); // Add a region with it's boundaries specified in km, either solid or not accoding to checkbox setting, and with the currently selected thermal properties.
                selectedThermalRegionIndex=thermalRegions.size()-1; // Make the new region be the selected one.
                updateThermalRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint(); // We've added a region - so now we need to get the on screen view updated.
            }

            void addCircle(double x, double z, double radius) {
                thermalRegions.add(new ThermalCircle(x, z, radius, drawSolidShapesCheckBox.isSelected(), Double.parseDouble(halfWidthNumberBoxBean.getText()), Double.parseDouble(temperatureNumberBoxBean.getText()), currentlySelectedOverlap()));
                selectedThermalRegionIndex=thermalRegions.size()-1; // Make the new region be the selected one.
                updateThermalRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint();
            }
            
            void paintAllRegions(Graphics graphics) { // Draw all thermal regions in the window pointed to by graphics.
                Iterator currentRegion=thermalRegions.iterator();
                while (currentRegion.hasNext()) {
                    ((Region)currentRegion.next()).draw(graphics); // Call the base class draw function to draw each region - it's polymorphic, so can be used to draw all types of region.
                }
            }
            
            boolean paintSelectedRegionNextTime=true;
            void paintSelectedRegion(Graphics graphicsContext) { // Paints the selected region (if there is one) with a blue XOR border.
                if (selectedThermalRegionIndex==-1) { // If there's no selected region there's nothing to do.
                    return;
                } else {
                    if (paintSelectedRegionNextTime || !animateSelectedRegionsCheckBox.isSelected()) { // Every second time, draw the selected region in XOR birghtpink.
                        Graphics copyOfGraphicsContext=graphicsContext.create(); // Make a copy of the context, so the changes we make won't affect any other drawing operations.
                        
                        copyOfGraphicsContext.setXORMode(Colours.brightPink); // Draw shape by XORing between black and white.
                        
                        ((Region)thermalRegions.get(selectedThermalRegionIndex)).draw(copyOfGraphicsContext);
                    }
                }
                paintSelectedRegionNextTime= ! paintSelectedRegionNextTime;
            }
            
            void updateControlsForSelectedItem() { // Update the controls to reflect the selected item (if any), else clear them.
                if (thermalRegions.size()==0 || selectedThermalRegionIndex==-1) { // If there aren't any regions, or none is selected, blank out the controls.
                    if (rectangleControls.isShowing()) {
                        blankRectangleControls();
                    } else if (circleControls.isShowing()) {
                        blankCircleControls();
                    }
                } else { // Otherwise update the controls based on the selected region, and make sure the correct controls are showing.
                    Region selectedRegion=(Region)thermalRegions.get(selectedThermalRegionIndex);
                    selectedRegion.showControls(); // Polymorphic code to show the appropriate kind of controls.
                    selectedRegion.updateControls(); // And to update them.
                }
            }
            
        } // End of ThermalRegionContainer

        class DrawingRegionContainer {
            private LinkedList drawingRegions=new LinkedList();
            private int selectedDrawingRegionIndex=-1; // This will be used to keep track of the index of which region is selected.
            
/*            String getFileSettings() { // I may want to implement this function to add comment lines to the input file specifying any drawings.
                return "";
            }*/
            
            void updateSelectedRectangle() {
                if (selectedDrawingRegionIndex==-1) return; // If no region is selected, do nothing.
                
                RectangleCoordinates coordinates=new RectangleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((MyRectangle)drawingRegions.get(selectedDrawingRegionIndex)).update(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                repaint();
            }
            
            void updateSelectedLine() {
                if (selectedDrawingRegionIndex==-1) return; // If no region is selected, do nothing.
                
                LineCoordinates coordinates=new LineCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((Line)drawingRegions.get(selectedDrawingRegionIndex)).update(coordinates.x1, coordinates.z1, coordinates.x2, coordinates.z2); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                repaint();
            }
            
            void updateSelectedCircle() {
                if (selectedDrawingRegionIndex==-1) return; // If no region is selected, do nothing.
                
                CircleCoordinates coordinates=new CircleCoordinates();
                if (!coordinates.getCoordinates()) return; // Get the coordinates which are set in the main window, returning if there's an error.
                
                ((Circle)drawingRegions.get(selectedDrawingRegionIndex)).update(coordinates.x, coordinates.z, coordinates.radius, drawSolidShapesCheckBox.isSelected()); // Update the region based on the coordinates and whether we're doing solid shapes.
                
                repaint();
            }
            
            void deleteSelectedRegion() {
                if (selectedDrawingRegionIndex==-1) return; // If no region is selected, do nothing.
                drawingRegions.remove(selectedDrawingRegionIndex); // Remove the region from the list of regions.
                selectedDrawingRegionIndex=-1;
                
                updateDrawingRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted without this region.
            }
            
            void moveSelectedRegionForward() {
                if (selectedDrawingRegionIndex==-1 || selectedDrawingRegionIndex==drawingRegions.size()-1) return; // If no region is selected, or the region is already at the front, do nothing.
                Region theRegion=(Region)drawingRegions.remove(selectedDrawingRegionIndex); // Take the region out of the list
                drawingRegions.add(selectedDrawingRegionIndex+1, theRegion); // And put it back one further down.
                ++selectedDrawingRegionIndex; // Keep the same region selected.
                
                updateDrawingRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted with this region in its new position.
            }
            
            void moveSelectedRegionBackward() {
                if (selectedDrawingRegionIndex==-1 || selectedDrawingRegionIndex==0) return; // If no region is selected, or the region is already at the back, do nothing.
                Region theRegion=(Region)drawingRegions.remove(selectedDrawingRegionIndex); // Take the region out of the list
                drawingRegions.add(selectedDrawingRegionIndex-1, theRegion); // And put it back one further up.
                --selectedDrawingRegionIndex; // Keep the same region selected.
                
                updateDrawingRegionList(); // Update the Jlist to reflect the change.
                backgroundImagePanel.repaint(); // And get the background image panel repainted with this region in its new position.
            }
            
            void setSelectedRegion(int newSelectedRegionIndex) { // If we've changed regions, then we need to update the drawing controls to reflect that.
                selectedDrawingRegionIndex=newSelectedRegionIndex; // Do this twice, because if the controls are switched to those for a different colour the first time, then the shape will become deselected.
                updateControlsForSelectedItem();
                selectedDrawingRegionIndex=newSelectedRegionIndex;
                updateControlsForSelectedItem();
                
                drawingRegionList.setSelectedIndex(newSelectedRegionIndex); // And update the JList
            }
            
            void updateDrawingRegionList() {
                DefaultListModel theListModel=(DefaultListModel)drawingRegionList.getModel(); // Get the list model.
                int theSelectedDrawingRegionIndex=selectedDrawingRegionIndex; // We need to cache this here because the value will get upset when we remove all the items from the list - that will cause messages to be sent about the selection having changed.
                theListModel.removeAllElements(); // Remove all the items from the list.
                Iterator currentRegion=drawingRegions.iterator();
                while (currentRegion.hasNext()) {
                    Region theRegion=(Region)currentRegion.next(); // Get the current Region.
                    String listItem=theRegion.getShapeDescription(); // Create a string describing the region, made up of the type of shape and nothing else.
                    theListModel.addElement(listItem); // And add it to the list.
                }
                
                selectedDrawingRegionIndex=theSelectedDrawingRegionIndex; // Now we can restore this because the list is back to its proper state.
                
                if (selectedDrawingRegionIndex!=-1) { // If a region is selected then set it to be selected in the Jlist.
                    drawingRegionList.setSelectedIndex(selectedDrawingRegionIndex);
                }
                
                drawingRegionList.scrollRectToVisible(new java.awt.Rectangle(0, drawingRegionList.getHeight()+50, 1, 1)); // Make sure that the new item is visible in the scroll pane by scrolling to the bottom. (And scroll to the left of the pane.) The +50 ensures that we get right to the bottom of the scrollpane - I think the lists size hasn't been updated by this point so this is necessary.
            }
            
            void deselectRegion() { // Set the container so that no region is selected.
                selectedDrawingRegionIndex=-1;
                drawingRegionList.clearSelection(); // And deselect the region in the JList
            }
            
            void addRectangle(double x1, double y1, double x2, double y2) { // We're adding a drawing region with these coordinates in KM.
                drawingRegions.add(new DrawingRectangle(x1, y1, x2, y2, drawSolidShapesCheckBox.isSelected(), drawingChangeColourButton.getBackground())); // Add a region with it's boundaries specified in km, either solid or not accoding to checkbox setting, and of the current drawing type.
                selectedDrawingRegionIndex=drawingRegions.size()-1; // Make the new region be the selected one.
                updateDrawingRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint(); // We've added a region - so now we need to get the on screen view updated.
            }
            
            void addLine(double x1, double y1, double x2, double y2) { // We're adding a drawing region with these coordinates in KM.
                drawingRegions.add(new DrawingLine(x1, y1, x2, y2, drawingChangeColourButton.getBackground())); // Add a region with it's boundaries specified in km, and of the current drawing type.
                selectedDrawingRegionIndex=drawingRegions.size()-1; // Make the new region be the selected one.
                updateDrawingRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint(); // We've added a region - so now we need to get the on screen view updated.
            }
            
            void addCircle(double x, double z, double radius) {
                drawingRegions.add(new DrawingCircle(x, z, radius, drawSolidShapesCheckBox.isSelected(), drawingChangeColourButton.getBackground()));
                selectedDrawingRegionIndex=drawingRegions.size()-1; // Make the new region be the selected one.
                updateDrawingRegionList(); // Update the JList to reflect the current regions, and set the selected region as appropriate.
                backgroundImagePanel.repaint();
            }
            
            void paintAllRegions(Graphics graphics) { // Draw all drawing regions in the window pointed to by graphics.
                Iterator currentRegion=drawingRegions.iterator();
                while (currentRegion.hasNext()) {
                    ((Region)currentRegion.next()).draw(graphics); // Call the base class draw function to draw each region - it's polymorphic, so can be used to draw all types of region.
                }
            }
            
            boolean paintSelectedRegionNextTime=true;
            void paintSelectedRegion(Graphics graphicsContext) { // Paints the selected region (if there is one) with a blue XOR border.
                if (selectedDrawingRegionIndex==-1) { // If there's no selected region there's nothing to do.
                    return;
                } else {
                    if (paintSelectedRegionNextTime || !animateSelectedRegionsCheckBox.isSelected()) { // Every second time, draw the selected region in XOR birghtpink.
                        Graphics copyOfGraphicsContext=graphicsContext.create(); // Make a copy of the context, so the changes we make won't affect any other drawing operations.
                        
                        copyOfGraphicsContext.setXORMode(Colours.brightPink); // Draw shape by XORing between black and white.
                        
                        ((Region)drawingRegions.get(selectedDrawingRegionIndex)).draw(copyOfGraphicsContext);
                    }
                }
                paintSelectedRegionNextTime= ! paintSelectedRegionNextTime;
            }
            
            void updateControlsForSelectedItem() { // Update the controls to reflect the selected item (if any), else clear them.
                if (drawingRegions.size()==0 || selectedDrawingRegionIndex==-1) { // If there aren't any regions, or none is selected, blank out the controls.
                    if (rectangleControls.isShowing()) {
                        blankRectangleControls();
                    } else if (circleControls.isShowing()) {
                        blankCircleControls();
                    } else if (lineControls.isShowing()) {
                        blankLineControls();
                    }
                } else { // Otherwise update the controls based on the selected region, and make sure the correct controls are showing.
                    Region selectedRegion=(Region)drawingRegions.get(selectedDrawingRegionIndex);
                    selectedRegion.showControls(); // Polymorphic code to show the appropriate kind of controls.
                    selectedRegion.updateControls(); // And to update them.
                }
            }
            
        } // End of Drawing RegionContainer
        
        // ********* THESE ARE ALL THE REGIONS USED IN DRAWING THEY ARE SUBCLASSES OF BACKGROUNDIMAGEPANEL **********
        
        abstract class Region {
            boolean filled; // Is this shape drawn as a solid block or an outline. In the case of lines this will just be ignored.
            
            abstract void draw(Graphics graphicsContext); // Draws the region - will probably need an input specifying some sort of handle for where it is to be drawn.
            
            abstract void updateControls(); // Update the controls for the type of object to reflect the settings of the current object, and show the controls.
            
            abstract protected Color getColour();
            
            abstract String getShapeDescription(); // Returns a description of the type of shape (e.g. rectangle/circle etc.).
            
            abstract String getName(); // So far this is only applicable to material regions.
            Material getMaterial() { return null; } // This is only relevant for Material regions. The definition is just to keep the compiler happy.
            
            abstract void showControls(); // Show the controls for this type of region.
        }
        
        abstract class Line extends Region {
            double x1, x2, z1, z2;
            
            Line(double newx1, double newz1, double newx2, double newz2) {
                x1=newx1;
                x2=newx2;
                z1=newz1;
                z2=newz2;
            }
            
            void draw(Graphics graphicsContext) {
                graphicsContext.setColor(getColour()); // Set to draw in the current region's colour. (How this is defined will depend on the type of line (although in fact there will only be drawing lines.)
                graphicsContext.drawLine(xKm2Pixel(x1), yKm2Pixel(z1), xKm2Pixel(x2), yKm2Pixel(z2)); // Draw the line - need to convert all coordinates to pixels based on the current size of the window it's size in km.
            }
            
            protected Color getColour() { return null; }; // Will only be called by overridden versions of the class, so this is just a dummy to keep the compiler happy.
            
            String getName() { return null; } // So far this is only applicable to material regions, so the implementation is a dummy to keep the compiler happy.
            
            String getShapeDescription() {
                return "line";
            }
            
            void update(double newx1, double newz1, double newx2, double newz2) {
                x1=newx1;
                z1=newz1;
                x2=newx2;
                z2=newz2;
            }
            
            void updateControls() {
                rectangleX1TextField.setText(Useful.double2String(x1));
                rectangleX2TextField.setText(Useful.double2String(x2));
                rectangleY1TextField.setText(Useful.double2String(z1));
                rectangleY2TextField.setText(Useful.double2String(z2));
            }
            
            void showControls() {
                shapeChooser.setSelectedItem("Line");
            }
            
        }
        
        class DrawingLine extends Line {
            Color lineColour;
            
            DrawingLine(double x1, double z1, double x2, double z2, Color theColour) {
                super(x1, z1, x2, z2);
                lineColour=theColour;
            }
            
            protected Color getColour() {
                return lineColour;
            }
            
            String getName() { // Only applies to material regions, so this is a dummy to keep the compiler happy.
                return "";
            }
            
        }
        
        class MyRectangle extends Region {
            double x1, z1, x2, z2; // The coordinates of the top left and bottom right of the region (in kilometres).
            
            MyRectangle(double newx1, double newz1, double newx2, double newz2, boolean isFilled) { // Create a new rectangle with these coordinates.
                x1=newx1;
                z1=newz1;
                x2=newx2;
                z2=newz2;
                filled=isFilled;
            }
            
            void draw(Graphics graphicsContext) {
                graphicsContext.setColor(getColour()); // Set to draw in the current region's colour. (How this is defined will depend on the type of region.)
                if (filled) {
                    graphicsContext.fillRect(xKm2Pixel(x1), yKm2Pixel(z1), xKm2Pixel(x2-x1)+1, yKm2Pixel(z2-z1)+1); // Draw the rectangle - need to convert all coordinates to pixels based on the current size of the window it's size in km. The width and height have 1 added to them because width and height are counted from 1 not 0 like coordinates are.
                } else {
                    graphicsContext.drawRect(xKm2Pixel(x1), yKm2Pixel(z1), xKm2Pixel(x2-x1), yKm2Pixel(z2-z1)); // Draw the rectangle - need to convert all coordinates to pixels based on the current size of the window it's size in km. The width and height don't have 1 added to them here because of the way the JAVA coordinate system works with outlines vs filled objects.
                }
            }
            
            protected Color getColour() { return Colours.white; }; // Will only be called by overridden versions of the class, so this is just a dummy to keep the compiler happy.
            
            String getName() { return ""; } // So far this is only applicable to material regions, so the implementation is a dummy to keep the compiler happy.
            
            String getShapeDescription() {
                return "rectangle";
            }
            
            void update(double newx1, double newz1, double newx2, double newz2, boolean isFilled) {
                x1=newx1;
                z1=newz1;
                x2=newx2;
                z2=newz2;
                filled=isFilled;
            }
            
            void updateControls() {
                rectangleX1TextField.setText(Useful.double2String(x1));
                rectangleX2TextField.setText(Useful.double2String(x2));
                rectangleY1TextField.setText(Useful.double2String(z1));
                rectangleY2TextField.setText(Useful.double2String(z2));
            }
            
            void showControls() {
                shapeChooser.setSelectedItem("Rectangle");
            }
            
        }
        
        class Triangle extends Region {
            double x1, z1, x2, z2, x3, z3; // The coordinates of the corners of the triangle (in kilometres).
            
            Triangle(double newx1, double newz1, double newx2, double newz2, double newx3, double newz3, boolean isFilled) { // Create a new triangle with these coordinates.
                x1=newx1;
                z1=newz1;
                x2=newx2;
                z2=newz2;
                x3=newx3;
                z3=newz3;
                filled=isFilled;
            }
            
            void draw(Graphics graphicsContext) {                
                int[] xCoordinates=new int[3];
                int[] zCoordinates=new int[3];
                
                xCoordinates[0]=xKm2Pixel(x1);
                xCoordinates[1]=xKm2Pixel(x2);
                xCoordinates[2]=xKm2Pixel(x3);
                zCoordinates[0]=yKm2Pixel(z1);
                zCoordinates[1]=yKm2Pixel(z2);
                zCoordinates[2]=yKm2Pixel(z3);
                
                graphicsContext.setColor(getColour()); // Set to draw in the current region's colour. (How this is defined will depend on the type of region.)
                if (filled) {
                    graphicsContext.fillPolygon(xCoordinates, zCoordinates, 3); // Draw the triangle 
                } else {
                    graphicsContext.drawPolygon(xCoordinates, zCoordinates, 3); // Draw the triangle.
                }
            }
            
            protected Color getColour() { return Colours.white; }; // Will only be called by overridden versions of the class, so this is just a dummy to keep the compiler happy.
            
            String getName() { return ""; } // So far this is only applicable to material regions, so the implementation is a dummy to keep the compiler happy.
            
            String getShapeDescription() {
                return "triangle";
            }
            
            void update(double newx1, double newz1, double newx2, double newz2, double newx3, double newz3, boolean isFilled) {
                x1=newx1;
                z1=newz1;
                x2=newx2;
                z2=newz2;
                x3=newx3;
                z3=newz3;
                filled=isFilled;
            }
            
            void updateControls() {
                triangleX1TextField.setText(Useful.double2String(x1));
                triangleX2TextField.setText(Useful.double2String(x2));
                triangleX3TextField.setText(Useful.double2String(x3));
                triangleY1TextField.setText(Useful.double2String(z1));
                triangleY2TextField.setText(Useful.double2String(z2));
                triangleY3TextField.setText(Useful.double2String(z3));
            }
            
            void showControls() {
                shapeChooser.setSelectedItem("Triangle");
            }
            
        }
        
        class Circle extends Region { // This class is not abstract as we might want to define some elements during drawing, when we won't care what kind of circle this is.
            double x, z; // The coordinates of the circles centre (in kilometres).
            double radius; // The circles radius, also in kilometres.
            
            Circle(double newx, double newz, double newRadius, boolean isFilled) { // Create a new circle with these properties. (distances are in km)
                x=newx;
                z=newz;
                radius=newRadius;
                filled=isFilled;
            }
            
            void update(double newx, double newz, double newRadius, boolean isFilled) {
                x=newx;
                z=newz;
                radius=newRadius;
                filled=isFilled;
            }
            
            void draw(Graphics graphicsContext) {
                graphicsContext.setColor(getColour());
                if (filled) {
                    graphicsContext.fillOval(xKm2Pixel(x-radius), yKm2Pixel(z-radius), xKm2Pixel(radius*2)+1, yKm2Pixel(radius*2)+1); // +1 is added because width and height are counted from 1, not zero.
                } else {
                    graphicsContext.drawOval(xKm2Pixel(x-radius), yKm2Pixel(z-radius), xKm2Pixel(radius*2), yKm2Pixel(radius*2)); // No +1 is added because JAVA makes unfilled shapes one pixel bigger than specified by the coordinates.
                }
            }
            
            protected Color getColour() { return Colours.white; }; // Will only be called by overridden versions of the class, so this is just a dummy to keep the compiler happy.
            
            String getName() { return ""; } // So far this is only applicable to material regions, so this is a dummy to keep the compiler happy.
            
            String getShapeDescription() {
                return "circle";
            }
            
            void updateControls() {
                circleXTextField.setText(Useful.double2String(x));  
                circleZTextField.setText(Useful.double2String(z));
                circleRadiusTextField.setText(Useful.double2String(radius));
            }
            
            void showControls() {
                shapeChooser.setSelectedItem("Circle");
            }
            
        }
        
        class MaterialCircle extends Circle {
            Material circleMaterial;
            
            MaterialCircle(double x, double z, double radius, boolean isFilled, Material theMaterial) {
                super(x, z, radius, isFilled);
                circleMaterial=theMaterial;
            }
            
            Material getMaterial() {
                return circleMaterial;
            }
            
            protected Color getColour() {
                return circleMaterial.materialRheologyWindow.materialColourButton.getBackground();
            }
            
            String getName() { // Returns the materials name.
                return circleMaterial.materialRheologyWindow.materialNameTextBox.getText();
            }
        }
        
        class MaterialTriangle extends Triangle {
            Material triangleMaterial;
            
            MaterialTriangle(double x1, double z1, double x2, double z2, double x3, double z3, boolean isFilled, Material theMaterial) {
                super(x1, z1, x2, z2, x3, z3, isFilled);
                triangleMaterial=theMaterial;
            }
            
            Material getMaterial() {
                return triangleMaterial;
            }
            
            protected Color getColour() {
                return triangleMaterial.materialRheologyWindow.materialColourButton.getBackground();
            }
            
            String getName() {
                return triangleMaterial.materialRheologyWindow.materialNameTextBox.getText();
            }
        }
        
        class DrawingCircle extends Circle {
            Color circleColour;
            
            DrawingCircle(double x, double z, double radius, boolean isFilled, Color theColour) {
                super(x, z, radius, isFilled);
                circleColour=theColour;
            }
            
            protected Color getColour() {
                return circleColour;
            }
            
            String getName() { // Only applies to material regions, so this is a dummy to keep the compiler happy.
                return "";
            }
            
        }
        
        class MaterialRectangle extends MyRectangle {
            Material rectangleMaterial;
            
            MaterialRectangle(double x1, double z1, double x2, double z2, boolean isFilled, Material theMaterial) {
                super(x1, z1, x2, z2, isFilled);
                rectangleMaterial=theMaterial;
            }
            
            Material getMaterial() {
                return rectangleMaterial;
            }
            
            protected Color getColour() {
                return rectangleMaterial.materialRheologyWindow.materialColourButton.getBackground();
            }

            String getName() { // Returns the materials name.
                return rectangleMaterial.materialRheologyWindow.materialNameTextBox.getText();
            }         
            
        }
        
        class DrawingRectangle extends MyRectangle {
            Color rectangleColour;
            
            DrawingRectangle(double x1, double z1, double x2, double z2, boolean isFilled, Color newColour) {
                super(x1, z1, x2, z2, isFilled);
                rectangleColour=newColour;
            }
            
            protected Color getColour() {
                return rectangleColour;
            }
            
            String getName() { // Only applies to material regions, so this is a dummy to keep the compiler happy.
                return "";
            }
            
        }
        
        class ThermalRectangle extends MyRectangle {
            double halfWidth;
            double temperature;
            char overlaps;
            
            ThermalRectangle(double x1, double z1, double x2, double z2, boolean isFilled, double theHalfWidth, double theTemperature, char theOverlaps) {
                super(x1, z1, x2, z2, isFilled);
                halfWidth=theHalfWidth;
                temperature=theTemperature;
                overlaps=theOverlaps;
            }
            
            protected Color getColour() { // The higher the temperature, the lighter the colour in which the region is drawn.
                return Colours.temperature2Colour(temperature);
            }
            
            String getName() {
                return "";
            }
            
            void updateControls() {
                super.updateControls();
                halfWidthNumberBoxBean.setText(Useful.double2String(halfWidth));
                temperatureNumberBoxBean.setText(Useful.double2String(temperature));
                
                if (overlaps=='A') {
                    addRadioButton.setSelected(true);
                } if (overlaps=='M') {
                    multiplyRadioButton.setSelected(true);
                } if (overlaps=='R') {
                    replaceRadioButton.setSelected(true);
                }
            }
            
            void update(double newx1, double newz1, double newx2, double newz2, boolean isFilled) {
                super.update(newx1, newz1, newx2, newz2, isFilled);
                halfWidth=Double.parseDouble(halfWidthNumberBoxBean.getText());
                temperature=Double.parseDouble(temperatureNumberBoxBean.getText());
                if (addRadioButton.isSelected()) {
                    overlaps='A';
                } else {
                    if (multiplyRadioButton.isSelected()) {
                        overlaps='M';
                    } else {
                        overlaps='R';
                    }
                }
            }

        } // End of class ThermalRectangle

        class TracerRectangle extends MyRectangle {
            Color rectangleColour;
            int tracerDensity;
            
            TracerRectangle(double x1, double z1, double x2, double z2, boolean isFilled, Color newColour, int theDensity) {
                super(x1, z1, x2, z2, isFilled);
                rectangleColour=newColour;
                tracerDensity=theDensity;
            }
            
            protected Color getColour() {
                return rectangleColour;
            }
            
            String getName() { // Only applies to material regions, so this is a dummy to keep the compiler happy.
                return "";
            }
            
            void updateControls() {
                super.updateControls(); // Update the controls relevant to the shape.
                tracerDensityTextField.setText(""+tracerDensity); // Now update the controls relevant to the tracer.
                updateTracerColourChangeButton();
            }
            
            void updateTracerColourChangeButton() {
                tracerChangeColourButton.setBackground(rectangleColour);
                if ((rectangleColour.getRed()+rectangleColour.getBlue()+rectangleColour.getGreen())>382) { // If the chosen colour is light, use black text on the button, otherwise use white text.
                    tracerChangeColourButton.setForeground(java.awt.Color.black);
                } else {
                    tracerChangeColourButton.setForeground(java.awt.Color.white);
                }
            }
            
            void update(double newx1, double newz1, double newx2, double newz2, boolean isFilled) {
                super.update(newx1, newz1, newx2, newz2, isFilled);
                tracerDensity=Integer.parseInt(tracerDensityTextField.getText());
                rectangleColour=tracerChangeColourButton.getBackground();
            }

        }
        
        class BoundaryRectangle extends MyRectangle {
            Color rectangleColour;
            Boundary boundary;
            
            BoundaryRectangle(double x1, double z1, double x2, double z2, boolean isFilled, Color newColour, Boundary theBoundary) {
                super(x1, z1, x2, z2, isFilled);
                rectangleColour=newColour;
                boundary=theBoundary;
            }
            
            protected Color getColour() {
                return rectangleColour;
            }
            
            String getName() { // Return the name of the type of property.
                return boundary.getName();
            }
            
            void updateControls() {
                super.updateControls(); // Update the controls relevant to the shape.
                boundary.updateControls(); // Now update the controls relevant to the boundary.
                updateBoundaryColourChangeButton();
            }
            
            void updateBoundaryColourChangeButton() {
                boundaryChangeColourButton.setBackground(rectangleColour);
                if ((rectangleColour.getRed()+rectangleColour.getBlue()+rectangleColour.getGreen())>382) { // If the chosen colour is light, use black text on the button, otherwise use white text.
                    boundaryChangeColourButton.setForeground(java.awt.Color.black);
                } else {
                    boundaryChangeColourButton.setForeground(java.awt.Color.white);
                }
            }

            void update(double newx1, double newz1, double newx2, double newz2, boolean isFilled) {
                super.update(newx1, newz1, newx2, newz2, isFilled);
                boundary=new Boundary(); // The new boundary will get all the current parameters as set in the window objects.
                rectangleColour=boundaryChangeColourButton.getBackground();
            }
            
        }
        
        class ThermalCircle extends Circle {
            double halfWidth;
            double temperature;
            char overlaps;
            
            ThermalCircle(double x, double z, double radius, boolean isFilled, double theHalfWidth, double theTemperature, char theOverlaps) {
                super(x, z, radius, isFilled);
                halfWidth=theHalfWidth;
                temperature=theTemperature;
                overlaps=theOverlaps;
            }
            
            protected Color getColour() { // The higher the temperature, the lighter the colour in which the region is drawn.
                return Colours.temperature2Colour(temperature);
            }
            
            String getName() {
                return "";
            }
            
            void updateControls() {
                super.updateControls();
                halfWidthNumberBoxBean.setText(Useful.double2String(halfWidth));
                temperatureNumberBoxBean.setText(Useful.double2String(temperature));
                
                if (overlaps=='A') {
                    addRadioButton.setSelected(true);
                } if (overlaps=='M') {
                    multiplyRadioButton.setSelected(true);
                } if (overlaps=='R') {
                    replaceRadioButton.setSelected(true);
                }
            }
            
            void update(double newx, double newz, double newRadius, boolean isFilled) {
                super.update(newx, newz, newRadius, isFilled);
                halfWidth=Double.parseDouble(halfWidthNumberBoxBean.getText());
                temperature=Double.parseDouble(temperatureNumberBoxBean.getText());
                if (addRadioButton.isSelected()) {
                    overlaps='A';
                } else {
                    if (multiplyRadioButton.isSelected()) {
                        overlaps='M';
                    } else {
                        overlaps='R';
                    }
                }
            }

        } // End of class ThermalCircle

        class ThermalTriangle extends Triangle {
            double halfWidth;
            double temperature;
            char overlaps;
            
            ThermalTriangle(double x1, double z1, double x2, double z2, double x3, double z3, boolean isFilled, double theHalfWidth, double theTemperature, char theOverlaps) {
                super(x1, z1, x2, z2, x3, z3, isFilled);
                halfWidth=theHalfWidth;
                temperature=theTemperature;
                overlaps=theOverlaps;
            }
            
            protected Color getColour() { // The higher the temperature, the lighter the colour in which the region is drawn.
                return Colours.temperature2Colour(temperature);
            }
            
            String getName() {
                return "";
            }
            
            void updateControls() {
                super.updateControls();
                halfWidthNumberBoxBean.setText(Useful.double2String(halfWidth));
                temperatureNumberBoxBean.setText(Useful.double2String(temperature));
                
                if (overlaps=='A') {
                    addRadioButton.setSelected(true);
                } if (overlaps=='M') {
                    multiplyRadioButton.setSelected(true);
                } if (overlaps=='R') {
                    replaceRadioButton.setSelected(true);
                }
            }
            
            void update(double newx1, double newz1, double newx2, double newz2, double newx3, double newz3, boolean isFilled) {
                super.update(newx1, newz1, newx2, newz2, newx3, newz3, isFilled);
                halfWidth=Double.parseDouble(halfWidthNumberBoxBean.getText());
                temperature=Double.parseDouble(temperatureNumberBoxBean.getText());
                if (addRadioButton.isSelected()) {
                    overlaps='A';
                } else {
                    if (multiplyRadioButton.isSelected()) {
                        overlaps='M';
                    } else {
                        overlaps='R';
                    }
                }
            }

        } // End of class ThermalTriangle

        class BoundaryCircle extends Circle {
            Color circleColour;
            Boundary boundary;
            
            BoundaryCircle(double x, double z, double radius, boolean isFilled, Color theColour, Boundary theBoundary) {
                super(x, z, radius, isFilled);
                circleColour=theColour;
                boundary=theBoundary;
            }
            
            protected Color getColour() {
                return circleColour;
            }
            
            String getName() { // Return the name of the type of property.
                return boundary.getName();
            }
            
            void updateControls() {
                super.updateControls(); // Update the controls relevant to the shape.
                boundary.updateControls(); // Now update the controls relevant to the boundary.
                updateBoundaryColourChangeButton();
            }
            
            void updateBoundaryColourChangeButton() {
                boundaryChangeColourButton.setBackground(circleColour);
                if ((circleColour.getRed()+circleColour.getBlue()+circleColour.getGreen())>382) { // If the chosen colour is light, use black text on the button, otherwise use white text.
                    boundaryChangeColourButton.setForeground(java.awt.Color.black);
                } else {
                    boundaryChangeColourButton.setForeground(java.awt.Color.white);
                }
            }
            
            void update(double newx, double newz, double newradius, boolean isFilled) {
                super.update(newx, newz, newradius, isFilled);
                boundary=new Boundary(); // The new boundary will get all the current parameters as set in the window objects.
                circleColour=boundaryChangeColourButton.getBackground();
            }
            
        }
        
        class Boundary { // Just a class to hold the properties of each boundary (except for the boundary's colour, which is stored seperately).
            static final char xAxis='X'; // Define some symbolic constants.
            static final char yAxis='Y';
            static final char zAxis='Z';
            
            static final int temperature=0;
            static final int stress_x=1;
            static final int stress_z=2;
            static final int velocity_x=3;
            static final int velocity_z=4;
            
            char axis;
            double intercept;
            int property;
            double halfWidth;
            double value;

            Boundary() { // Create a boundary based on the current states of the window objects.
                getProperties();
            }
            
            Boundary(char newAxis, double newIntercept, int newProperty, double newHalfWidth, double newValue) { 
                axis=newAxis;
                property=newProperty;
                halfWidth=newHalfWidth;
                value=newValue;
                intercept=newIntercept;
            }
            
            double scaleValue(double value, int property) { // This converts from ellipsis units to SI ones or KMs.
                if (property==temperature)
                    return scaling.ellipsisUnits2kelvin(value);
                else if (property==stress_x || property==stress_z)
                    return scaling.ellipsisUnits2stress(value);

                return scaling.ellipsisUnits2velocity(value);
            }
            
            double scaleValueToEllipsisUnits(double value, int property) { // This converts from SI/KM to ellipsis units.
                if (property==temperature)
                    return scaling.kelvin2EllipsisUnits(value);
                else if (property==stress_x || property==stress_z)
                    return scaling.stress2EllipsisUnits(value);

                return scaling.velocity2EllipsisUnits(value);
            }
            
            void updateControls() { // Update the window controls based on this boundary's properties.
                boundaryPropertyComboBox.setSelectedIndex(property);

                if (axis==xAxis) {
                    boundaryXAxisRadioButton.setSelected(true);
                } else if (axis==yAxis) {
                    boundaryYAxisRadioButton.setSelected(true);
                } else {
                    boundaryZAxisRadioButton.setSelected(true);
                }
                
                boundaryInterceptNumberBoxBean.setText(Useful.double2String(intercept));
                boundaryHalfWidthTextField.setText(Useful.double2String(halfWidth));
                boundaryValueTextField.setText(Useful.double2String(value));
            }
            
            void getProperties() { // Set this boundary based on the values in the window.
                property=boundaryPropertyComboBox.getSelectedIndex();

                if (boundaryXAxisRadioButton.isSelected()) {
                    axis=xAxis;
                } else if (boundaryYAxisRadioButton.isSelected()) {
                    axis=yAxis;
                } else {
                    axis=zAxis;
                }
                
                intercept=Double.parseDouble(boundaryInterceptNumberBoxBean.getText());
                halfWidth=Double.parseDouble(boundaryHalfWidthTextField.getText());
                value=Double.parseDouble(boundaryValueTextField.getText());               
            }
            
            String getName() {
                if (property==temperature)
                    return "Temperature";
                if (property==stress_x)
                    return "Stress (x axis)";
                if (property==stress_z)
                    return "Stress (z axis)";
                if (property==velocity_x)
                    return "Velocity (x axis)";
                if (property==velocity_z)
                    return "Velocity (z axis)";
                return null; // We should never get here!
            }
                        
        } // End of class Boundary.
        
    } // End of BackgroundImageClass

    // N.B. it is probably best to leave gravity unscaled and specified in m/s/s
    class Scaling { // Controls all scaling of parameters between SI units and KM, and ellipsis units
        // In GUI we will use KM and Millions of years
        // But in scaling we will use m and s
        // This is because a lot of other SI units incorporate m and s
        
	//*********************** Declare and initialise INDEPENDENT SCALING PARAMETERS used in the Autoscale function (this is below). SD
	
        double ellipsisUnitsPerM=1.0;		// Distance
        double ellipsisUnitsPerKG=1.0;		// Density
        double ellipsisUnitsPerKelvin=1.0;	// Temperature
        double ellipsisUnitsPerSecond=1.0;	// Time
        
        //*********************** This is not valid anymore. This comes from Craigs scaling which we dont use anymore and Specific heat is now a dependent variabled scaled as a function of the independent variables. SD
        
        double ellipsisUnitsPerSpecificHeat=1.0; // N.B. adding this makes scaling inconsistent, as this isn't an independent dimension. But it's done this way because we want to be able to scale specific heat to small numbers.
        // Are there any other dimensions???
        // Are we definetly going to use KM not metres.
        
	
        Scaling() { // Initialise the conversion parameters to those chosen by autoscale.
            autoscale();
        }
        
	//*********************** This decides which scaling method to use. If autoscaling is not selected then parameter values are obtained from NumberBoxBean. SD
        //*********************** This needs to be changed so that there are only 2 options for scaling: (1) Scaled (2) Do not scale SD
        //*********************** For option (1) the user can define the 5 Independent variables and for option (2) the independent sacling factors are equal to one SD
	
        void updateScalingFactors() { // Set the scaling factors above, based on the current state of the GUI. This just reads in the four basic scaling factors.
            if (automaticallyScaleParametersRadioButton.isSelected()) { // If we'return using autscaling the factors are calculated.
                autoscale();
            } else { // Otherwise we get them from the GUI.
                
                //*********************** These are the current Independent Variables. They are wrong, should be Length, Temperature, Gravity, Diffusivity and Viscosity. SD
                
                ellipsisUnitsPerM=Double.parseDouble(ellipsisUnitsPerMNumberBoxBean.getText());
//                ellipsisUnitsPerKG=Double.parseDouble(ellipsisUnitsPerKGNumberBoxBean.getText());
                ellipsisUnitsPerSecond=ellipsisUnitsPerM*ellipsisUnitsPerM/Double.parseDouble(thermalDiffusivityScalingNumberBoxBean.getText()); // New basic scaling factor, but must use it to set seconds.
//                ellipsisUnitsPerSecond=Double.parseDouble(ellipsisUnitsPerSecondNumberBoxBean.getText());
                ellipsisUnitsPerKelvin=Double.parseDouble(ellipsisUnitsPerKelvinNumberBoxBean.getText());
                ellipsisUnitsPerKG=Double.parseDouble(viscosityScalingNumberBoxBean.getText())*ellipsisUnitsPerM*ellipsisUnitsPerSecond;
                ellipsisUnitsPerSpecificHeat=Double.parseDouble(ellipsisUnitsPerSpecificHeatNumberBoxBean.getText());
            }
            
            setAlternativeScalingFactors(); // Set the other scaling factors based on the five basic ones.
        }

        //*********************** This section calculates the Alternative (Dependent) scaling parameters based on the Independent scaling paramaters declared in previous statment. SD
        //*********************** This section needs to be expanded to included all dependent variables. SD
        
        void setAlternativeScalingFactors() {
//            gravityScalingNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerM/ellipsisUnitsPerSecond/ellipsisUnitsPerSecond)); // Gravity is now never scaled at all.
            ellipsisUnitsPerSecondNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerSecond));
            ellipsisUnitsPerKGNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerKG));
//            thermalDiffusivityScalingNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerM*ellipsisUnitsPerM/ellipsisUnitsPerSecond));
            densityScalingNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerKG/ellipsisUnitsPerM/ellipsisUnitsPerM/ellipsisUnitsPerM));
            thermalExpansionScalingNumberBoxBean.setText(Useful.double2String(1/ellipsisUnitsPerKelvin));
//            viscosityScalingNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerKG/ellipsisUnitsPerM/ellipsisUnitsPerSecond));
        }
        
	//*********************** This is the beginning of the Autoscale section. NOTE that length scale is reset here so that the depth of the box always scales to one. SD
                
        void autoscale() { // Automatically work out scaling factors for all of the dimensions, and store them in the object. and update the GUI NUmberBoxBeans.
            ellipsisUnitsPerM=1.0/(windowDimensionY*1000.0); // First set the lenth factor so that the screen is one unit high. (N.B. *1000 is kilometres to metres conversion).

	    //*********************** The below scaling for Time is wrong. Seconds should be scaled as a function of Length and Thermal Diffusivity. Time is NOT an independent variable. SD
	    
            ellipsisUnitsPerSecond=Math.sqrt(ellipsisUnitsPerM); // Scale seconds so that gravitational acceleration is actually not scaled at all.
            // KG are not currently scaled - probably they should be, perhaps based on the maximum viscosity value specified.
            // Kelvin is not currently scaled - probably this should be based on the maximum temperature specified anywhere. - or on the coefficient of thermal expansion.

            setRegularScalingFactors(); // Now update the window to reflect the new values.
            setAlternativeScalingFactors();
        }
        
        void setRegularScalingFactors() { // Set the text boxes showing the regular scaling factors based on the values stored in this object.
            ellipsisUnitsPerMNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerM));
//            ellipsisUnitsPerKGNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerKG));
//            ellipsisUnitsPerSecondNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerSecond));
            ellipsisUnitsPerKelvinNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerKelvin));
            thermalDiffusivityScalingNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerM*ellipsisUnitsPerM/ellipsisUnitsPerSecond));
            viscosityScalingNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerKG/ellipsisUnitsPerM/ellipsisUnitsPerSecond));
            ellipsisUnitsPerSpecificHeatNumberBoxBean.setText(Useful.double2String(ellipsisUnitsPerSpecificHeat));
        }
        
        //*********************** Not really sure what the below code means SD
        
/*        // The following functions are to allow scaling to be done manually. -- not needed as scaling factors are class scope.
        void setKMScale(double newEllipsisUnitsPerM) {
            ellipsisUnitsPerM=newEllipsisUnitsPerM;
        }
        
        double getKMScale() {
            return ellipsisUnitsPerM;
        }
        
        // Etc.
  */      
        UnitScaler getEllipsisUnits2KMScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2KM(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2KelvinScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2kelvin(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2SScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2S(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2VelocityScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2velocity(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2ViscosityScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2viscosity(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2powerPerUnitMassScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2powerPerUnitMass(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2SpecificHeatScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2specificHeat(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2ThermalDiffusivityScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2thermalDiffusivity(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2CoefficientOfThermalExpansionScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2CoefficientOfThermalExpansion(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2AccelerationScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2acceleration(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2StressScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2stress(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2PerSecondScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2perSecond(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2PascalsScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2pascals(value);
                }
            };
        }
        
        UnitScaler getEllipsisUnits2DensityScaler() {
            return new UnitScaler() {
                public double scale(double value) {
                    return ellipsisUnits2density(value);
                }
            };
        }
        
        // The following functions are to be used to convert between ellipsis units, SI units and units used in the GUI and output.
        
        //*********************** A lot of this seems to be redundant. We will instead implement the same method of Scaling that Scott and Patrice use. SD
        //*********************** Need to set the below variables as functions of Independent scaling parameters. (How does this compare to scaling calculations carried out in AlternativeScalingFactor section?) SD
        //*********************** What is the code below doing? Is it defining methods for sacling conversion? ie. perSecond2ellipsisUnits is a method for scaling a "real world" value (value of type double) SD
        //*********************** to a scaled ellipsis input parameter? Is so the conversion is wrong (as I read it) since we will use the non-dimensional approach where the returned ellipsis input variable should be 
        //*********************** calculated as value/dimensionalScalingFactorTime

        double perSecond2ellipsisUnits(double value) {
            return value/ellipsisUnitsPerSecond;
        }
        
        double perSecond2ellipsisUnits(String value) {
            return perSecond2ellipsisUnits(Double.parseDouble(value));
        }
        
        double ellipsisUnits2perSecond(double value) {
            return value*ellipsisUnitsPerSecond;
        }
        
        double joulePerMolePerKelvin2ellipsisUnits(double value) {
            return joulesPerMole2ellipsisUnits(value)/ellipsisUnitsPerKelvin;
        }
        
        double ellipsisUnits2joulePerMolePerKelvin(double value) {
            return ellipsisUnits2joulesPerMole(value)*ellipsisUnitsPerKelvin;
        }
        
        double joulesPerMole2ellipsisUnits(double value) { // This and the next function assume that moles don't have to be scaled - this is just a constant (Is this correct?)
            return joules2ellipsisUnits(value);
        }
        
        double ellipsisUnits2joulesPerMole(double value) {
            return ellipsisUnits2joules(value);
        }
        
                //*********************** Is joules a class? Where was it defined? Or is it just a variable declaration? SD
        
        double joules2ellipsisUnits(double joules) {
            return joules*ellipsisUnitsPerKG*ellipsisUnitsPerM*ellipsisUnitsPerM/ellipsisUnitsPerSecond/ellipsisUnitsPerSecond;
        }
        
        double ellipsisUnits2joules(double ellipsisEnergy) {
            return ellipsisEnergy/ellipsisUnitsPerKG/ellipsisUnitsPerM/ellipsisUnitsPerM*ellipsisUnitsPerSecond*ellipsisUnitsPerSecond;
        }
        
        double pascals2ellipsisUnits(double pascals) {
            return pascals*ellipsisUnitsPerKG/ellipsisUnitsPerM/ellipsisUnitsPerSecond/ellipsisUnitsPerSecond;
        }
        
        double pascals2ellipsisUnits(String pascals) {
            return pascals2ellipsisUnits(Double.parseDouble(pascals));
        }
        
        double ellipsisUnits2pascals(double ellipsisPressure) {
            return ellipsisPressure/ellipsisUnitsPerKG*ellipsisUnitsPerM*ellipsisUnitsPerSecond*ellipsisUnitsPerSecond;
        }
        
        double pascalSeconds2ellipsisUnits(double pascalSeconds) {
            return pascalSeconds*ellipsisUnitsPerKG/ellipsisUnitsPerM/ellipsisUnitsPerSecond;
        }
        
        double ellipsisUnits2pascalSeconds(double value) {
            return value/ellipsisUnitsPerKG*ellipsisUnitsPerM*ellipsisUnitsPerSecond;
        }
        
        double ellipsisUnits2powerPerUnitMass(double value) { // Corrected on 18-2-4 (mass term removed).
            return value/ellipsisUnitsPerM/ellipsisUnitsPerM*ellipsisUnitsPerSecond*ellipsisUnitsPerSecond*ellipsisUnitsPerSecond; // Dimension is kgm2s-3
        }
        
        double powerPerUnitMass2ellipsisUnits(String value) {
            return powerPerUnitMass2ellipsisUnits(Double.parseDouble(value));
        }
        
        double powerPerUnitMass2ellipsisUnits(double value) { // Corrected on 18-2-4
            return value*ellipsisUnitsPerM*ellipsisUnitsPerM/ellipsisUnitsPerSecond/ellipsisUnitsPerSecond/ellipsisUnitsPerSecond;
        }
        
        double ellipsisUnits2specificHeat(double value) {
            return value/ellipsisUnitsPerSpecificHeat;
            //            return value/ellipsisUnitsPerM/ellipsisUnitsPerM*ellipsisUnitsPerSecond*ellipsisUnitsPerSecond*ellipsisUnitsPerKelvin; // Specific hear is m2s-2k-1
        }
        
        double specificHeat2ellipsisUnits(String value) {
            return specificHeat2ellipsisUnits(Double.parseDouble(value));
        }
        
        double specificHeat2ellipsisUnits(double value) {
            return value*ellipsisUnitsPerSpecificHeat; // This is the new version where specific heat is scaled separately to the other parameters.
            //return value*ellipsisUnitsPerM*ellipsisUnitsPerM/ellipsisUnitsPerSecond/ellipsisUnitsPerSecond/ellipsisUnitsPerKelvin;
        }
        
        double ellipsisUnits2thermalDiffusivity(double value) {
            return value/ellipsisUnitsPerM/ellipsisUnitsPerM*ellipsisUnitsPerSecond; // Thermal diffusivity is m2s-1
        }
        
        double thermalDiffusivity2ellipsisUnits(String value) {
            return thermalDiffusivity2ellipsisUnits(Double.parseDouble(value));
        }
        
        double thermalDiffusivity2ellipsisUnits(double value) {
            return value*ellipsisUnitsPerM*ellipsisUnitsPerM/ellipsisUnitsPerSecond;
        }
        
        double ellipsisUnits2CoefficientOfThermalExpansion(double value) {
            return value*ellipsisUnitsPerKelvin; // Dimension is k-1, so this is correct.
        }
        
        double coefficientOfThermalExpansion2ellipsisUnits(String value) {
            return coefficientOfThermalExpansion2ellipsisUnits(Double.parseDouble(value));
        }
        
        double coefficientOfThermalExpansion2ellipsisUnits(double value) {
            return value/ellipsisUnitsPerKelvin;
        }
        
        double M2EllipsisUnits(double M) {
            return M*ellipsisUnitsPerM;
        }
        
        double KM2EllipsisUnits(double KM) {
            return KM*1000.0*ellipsisUnitsPerM;
        }
        
        double ellipsisUnits2M(double ellipsisLengthUnits) {
            return ellipsisLengthUnits/ellipsisUnitsPerM;
        }
        
        double ellipsisUnits2KM(double ellipsisLengthUnits) {
            return ellipsisUnits2M(ellipsisLengthUnits)/1000.0;
        }
        
        double ellipsisUnits2S(double ellipsisTimeUnits) {
            return ellipsisTimeUnits/ellipsisUnitsPerSecond;
        }
        
        double S2EllipsisUnits(double seconds) {
            return seconds*ellipsisUnitsPerSecond;
        }
        
        double S2EllipsisUnits(String seconds) {
            return S2EllipsisUnits(Double.parseDouble(seconds));
        }
        
        double kelvin2EllipsisUnits(double kelvin) {
            return kelvin*ellipsisUnitsPerKelvin; 
        }
        
        double kelvin2EllipsisUnits(String kelvin) {
            return kelvin2EllipsisUnits(Double.parseDouble(kelvin));
        }
        
        double ellipsisUnits2kelvin(double ellipsisTemperatureUnits) {
            return ellipsisTemperatureUnits/ellipsisUnitsPerKelvin;
        }
        
        double stress2EllipsisUnits(double stress) {
            return stress*ellipsisUnitsPerKG/ellipsisUnitsPerSecond/ellipsisUnitsPerSecond/ellipsisUnitsPerM;
        }
        
        double stress2EllipsisUnits(String stress) {
            return stress2EllipsisUnits(Double.parseDouble(stress));
        }
        
        double ellipsisUnits2stress(double ellipsisStress) {
            return ellipsisStress/ellipsisUnitsPerKG*ellipsisUnitsPerSecond*ellipsisUnitsPerSecond*ellipsisUnitsPerM;
        }
        
        double ellipsisUnits2viscosity(double ellipsisViscosity) {
            return ellipsisViscosity/ellipsisUnitsPerKG*ellipsisUnitsPerM*ellipsisUnitsPerSecond; 
        }
        
        double viscosity2EllipsisUnits(double viscosity) {
            return viscosity*ellipsisUnitsPerKG/ellipsisUnitsPerM/ellipsisUnitsPerSecond; 
        }
        
        double viscosity2EllipsisUnits(String viscosity) {
            return viscosity2EllipsisUnits(Double.parseDouble(viscosity));
        }
        
        double density2EllipsisUnits(double density) {
            return density*ellipsisUnitsPerKG/ellipsisUnitsPerM/ellipsisUnitsPerM/ellipsisUnitsPerM;
        }
        
        double density2EllipsisUnits(String density) {
            return Double.parseDouble(density);
        }
        
        double ellipsisUnits2density(double ellipsisDensity) {
            return ellipsisDensity/ellipsisUnitsPerKG*ellipsisUnitsPerM*ellipsisUnitsPerM*ellipsisUnitsPerM;
        }
        
        double velocity2EllipsisUnits(double velocity) {
            return velocity*ellipsisUnitsPerM/ellipsisUnitsPerSecond;
        }
        
        double velocity2EllipsisUnits(String velocity) {
            return velocity2EllipsisUnits(Double.parseDouble(velocity));
        }
                
        double ellipsisUnits2velocity(double ellipsisVelocity) {
            return ellipsisVelocity/ellipsisUnitsPerM*ellipsisUnitsPerSecond;
        }
        
        double acceleration2EllipsisUnits(double acceleration) {
            return acceleration; // Gravity is always lleft unscaled now.
//            return acceleration*ellipsisUnitsPerM/ellipsisUnitsPerSecond/ellipsisUnitsPerSecond;
        }
        
        double acceleration2EllipsisUnits(String acceleration) {
            return acceleration2EllipsisUnits(Double.parseDouble(acceleration));
        }
        
        double ellipsisUnits2acceleration(double ellipsisAcceleration) {
            return ellipsisAcceleration; // Gravity is always left unscaled.
  //          return ellipsisAcceleration/ellipsisUnitsPerM*ellipsisUnitsPerSecond*ellipsisUnitsPerSecond;
        }
            
    } // End of Scaling Class
    
    class SamplingTracer {
        static final int TEMPERATURE=1;
        static final int X_VELOCITY=2;
        static final int Z_VELOCITY=3;
        static final int NODAL_PRESSURE=4;
        static final int STRAIN_RATE=5;
        static final int STRESS=6;
        static final int TOTAL_STRESS=7;
        static final int DEPLETION=14;
        
        static final int UNRECOGNISED_OPTION=-1;
        
        static final int X_AXIS=1;
        static final int Z_AXIS=2; //*** Note screwed around numbering system.
        static final int Y_AXIS=3;

        int Sampling_lagrangian;
        int Sampling_plot_num;
        double Sampling_x;
        double Sampling_z;
        int Sampling_field;
        int Sampling_dirn;
        int Sampling_normalize;
        Color profileColour; // The color is stored in Java format, although it will be input and output as three separate components.

        SamplingTracer() { // Create a new sampling tracer based on the settings of the window objects.
            if (eulerianRadioButton.isSelected()) {
                Sampling_lagrangian=0;
            } else {
                Sampling_lagrangian=1;
            }
            Sampling_plot_num=samplingPPMComboBox.getSelectedIndex(); // N.B. This assumes that PPM files are numbered from 0 - which might not always be correct.
            Sampling_x=Double.parseDouble(Sampling_xNumberBoxBean.getText());
            Sampling_z=Double.parseDouble(Sampling_zNumberBoxBean.getText());
            
            Sampling_field=comboBoxItem2Field(Sampling_fieldComboBox.getSelectedIndex());
            Sampling_dirn=Sampling_dirnComboBox.getSelectedIndex()+1;
            
            if (Sampling_normalizeCheckBox.isSelected()) {
                Sampling_normalize=1;
            } else {
                Sampling_normalize=0;
            }
            
            profileColour=profileColourButton.getBackground();
        }
        
        SamplingTracer(int initialSampling_lagrangian, int initialSampling_plot_num, double initialSampling_x, double initialSampling_z, int initialSampling_field, int initialSampling_dirn, int initialSampling_normalize, double initialSampling_R, double initialSampling_G, double initialSampling_B) {
            if (initialSampling_lagrangian!=0 && initialSampling_lagrangian!=1) {
                JOptionPane.showMessageDialog(MainWindow.this, "One of the values specified in the parameter \"Sampling_lagrangian\" was invalid.\nThis value has been changed to 0 (Eulerian).", "Warning", JOptionPane.WARNING_MESSAGE);
                initialSampling_lagrangian=0;
            }
            Sampling_lagrangian=initialSampling_lagrangian;

            if (initialSampling_plot_num<0) {
                JOptionPane.showMessageDialog(MainWindow.this, "One of the values specified in the parameter \"Sampling_plot_num\" was invalid.\nThis value has been changed to 0.", "Warning", JOptionPane.WARNING_MESSAGE);
                initialSampling_plot_num=0;
            }
            Sampling_plot_num=initialSampling_plot_num;

            Sampling_x=initialSampling_x;
            Sampling_z=initialSampling_z;
  
            // Don't warn about bad values for sampling field - just store them the GUI doesn't recognise all the allowable values.
//            if (initialSampling_field<1 || initialSampling_field>7) {
//                JOptionPane.showMessageDialog(MainWindow.this, "One of the values specified in the parameter \"Sampling_field\" was invalid.\nThis value has been changed to 1 (Temperature).", "Warning", JOptionPane.WARNING_MESSAGE);
//                initialSampling_field=1;
//            }
            Sampling_field=initialSampling_field;
            
            if (initialSampling_dirn<1 || initialSampling_dirn>3) {
                JOptionPane.showMessageDialog(MainWindow.this, "One of the values specified in the parameter \"Sampling_dirn\" was invalid.\nThis value has been changed to 1 (Parallel to X axis).", "Warning", JOptionPane.WARNING_MESSAGE);
                initialSampling_dirn=1;
            }
            Sampling_dirn=initialSampling_dirn;
            
            if (initialSampling_normalize!=0 && initialSampling_normalize!=1) {
                JOptionPane.showMessageDialog(MainWindow.this, "One of the values specified in the parameter \"Sampling_normalize\" was invalid.\nThis value has been changed to 0 (Unnormalised).", "Warning", JOptionPane.WARNING_MESSAGE);
                initialSampling_normalize=0;
            }
            Sampling_normalize=initialSampling_normalize;
            
            profileColour=Colours.RGB2JavaFormat(initialSampling_R, initialSampling_G, initialSampling_B);
        }
        
        void updateWindowObjects() { // Update the window object to reflect the properties of this SamplingTracer.
            
            if (Sampling_lagrangian==0) {
                eulerianRadioButton.setSelected(true);
            } else {
                lagrangianRadioButton.setSelected(true);
            }
            
            if (samplingPPMComboBox.getItemCount()<=Sampling_plot_num || Sampling_plot_num<0) { // If there is no PPM defined corresponding to Sampling_plot_num we can't choose it in the combobox - so warn about this, and choose the first PPM instead. Also, if for any reason samplingPlotNumber is zero, display this message.
                JOptionPane.showMessageDialog(MainWindow.this, "The PPM file in which the currently selected sampling tracer's profile should be stored does not exist.\nPPM file 1 will be displayed instead, but this problem should be fixed before an attempt is made to use this file with Ellipsis.", "Warning", JOptionPane.WARNING_MESSAGE);
                if (samplingPPMComboBox.getItemCount()!=0) { // If the combo box is currently empty, don't try to set it.
                    samplingPPMComboBox.setSelectedIndex(0); 
                }
            } else {
                samplingPPMComboBox.setSelectedIndex(Sampling_plot_num); // Otherwise set the combo box to reflect the chosen sampling tracer.
            }
            
            Sampling_xNumberBoxBean.setText(""+Sampling_x);
            Sampling_zNumberBoxBean.setText(""+Sampling_z);
            
            Sampling_fieldComboBox.setSelectedIndex(field2ComboBoxItem(Sampling_field)); 

            Sampling_dirnComboBox.setSelectedIndex(Sampling_dirn-1);
            
            if (Sampling_normalize==0) {
                Sampling_normalizeCheckBox.setSelected(false);
            } else {
                Sampling_normalizeCheckBox.setSelected(true);
            }
            
            profileColourButton.setBackground(profileColour);
            profileColourButton.setForeground(Colours.contrast(profileColour));
        }
        
        int field2ComboBoxItem(int field) { // convert between which field has been chosen, and its index in the combo box. (Combo box elements are indexed from zero, depletion's code is 14 - but we can have unrecognised options too).
            if (field==DEPLETION)
                return 7;
            if (field>=TEMPERATURE && field<=TOTAL_STRESS)
                return field-1;
            return 8; // If no other options apply, we need to select unrecognised option in the combo box.
        }
        
        int comboBoxItem2Field(int item) { // Converts from the item chosen in the combo box to the code for the corresponding field.
            if (item==7)
                return DEPLETION;
            if (item==8) // This is if the user has selected unrecognised option - not something they should really do.
                return UNRECOGNISED_OPTION;
            return item+1; // Otherwise the code is just one more than the index of the corresponding field in the combo box.
        }
    } // End of class SamplingTracer
        
    class PPM_file {
        static final int TEMPERATURE=1;
        static final int TRACER_WEIGHT=2;
        static final int DEVIATORIC_STRESS=3;
        static final int PRESSURE=4;
        static final int ACCUMULATED_PLASTIC_STRAIN=8;
        static final int DEPLETION=13;

        static final int UNRECOGNISED_OPTION=-1;
        
        int height;
        int coloring;
        
        PPM_file() { // Create a new PPM_file, based on the settings of the window objects.
            height=Integer.parseInt(PPM_heightNumberBoxBean.getText());
            coloring=comboBoxItem2Field(PPMColoringComboBox.getSelectedIndex());
        }
        
        PPM_file(int initialHeight, int initialColoring) {
//            if (initialColoring>=5) { // If we don't recognise the option, warn about it, then just set it to temperature.
//               JOptionPane.showMessageDialog(MainWindow.this, "One of the values specified in the parameter \"PPM_coloring\" was "+initialColoring+".\nThis value was not recognised by Ellipsis GUI, and it has been changed to 1 (Temperature).", "Warning", JOptionPane.WARNING_MESSAGE);
//                initialColoring=1;
//            }
            height=initialHeight;
            coloring=initialColoring;
        }
        
        void updateWindowObjects() { // Update the window, so that it displays the settings for this PPM_file
            PPM_heightNumberBoxBean.setText(""+height);
            PPMColoringComboBox.setSelectedIndex(field2ComboBoxItem(coloring));
        }
        
        int field2ComboBoxItem(int field) { // convert between which field has been chosen, and its index in the combo box. (Combo box elements are indexed from zero, accumulated plastic strain is the 5th item, not the 8th one.
            if (field==ACCUMULATED_PLASTIC_STRAIN) {
                return 4;
            } else if (field==DEPLETION) {
                return 5;
            } else if (field>=TEMPERATURE && field<=PRESSURE) {
                return field-1;
            } else { // If we can't match the field to anything, just report that it is an unrecognised option.
                return 6;
            }
        }
        
        int comboBoxItem2Field(int comboBoxIndex) { // convert between the index of the field chosen in the combo box, and its field number.
            if (comboBoxIndex==4) 
                return ACCUMULATED_PLASTIC_STRAIN;
            if (comboBoxIndex==5)
                return DEPLETION;
            if (comboBoxIndex==6) // If the user has selected unrecognised option in the combo box, then return the code for it - they should never really do this of course.
                return UNRECOGNISED_OPTION;
            else
                return comboBoxIndex+1;
        }
        
    } // End of PPM_file class
    
} // End of MainWindow Class

